@@ -1,39 +1,30 @@
 /*
- * static char *rcsid_anim_c =
- *   "$Id: anim.c 16220 2011-12-18 19:14:52Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2003 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file anim.c
  * This file contains animation-related code.
  **/
 
-#include <global.h>
-#include <stdio.h>
+#include "global.h"
+
 #include <assert.h>
+#include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+static int animations_allocated = 0;
 
 /**
  * Clears all animation-related memory.
@@ -56,7 +47,8 @@ void init_anim(void) {
     char buf[MAX_BUF];
     FILE *fp;
     int num_frames = 0, i;
-    const New_Face *faces[MAX_ANIMATIONS];
+    const Face *faces[MAX_ANIMATIONS];
+    Animations *alloc_ptr; // Used to temporarily catch the result of realloc, in case of failure.
 
     animations_allocated = 9;
     num_animations = 0;
@@ -71,14 +63,14 @@ void init_anim(void) {
      */
     animations[0].name = add_string("###none");
     animations[0].num_animations = 1;
-    animations[0].faces = malloc(sizeof(New_Face*));
-    animations[0].faces[0] = &new_faces[find_face("bug.111", 0)];
+    animations[0].num = 0;
+    animations[0].faces = malloc(sizeof(Face*));
+    animations[0].faces[0] = find_face("bug.111", NULL);
     animations[0].facings = 0;
 
     snprintf(buf, sizeof(buf), "%s/animations", settings.datadir);
-    LOG(llevDebug, "Reading animations from %s...\n", buf);
     if ((fp = fopen(buf, "r")) == NULL) {
-        LOG(llevError, "Cannot open animations file %s: %s\n", buf, strerror_local(errno, buf, sizeof(buf)));
+        LOG(llevError, "Cannot open animations file %s: %s\n", buf, strerror(errno));
         exit(-1);
     }
     while (fgets(buf, MAX_BUF-1, fp) != NULL) {
@@ -93,45 +85,73 @@ void init_anim(void) {
                 LOG(llevError, "Didn't get a mina before %s\n", buf);
                 num_frames = 0;
             }
-            num_animations++;
+            ++num_animations;
             if (num_animations == animations_allocated) {
-                animations = realloc(animations, sizeof(Animations)*(animations_allocated+10));
-                animations_allocated += 10;
+                /*
+                 * We will realloc fewer times if we take larger steps for the realloc size.
+                 * This should prevent some copy operations during bootup, albeit at the expense of some RAM.
+                 * Most implementations of variable-sized things double each realloc for balance between
+                 * space and time efficiency.
+                 * Since animations_allocated is one less than the actual amount,
+                 * we aren't quite doubling the first allocation size.
+                 * Nevertheless, we skip by larger after the first realloc.
+                 * We should have enough animations for this to be worthwhile.
+                 *
+                 * While we are here, handle an allocation failure more gracefully than the inevitable segfault.
+                 *
+                 * Daniel Hawkins 2018-01-23
+                 */
+                alloc_ptr = realloc(animations, sizeof(Animations)*(animations_allocated << 1));
+                if (alloc_ptr)
+                {
+                    animations = alloc_ptr;
+                    animations_allocated <<= 1;
+                }
+                else
+                {
+                    LOG(llevError, "anim: animation array reallocation failed at size %d resizing to %d.\n",
+                        animations_allocated, animations_allocated << 1);
+                    // We will not have our animations, so terminate gracefully.
+                    fatal(OUT_OF_MEMORY);
+                }
             }
             animations[num_animations].name = add_string(buf+5);
             animations[num_animations].num = num_animations; /* for bsearch */
             animations[num_animations].facings = 1;
         } else if (!strncmp(buf, "mina", 4)) {
-            animations[num_animations].faces = malloc(sizeof(New_Face*)*num_frames);
+            assert(num_frames > 0);
+            animations[num_animations].faces = malloc(sizeof(Face*)*num_frames);
             for (i = 0; i < num_frames; i++)
                 animations[num_animations].faces[i] = faces[i];
             animations[num_animations].num_animations = num_frames;
             if (num_frames <= 1) {
-                LOG(llevDebug, "Animation %s has only one or none face!\n", animations[num_animations].name);
+                LOG(llevDebug, "anim: %s has less then two faces\n",
+                        animations[num_animations].name);
             }
             if (num_frames%animations[num_animations].facings) {
-                LOG(llevDebug, "Animation %s frame numbers (%d) is not a multiple of facings (%d)\n",
-                    animations[num_animations].name, num_frames, animations[num_animations].facings);
+                LOG(llevDebug, "anim: %s has %d frames: not a multiple of facings (%d)\n",
+                        animations[num_animations].name, num_frames,
+                        animations[num_animations].facings);
             }
             num_frames = 0;
         } else if (!strncmp(buf, "facings", 7)) {
             if (!(animations[num_animations].facings = atoi(buf+7))) {
-                LOG(llevDebug, "Animation %s has 0 facings, line=%s\n",
+                LOG(llevDebug, "anim: %s has 0 facings (line %s)\n",
                     animations[num_animations].name, buf);
                 animations[num_animations].facings = 1;
             }
         } else {
-            unsigned face = find_face(buf, (unsigned)-1);
-            if (face == (unsigned)-1) {
+            const Face *face = find_face(buf, NULL);
+            if (face == NULL) {
                 LOG(llevError, "Could not find face %s for animation %s\n",
                     buf, animations[num_animations].name);
-                face = find_face("bug.111", 0);
+                face = find_face("bug.111", NULL);
             }
-            faces[num_frames++] = &new_faces[face];
+            faces[num_frames++] = face;
         }
     }
     fclose(fp);
-    LOG(llevDebug, "done. got (%d)\n", num_animations);
+    LOG(llevDebug, "anim: loaded %d animations\n", num_animations);
 }
 
 /**
@@ -143,39 +163,36 @@ static int anim_compare(const Animations *a, const Animations *b) {
 }
 
 /**
- * Finds the animation id that matches name. Will LOG() an error if not found.
+ * Finds the animation that matches name. Will LOG() an error if not found.
  * @param name
  * the animation's name.
  * @return
- * animation number, or 0 if no match found (animation 0 is initialized as the 'bug' face).
+ * animation, or the "###none" animation if no match found.
  * @see try_find_animation
  */
-int find_animation(const char *name) {
-    int face = try_find_animation(name);
-    if (!face)
+const Animations *find_animation(const char *name) {
+    const Animations *anim = try_find_animation(name);
+    if (!anim)
         LOG(llevError, "Unable to find animation %s\n", name);
-    return face;
+    return anim ? anim : &animations[0];
 }
 
 /**
- * Tries to find the animation id that matches name, don't LOG() an error if not found.
+ * Tries to find the animation that matches name, don't LOG() an error if not found.
  * @param name
  * the animation's name.
  * @return
- * animation number, or 0 if no match found (animation 0 is initialized as the 'bug' face).
+ * animation, or NULL if no match found.
  * @see find_animation
  */
-int try_find_animation(const char *name) {
+const Animations *try_find_animation(const char *name) {
     Animations search, *match;
 
     search.name = name;
 
     match = (Animations *)bsearch(&search, animations, (num_animations+1), sizeof(Animations), (int (*)(const void *, const void *))anim_compare);
 
-
-    if (match)
-        return match->num;
-    return 0;
+    return match;
 }
 
 /**
@@ -197,7 +214,7 @@ void animate_object(object *op, int dir) {
     int base_state; /* starting index # to draw from */
     int oldface = op->face->number;
 
-    if (!op->animation_id || !NUM_ANIMATIONS(op)) {
+    if (!op->animation || !NUM_ANIMATIONS(op)) {
         StringBuffer *sb;
         char *diff;
 
@@ -253,8 +270,8 @@ void animate_object(object *op, int dir) {
     /* If beyond drawable states, reset */
     if (op->state >= max_state) {
         op->state = 0;
-        if (op->temp_animation_id) {
-            op->temp_animation_id = 0;
+        if (op->temp_animation) {
+            op->temp_animation = 0;
             animate_object(op, dir);
             return;
         }
@@ -298,29 +315,40 @@ void animate_object(object *op, int dir) {
  * @param suffix
  * animation suffix to apply. Must not be NULL.
  */
-void apply_anim_suffix(object *who, sstring suffix) {
-    int anim;
+void apply_anim_suffix(object *who, const char *suffix) {
+    const Animations *anim;
     object *head, *orig;
     char buf[MAX_BUF];
 
     assert(who);
     assert(suffix);
 
-    if (who->temp_animation_id != 0)
+    if (who->temp_animation)
         /* don't overlap animation, let the current one finish. */
         return;
 
     head = HEAD(who);
     orig = head;
-    snprintf(buf, MAX_BUF, "%s_%s", animations[head->animation_id].name, suffix);
+    snprintf(buf, MAX_BUF, "%s_%s", (head->animation ? head->animation->name : ""), suffix);
     anim = try_find_animation(buf);
     if (anim) {
         for (; head != NULL; head = head->more) {
-            head->temp_animation_id = anim;
-            head->temp_anim_speed = animations[anim].num_animations/animations[anim].facings;
+            head->temp_animation = anim;
+            head->temp_anim_speed = anim->num_animations / anim->facings;
             head->last_anim = 0;
             head->state = 0;
         }
         animate_object(orig, orig->facing);
     }
 }
+
+/**
+ * Dump all animations to stderr, for debugging purposes.
+ */
+void dump_animations(void) {
+    int a;
+    fprintf(stderr, "id    name                                               faces\n");
+    for (a = 0; a < num_animations; a++) {
+        fprintf(stderr, "%5d %50s %5d\n", animations[a].num, animations[a].name, animations[a].num_animations);
+    }
+}
