@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_arch_c =
- *   "$Id: arch.c 15665 2011-11-11 10:37:59Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file arch.c
  * All archetype-related functions.
@@ -40,11 +25,13 @@
  * make the functions use the same order for parameters (type first, then name, or the opposite).
  */
 
-#include <global.h>
-#include <loader.h>
+#include "global.h"
 
-/** If set, does a little timing on the archetype load. */
-#define TIME_ARCH_LOAD 0
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "loader.h"
 
 static void add_arch(archetype *at);
 
@@ -69,18 +56,19 @@ static void load_archetypes(void);
  */
 archetype *find_archetype_by_object_name(const char *name) {
     archetype *at;
-    const char *tmp;
+    sstring tmp;
 
     if (name == NULL)
         return (archetype *)NULL;
-    tmp = add_string(name);
+    tmp = find_string(name);
+    if (!tmp)
+        return NULL;
+
     for (at = first_archetype; at != NULL; at = at->next) {
         if (at->clone.name == tmp) {
-            free_string(tmp);
             return at;
         }
     }
-    free_string(tmp);
     return NULL;
 }
 
@@ -199,6 +187,8 @@ void init_archetypes(void) {
     load_archetypes();
     arch_init = 0;
     empty_archetype = find_archetype("empty_archetype");
+    if (empty_archetype == NULL)
+        fatal(SEE_LAST_ERROR);
     /*  init_blocksview();*/
 }
 
@@ -225,7 +215,7 @@ void clear_archetable(void) {
 static void init_archetable(void) {
     archetype *at;
 
-    LOG(llevDebug, " Setting up archetable...\n");
+    LOG(llevDebug, "arch: setting up archetable\n");
     for (at = first_archetype; at != NULL; at = (at->more == NULL) ? at->next : at->more) {
         if (at->name == NULL) {
             LOG(llevError, "archetype without name? %s\n", at->clone.name ? at->clone.name : "(no clone name)");
@@ -233,7 +223,6 @@ static void init_archetable(void) {
         }
         add_arch(at);
     }
-    LOG(llevDebug, "done\n");
 }
 
 /**
@@ -289,7 +278,6 @@ void free_arch(archetype *at) {
         free_string(at->clone.slaying);
     if (at->clone.msg)
         free_string(at->clone.msg);
-    free(at->clone.discrete_damage);
     object_free_key_values(&at->clone);
     free(at);
 }
@@ -467,7 +455,6 @@ static void second_arch_pass(FILE *fp) {
             load_object(fp, inv, LO_LINEMODE, 0);
             if (at) {
                 object_insert_in_ob(inv, &at->clone);
-                /*LOG(llevDebug, "Put %s in %s\n", inv->name, at->clone.name);*/
             } else {
                 LOG(llevError, "Got an arch %s not inside an Object.\n", argument);
                 object_free_drop_inventory(inv);
@@ -547,54 +534,27 @@ static void check_spells(void) {
 static void load_archetypes(void) {
     FILE *fp;
     char filename[MAX_BUF];
-#if TIME_ARCH_LOAD
-    struct timeval tv1, tv2;
-#endif
 
     snprintf(filename, sizeof(filename), "%s/%s", settings.datadir, settings.archetypes);
-    LOG(llevDebug, "Reading archetypes from %s...\n", filename);
     if ((fp = fopen(filename, "r")) == NULL) {
         LOG(llevError, " Can't open archetype file.\n");
         return;
     }
     clear_archetable();
-    LOG(llevDebug, " arch-pass 1...\n");
-#if TIME_ARCH_LOAD
-    GETTIMEOFDAY(&tv1);
-#endif
+
+    LOG(llevDebug, "arch: starting pass 1...\n");
     first_arch_pass(fp);
-#if TIME_ARCH_LOAD
-    {
-        int sec, usec;
-
-        GETTIMEOFDAY(&tv2);
-        sec = tv2.tv_sec-tv1.tv_sec;
-        usec = tv2.tv_usec-tv1.tv_usec;
-        if (usec < 0) {
-            usec += 1000000;
-            sec--;
-        }
-        LOG(llevDebug, "Load took %d.%06d seconds\n", sec, usec);
-    }
-#endif
-
-    LOG(llevDebug, " done\n");
     init_archetable();
-    warn_archetypes = 1;
-
     rewind(fp);
 
-    LOG(llevDebug, " loading treasure...\n");
+    LOG(llevDebug, "arch: loading treasures\n");
     load_treasures();
-    LOG(llevDebug, " done\n");
-    LOG(llevDebug, "arch-pass 2...\n");
+    LOG(llevDebug, "arch: starting pass 2...\n");
     second_arch_pass(fp);
-    LOG(llevDebug, " done\n");
     check_generators();
     check_spells();
     check_summoned();
     fclose(fp);
-    LOG(llevDebug, " done\n");
 }
 
 /**
@@ -612,8 +572,7 @@ object *arch_to_object(archetype *at) {
     object *op;
 
     if (at == NULL) {
-        if (warn_archetypes)
-            LOG(llevError, "Couldn't find archetype.\n");
+        LOG(llevError, "Couldn't find archetype.\n");
         return NULL;
     }
     op = object_new();
@@ -641,6 +600,7 @@ object *create_singularity(const char *name) {
 
     snprintf(buf, sizeof(buf), "%s (%s)", ARCH_SINGULARITY, name);
     op = object_new();
+    op->arch = empty_archetype;
     op->name = add_string(buf);
     op->name_pl = add_string(buf);
     SET_FLAG(op, FLAG_NO_PICK);
@@ -730,7 +690,7 @@ archetype *try_find_archetype(const char *name) {
  * @return
  * pointer to the found archetype, otherwise NULL.
  * @see try_find_archetype
- * @todo replace by try_find_archetype() when suitable and trash ::warn_archetypes.
+ * @todo replace by try_find_archetype() when suitable
  */
 archetype *find_archetype(const char *name) {
     archetype *at;
@@ -738,7 +698,7 @@ archetype *find_archetype(const char *name) {
     if (name == NULL)
         return (archetype *)NULL;
     at = try_find_archetype(name);
-    if (at == NULL && warn_archetypes)
+    if (at == NULL)
         LOG(llevError, "Couldn't find archetype %s\n", name);
     return at;
 }
@@ -790,42 +750,4 @@ object *object_create_arch(archetype *at) {
     return (head);
 }
 
-/**
- * Checks if the specified type is a valid one for a Crossfire object.
- * It would be better (not sure how) if it could automatically
- * extract this somehow from define.h .  Otherwise, it will complain
- * if a new type is added, but if a type is removed and this
- * function not updated, that will no be noticed.
- *
- * @param type value to check.
- * @return 1 if the type is valid, 0 else.
- */
-int is_type_valid(uint8 type) {
-    if (type >= OBJECT_TYPE_MAX)
-        return 0;
-
-    if (type == 11 || type == 12)
-        return 0;
-    if (type == 19 || type == 25 || type == 35 || type == 38 || type == 44 || type == 47)
-        return 0;
-    if (type == 61 || type == 63 || type == 76 || type == 78 || type == 81)
-        return 0;
-    if (type == 81 || type == 84 || type == 86 || type == 88 || type == 89)
-        return 0;
-    if (type == 96 || type == 97 || type == 107 || type == 108 || type == 110)
-        return 0;
-    if (type >= 117 && type <= 120)
-        return 0;
-    if (type >= 125 && type <= 129)
-        return 0;
-    if (type >= 131 && type <= 137)
-        return 0;
-    if (type >= 140 && type <= 149)
-        return 0;
-    if (type >= 151 && type <= 153)
-        return 0;
-
-    return 1;
-}
-
 /*** end of arch.c ***/
