@@ -1,39 +1,28 @@
 /*
- * static char *rcs_artifact_c =
- *   "$Id: artifact.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Everything concerning artifacts.
  * @see page_treasure_list
  */
 
-#include <stdlib.h>
 #include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
+
 #include "loader.h"
 
 int artifact_init;  /**< 1 if doing archetypes initialization */
@@ -261,6 +250,37 @@ int legal_artifact_combination(const object *op, const artifact *art) {
     return success;
 }
 
+/**
+ * Compute the name of a face with a suffix, taking into account names like '.123' or '.1xx'.
+ * @param buf where to put the resulting name.
+ * @param size length of buf.
+ * @param name base face name.
+ * @param suffix suffix to add to the face name.
+ */
+static void compute_face_name(char* buf, size_t size, const char* name, const char* suffix)
+{
+    const char* dot = name + strlen(name) - 1;
+    while (dot > name && (isdigit(*dot) || (*dot == 'x')))
+    {
+      dot--;
+    }
+
+    if (*dot == '.')
+    {
+        buf[0] = '0';
+        strlcpy(buf, name, dot - name + 1);
+        snprintf(buf + strlen(buf), size - strlen(buf), "_%s%s", suffix, dot);
+    }
+    else
+    {
+        snprintf(buf, size, "%s_%s", name, suffix);
+    }
+}
+
+/* Keys used for artifact stuff, not copied */
+#define KEY_FACE_SUFFIX         "face_suffix"
+#define KEY_ANIMATION_SUFFIX    "animation_suffix"
+
 /**
  * Used in artifact generation.  The bonuses of the first object
  * is modified by the bonuses of the second object.
@@ -268,6 +288,8 @@ int legal_artifact_combination(const object *op, const artifact *art) {
 void add_abilities(object *op, const object *change) {
     int i, tmp;
     char buf[MAX_BUF];
+    sstring key;
+
 
     if (change->face != blank_face) {
 #ifdef TREASURE_VERBOSE
@@ -275,6 +297,11 @@ void add_abilities(object *op, const object *change) {
 #endif
 
         object_set_value(op, "identified_face", change->face->name, 1);
+    } else if ((key = object_get_value(change, KEY_FACE_SUFFIX)) != NULL) {
+        const Face* face;
+        compute_face_name(buf, sizeof(buf), op->face->name, key);
+        face = find_face(buf, op->face);
+        object_set_value(op, "identified_face", face->name, 1);
     }
     if (QUERY_FLAG(change, FLAG_CLIENT_ANIM_RANDOM)) {
         object_set_value(op, "identified_anim_random", "1", 1);
@@ -283,12 +310,33 @@ void add_abilities(object *op, const object *change) {
         snprintf(buf, sizeof(buf), "%d", change->anim_speed);
         object_set_value(op, "identified_anim_speed", buf, 1);
     }
-    if (change->animation_id != 0 && op->arch != NULL) {
+    if (change->animation != NULL && op->arch != NULL) {
         /* op->arch can be NULL when called from artifact_msg(). */
-        snprintf(buf, sizeof(buf),"%d", change->animation_id);
-        object_set_value(op, "identified_animation", buf, 1);
+        object_set_value(op, "identified_animation", change->animation->name, 1);
+    } else if (op->animation != NULL && (key = object_get_value(change, KEY_ANIMATION_SUFFIX)) != NULL) {
+        const Animations *anim;
+        snprintf(buf, sizeof(buf), "%s_%s", op->animation->name, key);
+        anim = try_find_animation(buf);
+        if (anim != NULL) {
+            object_set_value(op, "identified_animation", anim->name, 1);
+        }
     }
 
+    /**
+     * For items that do not need identifying, use their identified face/animation
+     * immediately. This should make seas of generated creatures look more interesting.
+     *
+     * Daniel Hawkins
+     * 2018-01-07
+     *
+     * Add query flags to perform the same treatment for identified items or artifacts that come identified.
+     * Also make sure we have op->arch. Otherwise it segfaults when we don't.
+     *
+     * 2018-01-08
+     */
+    if (op->arch && (is_identified(op) || QUERY_FLAG(change, FLAG_IDENTIFIED)))
+        object_give_identified_properties(op);
+
     for (i = 0; i < NUM_STATS; i++)
         change_attr_value(&(op->stats), i, get_attr_value(&(change->stats), i));
 
@@ -317,6 +365,8 @@ void add_abilities(object *op, const object *change) {
         SET_FLAG(op, FLAG_XRAYS);
     if (QUERY_FLAG(change, FLAG_BLIND))
         SET_FLAG(op, FLAG_BLIND);
+    if (QUERY_FLAG(change, FLAG_CONFUSED))
+        SET_FLAG(op, FLAG_CONFUSED);
     if (QUERY_FLAG(change, FLAG_SEE_IN_DARK))
         SET_FLAG(op, FLAG_SEE_IN_DARK);
     if (QUERY_FLAG(change, FLAG_REFL_MISSILE))
@@ -449,6 +499,16 @@ void add_abilities(object *op, const object *change) {
     if (change->msg)
         object_set_msg(op, change->msg);
 
+    if (change->key_values) {
+        const key_value *kv = change->key_values;
+        while (kv) {
+            if (strcmp(kv->key, KEY_FACE_SUFFIX) != 0 && strcmp(kv->key, KEY_ANIMATION_SUFFIX) != 0) {
+                object_set_value(op, kv->key, kv->value, 1);
+            }
+            kv = kv->next;
+        }
+    }
+
     if (change->inv) {
         object *copy;
 
@@ -544,9 +604,9 @@ void init_artifacts(void) {
                 art->allowed_size++;
             } while ((cp = next) != NULL);
         } else if (sscanf(cp, "chance %d", &value))
-            art->chance = (uint16)value;
+            art->chance = (uint16_t)value;
         else if (sscanf(cp, "difficulty %d", &value))
-            art->difficulty = (uint8)value;
+            art->difficulty = (uint8_t)value;
         else if (!strncmp(cp, "Object", 6)) {
             art->item = (object *)calloc(1, sizeof(object));
             if (art->item == NULL) {
@@ -578,7 +638,7 @@ void init_artifacts(void) {
     for (al = first_artifactlist; al != NULL; al = al->next) {
         for (art = al->items; art != NULL; art = art->next) {
             if (!art->chance)
-                LOG(llevError, "Warning: artifact with no chance: %s\n", art->item->name);
+                LOG(llevDebug, "Artifact with no chance: %s\n", art->item->name);
             else
                 al->total_chance += art->chance;
         }
