@@ -1,38 +1,27 @@
 /*
- * static char *rcsid_button_c =
- *   "$Id: button.c 14671 2011-06-21 17:17:13Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * This handles triggers, buttons, altars and associated objects.
  */
 
-#include <global.h>
-#include <sproto.h>
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "sproto.h"
 
 static objectlink *get_button_links(const object *button);
 
@@ -132,16 +121,22 @@ void trigger_connected(objectlink *ol, object *cause, const int state) {
             break;
 
         case DIRECTOR:
+            if ((tmp->stats.sp += tmp->stats.maxsp) > 8) /* next direction */
+                tmp->stats.sp = ((tmp->stats.sp-1)%8)+1;
+            animate_turning(tmp);
+            break;
         case FIREWALL:
-            if (!QUERY_FLAG(tmp, FLAG_ANIMATE) && tmp->type == FIREWALL)
+            // Since director is broken out, we don't need to check on firewall type now
+            if (!QUERY_FLAG(tmp, FLAG_ANIMATE)/* && tmp->type == FIREWALL*/)
                 move_firewall(tmp);
             else {
-                if ((tmp->stats.sp += tmp->stats.maxsp) > 8) /* next direction */
-                    tmp->stats.sp = ((tmp->stats.sp-1)%8)+1;
+                if ((tmp->direction += tmp->stats.maxsp) > 8) /* next direction */
+                    tmp->direction = ((tmp->direction-1)%8)+1;
                 animate_turning(tmp);
             }
             break;
 
+
         default:
             ob_trigger(tmp, cause, state);
         }
@@ -199,7 +194,7 @@ void update_button(object *op) {
                  * that are only triggered by flying objects.
                  */
                 if ((ab->move_type&tmp->move_on) || ab->move_type == 0)
-                    tot += ab->weight*(ab->nrof ? ab->nrof : 1)+ab->carrying;
+                    tot += ab->weight*NROF(ab)+ab->carrying;
             FOR_ABOVE_FINISH();
 
             tmp->value = (tot >= tmp->weight) ? 1 : 0;
@@ -280,12 +275,18 @@ void use_trigger(object *op) {
 void animate_turning(object *op) {
     if (++op->state >= NUM_ANIMATIONS(op)/8)
         op->state = 0;
-    SET_ANIMATION(op, (op->stats.sp-1)*NUM_ANIMATIONS(op)/8+op->state);
+    // Firewalls use direction instead of sp to avoid issues involving the spell transference.
+    if (op->type == FIREWALL){
+        SET_ANIMATION(op, (op->direction-1)*NUM_ANIMATIONS(op)/8+op->state);
+    }
+    else {
+        SET_ANIMATION(op, (op->stats.sp-1)*NUM_ANIMATIONS(op)/8+op->state);
+    }
     object_update(op, UP_OBJ_FACE);
 }
 
 #define ARCH_SACRIFICE(xyz) ((xyz)->slaying)
-#define NROF_SACRIFICE(xyz) ((uint32)(xyz)->stats.food)
+#define NROF_SACRIFICE(xyz) ((uint32_t)(xyz)->stats.food)
 
 /**
  * Helper function to check if the item matches altar's requested sacrifice.
@@ -350,7 +351,7 @@ static int matches_sacrifice(const object *altar, const object *sacrifice) {
  */
 int check_altar_sacrifice(const object *altar, const object *sacrifice, int remove_others, int *toremove) {
     int money;
-    uint32 wanted, rest;
+    uint32_t wanted, rest;
 
     if (!matches_sacrifice(altar, sacrifice))
         /* New dropped object doesn't match the altar, other objects already on top are not enough to
@@ -375,7 +376,7 @@ int check_altar_sacrifice(const object *altar, const object *sacrifice, int remo
         return 1;
     }
 
-    if (!money && NROF_SACRIFICE(altar) <= (sacrifice->nrof ? sacrifice->nrof : 1)) {
+    if (!money && NROF_SACRIFICE(altar) <= NROF(sacrifice)) {
         if (toremove)
             *toremove = NROF_SACRIFICE(altar);
         return 1;
@@ -384,7 +385,7 @@ int check_altar_sacrifice(const object *altar, const object *sacrifice, int remo
     if (money) {
         wanted = NROF_SACRIFICE(altar)-sacrifice->nrof*sacrifice->value;
     } else {
-        wanted = NROF_SACRIFICE(altar)-(sacrifice->nrof ? sacrifice->nrof : 1);
+        wanted = NROF_SACRIFICE(altar)-NROF(sacrifice);
     }
     rest = wanted;
 
@@ -399,7 +400,7 @@ int check_altar_sacrifice(const object *altar, const object *sacrifice, int remo
         if (money)
             wanted -= tmp->nrof*tmp->value;
         else
-            wanted -= (tmp->nrof ? tmp->nrof : 1);
+            wanted -= NROF(tmp);
     } FOR_ABOVE_FINISH();
 
     if (wanted > 0)
@@ -410,7 +411,7 @@ int check_altar_sacrifice(const object *altar, const object *sacrifice, int remo
 
     /* Last dropped object will be totally eaten in any case. */
     if (toremove)
-        *toremove = sacrifice->nrof ? sacrifice->nrof : 1;
+        *toremove = NROF(sacrifice);
 
     if (!remove_others)
         return 1;
@@ -434,8 +435,8 @@ int check_altar_sacrifice(const object *altar, const object *sacrifice, int remo
                 return 1;
             }
         } else
-            if (rest > (tmp->nrof ? tmp->nrof : 1)) {
-                rest -= (tmp->nrof ? tmp->nrof : 1);
+            if (rest > NROF(tmp)) {
+                rest -= NROF(tmp);
                 object_remove(tmp);
             } else {
                 object_decrease_nrof(tmp, rest);
@@ -536,7 +537,7 @@ int check_trigger(object *op, object *cause) {
                      * that are only triggered by flying objects.
                      */
                     if ((tmp->move_type&op->move_on) || tmp->move_type == 0) {
-                        tot += tmp->weight*(tmp->nrof ? tmp->nrof : 1)+tmp->carrying;
+                        tot += tmp->weight*NROF(tmp)+tmp->carrying;
                     }
                 FOR_ABOVE_FINISH();
                 if (tot >= op->weight)
@@ -772,6 +773,7 @@ int get_button_value(const object *button) {
  *              race = match object archetype name flag
  *              hp = match object type (excpt type '0'== PLAYER)
  *      title = match object title
+ *              unpaid = 1 -> match only unpaid, 0 -> match only not unpaid
  * Searching by title only is not recommended, as it can be a rather slow
  * operation; use it in combination with archetype or type.
 
@@ -789,7 +791,8 @@ object *check_inv_recursive(object *op, const object *trig) {
     if ((!trig->stats.hp || (op->type == trig->stats.hp))
     && (!trig->slaying || (op->slaying == trig->slaying))
     && (!trig->race || (op->arch->name == trig->race))
-    && (!trig->title || (op->title == trig->title)))
+    && (!trig->title || (op->title == trig->title))
+    && (QUERY_FLAG(op, FLAG_UNPAID) == QUERY_FLAG(trig, FLAG_UNPAID)))
         return op;
 
     FOR_INV_PREPARE(op, tmp) {
@@ -800,13 +803,15 @@ object *check_inv_recursive(object *op, const object *trig) {
         } else if ((!trig->stats.hp || (tmp->type == trig->stats.hp))
         && (!trig->slaying || (tmp->slaying == trig->slaying))
         && (!trig->race || (tmp->arch->name == trig->race))
-        && (!trig->title || (tmp->title == trig->title)))
+        && (!trig->title || (tmp->title == trig->title))
+        && (QUERY_FLAG(tmp, FLAG_UNPAID) == QUERY_FLAG(trig, FLAG_UNPAID)))
             return tmp;
     } FOR_INV_FINISH();
 
     return NULL;
 }
 
+
 /**
  * Function to search the inventory,
  * of a player and then based on a set of conditions,
