@@ -1,45 +1,29 @@
 /*
- * static char *rcsid_arch_c =
- *   "$Id: exp.c 14864 2011-07-17 08:36:07Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file exp.c
  * Experience management. reading data from files and such.
  */
 
+#include "global.h"
+
+#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <global.h>
 
-sint64 *levels; /**< Number of levels for which we have experience. */
-
-#define TRUE 1
-#define FALSE 0
+int64_t *levels; /**< Number of levels for which we have experience. */
 
 static const float exp_att_mult[NROFATTACKS] = {
     0.0,    /* AT_PHYSICAL */
@@ -113,7 +97,7 @@ static const float exp_prot_mult[NROFATTACKS] = {
  * @return
  * experience computed from object's properties.
  */
-sint64 new_exp(const object *ob) {
+int64_t new_exp(const object *ob) {
     double att_mult, prot_mult, spec_mult;
     double exp;
     int i;
@@ -158,7 +142,7 @@ sint64 new_exp(const object *ob) {
     if (QUERY_FLAG(ob, FLAG_STAND_STILL))
         exp /= 2;
 
-    return (sint64)exp;
+    return (int64_t)exp;
 }
 
 /**
@@ -183,11 +167,10 @@ int has_ability(const object *ob) {
 void init_experience(void) {
     char buf[MAX_BUF], *cp;
     int lastlevel = 0;
-    sint64 lastexp = -1, tmpexp;
+    int64_t lastexp = -1, tmpexp;
     FILE *fp;
 
     snprintf(buf, sizeof(buf), "%s/exp_table", settings.confdir);
-
     if ((fp = fopen(buf, "r")) == NULL) {
         LOG(llevError, "Fatal error: could not open experience table (%s)\n", buf);
         exit(1);
@@ -200,12 +183,13 @@ void init_experience(void) {
         if ((cp = strrchr(buf, '\n')) != NULL)
             *cp = '\0';
 
-        /* Skip over empty lines */
-        if (buf[0] == 0)
-            continue;
+        /* Skip over empty lines and whitespace */
         cp = buf;
         while (isspace(*cp) && *cp != 0)
             cp++;
+        if ( *cp == 0 ) continue;
+
+        /* Set max_level */
         if (!strncasecmp(cp, "max_level", 9)) {
             if (settings.max_level) {
                 LOG(llevDebug, "Got more than one max_level value from exp_table file?\n");
@@ -215,7 +199,7 @@ void init_experience(void) {
             if (!settings.max_level) {
                 LOG(llevDebug, "Got invalid max_level from exp_table file? %s\n", buf);
             } else {
-                levels = calloc(settings.max_level+1, sizeof(sint64));
+                levels = calloc(settings.max_level+1, sizeof(int64_t));
             }
         }
         while (isdigit(*cp) && *cp != 0) {
