@@ -1,42 +1,30 @@
 /*
- * static char *rcsid_holy_c =
- *   "$Id: holy.c 17403 2012-03-08 22:22:35Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file holy.c
  * God-related common functions.
  */
 
-#include <stdlib.h>
-#include <global.h>
-#include <living.h>
-#include <spells.h>
-#include <libproto.h>
+#include "global.h"
+
 #include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "libproto.h"
+#include "living.h"
+#include "spells.h"
 
 static void add_god_to_list(archetype *god_arch);
 
@@ -233,7 +221,7 @@ int describe_god(const object *god, int what, StringBuffer *buf, size_t maxlen)
     }
 
     if (what & GOD_BLESSED) {
-        char *cp = stringbuffer_finish(describe_resistance(god, 1, NULL));
+        char *cp = stringbuffer_finish(describe_resistance(god, 1, 1, NULL));
 
         if (*cp) {  /* This god does have protections */
             add = stringbuffer_new();
@@ -277,7 +265,7 @@ int describe_god(const object *god, int what, StringBuffer *buf, size_t maxlen)
 
     if (what & GOD_RESISTANCES) {
         char *cp;
-        cp = stringbuffer_finish(describe_resistance(god, 1, NULL));
+        cp = stringbuffer_finish(describe_resistance(god, 1, 1, NULL));
 
         if (*cp) {  /* This god does have protections */
             add = stringbuffer_new();
@@ -420,7 +408,7 @@ void dump_gods(void) {
             fprintf(stderr, " servant: NONE\n");
         fprintf(stderr, " aligned_race(s): %s\n", god->race);
         fprintf(stderr, " enemy_race(s): %s\n", (god->slaying ? god->slaying : "none"));
-        final = stringbuffer_finish(describe_resistance(god, 1, NULL));
+        final = stringbuffer_finish(describe_resistance(god, 1, 1, NULL));
         fprintf(stderr, "%s", final);
         free(final);
         snprintf(tmpbuf, sizeof(tmpbuf), " attacktype:");
