@@ -1,57 +1,48 @@
 /*
- * static char *rcsid_image_c =
- *   "$Id: image.c 15579 2011-11-08 21:14:12Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The maintainer of this code can be reached at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Handles face-related stuff, including the actual face data.
  */
 
-#include <global.h>
+#include "global.h"
+
+#include <assert.h>
+#include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include "image.h"
 
 /**
  * Contains face information, with names, numbers, magicmap color and such.
  * It is sorted by alphabetical order.
  */
-New_Face *new_faces;
+Face *new_faces;
 
 /**
  * Following can just as easily be pointers, but
  * it is easier to keep them like this.
  */
-New_Face *blank_face, *empty_face, *smooth_face;
+const Face *blank_face, *empty_face, *smooth_face;
 
 
 /**
  * Number of bitmaps loaded from the "bmaps" file.
  */
-unsigned int nrofpixmaps = 0;
+static unsigned int nrofpixmaps = 0;
 
 face_sets facesets[MAX_FACE_SETS];    /**< All facesets */
 
@@ -89,7 +80,7 @@ static const char *const colorname[] = {
  * @retval 0 if a == b
  * @retval 1 if a > b
  */
-static int compare_face(const New_Face *a, const New_Face *b) {
+static int compare_face(const Face *a, const Face *b) {
     if (strcmp(a->name, "bug.111") == 0) {
         if (strcmp(b->name, "bug.111") == 0)
             return 0;
@@ -99,6 +90,22 @@ static int compare_face(const New_Face *a, const New_Face *b) {
     return strcmp(a->name, b->name);
 }
 
+/**
+ * This returns the face with 'name'.
+ *
+ * @param name
+ * face to search for.
+ * @return matching face, NULL if not found.
+ */
+static Face *internal_find_face(const char *name) {
+    Face *bp, tmp;
+
+    tmp.name = name;
+    bp = (Face *)bsearch(&tmp, new_faces, nrofpixmaps, sizeof(Face), (int (*)(const void *, const void *))compare_face);
+
+    return bp;
+}
+
 /**
  * Finds a color by name.
  *
@@ -112,8 +119,8 @@ static int compare_face(const New_Face *a, const New_Face *b) {
  * 0 will actually be black, so there is no
  * way the calling function can tell if an error occurred or not
  */
-static uint8 find_color(const char *name) {
-    uint8 i;
+static uint8_t find_color(const char *name) {
+    uint8_t i;
 
     for (i = 0; i < sizeof(colorname)/sizeof(*colorname); i++)
         if (!strcmp(name, colorname[i]))
@@ -132,13 +139,12 @@ static uint8 find_color(const char *name) {
  */
 static void read_face_data(void) {
     char buf[MAX_BUF], *cp;
-    New_Face *on_face = NULL;
+    Face *on_face = NULL;
     FILE *fp;
 
     snprintf(buf, sizeof(buf), "%s/faces", settings.datadir);
-    LOG(llevDebug, "Reading faces from %s...\n", buf);
     if ((fp = fopen(buf, "r")) == NULL) {
-        LOG(llevError, "Cannot open faces file: %s\n", strerror_local(errno, buf, sizeof(buf)));
+        LOG(llevError, "faces: couldn't open file: %s\n", strerror(errno));
         exit(-1);
     }
 
@@ -148,20 +154,20 @@ static void read_face_data(void) {
         if (!strncmp(buf, "end", 3)) {
             on_face = NULL;
         } else if (!strncmp(buf, "face", 4)) {
-            unsigned tmp;
+            Face *tmp;
 
             cp = buf+5;
             cp[strlen(cp)-1] = '\0'; /* remove newline */
 
-            if ((tmp = find_face(cp, (unsigned)-1)) == (unsigned)-1) {
-                LOG(llevError, "Could not find face %s\n", cp);
+            if ((tmp = internal_find_face(cp)) == NULL) {
+                LOG(llevError, "faces: couldn't find '%s'\n", cp);
                 on_face = NULL;
                 continue;
             }
-            on_face = &new_faces[tmp];
+            on_face = tmp;
             on_face->visibility = 0;
         } else if (on_face == NULL) {
-            LOG(llevError, "Got line with no face set: %s\n", buf);
+            LOG(llevError, "faces: got line with no face set: %s\n", buf);
         } else if (!strncmp(buf, "visibility", 10)) {
             on_face->visibility = atoi(buf+11);
         } else if (!strncmp(buf, "magicmap", 8)) {
@@ -173,9 +179,8 @@ static void read_face_data(void) {
             if (value)
                 on_face->magicmap |= FACE_FLOOR;
         } else
-            LOG(llevDebug, "Got unknown line in faces file: %s\n", buf);
+            LOG(llevDebug, "faces: unknown line in %s\n", buf);
     }
-    LOG(llevDebug, "done\n");
     fclose(fp);
 }
 
@@ -189,39 +194,39 @@ static void read_face_data(void) {
  * will call exit() if file doesn't exist, and abort() in case of memory error.
  */
 void read_bmap_names(void) {
-    char buf[MAX_BUF], *p, *q;
+    char buf[MAX_BUF], *p;
     FILE *fp;
-    int value;
     unsigned int i;
     size_t l;
 
     bmaps_checksum = 0;
-    snprintf(buf, sizeof(buf), "%s/bmaps", settings.datadir);
-    LOG(llevDebug, "Reading bmaps from %s...\n", buf);
+    snprintf(buf, sizeof(buf), "%s/bmaps.paths", settings.datadir);
     if ((fp = fopen(buf, "r")) == NULL) {
-        LOG(llevError, "Cannot open bmaps file: %s\n", strerror_local(errno, buf, sizeof(buf)));
+        LOG(llevError, "bmaps: couldn't open: %s\n", strerror(errno));
         exit(-1);
     }
 
     nrofpixmaps = 0;
 
     /* First count how many bitmaps we have, so we can allocate correctly */
-    while (fgets(buf, MAX_BUF, fp) != NULL)
-        if (buf[0] != '#' && buf[0] != '\n')
+    while (fgets(buf, MAX_BUF, fp) != NULL) {
+        if (buf[0] != '#' && buf[0] != '\n') {
             nrofpixmaps++;
-    rewind(fp);
+        }
+    }
 
-    new_faces = (New_Face *)malloc(sizeof(New_Face)*nrofpixmaps);
+    rewind(fp);
+    assert(nrofpixmaps > 0);
+    new_faces = (Face *)malloc(sizeof(Face)*nrofpixmaps);
     if (new_faces == NULL) {
-        LOG(llevError, "read_bmap_names: new_faces memory allocation failure.\n");
-        abort();
+        fatal(OUT_OF_MEMORY);
     }
 
     for (i = 0; i < nrofpixmaps; i++) {
         new_faces[i].name = NULL;
         new_faces[i].visibility = 0;
         new_faces[i].magicmap = 255;
-        new_faces[i].smoothface = (uint16)-1;
+        new_faces[i].smoothface = NULL;
     }
 
     i = 0;
@@ -229,13 +234,13 @@ void read_bmap_names(void) {
         if (*buf == '#')
             continue;
 
-        p = (*buf == '\\') ? (buf+1) : buf;
-        if (!(p = strtok(p, " \t")) || !(q = strtok(NULL, " \t\n"))) {
-            LOG(llevDebug, "Warning, syntax error: %s\n", buf);
-            continue;
+        p = strrchr(buf, '/');
+        if ((p == NULL) || (strtok(p, " \t\n") == NULL)) {
+            LOG(llevError, "bmaps: syntax error in %s\n", buf);
+            fatal(SEE_LAST_ERROR);
         }
-        value = atoi(p);
-        new_faces[i].name = strdup_local(q);
+        /* strtok converted the final newline or tab to NULL so all is ok */
+        new_faces[i].name = strdup_local(p + 1);
 
         /* We need to calculate the checksum of the bmaps file
          * name->number mapping to send to the client.  This does not
@@ -245,15 +250,15 @@ void read_bmap_names(void) {
          * the file has the same data or not.
          */
         ROTATE_RIGHT(bmaps_checksum);
-        bmaps_checksum += value&0xff;
+        bmaps_checksum += i&0xff;
         bmaps_checksum &= 0xffffffff;
 
         ROTATE_RIGHT(bmaps_checksum);
-        bmaps_checksum += (value>>8)&0xff;
+        bmaps_checksum += (i>>8)&0xff;
         bmaps_checksum &= 0xffffffff;
-        for (l = 0; l < strlen(q); l++) {
+        for (l = 0; l < strlen(p); l++) {
             ROTATE_RIGHT(bmaps_checksum);
-            bmaps_checksum += q[l];
+            bmaps_checksum += p[l];
             bmaps_checksum &= 0xffffffff;
         }
 
@@ -266,9 +271,9 @@ void read_bmap_names(void) {
         fatal(SEE_LAST_ERROR);
     }
 
-    LOG(llevDebug, "done (got %d faces)\n", nrofpixmaps);
+    LOG(llevDebug, "bmaps: loaded %d faces\n", nrofpixmaps);
 
-    qsort(new_faces, nrofpixmaps, sizeof(New_Face), (int (*)(const void *, const void *))compare_face);
+    qsort(new_faces, nrofpixmaps, sizeof(Face), (int (*)(const void *, const void *))compare_face);
 
     for (i = 0; i < nrofpixmaps; i++) {
         new_faces[i].number = i;
@@ -284,40 +289,33 @@ void read_bmap_names(void) {
     /* Actually forcefully setting the colors here probably should not
      * be done - it could easily create confusion.
      */
-    blank_face = &new_faces[find_face(BLANK_FACE_NAME, 0)];
-    blank_face->magicmap = find_color("khaki")|FACE_FLOOR;
+    blank_face = internal_find_face(BLANK_FACE_NAME);
+    ((Face*)blank_face)->magicmap = find_color("khaki")|FACE_FLOOR; // TODO: doh :(
 
-    empty_face = &new_faces[find_face(EMPTY_FACE_NAME, 0)];
+    empty_face = internal_find_face(EMPTY_FACE_NAME);
 
-    smooth_face = &new_faces[find_face(SMOOTH_FACE_NAME, 0)];
+    smooth_face = internal_find_face(SMOOTH_FACE_NAME);
 }
 
 /**
- * This returns an the face number of face 'name'.  Number is constant
- * during an invocation, but not necessarily between versions (this
- * is because the faces are arranged in alphabetical order, so
- * if a face is removed or added, all faces after that will now
- * have a different number.
+ * This returns the face with 'name'.
  *
  * @param name
- * face to search for
+ * face to search for.
  * @param error
  * value to return if face was not found.
+ * @return found face, or error.
  *
  * @note
  * If a face is not found, then error is returned.  This can be useful if
- * you want some default face used, or can be set to negative
+ * you want some default face used, or can be set to NULL
  * so that it will be known that the face could not be found
  * (needed in client, so that it will know to request that image
  * from the server)
  */
-unsigned find_face(const char *name, unsigned error) {
-    New_Face *bp, tmp;
-
-    tmp.name = name;
-    bp = (New_Face *)bsearch(&tmp, new_faces, nrofpixmaps, sizeof(New_Face), (int (*)(const void *, const void *))compare_face);
-
-    return bp ? bp->number : error;
+const Face *find_face(const char *name, const Face *error) {
+    const Face *bp = internal_find_face(name);
+    return bp ? bp : error;
 }
 
 /**
@@ -334,13 +332,12 @@ unsigned find_face(const char *name, unsigned error) {
 int read_smooth(void) {
     char buf[MAX_BUF], *p, *q;
     FILE *fp;
-    unsigned regular, smoothed;
+    Face *regular, *smoothed;
     int nrofsmooth = 0;
 
     snprintf(buf, sizeof(buf), "%s/smooth", settings.datadir);
-    LOG(llevDebug, "Reading smooth from %s...\n", buf);
     if ((fp = fopen(buf, "r")) == NULL) {
-        LOG(llevError, "Cannot open smooth file: %s\n", strerror_local(errno, buf, sizeof(buf)));
+        LOG(llevError, "Cannot open smooth file: %s\n", strerror(errno));
         exit(-1);
     }
 
@@ -357,25 +354,25 @@ int read_smooth(void) {
 
         *p = '\0';
         q = buf;
-        regular = find_face(q, (unsigned)-1);
-        if (regular == (unsigned)-1) {
+        regular = internal_find_face(q);
+        if (regular == NULL) {
             LOG(llevError, "invalid regular face: %s\n", q);
             continue;
         }
         q = p+1;
-        smoothed = find_face(q, (unsigned)-1);
-        if (smoothed == (unsigned)-1) {
+        smoothed = internal_find_face(q);
+        if (smoothed == NULL) {
             LOG(llevError, "invalid smoothed face: %s\n", q);
             continue;
         }
 
-        new_faces[regular].smoothface = smoothed;
+        regular->smoothface = smoothed;
 
         nrofsmooth++;
     }
     fclose(fp);
 
-    LOG(llevDebug, "done (got %d smooth entries)\n", nrofsmooth);
+    LOG(llevDebug, "smooth: loaded %d entries\n", nrofsmooth);
     return nrofsmooth;
 }
 
@@ -388,14 +385,11 @@ int read_smooth(void) {
  *
  * @return 1=smooth face found, 0=no smooth face found
  */
-int find_smooth(uint16 face, uint16 *smoothed) {
-    (*smoothed) = 0;
+int find_smooth(const Face *face, const Face **smoothed) {
+    (*smoothed) = NULL;
 
-    if (face < nrofpixmaps) {
-        if (new_faces[face].smoothface == ((uint16)-1))
-            return 0;
-
-        (*smoothed) = new_faces[face].smoothface;
+    if (face && face->smoothface) {
+        (*smoothed) = face->smoothface;
         return 1;
     }
 
@@ -464,6 +458,7 @@ void read_client_images(void) {
     FILE *infile;
     int len, fileno, i;
     unsigned int num;
+    const Face *face;
 
     memset(facesets, 0, sizeof(facesets));
     snprintf(filename, sizeof(filename), "%s/image_info", settings.datadir);
@@ -506,7 +501,7 @@ void read_client_images(void) {
         facesets[fileno].faces = calloc(nrofpixmaps, sizeof(face_info));
 
         snprintf(filename, sizeof(filename), "%s/crossfire.%d", settings.datadir, fileno);
-        LOG(llevDebug, "Loading image file %s\n", filename);
+        LOG(llevDebug, "images: loading from %s\n", filename);
 
         if ((infile = fopen(filename, "rb")) == NULL) {
             LOG(llevError, "Unable to open %s\n", filename);
@@ -524,10 +519,12 @@ void read_client_images(void) {
                 abort();
             }
 
-            for ( ; *cp != ' ' && *cp != '\n' && *cp != '\0'; cp++)
-                ;
+            for ( ; *cp != ' ' && *cp != '\n' && *cp != '\0'; cp++) {
+                /* Increment pointer until next token. */
+            }
+
             if (*cp != ' ') {
-                LOG(llevError, "read_client_images: couldn't find name start for %d\n", num);
+                LOG(llevError, "read_client_images: couldn't read name\n");
                 abort();
             }
             cp++;
@@ -539,11 +536,12 @@ void read_client_images(void) {
                 cp[strlen(cp) - 1] = '\0';
 
             /* cp points to the start of the picture name itself */
-            num = find_face(cp, (unsigned)-1);
-            if (num == (unsigned)-1) {
+            face = find_face(cp, NULL);
+            if (face == NULL) {
                 LOG(llevError, "read_client_images: couldn't find picture %s\n", cp);
                 abort();
             }
+            num = face->number;
             if (num >= nrofpixmaps) {
                 LOG(llevError, "read_client_images: invalid picture number %d for %s\n", num, cp);
                 abort();
@@ -627,3 +625,35 @@ int get_face_fallback(int faceset, int imageno) {
         return faceset;
     return get_face_fallback(facesets[faceset].fallback, imageno);
 }
+
+/**
+ * Return the number of faces, including the "bug" one.
+ * @return number of faces.
+ */
+unsigned int get_faces_count() {
+    return nrofpixmaps;
+}
+
+/**
+ * Return the face at the specified index.
+ * @param index index, between 0 and get_faces_count() excluded.
+ * @return face, NULL if the index is invalid.
+ */
+const Face *get_face_by_index(int index) {
+    if (index < 0 || index >= nrofpixmaps)
+        return NULL;
+    return &new_faces[index];
+}
+
+/**
+ * Get a face from its unique identifier.
+ * @param id face identifier.
+ * @return matching face, NULL if no face with this identifier.
+ */
+const Face *get_face_by_id(uint16_t id) {
+    for (int f = 0; f < nrofpixmaps; f++) {
+        if (new_faces[f].number == id)
+            return &new_faces[f];
+    }
+    return NULL;
+}
