@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_init_c =
- *   "$Id: init.c 16018 2011-12-03 15:13:09Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Basic initialization for the common library.
@@ -33,8 +18,14 @@
 
 #define EXTERN
 #define INIT_C
-#include <global.h>
-#include <object.h>
+
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "object.h"
+#include "output_file.h"
 
 static void init_environ(void);
 static void init_defaults(void);
@@ -43,80 +34,65 @@ static void init_clocks(void);
 static void init_attackmess(void);
 
 /*
- * You unforunately need to looking in include/global.h to see what these
- * correspond to.
+ * Anything with non-zero defaults in include/global.h must be set here.
  */
 struct Settings settings = {
-    LOGFILE,   /* Logfile */
-    CSPORT,    /* Client/server port */
-
-    llevInfo,  /* Debug level */
-    0, NULL, 0,    /* dumpvalues, dumparg, daemonmode */
-    0, /* argc */
-    NULL, /* argv */
-    CONFDIR,
-    DATADIR,
-    LOCALDIR,
-    PLAYERDIR, MAPDIR, ARCHETYPES, REGIONS, TREASURES,
-    UNIQUE_DIR, TEMPLATE_DIR,
-    TMPDIR,
-    STAT_LOSS_ON_DEATH,
-    PK_LUCK_PENALTY,
-    PERMANENT_EXPERIENCE_RATIO,
-    DEATH_PENALTY_RATIO,
-    DEATH_PENALTY_LEVEL,
-    BALANCED_STAT_LOSS,
-    NOT_PERMADETH,
-    SIMPLE_EXP,
-    RESET_LOCATION_TIME,
-    SET_TITLE,
-    RESURRECTION,
-    SEARCH_ITEMS,
-    SPELL_ENCUMBRANCE,
-    SPELL_FAILURE_EFFECTS,
-    CASTING_TIME,
-    REAL_WIZ,
-    RECYCLE_TMP_MAPS,
-    SPELLPOINT_LEVEL_DEPEND,
-    SET_FRIENDLY_FIRE,
-    "", /* Who format specifier */
-    "", /* who wiz format specifier */
-    MOTD,
-    "rules",
-    "news",
-    "",  /* DM_MAIL */
-    0,  /* This and the next 3 values are metaserver values */
-    "",
-    "",
-    0,
-    "",
-    0, 0, 0, 0, 0, 0, 0,  /* worldmap settings*/
-    NULL, EMERGENCY_X, EMERGENCY_Y,
-    0,
-    1.0,
+    .csport = CSPORT,
+    .debug = llevInfo,
+    .confdir = CONFDIR,
+    .datadir = DATADIR,
+    .localdir = LOCALDIR,
+    .playerdir = PLAYERDIR,
+    .mapdir = MAPDIR,
+    .archetypes = ARCHETYPES,
+    .regions = REGIONS,
+    .treasures = TREASURES,
+    .uniquedir = UNIQUE_DIR,
+    .templatedir = TEMPLATE_DIR,
+    .tmpdir = TMPDIR,
+    .stat_loss_on_death = STAT_LOSS_ON_DEATH,
+    .pk_luck_penalty = PK_LUCK_PENALTY,
+    .permanent_exp_ratio = PERMANENT_EXPERIENCE_RATIO,
+    .death_penalty_ratio = DEATH_PENALTY_RATIO,
+    .death_penalty_level = DEATH_PENALTY_LEVEL,
+    .balanced_stat_loss = BALANCED_STAT_LOSS,
+    .not_permadeth = NOT_PERMADETH,
+    .simple_exp = SIMPLE_EXP,
+    .reset_loc_time = RESET_LOCATION_TIME,
+    .set_title = SET_TITLE,
+    .resurrection = RESURRECTION,
+    .search_items = SEARCH_ITEMS,
+    .spell_encumbrance = SPELL_ENCUMBRANCE,
+    .spell_failure_effects = SPELL_FAILURE_EFFECTS,
+    .casting_time = CASTING_TIME,
+    .real_wiz = REAL_WIZ,
+    .recycle_tmp_maps = RECYCLE_TMP_MAPS,
+    .always_show_hp = ALWAYS_SHOW_HP,
+    .spellpoint_level_depend = SPELLPOINT_LEVEL_DEPEND,
+    .set_friendly_fire = SET_FRIENDLY_FIRE,
+    .motd = MOTD,
+    .rules = "rules",
+    .news = "news",
+    .emergency_x = EMERGENCY_X,
+    .emergency_y = EMERGENCY_Y,
+    .item_power_factor = 1.0,
     /* Armor enchantment stuff */
-    ARMOR_MAX_ENCHANT,
-    ARMOR_WEIGHT_REDUCTION,
-    ARMOR_WEIGHT_LINEAR,
-    ARMOR_SPEED_IMPROVEMENT,
-    ARMOR_SPEED_LINEAR,
-    1, /* no_player_stealing */
-    0, /* create_home_portals */
-    1, /* personalized_blessings */
-    5000000, /* pk_max_experience */
-    10, /* pk_max_experience_percent */
-    0, /* allow_denied_spells_writing */
-    0, /* allow_broken_converters */
-    0, /* log_timestamp */
-    NULL, /* log_timestamp_format */
-    3,      /* starting_stat_min */
-    18,     /* starting_stat_max */
-    85,     /* starting_stat_points */
-    115,    /* roll_stat_points */
-    0,     /* max_stat - will be loaded from stats file */
-    1,     /* special_break_map, 1 for historical reasons */
-    NULL,  /* disabled_plugins */
-    0,      /* ignore_plugin_compatibility */
+    .armor_max_enchant = ARMOR_MAX_ENCHANT,
+    .armor_weight_reduction = ARMOR_WEIGHT_REDUCTION,
+    .armor_weight_linear = ARMOR_WEIGHT_LINEAR,
+    .armor_speed_improvement = ARMOR_SPEED_IMPROVEMENT,
+    .armor_speed_linear = ARMOR_SPEED_LINEAR,
+    .no_player_stealing = 1,
+    .create_home_portals = 0,
+    .personalized_blessings = 1,
+    .pk_max_experience = 5000000,
+    .pk_max_experience_percent = 10,
+    .starting_stat_min = 3,
+    .starting_stat_max = 18,
+    .starting_stat_points = 85,
+    .roll_stat_points = 115,
+    .max_stat = 0,     /* max_stat - will be loaded from stats file */
+    .special_break_map = 1,     /* special_break_map, 1 for historical reasons */
 };
 
 struct Statistics statistics;
@@ -192,7 +168,9 @@ static void init_emergency_mappath(void) {
         fclose(fp);
         if (online <= 2)
             LOG(llevError, "Online read partial data from %s\n", filename);
-        LOG(llevDebug, "Emergency mappath reset to %s (%d, %d)\n", settings.emergency_mapname, settings.emergency_x, settings.emergency_y);
+        LOG(llevDebug, "emergency map set to %s (%d, %d)\n",
+                settings.emergency_mapname,
+                settings.emergency_x, settings.emergency_y);
     }
 }
 
@@ -202,7 +180,7 @@ static void init_emergency_mappath(void) {
  * using this library.
  * If you want to lessen the size of the program using the library,
  * you can replace the call to init_library() with init_globals() and
- * init_function_pointers().  Good idea to also call init_vars and
+ * init_function_pointers().  Good idea to also call
  * init_hash_table if you are doing any object loading.
  */
 void init_library(void) {
@@ -214,7 +192,6 @@ void init_library(void) {
     init_hash_table();
     i18n_init();
     init_objects();
-    init_vars();
     init_block();
     read_bmap_names();
     read_smooth();
@@ -224,7 +201,13 @@ void init_library(void) {
     init_clocks();
     init_emergency_mappath();
     init_experience();
-    init_regions(); /* init_dynamic() loads a map, so needs a region */
+
+    /* init_dynamic() loads a map, so needs a region */
+    if (init_regions() != 0) {
+        LOG(llevError, "Please check that your maps are correctly installed.\n");
+        exit(EXIT_FAILURE);
+    }
+
     init_dynamic();
 }
 
@@ -273,14 +256,25 @@ static void init_environ(void) {
  */
 void init_globals(void) {
     memset(&statistics, 0, sizeof(struct Statistics));
-    if (settings.logfilename[0] == 0) {
-        logfile = stderr;
-    } else if ((logfile = fopen(settings.logfilename, "a")) == NULL) {
-        fprintf(stderr, "Unable to open %s as the logfile - will use stderr instead\n", settings.logfilename);
-        logfile = stderr;
-    } else {
-        setvbuf(logfile, NULL, _IOLBF, 0);
+
+    /* Log to stderr by default. */
+    logfile = stderr;
+
+    /* Try to open the log file specified on the command-line. */
+    if (settings.logfilename != NULL) {
+        logfile = fopen(settings.logfilename, "a");
+
+        /* If writable, set buffer mode to per-line. */
+        if (logfile != NULL) {
+            setvbuf(logfile, NULL, _IOLBF, 0);
+        } else {
+            logfile = stderr;
+
+            LOG(llevError, "Could not open '%s' for logging.\n",
+                    settings.logfilename);
+        }
     }
+
     exiting = 0;
     first_player = NULL;
     first_friendly_object = NULL;
@@ -289,18 +283,15 @@ void init_globals(void) {
     first_artifactlist = NULL;
     first_archetype = NULL;
     *first_map_ext_path = 0;
-    warn_archetypes = 0;
     nroftreasures = 0;
     nrofartifacts = 0;
     nrofallowedstr = 0;
     ring_arch = NULL;
     amulet_arch = NULL;
-    staff_arch = NULL;
     undead_name = add_string("undead");
     trying_emergency_save = 0;
     num_animations = 0;
     animations = NULL;
-    animations_allocated = 0;
     init_defaults();
 }
 
@@ -349,40 +340,6 @@ void free_globals(void) {
     }
 }
 
-/**
- * Sets up and initialises the linked list of free and used objects.
- * Allocates a certain chunk of objects and puts them on the free list.
- * Called by init_library();
- */
-void init_objects(void) {
-#ifndef MEMORY_DEBUG
-    int i;
-#endif
-    /* Initialize all objects: */
-    objects = NULL;
-    active_objects = NULL;
-
-#ifdef MEMORY_DEBUG
-    free_objects = NULL;
-#else
-    free_objects = objarray;
-    objarray[0].prev = NULL,
-    objarray[0].next = &objarray[1],
-    SET_FLAG(&objarray[0], FLAG_REMOVED);
-    SET_FLAG(&objarray[0], FLAG_FREED);
-    for (i = 1; i < STARTMAX-1; i++) {
-        objarray[i].next = &objarray[i+1];
-        objarray[i].prev = &objarray[i-1];
-        SET_FLAG(&objarray[i], FLAG_REMOVED);
-        SET_FLAG(&objarray[i], FLAG_FREED);
-    }
-    objarray[STARTMAX-1].next = NULL;
-    objarray[STARTMAX-1].prev = &objarray[STARTMAX-2];
-    SET_FLAG(&objarray[STARTMAX-1], FLAG_REMOVED);
-    SET_FLAG(&objarray[STARTMAX-1], FLAG_FREED);
-#endif
-}
-
 /**
  * Initialises global variables which can be changed by options.
  * Called by init_library().
@@ -404,12 +361,13 @@ static void init_dynamic(void) {
     while (at) {
         if (at->clone.type == MAP && at->clone.subtype == MAP_TYPE_LEGACY) {
             if (at->clone.race) {
-                strcpy(first_map_ext_path, at->clone.race);
+                safe_strncpy(first_map_ext_path, at->clone.race,
+                        sizeof(first_map_ext_path));
             }
             if (EXIT_PATH(&at->clone)) {
                 mapstruct *first;
 
-                snprintf(first_map_path, sizeof(first_map_path), "%s", EXIT_PATH(&at->clone));
+                strlcpy(first_map_path, EXIT_PATH(&at->clone), sizeof(first_map_path));
                 first = ready_map_name(first_map_path, 0);
                 if (!first) {
                     LOG(llevError, "Initial map %s can't be found! Please ensure maps are correctly installed.\n", first_map_path);
@@ -436,15 +394,14 @@ unsigned long todtick;
 void write_todclock(void) {
     char filename[MAX_BUF];
     FILE *fp;
+    OutputFile of;
 
     snprintf(filename, sizeof(filename), "%s/clockdata", settings.localdir);
-    fp = fopen(filename, "w");
-    if (fp == NULL) {
-        LOG(llevError, "Cannot open %s for writing\n", filename);
+    fp = of_open(&of, filename);
+    if (fp == NULL)
         return;
-    }
     fprintf(fp, "%lu", todtick);
-    fclose(fp);
+    of_close(&of);
 }
 
 /**
@@ -462,7 +419,6 @@ static void init_clocks(void) {
         has_been_done = 1;
 
     snprintf(filename, sizeof(filename), "%s/clockdata", settings.localdir);
-    LOG(llevDebug, "Reading clockdata from %s...\n", filename);
     fp = fopen(filename, "r");
     if (fp == NULL) {
         LOG(llevError, "Can't open %s.\n", filename);
@@ -472,7 +428,7 @@ static void init_clocks(void) {
     }
     /* Read TOD and default to 0 on failure. */
     if (fscanf(fp, "%lu", &todtick) == 1) {
-        LOG(llevDebug, "todtick=%lu\n", todtick);
+        LOG(llevDebug, "clockdata: todtick is %lu\n", todtick);
         fclose(fp);
     } else {
         LOG(llevError, "Couldn't parse todtick, using default value 0\n");
@@ -503,7 +459,6 @@ static void init_attackmess(void) {
         has_been_done = 1;
 
     snprintf(filename, sizeof(filename), "%s/attackmess", settings.datadir);
-    LOG(llevDebug, "Reading attack messages from %s...\n", filename);
     fp = fopen(filename, "r");
     if (fp == NULL) {
         LOG(llevError, "Can't open %s.\n", filename);
@@ -514,12 +469,18 @@ static void init_attackmess(void) {
     while (fgets(buf, MAX_BUF, fp) != NULL) {
         if (*buf == '#')
             continue;
+        // Find the end of the line and strip the newline
         cp = strchr(buf, '\n');
         if (cp != NULL)
             *cp = '\0';
-        cp = buf;
-        while (*cp == ' ') /* Skip blanks */
-            cp++;
+        /*
+         * Skip blanks -- strspn is slightly faster than a loop w/ optimization on
+         * Also, note we go from the beginning of the line again, since cp was at the end.
+         * While here, also skip tabs for more complete whitespace handling.
+         *
+         * SilverNexus 2018-01-21
+         */
+        cp = buf + strspn(buf, " \t");
 
         if (strncmp(cp, "TYPE:", 5) == 0) {
             p = strtok(buf, ":");
@@ -569,6 +530,6 @@ static void init_attackmess(void) {
             continue;
         }
     }
-    LOG(llevDebug, "got %d messages in %d categories.\n", total, mess+1);
+    LOG(llevDebug, "attackmsg: %d messages in %d categories\n", total, mess+1);
     fclose(fp);
 }
