@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_item_c =
- *   "$Id: item.c 16871 2012-01-28 13:23:12Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Those functions deal with objects in general, including description, body location, and such.
@@ -34,11 +19,15 @@
  * put const char *instead of char *when possible.
  */
 
-#include <stdio.h>
+#include "global.h"
+
 #include <assert.h>
-#include <global.h>
-#include <living.h>
-#include <spells.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "living.h"
+#include "spells.h"
 
 /**
  * The ordering of this is actually doesn't make a difference
@@ -79,26 +68,6 @@ body_locations_struct body_locations[NUM_BODY_LOCATIONS] = {
     /*{"body_dragon_torso", "your body", "a dragon's body"} */
 };
 
-/** Tens */
-static const char *const numbers_10[] = {
-    "zero", "ten", "twenty", "thirty", "fourty", "fifty", "sixty", "seventy",
-    "eighty", "ninety"
-};
-
-/** Levels as a full name and not a number. */
-static const char *const levelnumbers[] = {
-    "zeroth", "first", "second", "third", "fourth", "fifth", "sixth", "seventh",
-    "eighth", "ninth", "tenth", "eleventh", "twelfth", "thirteenth",
-    "fourteenth", "fifteenth", "sixteenth", "seventeenth", "eighteen",
-    "nineteen", "twentieth"
-};
-
-/** Tens for levels */
-static const char *const levelnumbers_10[] = {
-    "zeroth", "tenth", "twentieth", "thirtieth", "fortieth", "fiftieth", "sixtieth",
-    "seventieth", "eightieth", "ninetieth"
-};
-
 /**
  * The following is a large table of item types, the fields are:
  * item number, item name, item name (plural), and two numbers that are the skills
@@ -199,7 +168,7 @@ static const typedata item_types[] = {
     { GIRDLE, "girdle", "girdles", SK_SMITHERY, 0 },
     { FORCE, "force", "forces", 0, 0 },
     { POTION_RESIST_EFFECT, "potion effect", "potion effects", 0, 0 },
-    { CLOSE_CON, "closed container", "closed container", 0, 0 },
+    { CLOSE_CON, "closed container", "closed container", SK_ALCHEMY, 0 },
     { CONTAINER, "container", "containers", SK_ALCHEMY, 0 },
     { ARMOUR_IMPROVER, "armour improver", "armour improvers", SK_LITERACY, 0 },
     { WEAPON_IMPROVER, "weapon improver", "weapon improvers", SK_LITERACY, 0 },
@@ -209,12 +178,13 @@ static const typedata item_types[] = {
     { SHOP_INVENTORY, "inventory list", "inventory lists", 0, 0 },
     { RUNE, "rune", "runes", 0, 0 },
     { TRAP, "trap", "traps", 0, 0 },
-    { POWER_CRYSTAL, "power_crystal", "power_crystals", 0, 0 },
+    { POWER_CRYSTAL, "power_crystal", "power_crystals", SK_THAUMATURGY, 0 },
     { CORPSE, "corpse", "corpses", 0, 0 },
     { DISEASE, "disease", "diseases", 0, 0 },
     { SYMPTOM, "symptom", "symptoms", 0, 0 },
     { BUILDER, "item builder", "item builders", 0, 0 },
     { MATERIAL, "building material", "building materials", 0, 0 },
+    { MIMIC, "mimic", "mimics", SK_ALCHEMY, 0 },
 };
 
 /** Number of items in ::item_types array. */
@@ -244,6 +214,16 @@ int get_power_from_ench(int ench) {
     return enc_to_item_power[ench];
 }
 
+static int bits_set(uint32_t x, int start, int end) {
+    int enc = 0;
+    for (int i = start; i < end; i++) {
+        if (x&(1<<i)) {
+            enc++;
+        }
+    }
+    return enc;
+}
+
 /**
  * This takes an object 'op' and figures out what its item_power
  * rating should be.  This should only really be used by the treasure
@@ -282,9 +262,7 @@ int calc_item_power(const object *op) {
      * physical doesn't count against total.
      */
     if (op->type == WEAPON) {
-        for (i = 1; i < NROFATTACKS; i++)
-            if (op->attacktype&(1<<i))
-                enc++;
+        enc += bits_set(op->attacktype, 1, NROFATTACKS);
         if (op->slaying)
             enc += 2;     /* What it slays is probably more relevent */
     }
@@ -309,14 +287,9 @@ int calc_item_power(const object *op) {
     enc += op->stats.luck;
 
     /* Do spell paths now */
-    for (i = 1; i < NRSPELLPATHS; i++) {
-        if (op->path_attuned&(1<<i))
-            enc++;
-        else if (op->path_denied&(1<<i))
-            enc -= 2;
-        else if (op->path_repelled&(1<<i))
-            enc--;
-    }
+    enc += bits_set(op->path_attuned, 1, NRSPELLPATHS);
+    enc -= bits_set(op->path_repelled, 1, NRSPELLPATHS);
+    enc -= 2*bits_set(op->path_denied, 1, NRSPELLPATHS);
 
     if (QUERY_FLAG(op, FLAG_LIFESAVE))
         enc += 5;
@@ -384,11 +357,14 @@ const typedata *get_typedata_by_name(const char *name) {
  * @param newline
  * If TRUE, we don't put parens around the description
  * but do put a newline at the end. Useful when dumping to files
+ * @param use_media_tags
+ * if non-zero, then media tags (colors and such) are inserted in the description.
+ * This enables the player to more easily see some things.
  * @param buf
  * buffer that will receive the description. Can be NULL.
  * @return buf, a new StringBuffer the caller should free if buf was NULL.
  */
-StringBuffer *describe_resistance(const object *op, int newline, StringBuffer *buf) {
+StringBuffer *describe_resistance(const object *op, int newline, int use_media_tags, StringBuffer *buf) {
     int tmpvar;
 
     if (buf == NULL)
@@ -396,10 +372,26 @@ StringBuffer *describe_resistance(const object *op, int newline, StringBuffer *b
 
     for (tmpvar = 0; tmpvar < NROFATTACKS; tmpvar++) {
         if (op->resist[tmpvar] && (op->type != FLESH || atnr_is_dragon_enabled(tmpvar) == 1)) {
+            if (use_media_tags) {
+                if (resist_color[tmpvar] != NULL) {
+                    stringbuffer_append_printf(buf, "[color=%s]", resist_color[tmpvar]);
+                }
+                if (op->resist[tmpvar] == 100 || op->resist[tmpvar] == -100) {
+                    stringbuffer_append_string(buf, "[ul]");
+                }
+            }
             if (!newline)
                 stringbuffer_append_printf(buf, "(%s %+d)", resist_plus[tmpvar], op->resist[tmpvar]);
             else
                 stringbuffer_append_printf(buf, "%s %d\n", resist_plus[tmpvar], op->resist[tmpvar]);
+            if (use_media_tags) {
+                if (op->resist[tmpvar] == 100 || op->resist[tmpvar] == -100) {
+                    stringbuffer_append_string(buf, "[/ul]");
+                }
+                if (resist_color[tmpvar] != NULL) {
+                    stringbuffer_append_string(buf, "[/color]");
+                }
+            }
         }
     }
 
@@ -416,7 +408,7 @@ StringBuffer *describe_resistance(const object *op, int newline, StringBuffer *b
  * buffer size.
  */
 void query_weight(const object *op, char *buf, size_t size) {
-    sint32 i = (op->nrof ? op->nrof : 1)*op->weight+op->carrying;
+    int32_t i = NROF(op)*op->weight+op->carrying;
 
     if (op->weight < 0)
         snprintf(buf, size, "      ");
@@ -426,42 +418,19 @@ void query_weight(const object *op, char *buf, size_t size) {
         snprintf(buf, size, "%4d  ", i/1000);
 }
 
-/**
- * Formats a level.
- * @param i
- * level to format.
- * @param[out] buf
- * buffer which will contain the level.
- * @param size
- * size of the buffer.
- */
-void get_levelnumber(int i, char *buf, size_t size) {
-    if (i > 99 || i < 0) {
-        snprintf(buf, size, "%d.", i);
-        return;
-    }
-    if (i < 21) {
-        snprintf(buf, size, "%s", levelnumbers[i]);
-        return;
-    }
-    if (!(i%10)) {
-        snprintf(buf, size, "%s", levelnumbers_10[i/10]);
-        return;
-    }
-    snprintf(buf, size, "%s%s", numbers_10[i/10], levelnumbers[i%10]);
-    return;
-}
-
 /**
  * Describes a ring or amulet, or a skill.
  * @param op
  * item to describe, must be RING, AMULET or SKILL.
+ * @param use_media_tags
+ * if non-zero, then media tags (colors and such) are inserted in the description.
+ * This enables the player to more easily see some things.
  * @param buf
  * buffer that will contain the description. If NULL a new one is created.
  * @return buf, or a new StringBuffer the caller should free if buf was NULL.
  * @todo why does this also describe a SKILL?
  */
-static StringBuffer *ring_desc(const object *op, StringBuffer *buf) {
+static StringBuffer *ring_desc(const object *op, int use_media_tags, StringBuffer *buf) {
     int attr, val;
     size_t len;
 
@@ -489,7 +458,7 @@ static StringBuffer *ring_desc(const object *op, StringBuffer *buf) {
     if (op->stats.ac)
         stringbuffer_append_printf(buf, "(ac%+d)", op->stats.ac);
 
-    describe_resistance(op, 0, buf);
+    describe_resistance(op, 0, use_media_tags, buf);
 
     if (op->stats.food != 0)
         stringbuffer_append_printf(buf, "(sustenance%+d)", op->stats.food);
@@ -509,6 +478,8 @@ static StringBuffer *ring_desc(const object *op, StringBuffer *buf) {
         stringbuffer_append_printf(buf, "(reflect missiles)");
     if (QUERY_FLAG(op, FLAG_STEALTH))
         stringbuffer_append_printf(buf, "(stealth)");
+    if (op->glow_radius)
+        stringbuffer_append_string(buf, "(glowing)");
 
     describe_spellpath_attenuation("Attuned", op->path_attuned, buf);
     describe_spellpath_attenuation("Repelled", op->path_repelled, buf);
@@ -542,7 +513,7 @@ void query_short_name(const object *op, char *buf, size_t size) {
         return;
     }
     if (!op->nrof && !op->weight && !op->title && !is_magical(op)) {
-        snprintf(buf, size, "%s", op->name); /* To speed things up (or make things slower?) */
+        strlcpy(buf, op->name, size); /* To speed things up (or make things slower?) */
         return;
     }
     buf[0] = '\0';
@@ -563,12 +534,9 @@ void query_short_name(const object *op, char *buf, size_t size) {
     case WAND:
     case ROD:
         if (QUERY_FLAG(op, FLAG_IDENTIFIED) || QUERY_FLAG(op, FLAG_BEEN_APPLIED)) {
-            if (!op->title) {
+            if (!op->title && (op->inv)) {
                 safe_strcat(buf, " of ", &len, size);
-                if (op->inv)
-                    safe_strcat(buf, op->inv->name, &len, size);
-                else
-                    LOG(llevError, "Spellbook %s lacks inventory\n", op->name);
+                safe_strcat(buf, op->inv->name, &len, size);
             }
             if (op->type != SPELLBOOK) {
                 snprintf(buf+len, size-len, " (lvl %d)", op->level);
@@ -584,7 +552,7 @@ void query_short_name(const object *op, char *buf, size_t size) {
             /* If ring has a title, full description isn't so useful */
             char* desc;
 
-            desc = stringbuffer_finish(ring_desc(op, NULL));
+            desc = stringbuffer_finish(ring_desc(op, 0, NULL));
             if (desc[0]) {
                 safe_strcat(buf, " ", &len, size);
                 safe_strcat(buf, desc, &len, size);
@@ -595,7 +563,7 @@ void query_short_name(const object *op, char *buf, size_t size) {
 
     default:
         if (op->magic
-        && ((QUERY_FLAG(op, FLAG_BEEN_APPLIED) && need_identify(op)) || QUERY_FLAG(op, FLAG_IDENTIFIED))) {
+        && (QUERY_FLAG(op, FLAG_BEEN_APPLIED) || is_identified(op))) {
             snprintf(buf+len, size-len, " %+d", op->magic);
             len += strlen(buf+len);
         }
@@ -616,9 +584,16 @@ void query_name(const object *op, char *buf, size_t size) {
     size_t len = 0;
 
     buf[0] = '\0';
-
-    query_short_name(op, buf+len, size-len);
-    len += strlen(buf+len);
+    /*
+     * Since len is initialized to zero, but is not changed
+     * up to this point, I see no reason to have buf+len and size-len;
+     * buf and size should suffice, since we're just adding and
+     * subtracting zero here, anyway.
+     *
+     * SilverNexus 2014-06-04
+     */
+    query_short_name(op, buf, size);
+    len = strlen(buf);
 
     if (QUERY_FLAG(op, FLAG_INV_LOCKED))
         safe_strcat(buf, " *", &len, size);
@@ -718,7 +693,7 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
 
     buf[0] = '\0';
 
-    snprintf(buf, size, "%s", plural ? op->name_pl : op->name);
+    strlcpy(buf, plural ? op->name_pl : op->name, size);
     len = strlen(buf);
 
     if (op->title && QUERY_FLAG(op, FLAG_IDENTIFIED)) {
@@ -732,12 +707,9 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
     case WAND:
     case ROD:
         if (QUERY_FLAG(op, FLAG_IDENTIFIED) || QUERY_FLAG(op, FLAG_BEEN_APPLIED)) {
-            if (!op->title) {
+            if (!op->title && (op->inv)) {
                 safe_strcat(buf, " of ", &len, size);
-                if (op->inv)
-                    safe_strcat(buf, op->inv->name, &len, size);
-                else
-                    LOG(llevError, "Spellbook %s lacks inventory\n", op->name);
+                safe_strcat(buf, op->inv->name, &len, size);
             }
             if (op->type != SPELLBOOK) {
                 snprintf(buf+len, size-len, " (lvl %d)", op->level);
@@ -753,7 +725,7 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
             /* If ring has a title, full description isn't so useful */
             char* s;
 
-            s = stringbuffer_finish(ring_desc(op, NULL));
+            s = stringbuffer_finish(ring_desc(op, 0, NULL));
             if (s[0]) {
                 safe_strcat(buf, " ", &len, size);
                 safe_strcat(buf, s, &len, size);
@@ -764,7 +736,7 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
 
     default:
         if (op->magic
-        && ((QUERY_FLAG(op, FLAG_BEEN_APPLIED) && need_identify(op)) || QUERY_FLAG(op, FLAG_IDENTIFIED))) {
+        && (QUERY_FLAG(op, FLAG_BEEN_APPLIED) || is_identified(op))) {
             snprintf(buf+strlen(buf), size-strlen(buf), " %+d", op->magic);
         }
     }
@@ -775,6 +747,9 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
  *
  * @param op
  * monster to describe. Must not be NULL, and must have FLAG_MONSTER or be a PLAYER.
+ * @param use_media_tags
+ * if non-zero, then media tags (colors and such) are inserted in the description.
+ * This enables the player to more easily see some things.
  * @param buf
  * buffer that will contain the description. Can be NULL.
  * @return buf, or a new StringBuffer the caller should clear if buf was NULL.
@@ -783,9 +758,7 @@ void query_base_name(const object *op, int plural, char *buf, size_t size) {
  * Rename to describe_living (or equivalent) since called for player too.
  * Fix weird sustenance logic.
  */
-StringBuffer *describe_monster(const object *op, StringBuffer *buf) {
-    int i;
-
+StringBuffer *describe_monster(const object *op, int use_media_tags, StringBuffer *buf) {
     assert(op != NULL);
     assert(QUERY_FLAG(op, FLAG_MONSTER) || op->type == PLAYER);
 
@@ -912,11 +885,7 @@ StringBuffer *describe_monster(const object *op, StringBuffer *buf) {
     describe_spellpath_attenuation("Attuned", op->path_attuned &~ op->path_denied, buf);
     describe_spellpath_attenuation("Repelled", op->path_repelled &~ op->path_denied, buf );
     describe_spellpath_attenuation("Denied", op->path_denied, buf);
-    for (i = 0; i < NROFATTACKS; i++) {
-        if (op->resist[i]) {
-            stringbuffer_append_printf(buf, "(%s %+d)", resist_plus[i], op->resist[i]);
-        }
-    }
+    describe_resistance(op, 0, use_media_tags, buf);
 
     return buf;
 }
@@ -947,6 +916,9 @@ StringBuffer *describe_monster(const object *op, StringBuffer *buf) {
  * object to describe. Must not be NULL.
  * @param owner
  * player examining the object.
+ * @param use_media_tags
+ * if non-zero, then media tags (colors and such) are inserted in the description.
+ * This enables the player to more easily see some things.
  * @param buf
  * buffer that will contain the description. Can be NULL.
  * @return buf, or new StringBuffer the caller must free if buf was NULL.
@@ -960,22 +932,22 @@ StringBuffer *describe_monster(const object *op, StringBuffer *buf) {
  * @todo
  * Check whether owner is really needed.
  */
-StringBuffer *describe_item(const object *op, const object *owner, StringBuffer *buf) {
+StringBuffer *describe_item(const object *op, const object *owner, int use_media_tags, StringBuffer *buf) {
     int identified, i;
 
     if (buf == NULL)
         buf = stringbuffer_new();
 
     if (QUERY_FLAG(op, FLAG_MONSTER) || op->type == PLAYER) {
-        return describe_monster(op, buf);
+        return describe_monster(op, use_media_tags, buf);
     }
 
-    /* figure this out once, instead of making multiple calls to need_identify.
+    /* figure this out once, instead of making multiple calls to is_identified.
      * also makes the code easier to read.
      */
-    if (!need_identify(op) || QUERY_FLAG(op, FLAG_IDENTIFIED))
+    if (is_identified(op)) {
         identified = 1;
-    else {
+    } else {
         stringbuffer_append_string(buf, "(unidentified)");
         identified = 0;
     }
@@ -999,37 +971,6 @@ StringBuffer *describe_item(const object *op, const object *owner, StringBuffer
     case LAMP:
         break; /* just so we get the "glowing" part. */
 
-    case POWER_CRYSTAL:
-        /* Avoid division by zero... */
-        if (op->stats.maxsp == 0) {
-            stringbuffer_append_printf(buf, "(capacity %d).", op->stats.maxsp);
-        } else {
-            if (op->stats.maxsp > 1000) { /*higher capacity crystals*/
-                i = (op->stats.maxsp%1000)/100;
-                if (i)
-                    stringbuffer_append_printf(buf, "(capacity %d.%dk). It is ", op->stats.maxsp/1000, i);
-                else
-                    stringbuffer_append_printf(buf, "(capacity %dk). It is ", op->stats.maxsp/1000);
-            } else
-                stringbuffer_append_printf(buf, "(capacity %d). It is ", op->stats.maxsp);
-            i = (op->stats.sp*10)/op->stats.maxsp;
-            if (op->stats.sp == 0)
-                stringbuffer_append_string(buf, "empty.");
-            else if (i == 0)
-                stringbuffer_append_string(buf, "almost empty.");
-            else if (i < 3)
-                stringbuffer_append_string(buf, "partially filled.");
-            else if (i < 6)
-                stringbuffer_append_string(buf, "half full.");
-            else if (i < 9)
-                stringbuffer_append_string(buf, "well charged.");
-            else if (op->stats.sp == op->stats.maxsp)
-                stringbuffer_append_string(buf, "fully charged.");
-            else
-                stringbuffer_append_string(buf, "almost full.");
-        }
-        break;
-
     case FOOD:
     case FLESH:
     case DRINK:
@@ -1060,9 +1001,7 @@ StringBuffer *describe_item(const object *op, const object *owner, StringBuffer
         if (op->item_power) {
             stringbuffer_append_printf(buf, "(item_power %+d)", op->item_power);
         }
-        if (op->title) {
-            ring_desc(op, buf);
-        }
+        ring_desc(op, use_media_tags, buf);
         return buf;
 
     default:
@@ -1231,7 +1170,7 @@ StringBuffer *describe_item(const object *op, const object *owner, StringBuffer
          * non flesh, everyone can see its resistances
          */
         if (op->type != FLESH || (owner && is_dragon_pl(owner))) {
-            describe_resistance(op, 0, buf);
+            describe_resistance(op, 0, 1, buf);
         }
         describe_spellpath_attenuation("Attuned", op->path_attuned &~ op->path_denied, buf);
         describe_spellpath_attenuation("Repelled", op->path_repelled &~ op->path_denied, buf);
@@ -1332,54 +1271,50 @@ int is_magical(const object *op) {
 }
 
 /**
- * This function really should not exist - by default, any item not identified
- * should need it.
+ * Return true if this item's type is one that cares about whether or not it's
+ * been identified -- e.g. money and walls don't care (they are considered to
+ * be "always identified" no matter what FLAG_IDENTIFIED says) but weapons and
+ * scrolls do.
+ * In practice, this means anything with at least one identifyskill in the
+ * typedata, or skill tools (which have no identifyskill, but can still be IDed
+ * using magic).
  *
  * @param op
- * item to check.
+ * Object to check.
  * @return
- * true if the item should be identified.
+ * True if the object can be identified.
  * @todo
- * either remove this function, or fix comment above :)
+ * Ideally, this distinction would be meaningless and we'd just always check
+ * FLAG_IDENTIFIED, and assume that items that shouldn't ever need to be
+ * identified will always have the flag set. In practice we can't reliably
+ * make that assumption.
  */
-int need_identify(const object *op) {
-    switch (op->type) {
-    case RING:
-    case WAND:
-    case ROD:
-    case SCROLL:
-    case SKILL:
-    case SKILLSCROLL:
-    case SPELLBOOK:
-    case FOOD:
-    case POTION:
-    case BOW:
-    case ARROW:
-    case WEAPON:
-    case ARMOUR:
-    case SHIELD:
-    case HELMET:
-    case AMULET:
-    case BOOTS:
-    case GLOVES:
-    case BRACERS:
-    case GIRDLE:
-    case CONTAINER:
-    case DRINK:
-    case FLESH:
-    case INORGANIC:
-    case CLOSE_CON:
-    case CLOAK:
-    case GEM:
-    case POWER_CRYSTAL:
-    case POISON:
-    case BOOK:
-    case SKILL_TOOL:
-    case ARMOUR_IMPROVER:
-    case WEAPON_IMPROVER:
-        return 1;
-    }
-    return 0;
+int is_identifiable_type(const object *op) {
+    /* Special case -- skill tools don't have an associated identification
+     * skill but still need IDing. TODO: we should probably have per-tool ID
+     * skills, e.g. thievery for lockpicks and thaumaturgy for talismans, but
+     * currently there's no good way to do this because the identifyskill is
+     * tied to the itemtype rather than to the arch. */
+    if (op->type == SKILL_TOOL) return true;
+
+    /* Sometimes an itemtype has no associated typedata?! */
+    const typedata *td = get_typedata(op->type);
+    return td && td->identifyskill;
+}
+
+/**
+ * Return true if the item is identified, either because it is of a type that
+ * doesn't ever need identification (e.g. money), or because it has
+ * FLAG_IDENTIFIED set.
+ * Uses is_identifiable_type() to distinguish between the two cases.
+ *
+ * @param op
+ * Object to check.
+ * @return
+ * true if the item has been identified, or doesn't need identification.
+ */
+int is_identified(const object *op) {
+    return !is_identifiable_type(op) || QUERY_FLAG(op, FLAG_IDENTIFIED);
 }
 
 /**
@@ -1391,14 +1326,14 @@ void object_give_identified_properties(object *op) {
 
     key = object_get_value(op, "identified_face");
     if (key != NULL) {
-        op->face = &new_faces[find_face(key, op->face->number)];
+        op->face = find_face(key, op->face);
         /* if the face is defined, clean the animation, because else
          * the face can be lost ; if an animation is defined, it'll be
          * processed later on */
         CLEAR_FLAG(op, FLAG_CLIENT_ANIM_RANDOM);
         CLEAR_FLAG(op, FLAG_CLIENT_ANIM_SYNC);
         op->anim_speed = 0;
-        op->animation_id = 0;
+        op->animation = 0;
         object_set_value(op, "identified_face", NULL, 0);
     }
 
@@ -1416,7 +1351,7 @@ void object_give_identified_properties(object *op) {
 
     key = object_get_value(op, "identified_animation");
     if (key != NULL) {
-        op->animation_id = atoi(key);
+        op->animation = find_animation(key);
         if (!QUERY_FLAG(op, FLAG_IS_TURNABLE))
             SET_FLAG(op, FLAG_ANIMATE);
         animate_object(op, op->facing);
