@@ -1,34 +1,21 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
-/**
- * @file
- * i18n support routines. See @ref page_i18n.
- * @warning
- * This file is encoded in UTF-8, take care to not mess the language_names array.
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-#include <global.h>
+#include "global.h"
+
+#include <errno.h>
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * One message.
@@ -101,7 +88,7 @@ const char *i18n(const object *who, const char *code) {
 }
 
 /**
- * Attenmpt to find the identifier of a language from its code.
+ * Attempt to find the identifier of a language from its code.
  * @param code language code.
  * @return index, -1 if not found.
  */
@@ -179,7 +166,7 @@ static void convert_newline(char *line) {
  * If there is an error, calls fatal().
  */
 void i18n_init(void) {
-    char dirname[MAX_BUF], filename[MAX_BUF], line[HUGE_BUF];
+    char dirname[MAX_BUF], filename[MAX_BUF*2], line[HUGE_BUF];
     FILE *fp;
     char *token;
     DIR *dir;
@@ -190,7 +177,7 @@ void i18n_init(void) {
 
     dir = opendir(dirname);
     if (dir == NULL) {
-        LOG(llevError, "couldn't open i18n directory %s\n", dirname);
+        LOG(llevError, "i18n: couldn't open %s\n", dirname);
         fatal(SEE_LAST_ERROR);
     }
 
@@ -202,7 +189,8 @@ void i18n_init(void) {
 
         snprintf(filename, sizeof(filename), "%s%s", dirname, file->d_name);
         if ((fp = fopen(filename, "r")) == NULL) {
-            LOG(llevError, "Cannot open i18n file %s: %s\n", filename, strerror_local(errno, line, sizeof(line)));
+            LOG(llevError, "i18n: couldn't open %s\n",
+                    filename, strerror(errno));
             fatal(SEE_LAST_ERROR);
         }
 
@@ -235,12 +223,13 @@ void i18n_init(void) {
         qsort(i18n_files[i18n_count].messages, i18n_files[i18n_count].count, sizeof(i18n_message), (int (*)(const void *, const void *))i18n_message_compare_code);
         found = bsearch(&code, i18n_files[i18n_count].messages, i18n_files[i18n_count].count, sizeof(i18n_message), (int (*)(const void *, const void *))i18n_message_compare_code);
         if (found == NULL) {
-            LOG(llevError, "couldn't find language name (LN) for %s\n", filename);
+            LOG(llevError, "i18n: no language set in %s\n", filename);
             fatal(SEE_LAST_ERROR);
         }
 
         i18n_files[i18n_count].name = found->message;
-        LOG(llevDebug, "Read %i strings for language: %s\n", i18n_files[i18n_count].count, found->message);
+        LOG(llevDebug, "i18n: %d strings for %s\n",
+                i18n_files[i18n_count].count, found->message);
 
         if (strcmp(i18n_files[i18n_count].code, "en") == 0)
             i18n_default = i18n_count;
@@ -252,7 +241,7 @@ void i18n_init(void) {
     free_string(code.code);
 
     if (i18n_default == -1) {
-        LOG(llevError, "couldn't find default language en!\n");
+        LOG(llevError, "i18n: couldn't find default language (en)\n");
         fatal(SEE_LAST_ERROR);
     }
 }
