@@ -1,50 +1,38 @@
 /*
- * static char *rcsid_loger_c =
- *   "$Id: logger.c 17401 2012-03-08 22:13:26Z ryo_saeba $ ";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-int reopen_logfile = 0; /* May be set in SIGHUP handler */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file logger.c
  * This handles logging, to file or strerr/stdout.
  */
 
+#include "global.h"
+
 #include <stdarg.h>
-#include <global.h>
-#include <sproto.h>
+#include <stdlib.h>
+
+#include "sproto.h"
+
+int reopen_logfile = 0; /* May be set in SIGHUP handler */
 
 /**
  * Human-readable name of log levels.
  */
 static const char *const loglevel_names[] = {
-    "[Error]   ",
-    "[Info]    ",
-    "[Debug]   ",
-    "[Monster] "
+    "[EE] ",
+    "[II] ",
+    "[DD] ",
+    "[MM] ",
 };
 
 /**
