@@ -1,40 +1,26 @@
 /*
- * static char *rcsid_los_c =
- *   "$Id: los.c 14671 2011-06-21 17:17:13Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file los.c
  * This handles the "line of sight" for players and monsters.
  */
 
-/* Nov 95 - inserted USE_LIGHTING code stuff in here - b.t. */
+#include "global.h"
 
-#include <global.h>
 #include <math.h>
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * Distance must be less than this for the object to be blocked.
@@ -117,14 +103,14 @@ void init_block(void) {
      */
     for (x = 1; x <= MAP_CLIENT_X/2; x++) {
         for (y = 1; y <= MAP_CLIENT_Y/2; y++) {
+            /* center space never blocks */
+            if (x == MAP_CLIENT_X/2 && y == MAP_CLIENT_Y/2)
+                continue;
+
             for (i = 0; i < 3; i++) {
                 dx = x+block_x[i];
                 dy = y+block_y[i];
 
-                /* center space never blocks */
-                if (x == MAP_CLIENT_X/2 && y == MAP_CLIENT_Y/2)
-                    continue;
-
                 /* If its a straight line, its blocked */
                 if ((dx == x && x == MAP_CLIENT_X/2)
                 || (dy == y && y == MAP_CLIENT_Y/2)) {
@@ -175,33 +161,45 @@ void init_block(void) {
  * In this way, the chain of visibility is set.
  * Used to initialise the array used by the LOS routines.
  *
- * @param op
+ * @param pl
  * player for which we're computing.
  * @param x
  * @param y
  * indexes into the blocked[][] array.
  */
-static void set_wall(object *op, int x, int y) {
-    int i;
+static void set_wall(player *pl, int x, int y) {
+    // Get this outside the loop -- now we can recycle x and y in the loop
+    // Trying less dereferencing for better efficiency
+    // Equivalent to &(block[x][y]), but faster.
+    const blocks * const at = block[x] + y;
+    /*
+     * Assume the mapsize will not change mid-drawing. This seemed to work in my testing
+     * Daniel Hawkins 2019-12-03
+     */
+    const int map_sock_x = (MAP_CLIENT_X-pl->socket.mapx)>>1,
+              map_sock_y = (MAP_CLIENT_Y-pl->socket.mapy)>>1;
+    for (int i = 0; i < at->index; i++) {
+        int dx = at->x[i], dy = at->y[i];
 
-    for (i = 0; i < block[x][y].index; i++) {
-        int dx = block[x][y].x[i], dy = block[x][y].y[i], ax, ay;
-
-        /* ax, ay are the values as adjusted to be in the
-        * socket look structure.
+        /* x, y are the values as adjusted to be in the
+        * socket look structure. Since the values are copied,
+        * we can safely store this in the x and y passed to the function,
+        * since they were only needed to find the element of the blocks array.
         */
-        ax = dx-(MAP_CLIENT_X-op->contr->socket.mapx)/2;
-        ay = dy-(MAP_CLIENT_Y-op->contr->socket.mapy)/2;
+        x = dx-map_sock_x;
+        y = dy-map_sock_y;
 
-        if (ax < 0 || ax >= op->contr->socket.mapx
-        || ay < 0 || ay >= op->contr->socket.mapy)
+        if (x < 0 || x >= pl->socket.mapx
+        || y < 0 || y >= pl->socket.mapy
+        /* If already set to 100, we probably hit this space through recursion already. */
+        || pl->blocked_los[x][y] == 100)
             continue;
         /* we need to adjust to the fact that the socket
          * code wants the los to start from the 0,0
          * and not be relative to middle of los array.
          */
-        op->contr->blocked_los[ax][ay] = 100;
-        set_wall(op, dx, dy);
+        pl->blocked_los[x][y] = 100;
+        set_wall(pl, dx, dy);
     }
 }
 
@@ -241,26 +239,23 @@ static void check_wall(object *op, int x, int y) {
 
 
     if (get_map_flags(op->map, NULL, op->x+x-MAP_CLIENT_X/2, op->y+y-MAP_CLIENT_Y/2, NULL, NULL)&(P_BLOCKSVIEW|P_OUT_OF_MAP))
-        set_wall(op, x, y);
+        set_wall(op->contr, x, y);
 }
 
 /**
  * Clears/initialises the los-array associated to the player
  * controlling the object.
  *
- * @param op
- * player's object.
- *
- * @todo
- * use player *instead of object *to show it must be a player?
+ * @param pl
+ * player to clear.
  */
-void clear_los(object *op) {
+void clear_los(player *pl) {
     /* This is safer than using the socket->mapx, mapy because
      * we index the blocked_los as a 2 way array, so clearing
      * the first z spaces may not not cover the spaces we are
      * actually going to use
      */
-    (void)memset((void *)op->contr->blocked_los, 0, MAP_CLIENT_X*MAP_CLIENT_Y);
+    (void)memset((void *)pl->blocked_los, 0, MAP_CLIENT_X*MAP_CLIENT_Y);
 }
 
 /**
@@ -334,7 +329,7 @@ int has_carried_lights(const object *op) {
 static void expand_lighted_sight(object *op) {
     int x, y, darklevel, ax, ay, basex, basey, mflags, light, x1, y1;
     mapstruct *m = op->map;
-    sint16 nx, ny;
+    int16_t nx, ny;
 
     darklevel = MAP_DARKNESS(m);
 
@@ -441,17 +436,17 @@ static void expand_lighted_sight(object *op) {
  * odd that you can see yourself (and what your standing on), but
  * really need for any reasonable game play.
  *
- * @param op
- * player's object for which to reset los. Must have a valid contr.
+ * @param pl
+ * player for which to reset los.
  */
-static void blinded_sight(object *op) {
+static void blinded_sight(player *pl) {
     int x, y;
 
-    for (x = 0; x < op->contr->socket.mapx; x++)
-        for (y = 0; y <  op->contr->socket.mapy; y++)
-            op->contr->blocked_los[x][y] = 100;
+    for (x = 0; x < pl->socket.mapx; x++)
+        for (y = 0; y <  pl->socket.mapy; y++)
+            pl->blocked_los[x][y] = 100;
 
-    op->contr->blocked_los[op->contr->socket.mapx/2][op->contr->socket.mapy/2] = 0;
+    pl->blocked_los[pl->socket.mapx/2][pl->socket.mapy/2] = 0;
 }
 
 /**
@@ -467,7 +462,7 @@ void update_los(object *op) {
     if (QUERY_FLAG(op, FLAG_REMOVED))
         return;
 
-    clear_los(op);
+    clear_los(op->contr);
     if (QUERY_FLAG(op, FLAG_WIZ) /* || XRAYS(op) */)
         return;
 
@@ -483,7 +478,7 @@ void update_los(object *op) {
 
     /* do the los of the player. 3 (potential) cases */
     if (QUERY_FLAG(op, FLAG_BLIND)) /* player is blind */
-        blinded_sight(op);
+        blinded_sight(op->contr);
     else
         expand_sight(op);
 
