@@ -1,37 +1,25 @@
 /*
- * static char *rcsid_player_c =
- *   "$Id: player.c 12599 2010-03-08 18:33:07Z akirschbaum $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Player-structure related functions.
  */
 
-#include <global.h>
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * Clears data in player structure.
@@ -61,7 +49,7 @@ void clear_player(player *pl) {
 }
 
 /**
- * Clears player structure, including pointed object (through object_free_drop_inventory()).
+ * Frees player structure, including pointed object (through object_free_drop_inventory()).
  *
  * @param pl
  * player to clear. Pointer is invalid after this call.
@@ -90,7 +78,7 @@ void free_player(player *pl) {
 
     free(pl->socket.faces_sent);
 
-    CFREE(pl);
+    free(pl);
 }
 
 /**
@@ -153,10 +141,13 @@ client_spell *get_client_spell_state(player *pl, object *spell) {
             return info;
         info = info->next;
     }
-    info = (client_spell *)malloc(sizeof(client_spell));
+    /*
+     * Why take the time to malloc() and then memset()?
+     * Just calloc and its good to go!
+     */
+    info = (client_spell *)calloc(1, sizeof(client_spell));
     if (info == NULL)
         fatal(OUT_OF_MEMORY);
-    memset(info, 0, sizeof(client_spell));
     info->next = pl->spell_state;
     info->spell = spell;
     pl->spell_state = info;
@@ -234,7 +225,7 @@ void player_get_title(const struct pl *pl, char *buf, size_t bufsize) {
     if (pl->own_title[0] == '\0')
         snprintf(buf, bufsize, "the %s", pl->title);
     else
-        snprintf(buf, bufsize, "%s", pl->own_title);
+        strlcpy(buf, pl->own_title, bufsize);
 }
 
 /**
@@ -271,6 +262,36 @@ const char *player_get_own_title(const struct pl *pl) {
  * the new title to set; empty string to unset
  */
 void player_set_own_title(struct pl *pl, const char *title) {
-    snprintf(pl->own_title, sizeof(pl->own_title), "%s", title);
+    strlcpy(pl->own_title, title, sizeof(pl->own_title));
     replace_unprintable_chars(pl->own_title);
 }
+
+/**
+ * This function goes through the player inventory and sets
+ * up the last_skills[] array in the player object.
+ * The last_skills[] is used to more quickly lookup skills -
+ * mostly used for sending exp.
+ * This function should be called anytime the player gains a skill.
+ *
+ * @param op
+ * player to link skills for. Must be a player.
+ */
+void link_player_skills(object *op) {
+    int skill;
+    FOR_INV_PREPARE(op, tmp) {
+        if (tmp->type == SKILL) {
+            for (skill = 0; skill < MAX_SKILLS; skill++) {
+                if (op->contr->last_skill_ob[skill] == NULL) {
+                    /* Didn't find the skill, so add it */
+                    op->contr->last_skill_ob[skill] = tmp;
+                    op->contr->last_skill_exp[skill] = -1;
+                    break;
+                }
+                if (op->contr->last_skill_ob[skill] == tmp) {
+                    /* Skill already linked, nothing to do */
+                    break;
+                }
+            }
+        }
+    } FOR_INV_FINISH();
+}
