@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_readable_c =
- *   "$Id: readable.c 16408 2012-01-01 13:03:00Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file readable.c
  * This file contains code relevant to the BOOKS hack -- designed
@@ -37,12 +22,18 @@
 
 /* laid down initial file - dec 1995. -b.t. thomas@astro.psu.edu */
 
-#include <stdio.h>
-#include <global.h>
-#include <book.h>
-#include <living.h>
-#include <spells.h>
+#include "global.h"
+
 #include <assert.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+
+#include "book.h"
+#include "living.h"
+#include "output_file.h"
+#include "spells.h"
 
 /* Define this if you want to archive book titles by contents.
  * This option should enforce UNIQUE combinations of titles,authors and
@@ -82,12 +73,19 @@
 /** How many times to try to generate a unique name for a book. */
 #define MAX_TITLE_CHECK 20
 
+/** Message from the lib/messages file. */
 #define MSGTYPE_LIB 0
+/** Monster-related information. */
 #define MSGTYPE_MONSTER 1
+/** Artifact-related information. */
 #define MSGTYPE_ARTIFACT 2
+/** Spellpath-related information. */
 #define MSGTYPE_SPELLPATH 3
+/** Alchemy-related information. */
 #define MSGTYPE_ALCHEMY 4
+/** God-related information. */
 #define MSGTYPE_GODS 5
+/** Message from the lib/messages file. */
 #define MSGTYPE_MSGFILE 6
 
 /**
@@ -101,17 +99,23 @@
  * are not needed anyplace else, so why have them globally declared?
  */
 
-/** 'title' and 'titlelist' are used by the readable code */
+/**
+ * Information on one title.
+ * 'title' and 'titlelist' are used by the readable code
+ */
 typedef struct titlestruct {
     const char *name;      /**< the name of the book */
     const char *authour;   /**< the name of the book authour */
     const char *archname;  /**< the archetype name of the book */
-    int level;             /**< level of difficulty of this message */
+    unsigned int level;    /**< level of difficulty of this message */
     size_t size;           /**< size of the book message */
     int msg_index;         /**< an index value derived from book message */
-    struct titlestruct *next;
+    struct titlestruct *next;   /**< next item in the list */
 } title;
 
+/**
+ * Titles for one message type.
+ */
 typedef struct titleliststruct {
     int number;       /**< number of items in the list */
     struct titlestruct *first_book;     /**< pointer to first book in this list */
@@ -134,25 +138,23 @@ struct GeneralMessage {
     sstring title;          /**< The message's title, only used for knowledge. */
     sstring message;        /**< The message's body. */
     sstring quest_code;     /**< Optional quest code and state this message will start. */
+    const Face *face;   /**< Face the message displays at in the knowledge dialog, NULL if no face defined. */
     GeneralMessage *next;   /**< Next message in the list. */
 };
 
 static void add_book(title *book, int type, const char *fname, int lineno);
 
 /**
- * booklist is the buffer of books read in from the bookarch file. It's element
+ * Buffer of books read in from the bookarch file. It's element
  * size does not exceed arraysize(max_titles).
  */
 static titlelist *booklist = NULL;
 
-/** Information on monsters */
+/** Information on monsters. */
 static objectlink *first_mon_info = NULL;
 
-/**
- * these are needed for creation of a linked list of
- * pointers to all (hostile) monster objects
- */
-static int nrofmon = 0, need_to_write_bookarchive = 0;
+static int nrofmon = 0, /**< Number of monsters in the ::first_mon_info list. */
+    need_to_write_bookarchive = 0; /**< If set then we have information to save. */
 
 /**
  * First message from data read from the messages file.
@@ -167,9 +169,9 @@ static GeneralMessage *first_msg = NULL;
 static int msg_total_chance = 0;
 
 /**
- * Spellpath information
+ * Spellpath information.
  */
-static const uint32 spellpathdef[NRSPELLPATHS] = {
+static const uint32_t spellpathdef[NRSPELLPATHS] = {
     PATH_PROT,
     PATH_FIRE,
     PATH_FROST,
@@ -196,14 +198,16 @@ static const uint32 spellpathdef[NRSPELLPATHS] = {
 static const char *const path_book_name[] = {
     "codex",
     "compendium",
+    "documentary",
     "exposition",
     "tables",
     "treatise"
 };
 
-/** Used by spellpath texts */
+/** Used by spellpath texts. */
 static const char *const path_author[] = {
     "aether",
+    "arcana",
     "astral byways",
     "connections",
     "the Grey Council",
@@ -255,6 +259,7 @@ static const char *const art_book_name[] = {
     "inventory",
     "list",
     "listing",
+    "omnibus",
     "record",
     "record book"
 };
@@ -267,6 +272,7 @@ static const char *const art_author[] = {
     "items",
     "lost artifacts",
     "the ancients",
+    "treasures",
     "useful things"
 };
 
@@ -284,12 +290,13 @@ static const char *const mon_book_name[] = {
     "list",
     "manual",
     "notes",
+    "omnibus",
     "record",
     "register",
     "volume"
 };
 
-/** Used by monster beastuary texts */
+/** Used by monster beastuary texts. */
 static const char *const mon_author[] = {
     "beasts",
     "creatures",
@@ -307,7 +314,7 @@ static const char *const mon_author[] = {
 };
 
 /**
- * God book information
+ * God book information.
  */
 static const char *const gods_book_name[] = {
     "devotional",
@@ -316,13 +323,14 @@ static const char *const gods_book_name[] = {
     "divine work",
     "holy book",
     "holy record",
+    "illuminated text",
     "moral text",
     "sacred guide",
     "testament",
     "transcript"
 };
 
-/** Used by gods texts */
+/** Used by gods texts. */
 static const char *const gods_author[] = {
     "cults",
     "joy",
@@ -339,7 +347,7 @@ static const char *const gods_author[] = {
 };
 
 /**
- * Alchemy (formula) information
+ * Alchemy (formula) information.
  */
 static const char *const formula_book_name[] = {
     "cookbook",
@@ -349,10 +357,12 @@ static const char *const formula_book_name[] = {
     "recipe book",
     "experiment record",
     "work plan",
-    "design notes"
+    "design notes",
+    "research notes",
+    "crafting manual"
 };
 
-/** This isn't used except for empty books */
+/** This isn't used except for empty books. */
 static const char *const formula_author[] = {
     "Albertus Magnus",
     "alchemy",
@@ -374,17 +384,19 @@ static const char *const formula_author[] = {
  * Generic book information
  */
 
-/** Used by msg file and 'generic' books */
+/** Used by msg file and 'generic' books. */
 static const char *const light_book_name[] = {
     "calendar",
     "datebook",
     "diary",
+    "essay",
     "guidebook",
     "handbook",
     "ledger",
     "notes",
     "notebook",
     "octavo",
+    "opuscule",
     "pamphlet",
     "practicum",
     "script",
@@ -395,6 +407,7 @@ static const char *const light_book_name[] = {
 static const char *const heavy_book_name[] = {
     "catalog",
     "compendium",
+    "dissertation",
     "guide",
     "manual",
     "opus",
@@ -404,7 +417,7 @@ static const char *const heavy_book_name[] = {
     "work"
 };
 
-/** Used by 'generic' books */
+/** Used by 'generic' books. */
 static const char *const book_author[] = {
     "Abdulah",
     "Al'hezred",
@@ -430,6 +443,7 @@ static const char *const book_author[] = {
     "Ichibod",
     "Janus",
     "June",
+    "Laplace",
     "Magnuson",
     "Nandii",
     "Nitfeder",
@@ -456,9 +470,9 @@ static const char *const book_descrpt[] = {
     "cryptic",
     "cryptical",
     "dusty",
-    "hiearchical",
+    "hierarchical",
     "grizzled",
-    "gold-guilt",
+    "gold-gilt",
     "great",
     "lost",
     "magnificent",
@@ -473,7 +487,7 @@ static const char *const book_descrpt[] = {
 };
 
 /**
- * Each line of this array is a readable subtype
+ * Each line of this array is a readable subtype.
  * Be careful to keep the order. If you add readable subtype, add them
  * at the bottom of the list. Never delete a subtype because index is used as
  * subtype parameter in arch files!
@@ -541,7 +555,7 @@ static const readable_message_type readable_message_types[] = {
                     { MSG_TYPE_SIGN, MSG_TYPE_SIGN_MAGIC_MOUTH }
 };
 
-/** Number of elements in ::readable_message_types */
+/** Number of elements in ::readable_message_types. */
 static const int last_readable_subtype = arraysize(readable_message_types);
 
 /** Number of titles for different name lists. */
@@ -652,7 +666,7 @@ int nstrtok(const char *buf1, const char *buf2) {
     if (!buf1 || !buf2)
         return 0;
 
-    snprintf(buf, sizeof(buf), "%s", buf1);
+    strlcpy(buf, buf1, sizeof(buf));
     for (tbuf = strtok(buf, buf2); tbuf; tbuf = strtok(NULL, buf2)) {
         number++;
     }
@@ -680,7 +694,7 @@ char *strtoktolin(const char *buf1, const char *buf2, char *retbuf, size_t size)
     char *tbuf, buf[MAX_BUF];
 
     maxi = i;
-    snprintf(buf, sizeof(buf), "%s", buf1);
+    strlcpy(buf, buf1, sizeof(buf));
     snprintf(retbuf, size, " ");
     for (tbuf = strtok(buf, buf2); tbuf && i > 0; tbuf = strtok(NULL, buf2)) {
         snprintf(retbuf+strlen(retbuf), size-strlen(retbuf), "%s", tbuf);
@@ -748,12 +762,27 @@ static void init_msgfile(void) {
                 continue;
             cp = strchr(buf, '\n');
             if (cp != NULL) {
+                // Remove trailing whitespace
                 while (cp > buf && (cp[-1] == ' ' || cp[-1] == '\t'))
                     cp--;
-                *cp = '\0';
+                /* If we make sure there is a newline here,
+                 * we can avoid the auto-append of it and make long
+                 * blocks of text not get split.
+                 * But only do that if we are getting the message.
+                 * Everywhere else we do not want the newline.
+                 * Daniel Hawkins 2018-10-24
+                 */
+                if (text)
+                {
+                    *cp = '\n';
+                    // to have found a newline means we have room for a null terminator, too
+                    *(++cp)= '\0';
+                }
+                else
+                    *cp = '\0';
             }
             if (tmp != NULL) {
-                if (text && strcmp(buf, "ENDMSG") == 0) {
+                if (text && strncmp(buf, "ENDMSG", 6) == 0) {
                     if (strlen(msgbuf) > BOOK_BUF) {
                         LOG(llevDebug, "Warning: this string exceeded max book buf size:\n");
                         LOG(llevDebug, "  %s\n", msgbuf);
@@ -771,7 +800,8 @@ static void init_msgfile(void) {
                 } else if (text) {
                     if (!buf_overflow(msgbuf, buf, HUGE_BUF-1)) {
                         strcat(msgbuf, buf);
-                        strcat(msgbuf, "\n");
+                        // If there is a newline in the text, it will be included in the output where it belongs
+                        // We should avoid really long lines of text getting split up this way.
                     } else if (error_lineno != 0) {
                         LOG(llevInfo, "Warning: truncating book at %s, line %d\n", fname, error_lineno);
                     }
@@ -784,13 +814,21 @@ static void init_msgfile(void) {
                     tmp->title = add_string(buf + 6);
                 } else if (strncmp(buf, "QUEST ", 6) == 0) {
                     tmp->quest_code = add_string(buf + 6);
+                } else if (strncmp(buf, "FACE ", 5) == 0) {
+                    const Face *face = find_face(buf + 5, NULL);
+                    if (face != NULL) {
+                        tmp->face = face;
+                    } else {
+                        LOG(llevInfo, "Warning: unknown face %s for message %s, line %d\n", buf + 5, tmp->identifier, error_lineno);
+                    }
                 } else if (error_lineno != 0) {
                     LOG(llevInfo, "Warning: unknown line %s, line %d\n", buf, error_lineno);
                 }
             } else if (strncmp(buf, "MSG", 3) == 0) {
                 error_lineno = lineno;
                 tmp = (GeneralMessage *)calloc(1, sizeof(GeneralMessage));
-                strcpy(msgbuf, " ");  /* reset msgbuf for new message */
+                tmp->face = NULL;
+                strcpy(msgbuf, "");  /* reset msgbuf for new message */
                 if (buf[3] == ' ') {
                     int i = 4;
                     while (buf[i] == ' ' && buf[i] != '\0')
@@ -1424,7 +1462,7 @@ object *get_random_mon(int level) {
 static StringBuffer *mon_desc(const object *mon) {
     StringBuffer *desc = stringbuffer_new();
     stringbuffer_append_printf(desc, "\n---\n *** %s ***\n", mon->name);
-    describe_item(mon, NULL, desc);
+    describe_item(mon, NULL, 0, desc);
     return desc;
 }
 
@@ -1472,6 +1510,7 @@ static StringBuffer *mon_info_msg(int level, size_t booksize, object *book) {
     StringBuffer *marker = stringbuffer_new(), *desc = stringbuffer_new(), *mon = NULL;
     int added = 0;
     sstring final;
+    const char *sep = ":";
 
     /*preamble */
     stringbuffer_append_string(desc, "This beastiary contains:");
@@ -1491,13 +1530,16 @@ static StringBuffer *mon_info_msg(int level, size_t booksize, object *book) {
         if (stringbuffer_length(desc) + stringbuffer_length(mon) >= booksize)
             break;
         added++;
-        stringbuffer_append_printf(marker, ":%s", tmp->arch->name);
+        stringbuffer_append_printf(marker, "%s%s", sep, tmp->arch->name);
+        sep = "/";
         stringbuffer_append_stringbuffer(desc, mon);
         stringbuffer_delete(mon);
+        mon = NULL;
     }
 
-    if (mon != NULL)
+    if (mon != NULL) {
         stringbuffer_delete(mon);
+    }
 
     final = stringbuffer_finish_shared(marker);
     if (added)
@@ -1592,7 +1634,7 @@ static StringBuffer *artifact_describe(const artifact *art, const artifactlist *
     add_abilities(tmp, art->item);
     tmp->type = al->type;
     SET_FLAG(tmp, FLAG_IDENTIFIED);
-    sbuf = describe_item(tmp, NULL, NULL);
+    sbuf = describe_item(tmp, NULL, 0, NULL);
     if (stringbuffer_length(sbuf) > 1) {
         stringbuffer_append_string(desc, " Properties of this artifact include:\n ");
         stringbuffer_append_stringbuffer(desc, sbuf);
@@ -1615,7 +1657,7 @@ static StringBuffer *artifact_describe(const artifact *art, const artifactlist *
  * @return
  * new StringBuffer containing the dsecription.
  */
-static StringBuffer *artifact_msg(int level, size_t booksize) {
+static StringBuffer *artifact_msg(unsigned int level, size_t booksize) {
     const artifactlist *al;
     const artifact *art;
     int i, type, index;
@@ -1703,7 +1745,7 @@ static StringBuffer *artifact_msg(int level, size_t booksize) {
 static StringBuffer *spellpath_msg(int level, size_t booksize, StringBuffer *buf) {
     int path = RANDOM()%NRSPELLPATHS, prayers = RANDOM()%2;
     int did_first_sp = 0;
-    uint32 pnum = spellpathdef[path];
+    uint32_t pnum = spellpathdef[path];
     archetype *at;
 
     if (buf == NULL) {
@@ -1821,7 +1863,7 @@ static void make_formula_book(object *book, int level) {
         stringbuffer_append_printf(title, "%s: %s of %s", formula_book_name[RANDOM()%arraysize(formula_book_name)], op_name, formula->title);
     } else {
         stringbuffer_append_printf(text, "The %s", op_name);
-        stringbuffer_append_printf(title, formula_book_name[RANDOM()%arraysize(formula_book_name)], op_name);
+        stringbuffer_append_printf(title, "%s: %s", formula_book_name[RANDOM()%arraysize(formula_book_name)], op_name);
         if (at->clone.title) {
             stringbuffer_append_printf(text, " %s", at->clone.title);
             stringbuffer_append_printf(title, " %s", at->clone.title);
@@ -1888,9 +1930,11 @@ static StringBuffer *msgfile_msg(object *book, size_t booksize) {
     if (msg_total_chance > 0) {
         assert(first_msg != NULL);
         msg = first_msg;
-        weight = (RANDOM() % msg_total_chance) - msg->chance;
-        while (weight > 0 && msg) {
+        weight = (RANDOM() % msg_total_chance);
+        while (msg) {
             weight -= msg->chance;
+            if (weight < 0)
+                break;
             msg = msg->next;
         }
         /* if msg is NULL, then something is really wrong in the computation! */
@@ -1908,9 +1952,12 @@ static StringBuffer *msgfile_msg(object *book, size_t booksize) {
         }
         if (msg->quest_code) {
             /* add a 'apply' hook to launch the quest */
-            object *event = object_create_arch(find_archetype("quest_advance_apply"));
-            FREE_AND_COPY(event->name, msg->quest_code);
-            object_insert_in_ob(event, book);
+            archetype *at = find_archetype("quest_advance_apply");
+            if (at != NULL) {
+                object *event = object_create_arch(at);
+                FREE_AND_COPY(event->name, msg->quest_code);
+                object_insert_in_ob(event, book);
+            }
         }
     } else
         stringbuffer_append_string(ret, "\n <undecipherable text>");
@@ -2031,7 +2078,10 @@ void tailor_readable_ob(object *book, int msg_type) {
      * 4) you may want separate authour/book name arrays in read.h
      */
 
-    msg_type = msg_type > 0 ? msg_type : RANDOM()%6;
+    if (msg_type >= (int)arraysize(max_titles))
+        msg_type = 0;
+
+    msg_type = msg_type > 0 ? msg_type : (int)(RANDOM()%6);
     switch (msg_type) {
     case MSGTYPE_MONSTER:
         message = mon_info_msg(level, book_buf_size, book);
@@ -2078,6 +2128,9 @@ void tailor_readable_ob(object *book, int msg_type) {
  *
  *****************************************************************************/
 
+/**
+ * Free all readable-related information.
+ */
 void free_all_readable(void) {
     titlelist *tlist, *tnext;
     title *title1, *titlenext;
@@ -2129,6 +2182,7 @@ void free_all_readable(void) {
  */
 void write_book_archive(void) {
     FILE *fp;
+    OutputFile of;
     int index;
     char fname[MAX_BUF];
     title *book;
@@ -2141,11 +2195,9 @@ void write_book_archive(void) {
     snprintf(fname, sizeof(fname), "%s/bookarch", settings.localdir);
     LOG(llevDebug, "Updating book archive: %s...\n", fname);
 
-    fp = fopen(fname, "w");
-    if (fp == NULL) {
-        LOG(llevDebug, "Can't open book archive file %s\n", fname);
+    fp = of_open(&of, fname);
+    if (fp == NULL)
         return;
-    }
 
     for (bl = get_titlelist(0), index = 0; bl; bl = bl->next, index++) {
         for (book = bl->first_book; book; book = book->next)
@@ -2161,16 +2213,13 @@ void write_book_archive(void) {
                 fprintf(fp, "end\n");
             }
     }
-    if (ferror(fp)) {
-        LOG(llevError, "Error during book archive save.\n");
-        fclose(fp);
+    if (!of_close(&of))
         return;
+
+    if (chmod(fname, SAVE_MODE) != 0) {
+        LOG(llevError, "Could not set permissions on '%s'\n", fname);
     }
-    if (fclose(fp) != 0) {
-        LOG(llevError, "Error during book archive save.\n");
-        return;
-    }
-    chmod(fname, SAVE_MODE);
+
     need_to_write_bookarchive = 0;
 }
 
@@ -2182,7 +2231,7 @@ void write_book_archive(void) {
  * type of the book. Will never be NULL.
  */
 const readable_message_type *get_readable_message_type(object *readable) {
-    uint8 subtype = readable->subtype;
+    uint8_t subtype = readable->subtype;
 
     if (subtype > last_readable_subtype)
         return &readable_message_types[0];
@@ -2219,3 +2268,12 @@ sstring get_message_title(const GeneralMessage *message) {
 sstring get_message_body(const GeneralMessage *message) {
     return message->message;
 }
+
+/**
+ * Get a message's face.
+ * @param message message, must not be NULL.
+ * @return face, -1 if not defined.
+ */
+const Face *get_message_face(const GeneralMessage *message) {
+    return message->face;
+}
