@@ -1,24 +1,15 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2001-2006 Crossfire Development Team
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
 /**
  * @file recipe.c
@@ -41,10 +32,14 @@
  * An ingredient is a name, which can contain an initial number for how many are needed.
  */
 
-#include <stdlib.h>
-#include <global.h>
-#include <object.h>
+#include "global.h"
+
+#include <assert.h>
 #include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "object.h"
 
 static void build_stringlist(const char *str, char ***result_list, size_t *result_size);
 static void check_formulae(void);
@@ -80,30 +75,12 @@ static recipelist *init_recipelist(void) {
  * new structure initialized. Never NULL.
  */
 static recipe *get_empty_formula(void) {
-    recipe *t = (recipe *)malloc(sizeof(recipe));
+    // This used to be a malloc followed by setting everything to zero.
+    // So just use calloc to make it faster.
+    // SilverNexus -- 2018-10-22
+    recipe *t = (recipe *)calloc(1, sizeof(recipe));
     if (t == NULL)
         fatal(OUT_OF_MEMORY);
-    t->chance = 0;
-    t->index = 0;
-    t->transmute = 0;
-    t->yield = 0;
-    t->diff = 0;
-    t->exp = 0;
-    t->keycode = NULL;
-    t->title = NULL;
-    t->arch_names = 0;
-    t->arch_name = NULL;
-    t->skill = NULL;
-    t->cauldron = NULL;
-    t->ingred = NULL;
-    t->ingred_count = 0;
-    t->next = NULL;
-    t->failure_arch = NULL;
-    t->failure_message = NULL;
-    t->min_level = 0;
-    t->is_combination = 0;
-    t->tool = NULL;
-    t->tool_size = 0;
     return t;
 }
 
@@ -224,6 +201,11 @@ void init_formulae(void) {
                     formula->ingred_count++;
                 }
                 tmp = (linked_char *)malloc(sizeof(linked_char));
+                /* trim the string */
+                while (*cp == ' ')
+                  cp++;
+                while (*cp != '\0' && cp[strlen(cp) - 1] == ' ')
+                  cp[strlen(cp) - 1] = '\0';
                 tmp->name = add_string(cp);
                 tmp->next = formula->ingred;
                 formula->ingred = tmp;
@@ -289,17 +271,39 @@ void init_formulae(void) {
 static void check_formulae(void) {
     recipelist *fl;
     recipe *check, *formula;
-    int numb = 1;
+    int numb = 1, tool_match;
+    size_t tool_i,tool_j;
 
     LOG(llevDebug, "Checking formulae lists...\n");
 
     for (fl = formulalist; fl != NULL; fl = fl->next) {
         for (formula = fl->items; formula != NULL; formula = formula->next)
             for (check = formula->next; check != NULL; check = check->next)
-                if (check->index == formula->index && strcmp(check->cauldron, formula->cauldron) == 0) {
-                    /* if the recipes don't have the same facility, then no issue anyway. */
-                    LOG(llevError, " ERROR: On %d ingred list:\n", numb);
-                    LOG(llevError, "Formulae [%s] of %s and [%s] of %s have matching index id (%d)\n", formula->arch_name[0], formula->title, check->arch_name[0], check->title, formula->index);
+                /* If one recipe has a tool and another a caudron, we should be able to handle it */
+                if (check->index == formula->index &&
+                        ((check->cauldron && formula->cauldron && strcmp(check->cauldron, formula->cauldron) == 0) ||
+                        (check->tool_size == formula->tool_size && check->tool_size > 0))) {
+                    /* Check the tool list to make sure they have no matches */
+                    if (check->tool && formula->tool)
+                    {
+                        tool_match = 0;
+                        for (tool_i = 0; tool_i < formula->tool_size; ++tool_i)
+                            /* If it turns out these lists are sorted, then we could optimize this better. */
+                            for (tool_j = 0; tool_j < check->tool_size; ++tool_j)
+                                if (strcmp(formula->tool[tool_i], check->tool[tool_j]) == 0) {
+                                    tool_match = 1;
+                                    break; /* TODO: break out of the double loop */
+                                }
+                    }
+                    else
+                        tool_match = 1; /* If we get here, we matched on the cauldron */
+                    /* Check to see if we have a denoted match */
+                    if (tool_match) {
+                        /* if the recipes don't have the same facility, then no issue anyway. */
+                        LOG(llevError, " ERROR: On %d ingred list:\n", numb);
+                        LOG(llevError, "Formulae [%s] of %s and [%s] of %s have matching index id (%d)\n",
+                            formula->arch_name[0], formula->title, check->arch_name[0], check->title, formula->index);
+                    }
                 }
         numb++;
     }
@@ -339,7 +343,7 @@ void dump_alchemy(void) {
                         if (strcmp(formula->title, "NONE"))
                             snprintf(buf, sizeof(buf), "%s of %s", string, formula->title);
                         else
-                            snprintf(buf, sizeof(buf), "%s", string);
+                            strlcpy(buf, string, sizeof(buf));
                         fprintf(logfile, "%-30s(%d) bookchance %3d  ", buf, formula->index, formula->chance);
                         fprintf(logfile, "skill %s", formula->skill);
                         fprintf(logfile, "\n");
@@ -471,9 +475,9 @@ static long find_ingred_cost(const char *name) {
     /* second, try to match an artifact ("arch of something") */
     cp = strstr(name, " of ");
     if (cp != NULL) {
-        strcpy(part1, name);
+        safe_strncpy(part1, name, sizeof(part1));
         part1[cp-name] = '\0';
-        strcpy(part2, cp+4);
+        safe_strncpy(part2, cp + 4, sizeof(part2));
         /* find the first archetype matching the first part of the name */
         for (at = first_archetype; at != NULL; at = at->next)
             if (!strcasecmp(at->clone.name, part1) && at->clone.title == NULL)
@@ -491,9 +495,9 @@ static long find_ingred_cost(const char *name) {
     /* third, try to match a body part ("arch's something") */
     cp = strstr(name, "'s ");
     if (cp != NULL) {
-        strcpy(part1, name);
+        safe_strncpy(part1, name, sizeof(part1));
         part1[cp-name] = '\0';
-        strcpy(part2, cp+3);
+        safe_strncpy(part2, cp + 3, sizeof(part2));
         /* examine all archetypes matching the first part of the name */
         for (at = first_archetype; at != NULL; at = at->next)
             if (!strcasecmp(at->clone.name, part1) && at->clone.title == NULL) {
@@ -543,7 +547,7 @@ void dump_alchemy_costs(void) {
                         LOG(llevError, "Formula %s has no artifact\n", formula->title);
                     else {
                         if (!strcmp(formula->title, "NONE"))
-                            snprintf(buf, sizeof(buf), "%s", string);
+                            strlcpy(buf, string, sizeof(buf));
                         else
                             snprintf(buf, sizeof(buf), "%s of %s", string, formula->title);
                         fprintf(logfile, "\n%-40s bookchance %3d  skill %s\n", buf, formula->chance, formula->skill);
@@ -806,7 +810,8 @@ static void build_stringlist(const char *str, char ***result_list, size_t *resul
     for (p = strtok(dup, ","); p != NULL; p = strtok(NULL, ","))
         size++;
 
-    *result_list = malloc(size*sizeof(*result_list));
+    assert(size > 0);
+    *result_list = malloc(sizeof(**result_list) * size);
     if (*result_list == NULL)
         fatal(OUT_OF_MEMORY);
     *result_size = size;
