@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_map_c =
- *   "$Id: region.c 14865 2011-07-17 08:40:09Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2001-2003 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file region.c
  * Region management.
@@ -33,7 +18,11 @@
  * A region is a group of maps. It includes a "parent" region.
  */
 
-#include <global.h>
+#include "global.h"
+
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
 
 #ifndef WIN32 /* ---win32 exclude header */
 #include <unistd.h>
@@ -296,25 +285,27 @@ object *get_jail_exit(object *op) {
 
 /**
  * Initialises regions from the regions file.
+ *
+ * @return
+ * @li Returns '0' on success, anything else if failed.
  */
-void init_regions(void) {
+int init_regions(void) {
     FILE *fp;
     char filename[MAX_BUF];
 
     if (first_region != NULL) /* Only do this once */
-        return;
+        return 0;
 
     snprintf(filename, sizeof(filename), "%s/%s/%s", settings.datadir, settings.mapdir, settings.regions);
-    LOG(llevDebug, "Reading regions from %s...\n", filename);
     if ((fp = fopen(filename, "r")) == NULL) {
-        LOG(llevError, " Can't open regions file %s in init_regions.\n", filename);
-        return;
+        LOG(llevError, "Couldn't read regions file from \"%s\".\n", filename);
+        return 1;
     }
     parse_regions(fp);
     assign_region_parents();
-    LOG(llevDebug, " done\n");
 
     fclose(fp);
+    return 0;
 }
 
 /**
@@ -332,14 +323,10 @@ void init_regions(void) {
  * free those pointers someday? :)
  */
 region *get_region_struct(void) {
-    region *new;
-
-    new = (region *)CALLOC(1, sizeof(region));
+    region *new = (region *)calloc(1, sizeof(region));
     if (new == NULL)
         fatal(OUT_OF_MEMORY);
 
-    memset(new, '\0', sizeof(region));
-
     return new;
 }
 
@@ -452,11 +439,14 @@ static void parse_regions(FILE *fp) {
                 fatal(SEE_LAST_ERROR);
             }
             while (fgets(buf, HUGE_BUF-1, fp) != NULL) {
-                if (!strcmp(buf, "endmsg\n"))
+                key = buf;
+                while (isspace(*key))
+                    key++;
+                if (key && strcmp(key, "endmsg\n") == 0)
                     break;
                 else {
-                    strcpy(msgbuf+msgpos, buf);
-                    msgpos += strlen(buf);
+                    strcpy(msgbuf+msgpos, key);
+                    msgpos += strlen(key);
                 }
             }
             /*
@@ -497,6 +487,7 @@ static void parse_regions(FILE *fp) {
         } else if (!strcmp(key, "nomore")) {
             if (new) {
                 LOG(llevError, "region.c: Last region not properly closed.\n");
+                free(new);
             }
             /* we have reached the end of the region specs....*/
             break;
@@ -505,8 +496,10 @@ static void parse_regions(FILE *fp) {
             LOG(llevError, "Got unknown value in region file: %s %s\n", key, value);
         }
     }
-    if (!key || strcmp(key, "nomore"))
+    if (!key || strcmp(key, "nomore")) {
         LOG(llevError, "Got premature eof on regions file!\n");
+        free(new);
+    }
 }
 
 /**
@@ -514,8 +507,8 @@ static void parse_regions(FILE *fp) {
  */
 static void assign_region_parents(void) {
     region *reg;
-    uint32 parent_count = 0;
-    uint32 region_count = 0;
+    uint32_t parent_count = 0;
+    uint32_t region_count = 0;
 
     for (reg = first_region; reg != NULL; reg = reg->next) {
         if (reg->parent_name != NULL) {
@@ -527,5 +520,6 @@ static void assign_region_parents(void) {
         }
         region_count++;
     }
-    LOG(llevDebug, "Assigned %u regions with %u parents.\n", region_count, parent_count);
+    LOG(llevDebug, "regions: loaded %u with %u parents\n",
+            region_count, parent_count);
 }
