@@ -1,38 +1,27 @@
 /*
- * static char *rcsid_time_c =
- *    "$Id: time.c 13308 2010-05-28 06:12:12Z kbulgrien $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
- * Ingame time functions.
+ * In-game time functions.
  */
 
-#include <global.h>
-#include <tod.h>
+#include "global.h"
+
+#include <assert.h>
+#include <math.h>
+
+#include "tod.h"
 
 #ifndef WIN32 /* ---win32 exclude header */
 #include <stdio.h>
@@ -43,18 +32,18 @@
 /**
  * Gloabal variables:
  */
-uint32 max_time = MAX_TIME;
-struct timeval last_time;
+uint32_t max_time = MAX_TIME;
+static struct timespec game_time;
 
 /** Size of history buffer. */
 #define PBUFLEN 100
-static uint32 process_utime_save[PBUFLEN];   /**< Historic data. */
-static uint32 psaveind;                      /**< Index in ::process_utime_save. */
-static uint32 process_max_utime = 0;         /**< Longest cycle time. */
-static uint32 process_min_utime = 999999999; /**< Shortest cycle time. */
-static uint32 process_tot_mtime;             /**< ? */
-uint32 pticks;                               /**< ? */
-static uint32 process_utime_long_count;      /**< ? */
+static uint32_t process_utime_save[PBUFLEN];   /**< Historic data. */
+static uint32_t psaveind;                      /**< Index in ::process_utime_save. */
+static uint32_t process_max_utime = 0;         /**< Longest cycle time. */
+static uint32_t process_min_utime = 999999999; /**< Shortest cycle time. */
+static uint32_t process_tot_mtime;             /**< ? */
+uint32_t pticks;                               /**< ? */
+static uint32_t process_utime_long_count;      /**< Number of times server couldn't keep up with game time (tried to sleep for a negative time) */
 
 /** Ingame seasons. */
 static const char *const season_name[SEASONS_PER_YEAR+1] = {
@@ -149,13 +138,13 @@ void reset_sleep(void) {
     process_tot_mtime = 0;
     pticks = 0;
 
-    (void)GETTIMEOFDAY(&last_time);
+    clock_gettime(CLOCK_MONOTONIC, &game_time);
 }
 
 /**
  * Adds time to our history list.
  */
-static void log_time(uint32 process_utime) {
+static void log_time(uint32_t process_utime) {
     pticks++;
     if (++psaveind >= PBUFLEN)
         psaveind = 0;
@@ -168,82 +157,41 @@ static void log_time(uint32 process_utime) {
 }
 
 /**
- * Checks elapsed time since last tick.
- *
- * @return true if the time passed since last tick is more than max-time.
+ * Return the difference between two timespec's in microseconds.
  */
-int enough_elapsed_time(void) {
-    static struct timeval new_time;
-    uint32 elapsed_utime;
-
-    (void)GETTIMEOFDAY(&new_time);
-
-    elapsed_utime = (new_time.tv_sec-last_time.tv_sec)*1000000+new_time.tv_usec-last_time.tv_usec;
-    if (elapsed_utime > max_time) {
-        log_time(elapsed_utime);
-        last_time.tv_sec = new_time.tv_sec;
-        last_time.tv_usec = new_time.tv_usec;
-        return 1;
-    }
-    return 0;
+long timespec_diff(struct timespec *end, struct timespec *start) {
+    long long sec = (long long)end->tv_sec - (long long)start->tv_sec;
+    long nsec = end->tv_nsec - start->tv_nsec;
+    return sec * 1e6 + nsec / 1e3;
 }
 
 /**
- * sleep_delta checks how much time has elapsed since last tick.
- * If it is less than max_time, the remaining time is slept with select().
+ * Add 'usec' microseconds to the given timespec.
  */
-void sleep_delta(void) {
-    static struct timeval new_time;
-    long sleep_sec, sleep_usec;
+static void timespec_add(struct timespec *time, long usec) {
+    long nsec_sum = time->tv_nsec + usec * 1e3;
+    time->tv_sec += nsec_sum / 1e9;
+    time->tv_nsec = nsec_sum % (long)1e9;
+}
 
-    (void)GETTIMEOFDAY(&new_time);
+/*
+ * Add one tick length to the last tick time.
+ */
+void tick_game_time() {
+    timespec_add(&game_time, max_time);
+}
 
-    sleep_sec = last_time.tv_sec-new_time.tv_sec;
-    sleep_usec = max_time-(new_time.tv_usec-last_time.tv_usec);
+long get_sleep_remaining() {
+    struct timespec now;
+    clock_gettime(CLOCK_MONOTONIC, &now);
+    long time_since_last_sleep = timespec_diff(&now, &game_time);
+    log_time(time_since_last_sleep);
+    return max_time - time_since_last_sleep;
+}
 
-    /* This is very ugly, but probably the fastest for our use: */
-    while (sleep_usec < 0) {
-        sleep_usec += 1000000;
-        sleep_sec -= 1;
-    }
-    while (sleep_usec > 1000000) {
-        sleep_usec -= 1000000;
-        sleep_sec += 1;
-    }
-
-    log_time((new_time.tv_sec-last_time.tv_sec)*1000000+new_time.tv_usec-last_time.tv_usec);
-
-    if (sleep_sec >= 0 && sleep_usec > 0) {
-        static struct timeval sleep_time;
-
-        sleep_time.tv_sec = sleep_sec;
-        sleep_time.tv_usec = sleep_usec;
-
-#ifndef WIN32 /* 'select' doesn't work on Windows, 'Sleep' is used instead */
-        select(0, NULL, NULL, NULL, &sleep_time);
-#else
-        if (sleep_time.tv_sec)
-            Sleep(sleep_time.tv_sec*1000);
-        Sleep((int)(sleep_time.tv_usec/1000.));
-#endif
-    } else
-        process_utime_long_count++;
-    /*
-     * Set last_time to when we're expected to wake up:
-     */
-    last_time.tv_usec += max_time;
-    while (last_time.tv_usec > 1000000) {
-        last_time.tv_usec -= 1000000;
-        last_time.tv_sec++;
-    }
-    /*
-     * Don't do too much catching up:
-     * (Things can still get jerky on a slow/loaded computer)
-     */
-    if ((last_time.tv_sec-new_time.tv_sec)*1000000+(last_time.tv_usec-new_time.tv_usec) < 0) {
-        last_time.tv_sec = new_time.tv_sec;
-        last_time.tv_usec = new_time.tv_usec;
-    }
+void jump_time() {
+    process_utime_long_count++;
+    clock_gettime(CLOCK_MONOTONIC, &game_time);
 }
 
 /**
@@ -311,15 +259,12 @@ static void print_tod(object *op) {
     timeofday_t tod;
     const char *suf;
     int day;
+    char buf1[128];
 
     get_tod(&tod);
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                         "It is %d minute%s past %d o'clock %s, on %s",
-                         tod.minute+1, ((tod.minute+1 < 2) ? "" : "s"),
-                         ((tod.hour%14 == 0) ? 14 : ((tod.hour)%14)),
-                         ((tod.hour >= 14) ? "pm" : "am"),
-                         weekdays[tod.dayofweek]);
+        "It is %s, on %s", time_format_time(&tod, buf1, sizeof(buf1)), weekdays[tod.dayofweek]);
 
     day = tod.day+1;
     if (day == 1 || ((day%10) == 1 && day > 20))
@@ -348,30 +293,23 @@ static void print_tod(object *op) {
  */
 void time_info(object *op) {
     int tot = 0, long_count = 0;
-    uint32 maxt = 0, mint = 99999999, i;
+    uint32_t maxt = 0, mint = 99999999, i;
 
     print_tod(op);
 
     if (!QUERY_FLAG(op, FLAG_WIZ))
         return;
 
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
-                  "Total time:");
-
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
-                         "ticks=%d  time=%d.%2d",
-                         pticks, process_tot_mtime/1000, process_tot_mtime%1000);
-
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
-                         "avg time=%dms  max time=%dms  min time=%dms",
-                         process_tot_mtime/pticks, process_max_utime/1000,
-                         process_min_utime/1000);
-
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
-                         "ticks longer than max time (%dms) = %d (%d%%)",
-                         max_time/1000,
-                         process_utime_long_count, 100*process_utime_long_count/pticks);
+    draw_ext_info_format(
+        NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
+        "Statistics for last %d ticks:\n\tmin/avg/max = %d/%d/%d ms per tick",
+        pticks, process_min_utime / 1000, process_tot_mtime / pticks,
+        process_max_utime / 1000);
 
+    draw_ext_info_format(
+        NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
+        "\tticks longer than %d ms = %d (%d%%)", max_time / 1000,
+        process_utime_long_count, 100 * process_utime_long_count / pticks);
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
                          "Time last %d ticks:",
@@ -387,6 +325,8 @@ void time_info(object *op) {
             long_count++;
     }
 
+    assert(pticks > 0);
+
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
                          "avg time=%dms  max time=%dms  min time=%dms",
                          tot/MIN(pticks, PBUFLEN)/1000, maxt/1000,
@@ -399,14 +339,42 @@ void time_info(object *op) {
 }
 
 /**
- * Gets the seconds.
- *
- * @return
- * seconds.
+ * Return wall clock time in seconds.
  */
 long seconds(void) {
-    struct timeval now;
-
-    (void)GETTIMEOFDAY(&now);
+    struct timespec now;
+    clock_gettime(CLOCK_REALTIME, &now);
     return now.tv_sec;
 }
+
+/**
+ * Formats a timestamp in Crossfire time.
+ *
+ * @param tod
+ * the timestamp to format
+ *
+ * @param buf
+ * the buffer to fill
+ *
+ * @param bufsize
+ * the size of buf in bytes
+ *
+ * @return
+ * buf
+ */
+const char *time_format_time(const timeofday_t *tod, char *buf, size_t bufsize)
+{
+    snprintf(buf, bufsize, "%d minute%s past %d o'clock %s",
+        tod->minute+1,
+        tod->minute+1 < 2 ? "" : "s",
+        tod->hour%14 == 0 ? 14 : tod->hour%14,
+        tod->hour >= 14 ? "pm" : "am");
+    return buf;
+}
+
+/**
+ * Calculate the number of ticks that correspond to real time.
+ */
+unsigned int tick_length(float seconds) {
+    return (int)ceil(seconds * 1000000 / max_time);
+}
