@@ -1,40 +1,22 @@
-
 /*
- * static char *rcs_treasure_c =
- *   "$Id: treasure.c 15527 2011-11-06 17:43:01Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Everything concerning treasures.
  * @see page_treasure_list
  */
 
-#define ALLOWED_COMBINATION
-
 /**
  * TREASURE_DEBUG does some checking on the treasurelists after loading.
  * It is useful for finding bugs in the treasures file.  Since it only
@@ -43,15 +25,15 @@
  */
 #define TREASURE_DEBUG
 
-/* TREASURE_VERBOSE enables copious output concerning artifact generation */
-/* #define TREASURE_VERBOSE */
+#include "global.h"
 
+#include <ctype.h>
 #include <stdlib.h>
-#include <global.h>
-#include <treasure.h>
-#include <loader.h>
-#include <sproto.h>
+#include <string.h>
 
+#include "loader.h"
+#include "sproto.h"
+#include "treasure.h"
 
 /**
  * Resistances which can show up on rings and amulets.
@@ -80,18 +62,12 @@ extern const char *const spell_mapping[];
  * Initialize global archtype pointers:
  */
 void init_archetype_pointers(void) {
-    int prev_warn = warn_archetypes;
-
-    warn_archetypes = 1;
     if (ring_arch == NULL)
         ring_arch = find_archetype("ring");
     if (amulet_arch == NULL)
         amulet_arch = find_archetype("amulet");
-    if (staff_arch == NULL)
-        staff_arch = find_archetype("staff");
     if (crown_arch == NULL)
         crown_arch = find_archetype("crown");
-    warn_archetypes = prev_warn;
 }
 
 /**
@@ -169,8 +145,10 @@ static treasure *load_treasure(FILE *fp, int *line) {
             cp++;
 
         if (sscanf(cp, "arch %s", variable)) {
-            if ((t->item = find_archetype(variable)) == NULL)
+            if ((t->item = find_archetype(variable)) == NULL) {
                 LOG(llevError, "Treasure lacks archetype: %s\n", variable);
+                fatal(SEE_LAST_ERROR);
+            }
         } else if (sscanf(cp, "list %s", variable))
             t->name = add_string(variable);
         else if (sscanf(cp, "change_name %s", variable))
@@ -180,11 +158,11 @@ static treasure *load_treasure(FILE *fp, int *line) {
         else if (sscanf(cp, "change_slaying %s", variable))
             t->change_arch.slaying = add_string(variable);
         else if (sscanf(cp, "chance %d", &value))
-            t->chance = (uint8)value;
+            t->chance = (uint8_t)value;
         else if (sscanf(cp, "nrof %d", &value))
-            t->nrof = (uint16)value;
+            t->nrof = (uint16_t)value;
         else if (sscanf(cp, "magic %d", &value))
-            t->magic = (uint8)value;
+            t->magic = (uint8_t)value;
         else if (!strcmp(cp, "yes"))
             t->next_yes = load_treasure(fp, line);
         else if (!strcmp(cp, "no"))
@@ -198,6 +176,7 @@ static treasure *load_treasure(FILE *fp, int *line) {
             LOG(llevError, "Unknown treasure-command: '%s', last entry %s, line %d\n", cp, t->name ? t->name : "null", *line);
     }
     LOG(llevError, "treasure lacks 'end'.\n");
+    fatal(SEE_LAST_ERROR);
     return t;
 }
 
@@ -401,8 +380,7 @@ static void change_treasure(treasure *t, object *op) {
 static void create_all_treasures(treasure *t, object *op, int flag, int difficulty, int tries) {
     object *tmp;
 
-
-    if ((int)t->chance >= 100 || (RANDOM()%100+1) < (int)t->chance) {
+    if ((int)t->chance >= 100 || (RANDOM()%100+1) < t->chance) {
         if (t->name) {
             if (strcmp(t->name, "NONE") && difficulty >= t->magic)
                 create_treasure(find_treasurelist(t->name), op, flag, difficulty, tries);
@@ -519,11 +497,14 @@ void create_treasure(treasurelist *t, object *op, int flag, int difficulty, int
 }
 
 /**
+ * Generate a treasure from a list generating a single item.
  * This is similar to the old generate treasure function.  However,
  * it instead takes a treasurelist.  It is really just a wrapper around
  * create_treasure().  We create a dummy object that the treasure gets
  * inserted into, and then return that treasure.
  *
+ * @warning an error will be LOG()ed if more than one item is generated.
+ *
  * @param t
  * treasure list to generate from.
  * @param difficulty
@@ -1263,8 +1244,9 @@ void fix_generated_item(object *op, object *creator, int difficulty, int max_mag
             }
 
         case AMULET:
-                if (op->arch == amulet_arch)
-                    op->value *= 5; /* Since it's not just decoration */
+            if (op->arch == amulet_arch)
+                op->value *= 5; /* Since it's not just decoration */
+            /* fall through */
         case RING:
             if (op->arch == NULL) {
                 object_remove(op);
@@ -1548,6 +1530,16 @@ static void fix_flesh_item(object *item, const object *donor) {
         if (donor->attacktype&AT_ACID)
             item->stats.hp = -1*item->stats.food;
         SET_FLAG(item, FLAG_NO_STEAL);
+
+        /* attempt to change the face - will take a face named "donor's arch"_"item's face". We ignore the animation for now */
+        if (item->face != NULL) {
+            const Face *face;
+            snprintf(tmpbuf, sizeof(tmpbuf), "%s_%s", donor->arch->name, item->face->name);
+            face = find_face(tmpbuf, NULL);
+            if (face != NULL) {
+              item->face = face;
+            }
+        }
     }
 }
 
