@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_utils_c =
- *   "$Id: utils.c 14734 2011-06-29 17:18:42Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file utils.c
  * General convenience functions for crossfire.
@@ -38,8 +23,14 @@
  * numbers larger (ie, 1d1000 > 500 vs 1d6 > 3)
  */
 
+#include "global.h"
+
+#include <ctype.h>
+#include <math.h>
 #include <stdlib.h>
-#include <global.h>
+#include <string.h>
+
+#include "sproto.h"
 
 /**
  * Roll a random number between min and max.  Uses op to determine luck,
@@ -66,7 +57,7 @@ int random_roll(int min, int max, const object *op, int goodbad) {
         return((ran%diff)+min);
 
     luck = op->stats.luck;
-    if (RANDOM()%base < MIN(10, abs(luck))) {
+    if (RANDOM()%base < (unsigned int)MIN(10, abs(luck))) {
         /* we have a winner */
         ((luck > 0) ? (luck = 1) : (luck = -1));
         diff -= luck;
@@ -83,8 +74,9 @@ int random_roll(int min, int max, const object *op, int goodbad) {
  * This is a 64 bit version of random_roll() above.  This is needed
  * for exp loss calculations for players changing religions.
  */
-sint64 random_roll64(sint64 min, sint64 max, const object *op, int goodbad) {
-    sint64 omin, diff, luck, ran;
+int64_t random_roll64(int64_t min, int64_t max, const object *op, int goodbad) {
+    int64_t omin, diff, ran;
+    int8_t luck;
     int base;
 
     omin = min;
@@ -101,13 +93,13 @@ sint64 random_roll64(sint64 min, sint64 max, const object *op, int goodbad) {
      * a little byteshifting.  Do make sure the first one is only
      * 32 bit, so we don't get skewed results
      */
-    ran = (RANDOM()&0xffffffff)|((sint64)RANDOM()<<32);
+    ran = (RANDOM()&0xffffffff)|((int64_t)RANDOM()<<32);
 
     if (op->type != PLAYER)
         return((ran%diff)+min);
 
     luck = op->stats.luck;
-    if (RANDOM()%base < MIN(10, abs(luck))) {
+    if (RANDOM()%base < (unsigned int)MIN(10, abs(luck))) {
         /* we have a winner */
         ((luck > 0) ? (luck = 1) : (luck = -1));
         diff -= luck;
@@ -143,7 +135,7 @@ int die_roll(int num, int size, const object *op, int goodbad) {
         luck = op->stats.luck;
 
     for (i = 0; i < num; i++) {
-        if (RANDOM()%base < MIN(10, abs(luck)) && !gotlucky) {
+        if (RANDOM()%base < (unsigned int)MIN(10, abs(luck)) && !gotlucky) {
             /* we have a winner */
             gotlucky++;
             ((luck > 0) ? (luck = 1) : (luck = -1));
@@ -312,7 +304,7 @@ void set_materialname(object *op) {
     if (op->materialname != NULL)
         return;
 
-    for (mt = materialt; mt != NULL && mt->next != NULL; mt = mt->next) {
+    for (mt = materialt; mt != NULL; mt = mt->next) {
         if (op->material&mt->material) {
             break;
         }
@@ -365,7 +357,7 @@ void replace(const char *src, const char *key, const char *replacement, char *re
 
     /* special case to prevent infinite loop if key==replacement=="" */
     if (strcmp(key, replacement) == 0) {
-        snprintf(result, resultsize, "%s", src);
+        strlcpy(result, src, resultsize);
         return;
     }
 
@@ -409,8 +401,9 @@ void make_list_like(char *input) {
 
     strncpy(tmp, input, MAX_BUF-5);
     /*trim all trailing commas, spaces etc.*/
-    for (i = strlen(tmp); !isalnum(tmp[i]) && i >= 0; i--)
+    for (i = strlen(tmp); i >= 0 && !isalnum(tmp[i]); i--) {
         tmp[i] = '\0';
+    }
     strcat(tmp, ".");
 
     p = strrchr(tmp, ',');
@@ -586,3 +579,33 @@ StringBuffer *describe_attacktype(const char *attack, int value, StringBuffer *b
 
     return buf;
 }
+
+/**
+ * Compute the square root. This is now a wrapper around sqrt().
+ */
+int isqrt(int n) {
+    return (int)sqrt(n);
+}
+
+/**
+ * fatal() is meant to be called whenever a fatal signal is intercepted.
+ * It will call the emergency_save and the clean_tmp_files functions.
+ *
+ * @note
+ * this function never returns, as it calls exit().
+ */
+void fatal(enum fatal_error err) {
+    const char *fatalmsgs[] = {
+        "Failed to allocate memory",
+        "Failed repeatedly to load maps",
+        "Hashtable for archetypes is too small",
+        "Fatal issue in archetype file",
+        "See last error",
+    };
+
+    fprintf(logfile, "Fatal error: %s\n", fatalmsgs[err]);
+    emergency_save(0);
+    clean_tmp_files();
+    fprintf(logfile, "Exiting...\n");
+    exit(err);
+}
