@@ -1,31 +1,3 @@
-/*
- * static char *rcsid_attack_h =
- *   "$Id: attack.h 14856 2011-07-14 15:27:42Z ryo_saeba $";
- */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Attack-related definitions.
@@ -162,6 +134,7 @@ EXTERN attackmess_t attack_mess[NROFATTACKMESS][MAXATTACKMESS];
 EXTERN Chaos_Attacks ATTACKS[22];
 EXTERN const char *const change_resist_msg[NROFATTACKS];
 EXTERN const char *const resist_plus[NROFATTACKS];
+EXTERN const char *const resist_color[NROFATTACKS];
 EXTERN const char *const attacktype_desc[NROFATTACKS];
 EXTERN const char *const resist_save[NROFATTACKS];
 
@@ -199,6 +172,12 @@ EXTERN const char *const resist_plus[NROFATTACKS] = {
     "resist diseases"
 };
 
+/** Colors to add to the resistances for media tags. */
+EXTERN const char *const resist_color[NROFATTACKS] = {
+  "#FF15CD", "#930C76", "red", "blue", "#2CFFFF", NULL, NULL, NULL, NULL, NULL, "green", NULL, NULL,
+  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+};
+
 /**
  * These are the descriptions of the resistances displayed when a
  * player puts on/takes off an item. See change_abil() in living.c.
