@@ -1,31 +1,3 @@
-/*
- * static char *rcsid_config_h =
- *   "$Id: config.h 14733 2011-06-29 17:13:23Z ryo_saeba $";
- */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 #ifndef CONFIG_H
 #define CONFIG_H
 
@@ -185,6 +157,7 @@
 #define SPELL_FAILURE_EFFECTS FALSE
 #define REAL_WIZ TRUE
 #define RECYCLE_TMP_MAPS FALSE
+#define ALWAYS_SHOW_HP FALSE
 #define RESURRECTION FALSE
 #define SEARCH_ITEMS TRUE
 #define NOT_PERMADETH TRUE
@@ -303,7 +276,7 @@
  *
  * A few thoughts on these default value (by MSW)
  * gain ratio is pretty much meaningless until exp has been lost, as until
- * that poin, the value in the settings file will be used.
+ * that point, the value in the settings file will be used.
  * It is also impossible for the exp to actually be reduced to the permanent
  * exp ratio - since the loss ratio is .5, it will just get closer and
  * closer.  However, after about half a dozen hits, pretty much all the
@@ -331,27 +304,10 @@
  * SECTION 2 - Machine/Compiler specific stuff.
  *
  * Short list of items:
- * COMPRESS_SUFFIX - selection of compression programs
  * O_NDELAY - If you don't have O_NDELAY, uncomment it.
  *
  ***********************************************************************/
 
-/**
- * If you compress your files to save space, set the COMPRESS_SUFFIX below
- * to the compression suffix you want (.Z, .gz, .bz2).  The autoconf
- * should already find the program to use.  If you set the suffix to
- * something that autoconf did not find, you are likely to have serious
- * problems, so make sure you have the appropriate compression tool installed
- * before you set this.  You can look at the autoconf.h file to see
- * what compression tools it found (search for COMPRESS).
- * Note that this is used when saving files.  Crossfire will search all
- * methods when loading a file to see if it finds a match
- */
-
-#ifndef COMPRESS_SUFFIX
-/* #define COMPRESS_SUFFIX ".Z" */
-#endif
-
 /**
  * If you get a complaint about O_NDELAY not being known/undefined, try
  * uncommenting this.
@@ -380,13 +336,10 @@
  * DMFILE - file with dm/wizard access lists
  * LOGFILE - where to log if using -daemon option
  * MAP_ - various map timeout and swapping parameters
- * MAX_OBJECTS - how many objects to keep in memory.
- * MAX_OBJECTS_LWM - only swap maps out if below that value
  * MOTD - message of the day - printed each time someone joins the game
  * PERM_FILE - limit play times
  * SHUTDOWN - used when shutting down the server
- * SOCKETBUFSIZE - size of buffer used internally by the server for storing
- *    backlogged messages.
+ * SOCKETBUFSIZE - size of the output socket buffer
  * TMPDIR - directory to use for temp files
  * UNIQUE_DIR - directory to put unique item files into
  * USE_CALLOC for some memory requests
@@ -473,61 +426,17 @@
 #define MAP_DEFAULTRESET       7200
 
 /**
- * MAX_OBJECTS is no hard limit.  If this limit is exceeded, Crossfire
- * will look for maps which are already scheldued for swapping, and
- * promptly swap them out before new maps are being loaded.
- * If playing only by yourself, this number can probably be as low as
- * 3000.  If in server mode, probably figure about 1000-2000 objects per
- * active player (if they typically play on different maps), for some guess
- * on how many to define.  If it is too low, maps just get swapped out
- * immediately, causing a performance hit.  If it is too high, the program
- * consumes more memory.  If you have gobs of free memory, a high number
- * might not be a bad idea.  Each object is around 350 bytes right now.
- * 25000 is about 8.5 MB
- */
-#define MAX_OBJECTS     100000
-
-/**
- * Max objects low water mark (lwm).  If defined, the map swapping strategy
- * is a bit different:
- * 1) We only start swapping maps if the number of objects in use is
- *    greater than MAX_OBJECTS above.
- * 2) We keep swapping maps until there are no more maps to swap or the number
- *    of used objects drop below this low water mark value.
+ * Defining MEMORY_DEBUG disables Crossfire's object allocator, which allocates
+ * OBJ_EXPAND objects at a time and manages its own free list. It is faster at
+ * allocating lots of objects, but defeats memory debuggers and mitigation
+ * techniques by managing its own allocations and enabling use-after-free bugs.
  *
- * If this is not defined, maps are swapped out on the timeout value above,
- * or if the number of objects used is greater than MAX_OBJECTS above.
- *
- * Note:  While this will prevent the pauses noticed when saving maps, there
- * can instead be cpu performance penalties - any objects in memory get
- * processed.  So if there are 4000 objects in memory, and 1000 of them
- * are living objects, the system will process all 1000 objects each tick.
- * With swapping enable, maybe 600 of the objects would have gotten swapped
- * out.  This is less likely a problem with a smaller number of MAX_OBJECTS
- * than if it is very large.
- * Also, the pauses you do get can be worse, as if you enter a map with
- * a lot of new objects and go above MAX_OBJECTS, it may have to swap out
- * many maps to get below the low water mark.
+ * Unfortunately, much of the code does not work without use-after-free because
+ * even checking if an object was freed requires looking in the object. Until
+ * these issues are fixed, Crossfire needs MEMORY_DEBUG unset in order to run
+ * without crashing frequently.
  */
-/*#define MAX_OBJECTS_LWM       MAX_OBJECTS/2*/
-
-/**
- * Turning on MEMORY_DEBUG slows down execution, but makes it easier
- * to find memory corruption and leaks.  Currently, the main thing
- * that happens with this activated is that one malloc is done for
- * each object - thus whatever debugging mechanism the malloc library
- * (or other debugging tool provides, like purify), it can track this
- * individual malloc.  Default behaviour when turned off is that
- * enough memory is malloced for a large group of objects so malloc does
- * not need to be called as often.
- * This should only be turned on if some form of memory debugging tool
- * is being used - otherwise, turning this on will cause some performance
- * hit with no useful advantage.
- *
- * Define to 2 for stricter checks (known to currently break).
- * Define to 3 for even stricter checks (known to currently break even more).
- */
-/*#define MEMORY_DEBUG 1*/
+/*#define MEMORY_DEBUG*/
 
 /**
  * If you want to have a Message Of The Day file, define MOTD to be
@@ -554,14 +463,10 @@
 #endif
 
 /**
- * SOCKETBUFSIZE is the size of the buffer used internally by the server for
- * storing backlogged messages for the client.  This is not operating system
- * buffers or the like.  This amount is used per connection (client).
- * This buffer is in addition to OS buffers, so it may not need to be very
- * large.  When the OS buffer and this buffer is exhausted, the server
- * will drop the client connection for falling too far behind.  So if
- * you have very slow client connections, a larger value may be
- * warranted.
+ * SOCKETBUFSIZE is the size of the output socket buffer we request from the
+ * operating system. If this buffer is exhausted, the server will drop the
+ * client connection. Increase this if players with large player files are
+ * causing this buffer to fill up.
  */
 #define SOCKETBUFSIZE 256*1024
 
@@ -589,20 +494,6 @@
  */
 #define UNIQUE_DIR "unique-items"
 
-/**
- * If undefined, malloc is always used.
- * It looks like this can be oboleted.  However, it can be useful to
- * track down some bugs, as it will make sure that the entire data structure
- * is set to 0, at the expense of speed.
- * Rupert Goldie has run Purify against the code, and if this is disabled,
- * apparantly there are a lot of uninitialized memory reads - I haven't
- * seen any problem (maybe the memory reads are copies, and the destination
- * doesn't actually use the garbage values either?), but the impact on speed
- * of using this probably isn't great, and should make things more stable.
- * Msw 8-9-97
- */
-#define USE_CALLOC
-
 /**
  * These define the players starting map and location on that map, and where
  * emergency saves are defined.  This should be left as is unless you make
@@ -740,4 +631,11 @@
  */
 #define RESET_LOCATION_TIME     3600
 
+/**
+ * Interval between heartbeat requests. If a client hasn't sent a command
+ * within this time period, and heartbeat is enabled, that client lost
+ * connection.
+ */
+#define BEAT_INTERVAL 3
+
 #endif /* CONFIG_H */
