@@ -1,31 +1,3 @@
-/*
- * static char *rcsid_define_h =
- *   "$Id: define.h 17403 2012-03-08 22:22:35Z ryo_saeba $";
- */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2003-2005 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Core defines: object types, flags, etc.
@@ -42,17 +14,6 @@
 #ifndef DEFINE_H
 #define DEFINE_H
 
-/*
- * Crossfire requires ANSI-C, but some compilers "forget" to define it.
- * Thus the prototypes made by cextract don't get included correctly.
- */
-#if !defined(__STDC__)
-/* Removed # from start of following line.  makedepend was picking it up.
- * The following should still hopefully result in an error.
- */
-error - Your ANSI C compiler should be defining __STDC__;
-#endif
-
 #ifndef WIN32 /* ---win32 exclude unix configuration part */
 #include <autoconf.h>
 #endif
@@ -63,12 +24,6 @@ error - Your ANSI C compiler should be defining __STDC__;
 #ifdef __NetBSD__
 #include <sys/param.h>
 #endif
-#ifndef MIN
-#define MIN(x, y) ((x) < (y) ? (x) : (y))
-#endif
-#ifndef MAX
-#define MAX(x, y) ((x) > (y) ? (x) : (y))
-#endif
 
 /** NAME_MAX used by random maps may not be defined on pure ansi systems */
 #ifndef NAME_MAX
@@ -89,216 +44,13 @@ error - Your ANSI C compiler should be defining __STDC__;
 /**
  * Fatal variables; used as arguments to fatal().
  */
-/*@{*/
-#define OUT_OF_MEMORY           0
-#define MAP_ERROR               1
-#define ARCHTABLE_TOO_SMALL     2
-#define ARCHETYPE_ISSUE         3
-#define SEE_LAST_ERROR          4
-
-/*@}*/
-
-/**
- * @defgroup OBJECT_TYPE Object types.
- *
- * Only add new values to this list if somewhere in the program code, it is
- * actually needed.  Just because you add a new monster does not mean it has to
- * have a type defined here.  That only needs to happen if in some .c file, it
- * needs to do certain special actions based on the monster type, that can not
- * be handled by any of the numerous flags.  Also, if you add new entries, try
- * and fill up the holes in this list.  Additionally, when you add a new entry,
- * include it in the table in common/item.c
- *
- * type 0 is undefined and indicates non-valid type information.
- */
-/*@{*/
-#define PLAYER                      1
-#define TRANSPORT                   2     /**< see doc/Developers/objects */
-#define ROD                         3
-#define TREASURE                    4
-#define POTION                      5
-#define FOOD                        6
-#define POISON                      7
-#define BOOK                        8
-#define CLOCK                       9
-#define DRAGON_FOCUS                10    /**< Used during character creation */
-/*#define FBALL                     11 */
-/*#define LIGHTNING                 12 */
-#define ARROW                       13
-#define BOW                         14
-#define WEAPON                      15
-#define ARMOUR                      16
-#define PEDESTAL                    17
-#define ALTAR                       18
-/*#define CONFUSION                 19 */
-#define LOCKED_DOOR                 20
-#define SPECIAL_KEY                 21
-#define MAP                         22
-#define DOOR                        23
-#define KEY                         24
-/*#define MMISSILE                  25 */
-#define TIMED_GATE                  26
-#define TRIGGER                     27
-#define GRIMREAPER                  28
-#define MAGIC_EAR                   29
-#define TRIGGER_BUTTON              30
-#define TRIGGER_ALTAR               31
-#define TRIGGER_PEDESTAL            32
-#define SHIELD                      33
-#define HELMET                      34
-/*#define HORN                      35    has been merged into ROD */
-#define MONEY                       36
-#define CLASS                       37    /**< Object for applying character class
-                                           * modifications to someone */
-/*#define GRAVESTONE                38 */
-#define AMULET                      39
-#define PLAYERMOVER                 40
-#define TELEPORTER                  41
-#define CREATOR                     42
-#define SKILL                       43    /**< Also see ::SKILL_TOOL (74) below */
-#define EARTHWALL                   45
-#define GOLEM                       46
-/*#define BOMB                      47 */
-#define THROWN_OBJ                  48
-#define BLINDNESS                   49
-#define GOD                         50
-
-#define DETECTOR                    51    /**< peterm:  detector is an object
-                                           * which notices the presense of
-                                           * another object and is triggered
-                                           * like buttons. */
-#define TRIGGER_MARKER              52    /**< inserts an invisible, weightless
-                                           * force into a player with a
-                                           * specified string WHEN TRIGGERED. */
-#define DEAD_OBJECT                 53
-#define DRINK                       54
-#define MARKER                      55    /**< inserts an invisible, weightless
-                                           * force into a player with a
-                                           * specified string. */
-#define HOLY_ALTAR                  56
-#define PLAYER_CHANGER              57
-#define BATTLEGROUND                58    /**< battleground, by Andreas Vogl */
-
-#define PEACEMAKER                  59    /**< Object owned by a player which can
-                                           * convert a monster into a peaceful
-                                           * being incapable of attack.  */
-#define GEM                         60
-/*#define FIRECHEST                 61 */ /* FIRECHEST folded into FIREWALL */
-#define FIREWALL                    62
-/*#define ANVIL                     63 */
-#define CHECK_INV                   64    /**< b.t. thomas@nomad.astro.psu.edu */
-#define MOOD_FLOOR                  65    /**< b.t. thomas@nomad.astro.psu.edu
-                                           * values of last_sp set how to
-                                           * change:
-                                           * 0 = furious, all monsters become
-                                           *     aggressive
-                                           * 1 = angry, all but friendly become
-                                           *     aggressive
-                                           * 2 = calm, all aggressive monsters
-                                           *     calm down
-                                           * 3 = sleep, all monsters fall asleep
-                                           * 4 = charm, monsters become pets */
-#define EXIT                        66
-#define ENCOUNTER                   67
-#define SHOP_FLOOR                  68
-#define SHOP_MAT                    69
-#define RING                        70
-#define FLOOR                       71    /**< Floor tile -> native layer 0 */
-#define FLESH                       72    /**< animal 'body parts' -b.t. */
-#define INORGANIC                   73    /**< metals, minerals, dragon scales */
-#define SKILL_TOOL                  74    /**< Allows the use of a skill */
-#define LIGHTER                     75
-
-/* The trap_part, wall, light_source, misc_object, monster, and spawn_generator
- * types are not used in any archetypes, and should perhaps be removed.
- */
-/*#define TRAP_PART                 76 */ /* Needed by set traps skill -b.t. */
-
-#define WALL                        77    /**< Wall. Put it always in layer 1 if
-                                           * not set is_floor */
-/*#define LIGHT_SOURCE              78 */ /* torches, lamps, etc. */
-#define MISC_OBJECT                 79    /**< misc. objects are for objects
-                                           * without a function in the engine.
-                                           * Like statues, clocks, chairs...
-                                           *  If perhaps we create a function
-                                           * where we can sit on chairs, we
-                                           * create a new type and remove all
-                                           * chairs from here. */
-#define MONSTER                     80    /**< A real, living creature */
-/*#define SPAWN_GENERATOR           81 */ /* Spawn point or monster generator */
-#define LAMP                        82    /**< Lamp */
-#define DUPLICATOR                  83    /**< Duplicator/multiplier object */
-/*#define TOOL                      84 */ /* Tool for building objects */
-#define SPELLBOOK                   85
-/*#define BUILDFAC                  86 */ /* Facilities for building objects */
-#define CLOAK                       87
-/*#define CONE                      88 */
-/*#define AURA                      89 */ /* Aura spell object */
-
-#define SPINNER                     90
-#define GATE                        91
-#define BUTTON                      92
-#define CF_HANDLE                   93
-#define HOLE                        94    /* When open, objects fall through */
-#define TRAPDOOR                    95
-/*#define WORD_OF_RECALL            96 */
-/*#define PARAIMAGE                 97 */
-#define SIGN                        98
-#define BOOTS                       99
-#define GLOVES                      100
-#define SPELL                       101
-#define SPELL_EFFECT                102
-#define CONVERTER                   103
-#define BRACERS                     104
-#define POISONING                   105
-#define SAVEBED                     106
-/*#define POISONCLOUD               107*/
-/*#define FIREHOLES                 108*/
-#define WAND                        109
-/*#define ABILITY                   110*/
-#define SCROLL                      111
-#define DIRECTOR                    112
-#define GIRDLE                      113
-#define FORCE                       114
-#define POTION_RESIST_EFFECT        115   /**< A force, holding the effect of a
-                                           * resistance potion */
-#define EVENT_CONNECTOR             116   /**< Lauwenmark: an invisible object
-                                           * holding a plugin event hook */
-#define CLOSE_CON                   121   /**< Eneq((at)csd.uu.se): Id for
-                                           * close_container archetype. */
-#define CONTAINER                   122
-#define ARMOUR_IMPROVER             123
-#define WEAPON_IMPROVER             124
-
-/* unused: 125 - 129
- * type 125 was MONEY_CHANGER
- */
-#define SKILLSCROLL                 130   /**< can add a skill to player's
-                                           * inventory -bt.*/
-#define DEEP_SWAMP                  138
-#define IDENTIFY_ALTAR              139
-/*#define CANCELLATION              141*/ /* not used with new spell code */
-#define SHOP_INVENTORY              150   /**< Mark Wedel (mark@pyramid.com) Shop
-                                           * inventories */
-/*#define BALL_LIGHTNING            151*/ /* peterm:  ball lightning and color
-                                           * spray */
-/*#define SWARM_SPELL               153*/
-#define RUNE                        154
-#define TRAP                        155
-
-#define POWER_CRYSTAL               156
-#define CORPSE                      157
-
-#define DISEASE                     158
-#define SYMPTOM                     159
-
-#define BUILDER                     160   /**< Generic item builder, see subtypes
-                                           * below */
-#define MATERIAL                    161   /**< Material for building */
-
-#define OBJECT_TYPE_MAX             162   /**< Update if you add new types */
-/* END TYPE DEFINE */
-/*@}*/
+enum fatal_error {
+    OUT_OF_MEMORY = 0,
+    MAP_ERROR = 1,
+    ARCHTABLE_TOO_SMALL = 2,
+    ARCHETYPE_ISSUE = 3,
+    SEE_LAST_ERROR = 4,
+};
 
 /**
  * @defgroup TYPE_BUILDER Subtypes for ::BUILDER objects.
@@ -420,7 +172,7 @@ typedef struct typedata {
 #define IS_LIVE(op) \
         ((op->type == PLAYER || QUERY_FLAG(op, FLAG_MONSTER) || \
         (QUERY_FLAG(op, FLAG_ALIVE) && !QUERY_FLAG(op, FLAG_GENERATOR) && \
-        !op->type == DOOR)) && (!QUERY_FLAG(op, FLAG_IS_A_TEMPLATE)))
+        !(op->type == DOOR))) && (!QUERY_FLAG(op, FLAG_IS_A_TEMPLATE)))
 
 #define IS_ARROW(op) \
         (op->type == ARROW || \
@@ -468,19 +220,10 @@ typedef struct typedata {
  * So there are 18 available flags slots
  */
 /*@{*/
-#define SET_FLAG(xyz, p) \
-    ((xyz)->flags[p/32] |= (1U<<(p%32)))
-#define CLEAR_FLAG(xyz, p) \
-    ((xyz)->flags[p/32] &= ~(1U<<(p%32)))
-#define QUERY_FLAG(xyz, p) \
-    ((xyz)->flags[p/32]&(1U<<(p%32)))
-#define COMPARE_FLAGS(p, q) \
-    (                                                \
-        ((p)->flags[0] == (q)->flags[0]) &&          \
-        ((p)->flags[1] == (q)->flags[1]) &&          \
-        ((p)->flags[2] == (q)->flags[2]) &&          \
-        ((p)->flags[3] == (q)->flags[3])             \
-    )
+#define SET_FLAG(xyz, p) set_flag(xyz, p)
+#define CLEAR_FLAG(xyz, p) clear_flag(xyz, p)
+#define QUERY_FLAG(xyz, p) query_flag(xyz, p)
+#define COMPARE_FLAGS(p, q) compare_flags(p, q)
 
 /* the flags themselves. */
 
@@ -600,7 +343,7 @@ typedef struct typedata {
 #define FLAG_ONE_HIT            97 /**< Monster can only hit once before going
                                     * away (replaces ghosthit)
                                     */
-#define FLAG_CLIENT_SENT        98 /**< THIS IS A DEBUG FLAG ONLY.  We use it to
+#define FLAG_CLIENT_SENT        98 /**< We use it to
                                     * detect cases were the server is trying
                                     * to send an upditem when we have not
                                     * actually sent the item.
@@ -625,7 +368,10 @@ typedef struct typedata {
 #define FLAG_AFK                 111 /**< Player is AFK */
 #define FLAG_BLESSED             112 /**< Item has a blessing, opposite of cursed/damned. */
 #define FLAG_KNOWN_BLESSED       113 /**< Item is known to be blessed. */
-#define NUM_FLAGS                113 /**< Should always be equal to the last
+#define FLAG_PARALYZED           114 /**< Monster or player is paralyzed.
+                                      * Currently only used to display message to player
+                                      * of paralysis begin and end. */
+#define NUM_FLAGS                114 /**< Should always be equal to the last
                                       * defined flag.  If you change this,
                                       * make sure you update the flag_links
                                       * in common/loader.l
@@ -665,7 +411,7 @@ typedef struct typedata {
 /**
  * Typdef here to define type large enough to hold bitmask of
  * all movement types.  Make one declaration so easy to update.
- * uint8 is defined yet, so just use what that would define it
+ * uint8_t is defined yet, so just use what that would define it
  * at anyways.
  */
 typedef unsigned char MoveType;
@@ -687,7 +433,6 @@ typedef unsigned char MoveType;
     ((type != 0) && (ob1->move_type&type) == ob1->move_type)
 /*@}*/
 
-#define SET_GENERATE_TYPE(xyz, va) (xyz)->stats.sp = (va)
 #define GENERATE_TYPE(xyz)      ((xyz)->stats.sp)
 #define GENERATE_SPEED(xyz)     ((xyz)->stats.maxsp) /* if(!RANDOM()%<speed>) */
 
@@ -713,20 +458,7 @@ typedef unsigned char MoveType;
  */
 #define MAX_WEAPON_ITEM_POWER   100
 
-/**
- * @defgroup F_xxx Buy/sell flags.
- * Those flags are mostly used for query_cost() and similar functions.
- */
-/*@{*/
-#define F_BUY           0   /**< Item is being bought by player. */
-#define F_SELL          1   /**< Item is being sold by player. */
-#define F_TRUE          2   /**< True value of item, unadjusted. */
-#define F_NO_BARGAIN    4   /**< Combine with ::F_BUY or ::F_SELL to disable bargaining calc. */
-#define F_IDENTIFIED    8   /**< Flag to calculate value of identified item. */
-#define F_NOT_CURSED    16  /**< Flag to calculate value of uncursed item. */
-#define F_APPROX        32  /**< Flag to give a guess of item value. */
-#define F_SHOP          64  /**< Consider the effect that the shop that the player is in has. */
-/*@}*/
+static const int32_t MAX_FOOD = 999;
 
 #define DIRX(xyz)       freearr_x[(xyz)->direction]
 #define DIRY(xyz)       freearr_y[(xyz)->direction]
@@ -742,60 +474,59 @@ typedef unsigned char MoveType;
  * if your monsters start acting wierd, mail me.
  */
 /*@{*/
-/******************************************************************************/
-/* the following definitions are for the attack_movement variable in monsters */
-/* if the attack_variable movement is left out of the monster archetype, or is*/
-/* set to zero                                                                */
-/* the standard mode of movement from previous versions of crossfire will be  */
-/* used. the upper four bits of movement data are not in effect when the monst*/
-/* er has an enemy. these should only be used for non agressive monsters.     */
-/* to program a monsters movement add the attack movement numbers to the movem*/
-/* ment numbers example a monster that moves in a circle until attacked and   */
-/* then attacks from a distance:                                              */
-/*                                                      CIRCLE1 = 32          */
-/*                                              +       DISTATT = 1           */
-/*                                      -------------------                   */
-/*                      attack_movement = 33                                  */
-/******************************************************************************/
-#define DISTATT  1 /**< Move toward a player if far, but mantain some space,
-                    * attack from a distance - good for missile users only    */
-#define RUNATT   2 /**< Tun but attack if player catches up to object.        */
+/*
+ * The following definitions are for the attack_movement variable in monsters.
+ * If the attack_variable movement is left out of the monster archetype, or is
+ * set to zero, the standard mode of movement from previous versions of
+ * Crossfire will be used. The upper four bits of movement data are not in
+ * effect when the monster has an enemy. These should only be used for non
+ * agressive monsters. To program a monsters movement add the attack movement
+ * numbers to the movement numbers example a monster that moves in a circle
+ * until attacked and then attacks from a distance:
+ *                                                      CIRCLE1 = 32
+ *                                              +       DISTATT = 1
+ *                                      -------------------
+ *                      attack_movement = 33
+ */
+#define DISTATT  1 /**< Move toward a player if far, but maintain some space,
+                    * attack from a distance - good for missile users only.   */
+#define RUNATT   2 /**< Run but attack if player catches up to object.        */
 #define HITRUN   3 /**< Run to then hit player then run away cyclicly.        */
-#define WAITATT  4 /**< Wait for player to approach then hit, move if hit     */
-#define RUSH     5 /**< Rush toward player blindly, similiar to dumb monster  */
-#define ALLRUN   6 /**< Always run never attack good for sim. of weak player  */
-#define DISTHIT  7 /**< Attack from a distance if hit as recommended by Frank */
-#define WAIT2    8 /**< Monster does not try to move towards player if far
-                    * maintains comfortable distance                          */
-#define PETMOVE 16 /**< if the upper four bits of attack_movement
+#define WAITATT  4 /**< Wait for player to approach then hit, move if hit.    */
+#define RUSH     5 /**< Rush toward player blindly, similiar to dumb monster. */
+#define ALLRUN   6 /**< Always run, never attack good for sim. of weak player.*/
+#define DISTHIT  7 /**< Attack from a distance if hit as recommended by Frank.*/
+#define WAIT2    8 /**< Monster does not try to move towards player if far.
+                    * Maintains comfortable distance.                         */
+#define PETMOVE 16 /**< If the upper four bits of attack_movement
                     * are set to this number, the monster follows a player
                     * until the owner calls it back or off
-                    * player followed denoted by 0b->owner
+                    * player followed denoted by ob->owner
                     * the monster will try to attack whatever the player is
                     * attacking, and will continue to do so until the
                     * calls off the monster - a key command will be
-                    * inserted to do so                                       */
+                    * inserted to do so.                                      */
 #define CIRCLE1 32 /**< If the upper four bits of move_type / attack_movement
                     * are set to this number, the monster will move in a
                     * circle until it is attacked, or the enemy field is
-                    * set, this is good for non-aggressive monsters and NPC   */
-#define CIRCLE2 48 /**< Same as ::CIRCLE1 but a larger circle is used         */
-#define PACEH   64 /**< The monster will pace back and forth until attacked
-                    * this is HORIZONTAL movement                             */
+                    * set, this is good for non-aggressive monsters and NPC.  */
+#define CIRCLE2 48 /**< Same as ::CIRCLE1 but a larger circle is used.        */
+#define PACEH   64 /**< The monster will pace back and forth until attacked.
+                    * This is HORIZONTAL movement.                            */
 #define PACEH2  80 /**< The monster will pace as above but the length of the
                     * pace area is longer and the monster stops before
-                    * changing directions
-                    * this is HORIZONTAL movement                             */
+                    * changing directions.
+                    * This is HORIZONTAL movement.                            */
 #define RANDO   96 /**< The monster will go in a random direction until
                     * it is stopped by an obstacle, then it chooses another
                     * direction.                                              */
-#define RANDO2 112 /**< Constantly move in a different random direction       */
-#define PACEV  128 /**< The monster will pace back and forth until attacked
-                    * this is VERTICAL movement                               */
+#define RANDO2 112 /**< Constantly move in a different random direction.      */
+#define PACEV  128 /**< The monster will pace back and forth until attacked.
+                    * This is VERTICAL movement.                              */
 #define PACEV2 144 /**< The monster will pace as above but the length of the
                     * pace area is longer and the monster stops before
-                    * changing directions
-                    * this is VERTICAL movement                               */
+                    * changing directions.
+                    * This is VERTICAL movement.                              */
 #define LO4     15 /**< bitmasks for upper and lower 4 bits from 8 bit fields */
 #define HI4    240
 /*@}*/
@@ -832,29 +563,6 @@ typedef unsigned char MoveType;
 #define PREFER_HIGH     1
 #define PREFER_LOW      0
 
-/**
- * Simple function we use below to keep adding to the same string
- * but also make sure we don't overwrite that string.
- *
- * @param dest
- * string to append to.
- * @param orig
- * string to append.
- * @param[out] curlen
- * current length of dest. Will be updated by this function.
- * @param maxlen
- * maximum length of dest buffer.
- */
-static inline void safe_strcat(char *dest, const char *orig, size_t *curlen, size_t maxlen) {
-    if (*curlen == (maxlen-1))
-        return;
-    strncpy(dest+*curlen, orig, maxlen-*curlen-1);
-    dest[maxlen-1] = 0;
-    *curlen += strlen(orig);
-    if (*curlen > (maxlen-1))
-        *curlen = maxlen-1;
-}
-
 /**
  * @defgroup AP_xxx Flags for apply_special().
  *
@@ -865,6 +573,7 @@ static inline void safe_strcat(char *dest, const char *orig, size_t *curlen, siz
 #define AP_NULL         0   /**< Nothing specific. */
 #define AP_APPLY        1   /**< Item is to be applied. */
 #define AP_UNAPPLY      2   /**< Item is to be remvoed. */
+#define AP_OPEN         3   /**< Item is a container to be fully opened. */
 
 #define AP_BASIC_FLAGS  15
 
@@ -927,34 +636,11 @@ static inline void safe_strcat(char *dest, const char *orig, size_t *curlen, siz
 /** Cut off point of when an object is put on the active list or not */
 #define MIN_ACTIVE_SPEED        0.00001
 
-/*
- * random() is much better than rand().  If you have random(), use it instead.
- * You shouldn't need to change any of this
- *
- * 0.93.3: It looks like linux has random (previously, it was set below
- * to use rand).  Perhaps old version of linux lack rand?  IF you run into
- * problems, add || defined(__linux__) the #if immediately below.
- *
- * 0.94.2 - you probably shouldn't need to change any of the rand stuff
- * here.
- */
+uint32_t cf_random(void);
+void cf_srandom(unsigned long seed);
 
-#ifdef HAVE_SRANDOM
-#define RANDOM() random()
-#define SRANDOM(xyz) srandom(xyz)
-#else
-#  ifdef HAVE_SRAND48
-#  define RANDOM() lrand48()
-#  define SRANDOM(xyz) srand48(xyz)
-#  else
-#    ifdef HAVE_SRAND
-#      define RANDOM() rand()
-#      define SRANDOM(xyz) srand(xyz)
-#    else
-#      error "Could not find a usable random routine"
-#    endif
-#  endif
-#endif
+#define RANDOM()        cf_random()
+#define SRANDOM(seed)   cf_srandom(seed)
 
 /**
  * Returns the weight of the given object. Note: it does not take the number of
