@@ -1,31 +1,3 @@
-/*
- * static char *rcsid_newserver_h =
- *   "$Id: newserver.h 15065 2011-09-10 09:28:52Z ryo_saeba $";
- */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Defines various structures and values that are used for the
@@ -57,9 +29,8 @@
 
 /** One map cell, as sent to the client. */
 struct map_cell_struct {
-    uint16 faces[MAP_LAYERS];
-    uint16 smooth[MAP_LAYERS];
-    int darkness;
+    uint16_t faces[MAP_LAYERS];   /**< Face numbers. */
+    int darkness;               /**< Cell's darkness. */
 };
 
 /**
@@ -109,16 +80,6 @@ struct listen_info {
     struct sockaddr *addr;
 };
 
-/**
- * The following is the setup for a ring buffer for storing outbut
- * data that the OS can't handle right away.
- */
-typedef struct buffer_struct {
-    char    data[SOCKETBUFSIZE];
-    int     start;
-    int     len;
-} buffer_struct;
-
 /** How many times we are allowed to give the wrong password before being kicked. */
 #define MAX_PASSWORD_FAILURES 5
 
@@ -130,43 +91,43 @@ typedef struct socket_struct {
     int         fd;
     struct listen_info  *listen;
     struct Map  lastmap;
-    sint8       map_scroll_x, map_scroll_y;
+    int8_t       map_scroll_x, map_scroll_y;
     size_t      faces_sent_len;         /**< This is the number of elements allocated in faces_sent[]. */
-    uint8       *faces_sent;            /**< This is a bitmap on sent face status. */
-    uint8       anims_sent[MAXANIMNUM]; /**< What animations we sent. */
+    uint8_t       *faces_sent;            /**< This is a bitmap on sent face status. */
+    uint8_t       anims_sent[MAXANIMNUM]; /**< What animations we sent. */
     struct statsinfo stats;
     SockList    inbuf;                  /**< If we get an incomplete packet, this is used to hold the data. */
     char        *host;                  /**< Which host it is connected from (ip address). */
-    buffer_struct outputbuffer;         /**< For undeliverable data. */
-    uint8       password_fails;         /**< How many times the player has failed to give the right password. */
-    uint32      facecache:1;            /**< If true, client is caching images. */
-    uint32      darkness:1;             /**< True if client wants darkness information. */
-    uint32      update_look:1;          /**< If true, we need to send the look window. */
-    uint32      update_inventory:1;     /**< If true, we need to send the inventory list. */
-    uint32      can_write:1;            /**< Can we write to this socket? */
-    uint32      tick:1;                 /**< Client wishes to get tick commands. */
-    uint32      is_bot:1;               /**< Client shouldn't be reported to metaserver. */
-    uint32      want_pickup:1;          /**< Client wants pickup information when logging in. */
-    uint32      extended_stats:1;       /**< Client wants base and maximum statistics information. */
-    uint32      monitor_spells;         /**< Client wishes to be informed when their spell list changes. */
-    uint32      sound;                  /**< Client sound mode. */
-    uint32      cs_version, sc_version; /**< Versions of the client. */
-    uint16      look_position;          /**< Start of drawing of look window. */
-    uint16      container_position;     /**< Start of container contents to send to client. */
-    uint8       mapx, mapy;             /**< How large a map the client wants. */
-    uint8       faceset;                /**< Set the client is using, default 0. */
+    uint8_t       password_fails;         /**< How many times the player has failed to give the right password. */
+    uint32_t      facecache:1;            /**< If true, client is caching images. */
+    uint32_t      darkness:1;             /**< True if client wants darkness information. */
+    uint32_t      update_look:1;          /**< If true, we need to send the look window. */
+    uint32_t      update_inventory:1;     /**< If true, we need to send the inventory list. */
+    uint32_t      tick:1;                 /**< Client wishes to get tick commands. */
+    uint32_t      is_bot:1;               /**< Client shouldn't be reported to metaserver. */
+    uint32_t      want_pickup:1;          /**< Client wants pickup information when logging in. */
+    uint32_t      extended_stats:1;       /**< Client wants base and maximum statistics information. */
+    uint32_t      monitor_spells;         /**< Client wishes to be informed when their spell list changes. */
+    uint32_t      sound;                  /**< Client sound mode. */
+    bool          heartbeat;              /**< Client will send hearbeats. */
+    uint32_t      cs_version, sc_version; /**< Versions of the client. */
+    uint16_t      look_position;          /**< Start of drawing of look window. */
+    uint16_t      container_position;     /**< Start of container contents to send to client. */
+    uint8_t       mapx, mapy;             /**< How large a map the client wants. */
+    uint8_t       faceset;                /**< Set the client is using, default 0. */
 
     /* Below are flags for extedend infos to pass to client
      * with S->C mapextended command (note: this comment seems incorrect?) */
-    sint8       sounds_this_tick;       /**< Number of sounds sent this tick. */
-    uint8       num_look_objects;       /**< The maximum number of objects to show on the ground view;
+    int8_t       sounds_this_tick;       /**< Number of sounds sent this tick. */
+    uint8_t       num_look_objects;       /**< The maximum number of objects to show on the ground view;
                                              this number includes the prev/next group fake items.
                                              Can be set through "num_look_objects" setup option;
                                              defaults to DEFAULT_NUM_LOOK_OBJECTS. */
     char    *account_name;              /**< Name of the account logged in on this socket */
     Account_Char    *account_chars;     /**< Detailed information on characters on this account */
-    uint8   login_method;               /**< Login method this client is using */
-    uint16      notifications;          /**< Notifications this client wants to get. */
+    uint8_t login_method;   /**< Login method this client is using */
+    uint16_t notifications; /**< Notifications this client wants to get. */
+    uint32_t last_tick;     /**< Number of ticks since last communication. */
 } socket_struct;
 
 /**
