@@ -1,31 +1,3 @@
-/*
- * static char *rcsid_newclient_h =
- *   "$Id: newclient.h 14845 2011-07-14 13:39:14Z ryo_saeba $";
- */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2006,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Defines various flags that both the new client and new server use.  These
@@ -288,12 +260,6 @@ enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
 #define F_NOPICK        0x4000
 #define F_LOCKED        0x8000
 #define F_BLESSED       0x0100
-
-#define CF_FACE_NONE    0
-#define CF_FACE_BITMAP  1
-#define CF_FACE_XPM     2
-#define CF_FACE_PNG     3
-#define CF_FACE_CACHE   0x10
 /*@}*/
 
 /**
@@ -330,12 +296,15 @@ enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
 /*@}*/
 
 /**
- * @defgroup SOUND_xxx SOUND_xxx constants.
- *
+ * @defgroup Soundtypes Sound types
  */
 /*@{*/
-#define SOUND_NORMAL    0
-#define SOUND_SPELL     1
+#define SOUND_TYPE_LIVING       1
+#define SOUND_TYPE_SPELL        2
+#define SOUND_TYPE_ITEM         3
+#define SOUND_TYPE_GROUND       4
+#define SOUND_TYPE_HIT          5
+#define SOUND_TYPE_HIT_BY       6
 /*@}*/
 
 /**
