@@ -1,31 +1,3 @@
-/*
- * static char *rcsid_skills_h =
- *   "$Id: skills.h 16506 2012-01-07 18:22:35Z ryo_saeba $";
- */
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2003 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Skill-related defines, including subtypes.
@@ -35,7 +7,7 @@
 #define SKILLS_H
 
 /**
- * @defgroup SK_xxx Skill subtypes
+ * @enum SK Skill subtypes
  *
  * This list is just a subtype <-> skill (code wise) in the
  * server translation.  In theory, the processing of the different
@@ -44,60 +16,58 @@
  * The order of this list has no special meaning.  0 is not used
  * to denote improperly set objects.
  */
-/*@{*/
-#define SK_LOCKPICKING          1
-#define SK_HIDING               2
-#define SK_SMITHERY             3
-#define SK_BOWYER               4
-#define SK_JEWELER              5
-#define SK_ALCHEMY              6
-#define SK_STEALING             7
-#define SK_LITERACY             8
-#define SK_BARGAINING           9
-#define SK_JUMPING              10
-#define SK_DET_MAGIC            11
-#define SK_ORATORY              12
-#define SK_SINGING              13
-#define SK_DET_CURSE            14
-#define SK_FIND_TRAPS           15
-#define SK_MEDITATION           16
-#define SK_PUNCHING             17
-#define SK_FLAME_TOUCH          18
-#define SK_KARATE               19
-#define SK_CLIMBING             20
-#define SK_WOODSMAN             21
-#define SK_INSCRIPTION          22
-#define SK_ONE_HANDED_WEAPON    23
-#define SK_MISSILE_WEAPON       24
-#define SK_THROWING             25
-#define SK_USE_MAGIC_ITEM       26
-#define SK_DISARM_TRAPS         27
-#define SK_SET_TRAP             28
-#define SK_THAUMATURGY          29
-#define SK_PRAYING              30
-#define SK_CLAWING              31
-#define SK_LEVITATION           32
-#define SK_SUMMONING            33
-#define SK_PYROMANCY            34
-#define SK_EVOCATION            35
-#define SK_SORCERY              36
-#define SK_TWO_HANDED_WEAPON    37
-#define SK_WRAITH_FEED          38
-#define SK_HARVESTING           39
-#define SK_AIR_MAGIC            40
-#define SK_EARTH_MAGIC          41
-#define SK_WATER_MAGIC          42
-#define SK_FIRE_MAGIC           43
-
-/*@}*/
+enum SK {
+    SK_LOCKPICKING           = 1,   /**< Lockpicking. */
+    SK_HIDING                = 2,   /**< Hiding. */
+    SK_SMITHERY              = 3,   /**< Smithery. */
+    SK_BOWYER                = 4,   /**< Bowyer. */
+    SK_JEWELER               = 5,   /**< Jeweler. */
+    SK_ALCHEMY               = 6,   /**< Alchemy. */
+    SK_STEALING              = 7,   /**< Stealing. */
+    SK_LITERACY              = 8,   /**< Literacy. */
+    SK_BARGAINING            = 9,   /**< Bargaining. */
+    SK_JUMPING               = 10,  /**< Jumping. */
+    SK_DET_MAGIC             = 11,  /**< Detect magic. */
+    SK_ORATORY               = 12,  /**< Oratory. */
+    SK_SINGING               = 13,  /**< Singing. */
+    SK_DET_CURSE             = 14,  /**< Detect curse. */
+    SK_FIND_TRAPS            = 15,  /**< Find traps. */
+    SK_MEDITATION            = 16,  /**< Meditation. */
+    SK_PUNCHING              = 17,  /**< Punching. */
+    SK_FLAME_TOUCH           = 18,  /**< Flame-touch. */
+    SK_KARATE                = 19,  /**< Karate. */
+    SK_CLIMBING              = 20,  /**< Climbing. */
+    SK_WOODSMAN              = 21,  /**< Woodsman. */
+    SK_INSCRIPTION           = 22,  /**< Inscription.*/
+    SK_ONE_HANDED_WEAPON     = 23,  /**< One handed weapon. */
+    SK_MISSILE_WEAPON        = 24,  /**< Missile weapon. */
+    SK_THROWING              = 25,  /**< Throwing. */
+    SK_USE_MAGIC_ITEM        = 26,  /**< Use magic item. */
+    SK_DISARM_TRAPS          = 27,  /**< Disarm traps. */
+    SK_SET_TRAP              = 28,  /**< Set traps, unused. */
+    SK_THAUMATURGY           = 29,  /**< Thaumaturgy. */
+    SK_PRAYING               = 30,  /**< Praying. */
+    SK_CLAWING               = 31,  /**< Clawing. */
+    SK_LEVITATION            = 32,  /**< Levitation. */
+    SK_SUMMONING             = 33,  /**< Summoning. */
+    SK_PYROMANCY             = 34,  /**< Pyromancy. */
+    SK_EVOCATION             = 35,  /**< Evocation. */
+    SK_SORCERY               = 36,  /**< Sorcery. */
+    SK_TWO_HANDED_WEAPON     = 37,  /**< Two handed weapons. */
+    SK_WRAITH_FEED           = 38,  /**< Wraith feed. */
+    SK_HARVESTING            = 39,  /**< Harvesting. */
+    SK_AIR_MAGIC             = 40,  /**< Air magic, unused. */
+    SK_EARTH_MAGIC           = 41,  /**< Earth magic, unused. */
+    SK_WATER_MAGIC           = 42,  /**< Water magic, unused. */
+    SK_FIRE_MAGIC            = 43,  /**< Fire magic, unused. */
+};
 
 /**
- * This is the highest number skill in the table +1
- * This is used to store pointers to the actual skills -
- * to make life easier, we use the value above as index,
- * eg, SK_EVOCATION (35) will be in last_skills[35].
+ * This is the maximum number of skills the game may handle.
+ * There is no direct mapping to the subtype, as multiple skills may share
+ * the same subtype with a different name.
  */
-#define NUM_SKILLS              44
+#define MAX_SKILLS              50
 
 /**
  * @defgroup SK_EXP_xxx Experience flags
@@ -150,8 +120,10 @@
 #define IS_GRACE_SKILL(num) \
     (num == SK_PRAYING)
 
-extern const char *skill_names[NUM_SKILLS];
-extern int skill_faces[NUM_SKILLS];
+extern const char *skill_names[MAX_SKILLS];
+extern const Face *skill_faces[MAX_SKILLS];
+
+extern int get_skill_client_code(const char *skill_name);
 
 #ifdef WANT_UNARMED_SKILLS
 /** Table of unarmed attack skills.  Terminated by 0.  This
@@ -166,7 +138,7 @@ extern int skill_faces[NUM_SKILLS];
  * skill numbers above.
  * This should probably be removed and made a player preferance instead.
  */
-static uint8 unarmed_skills[] = {
+static uint8_t unarmed_skills[] = {
     SK_KARATE,
     SK_CLAWING,
     SK_FLAME_TOUCH,
