@@ -1,13 +1,13 @@
 /* account.c */
-void clear_accounts(void);
-void account_load_entries(void);
+void accounts_clear(void);
+void accounts_load(void);
 void accounts_save(void);
 const char *account_exists(const char *account_name);
-int account_check_name_password(const char *account_name, const char *account_password);
+int account_login(const char *account_name, const char *account_password);
 int account_check_string(const char *str);
-int account_add_account(const char *account_name, const char *account_password);
-int account_add_player_to_account(const char *account_name, const char *player_name);
-int account_remove_player_from_account(const char *account_name, const char *player_name);
+int account_new(const char *account_name, const char *account_password);
+int account_link(const char *account_name, const char *player_name);
+int account_remove_player(const char *account_name, const char *player_name);
 char **account_get_players_for_account(const char *account_name);
 const char *account_get_account_for_char(const char *charname);
 player *account_get_logged_in_player(const char *name);
@@ -20,7 +20,6 @@ void account_char_save(const char *account, Account_Char *chars);
 Account_Char *account_char_add(Account_Char *chars, player *pl);
 Account_Char *account_char_remove(Account_Char *chars, const char *pl_name);
 void account_char_free(Account_Char *chars);
-void account_char_init(void);
 /* alchemy.c */
 int use_alchemy(object *op);
 /* apply.c */
@@ -28,7 +27,7 @@ int transport_can_hold(const object *transport, const object *op, int nrof);
 int should_director_abort(const object *op, const object *victim);
 void apply_handle_yield(object *tmp);
 int set_object_face_main(object *op);
-int apply_container(object *op, object *sack);
+int apply_container(object *op, object *sack, int aflags);
 void do_learn_spell(object *op, object *spell, int special_prayer);
 void do_forget_spell(object *op, const char *spell);
 int apply_manual(object *op, object *tmp, int aflag);
@@ -41,14 +40,14 @@ int apply_auto(object *op);
 void apply_auto_fix(mapstruct *m);
 void scroll_failure(object *op, int failure, int power);
 void apply_changes_to_player(object *pl, object *change, int limit_stats);
-void legacy_apply_container(object *op, object *sack);
 /* attack.c */
-void save_throw_object(object *op, uint32 type, object *originator);
-int hit_map(object *op, int dir, uint32 type, int full_hit);
+void save_throw_object(object *op, uint32_t type, object *originator);
+int slow_living_by(object *op, const int speed_penalty);
+int hit_map(object *op, int dir, uint32_t type, int full_hit);
 int attack_ob(object *op, object *hitter);
 object *hit_with_arrow(object *op, object *victim);
 int friendly_fire(object *op, object *hitter);
-int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit);
+int hit_player(object *op, int dam, object *hitter, uint32_t type, int full_hit);
 void confuse_living(object *op, object *hitter, int dam);
 void blind_living(object *op, object *hitter, int dam);
 void paralyze_living(object *op, int dam);
@@ -168,6 +167,7 @@ void command_usekeys(object *op, const char *params);
 void command_resistances(object *op, const char *params);
 void command_help(object *op, const char *params);
 int onoff_value(const char *line);
+void command_delete(object *op, const char *params);
 void command_quit(object *op, const char *params);
 void command_sound(object *op, const char *params);
 void receive_player_name(object *op);
@@ -190,12 +190,14 @@ void command_southeast(object *op, const char *params);
 void command_southwest(object *op, const char *params);
 void command_west(object *op, const char *params);
 void command_stay(object *op, const char *params);
+void do_goto(object *op, const char *name, int x, int y);
 /* c_new.c */
 void execute_newserver_command(object *pl, char *command);
 void command_run(object *op, const char *params);
 void command_run_stop(object *op, const char *params);
 void command_fire(object *op, const char *params);
 void command_fire_stop(object *op, const char *params);
+void command_face(object *op, const char *params);
 /* c_object.c */
 void command_uskill(object *pl, const char *params);
 void command_rskill(object *pl, const char *params);
@@ -203,11 +205,11 @@ void command_search(object *op, const char *params);
 void command_disarm(object *op, const char *params);
 void command_throw(object *op, const char *params);
 void command_apply(object *op, const char *params);
-int sack_can_hold(const object *pl, const object *sack, const object *op, uint32 nrof);
+int sack_can_hold(const object *pl, const object *sack, const object *op, uint32_t nrof);
 void pick_up(object *op, object *alt);
 void command_take(object *op, const char *params);
-void put_object_in_sack(object *op, object *sack, object *tmp, uint32 nrof);
-object *drop_object(object *op, object *tmp, uint32 nrof);
+void put_object_in_sack(object *op, object *sack, object *tmp, uint32_t nrof);
+object *drop_object(object *op, object *tmp, uint32_t nrof);
 void drop(object *op, object *tmp);
 void command_dropall(object *op, const char *params);
 void command_drop(object *op, const char *params);
@@ -235,7 +237,6 @@ void command_cast(object *op, const char *params);
 void command_prepare(object *op, const char *params);
 void command_cast_spell(object *op, const char *params, char command);
 int legal_range(object *op, int r);
-void change_spell(object *op, char k);
 void command_rotateshoottype(object *op, const char *params);
 /* c_wiz.c */
 void command_loadtest(object *op, const char *params);
@@ -287,11 +288,11 @@ void command_style_map_info(object *op, const char *params);
 void command_follow(object *op, const char *params);
 void command_purge_quest(object *op, const char *param);
 void command_purge_quest_definitions(object *op, const char *param);
+void command_dumpabove(object *op, const char *params);
 void command_dumpbelow(object *op, const char *params);
+void command_settings(object *op, const char *ignored);
 /* commands.c */
 void init_commands(void);
-/* daemon.c */
-void become_daemon(void);
 /* disease.c */
 int move_disease(object *disease);
 int infect_object(object *victim, object *disease, int force);
@@ -314,6 +315,7 @@ void init(int argc, char **argv);
 void free_server(void);
 racelink *find_racelink(const char *name);
 /* knowledge.c */
+void knowledge_give(player *pl, const char *marker, const object *book);
 void knowledge_read(player *pl, object *book);
 void command_knowledge(object *pl, const char *params);
 void free_knowledge(void);
@@ -323,6 +325,7 @@ void knowledge_send_info(socket_struct *ns);
 void knowledge_send_known(player *pl);
 void knowledge_first_player_save(player *pl);
 void knowledge_process_incremental(void);
+void knowledge_show_monster_detail(object *op, const char *name);
 /* login.c */
 void emergency_save(int flag);
 void delete_character(const char *name);
@@ -338,7 +341,6 @@ int monster_compute_path(object *source, object *target, int default_dir);
 void monster_do_living(object *op);
 int monster_move(object *op);
 void monster_check_apply_all(object *monster);
-void monster_check_apply(object *mon, object *item);
 void monster_npc_call_help(object *op);
 void monster_check_earthwalls(object *op, mapstruct *m, int x, int y);
 void monster_check_doors(object *op, mapstruct *m, int x, int y);
@@ -353,7 +355,7 @@ int monster_can_see_enemy(object *op, object *enemy);
 int move_object(object *op, int dir);
 int move_ob(object *op, int dir, object *originator);
 int transfer_ob(object *op, int x, int y, int randomly, object *originator);
-int teleport(object *teleporter, uint8 tele_type, object *user);
+int teleport(object *teleporter, uint8_t tele_type, object *user);
 void recursive_roll(object *op, int dir, object *pusher);
 int push_ob(object *who, int dir, object *pusher);
 int move_to(object *op, int x, int y);
@@ -416,11 +418,12 @@ int check_race_and_class(living *stats, archetype *race, archetype *opclass);
 int apply_race_and_class(object *op, archetype *race, archetype *opclass, living *stats);
 void key_confirm_quit(object *op, char key);
 int check_pick(object *op);
-int fire_bow(object *op, object *arrow, int dir, int wc_mod, sint16 sx, sint16 sy);
+int fire_bow(object *op, object *arrow, int dir, int wc_mod, int16_t sx, int16_t sy);
 void fire(object *op, int dir);
 object *find_key(object *pl, object *container, object *door);
 void move_player_attack(object *op, int dir);
 int move_player(object *op, int dir);
+int face_player(object *op, int dir);
 int handle_newcs_player(object *op);
 void remove_unpaid_objects(object *op, object *env, int free_items);
 void do_some_living(object *op);
@@ -437,7 +440,7 @@ int player_can_view(object *pl, object *op);
 int op_on_battleground(object *op, int *x, int *y, archetype **trophy);
 void dragon_ability_gain(object *who, int atnr, int level);
 void player_unready_range_ob(player *pl, object *ob);
-void player_set_state(player *pl, uint8 state);
+void player_set_state(player *pl, uint8_t state);
 /* plugins.c */
 int user_event(object *op, object *activator, object *third, const char *message, int fix);
 int execute_event(object *op, int eventcode, object *activator, object *third, const char *message, int fix);
@@ -470,20 +473,6 @@ int dispel_rune(object *op, object *caster, object *spell, object *skill, int di
 int trap_see(object *op, object *trap);
 int trap_show(object *trap, object *where);
 int trap_disarm(object *disarmer, object *trap, int risk, object *skill);
-/* shop.c */
-uint64 query_cost(const object *tmp, object *who, int flag);
-StringBuffer *query_cost_string(const object *tmp, object *who, int flag, StringBuffer *buf);
-StringBuffer *cost_string_from_value(uint64 cost, int largest_coin, StringBuffer *buf);
-uint64 query_money(const object *op);
-int pay_for_amount(uint64 to_pay, object *pl);
-int pay_for_item(object *op, object *pl);
-int can_pay(object *pl);
-int get_payment(object *pl, object *op);
-void sell_item(object *op, object *pl);
-double shopkeeper_approval(const mapstruct *map, const object *player);
-int describe_shop(const object *op);
-int is_in_shop(object *ob);
-int coords_in_shop(mapstruct *map, int x, int y);
 /* skills.c */
 int steal(object *op, int dir, object *skill);
 int pick_lock(object *pl, int dir, object *skill);
@@ -503,13 +492,13 @@ int write_on_item(object *pl, const char *params, object *skill);
 int skill_throw(object *op, object *part, int dir, object *skill);
 /* skill_util.c */
 void init_skills(void);
-void link_player_skills(object *op);
 object *find_skill_by_name(object *who, const char *name);
+object *find_applied_skill_by_name(const object* op, const char* name);
 object *find_skill_by_number(object *who, int skillno);
 int change_skill(object *who, object *new_skill, int flag);
 void clear_skill(object *who);
 int do_skill(object *op, object *part, object *skill, int dir, const char *string);
-sint64 calc_skill_exp(const object *who, const object *op, const object *skill);
+int64_t calc_skill_exp(const object *who, const object *op, const object *skill);
 int learn_skill(object *pl, object *scroll);
 void show_skills(object *op, const char *search);
 int use_skill(object *op, const char *string);
@@ -568,7 +557,7 @@ void dump_spells(void);
 void spell_effect(object *spob, int x, int y, mapstruct *map, object *originator);
 int min_casting_level(const object *caster, const object *spell);
 int caster_level(const object *caster, const object *spell);
-sint16 SP_level_spellpoint_cost(object *caster, object *spell, int flags);
+int16_t SP_level_spellpoint_cost(object *caster, object *spell, int flags);
 int SP_level_dam_adjust(const object *caster, const object *spob);
 int SP_level_duration_adjust(const object *caster, const object *spob);
 int SP_level_range_adjust(const object *caster, const object *spob);
@@ -577,8 +566,8 @@ object *check_spell_known(object *op, const char *name);
 object *lookup_spell_by_name(object *op, const char *spname);
 int reflwall(mapstruct *m, int x, int y, object *sp_op);
 int cast_create_obj(object *op, object *new_op, int dir);
-int ok_to_put_more(mapstruct *m, sint16 x, sint16 y, object *op, uint32 immune_stop);
-int fire_arch_from_position(object *op, object *caster, sint16 x, sint16 y, int dir, object *spell);
+int ok_to_put_more(mapstruct *m, int16_t x, int16_t y, object *op, uint32_t immune_stop);
+int fire_arch_from_position(object *op, object *caster, int16_t x, int16_t y, int dir, object *spell);
 void regenerate_rod(object *rod);
 void drain_rod_charge(object *rod);
 void drain_wand_charge(object *wand);
@@ -595,7 +584,6 @@ void rod_adjust(object *rod);
 void read_map_log(void);
 int swap_map(mapstruct *map);
 void check_active_maps(void);
-void swap_below_max(const char *except_level);
 int players_on_map(mapstruct *m, int show_all);
 void flush_old_maps(void);
 /* time.c */
@@ -612,6 +600,8 @@ int process_object(object *op);
 void legacy_remove_force(object *op);
 void legacy_animate_trigger(object *op);
 void legacy_move_hole(object *op);
+unsigned int tick_length(float seconds);
+long timespec_diff(struct timespec *end, struct timespec *start);
 /* timers.c */
 void cftimer_process_timers(void);
 int cftimer_create(int id, long delay, object *ob, int mode);
@@ -621,12 +611,9 @@ void cftimer_init(void);
 /* weather.c */
 void set_darkness_map(mapstruct *m);
 void tick_the_clock(void);
-int similar_direction(int a, int b);
 /* server.c */
-void version(object *op);
-void start_info(object *op);
-char *crypt_string(const char *str, const char *salt);
-int check_password(const char *typed, const char *crypted);
+char const* newhash(char const *password);
+bool check_password(const char *typed, const char *crypted);
 void enter_player_savebed(object *op);
 void set_map_timeout(mapstruct *oldmap);
 void enter_exit(object *op, object *exit_ob);
@@ -635,4 +622,11 @@ void clean_tmp_files(void);
 void cleanup(void);
 void leave(player *pl, int draw_exit);
 int forbid_play(void);
-int server_main(int argc, char **argv);
+void server_main(int argc, char *argv[]);
+
+/**
+ * Move a player to its stored map level. This function is used to place the
+ * player just after logging in. Since the map may no longer exist, dump the
+ * player to an alternative 'emergency' location if that's the case.
+ */
+void enter_player_maplevel(object *op);
