@@ -32,7 +32,6 @@ void init_type_director(void);
 void init_type_dragon_focus(void);
 /* duplicator/duplicator.c */
 void init_type_duplicator(void);
-void move_duplicator(object *op);
 /* exit/exit.c */
 void init_type_exit(void);
 /* food/food.c */
@@ -49,7 +48,8 @@ void init_type_lamp(void);
 void init_type_lighter(void);
 /* marker/marker.c */
 void init_type_marker(void);
-void move_marker(object *op);
+/* mimic/mimic.c */
+void init_type_mimic(void);
 /* mood_floor/mood_floor.c */
 void init_type_mood_floor(void);
 /* peacemaker/peacemaker.c */
@@ -118,7 +118,7 @@ method_ret common_ob_move_on(ob_methods *context, object *trap, object *victim,
 method_ret common_pre_ob_move_on(object *trap, object *victim, object *originator);
 void common_post_ob_move_on(object *trap, object *victim, object *originator);
 /* common/describe.c */
-void common_ob_describe(const ob_methods *context, const object *op, const object *observer, char *buf, size_t size);
+void common_ob_describe(const ob_methods *context, const object *op, const object *observer, int use_media_tags, char *buf, size_t size);
 /* common/projectile.c */
 void stop_projectile(object *op);
 method_ret common_process_projectile(ob_methods *context, object *op);
@@ -126,6 +126,6 @@ method_ret common_projectile_move_on(ob_methods *context, object *trap, object *
 /* legacy/apply.c */
 method_ret legacy_ob_apply(ob_methods *context, object *op, object *applier, int aflags);
 /* legacy/legacy_describe.c */
-void legacy_ob_describe(const ob_methods *context, const object *op, const object *observer, char *buf, size_t size);
+void legacy_ob_describe(const ob_methods *context, const object *op, const object *observer, int use_media_tags, char *buf, size_t size);
 /* legacy/process.c */
 method_ret legacy_ob_process(ob_methods *context, object *op);
