@@ -30,9 +30,12 @@
 /* First let's include the header file needed                                */
 
 #include <assert.h>
-#include <cfanim.h>
 #include <stdarg.h>
-#include <svnversion.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "cfanim.h"
+#include "svnversion.h"
 
 CF_PLUGIN char SvnRevPlugin[] = SVN_REV;
 
@@ -238,14 +241,18 @@ static anim_move_result runapplyobject(struct CFanimation_struct *animation, lon
 }
 
 static long int initdropobject(const char *name, char *parameters, struct CFmovement_struct *move_entity) {
-    move_entity->parameters = parameters ? cf_strdup_local(parameters) : NULL;
+    move_entity->parameters = parameters ?
+        (void *)cf_add_string(parameters) : NULL;
     return 1;
 }
 
 static anim_move_result rundropobject(struct CFanimation_struct *animation, long int id, void *parameters) {
+    object *what;
     if (!parameters)
         return mr_finished;
-    cf_object_drop(animation->victim, parameters);
+    what = cf_object_find_by_name(animation->victim, parameters);
+    if (what != NULL)
+        cf_object_drop(what, animation->victim);
     cf_free_string(parameters);
     return mr_finished;
 }
@@ -347,10 +354,7 @@ static long int initteleport(const char *name, char *parameters, struct CFmoveme
     mapx = atoi(parameters);
     mapname++;
     parameters = mapname;
-    if (!parameters) {
-        cf_log(llevError, "CFAnim: Error - not enough parameters for teleport\n");
-        return 0;
-    }
+    assert(parameters != NULL);
     cf_log(llevDebug, ".(%s)\n", parameters);
     mapname = strstr(parameters, " ");
     cf_log(llevDebug, ".\n");
@@ -460,6 +464,50 @@ static anim_move_result runmessage(struct CFanimation_struct *animation, long in
     return mr_finished;
 }
 
+static long int inittrigger(const char *name, char *parameters, struct CFmovement_struct *move_entity) {
+    long int connection;
+
+    move_entity->parameters = NULL;
+    if (sscanf(parameters, "%ld", &connection) != 1 || connection <= 0) {
+        cf_log(llevError, "CFAnim: invalid connection %s\n", parameters);
+        return 0;
+    }
+    return connection;
+}
+
+static anim_move_result runtrigger(struct CFanimation_struct *animation, long int id, void *parameters) {
+    oblinkpt *olp;
+    mapstruct *map;
+    objectlink *ol = NULL;
+
+    if (animation->victim == NULL || animation->victim->map == NULL) {
+        cf_log(llevError, "CFAnim: trigger for victim not on map or NULL.\n");
+        return mr_finished;
+    }
+
+    map = animation->victim->map;
+
+    /* locate objectlink for this connected value */
+    if (!map->buttons) {
+        cf_log(llevError, "Map %s called for trigger on connected %d but there ain't any button list for that map!\n", cf_map_get_sstring_property(map, CFAPI_MAP_PROP_PATH), id);
+        return mr_finished;
+    }
+    for (olp = map->buttons; olp; olp = olp->next) {
+        if (olp->value == id) {
+            ol = olp->link;
+            break;
+        }
+    }
+    if (ol == NULL) {
+        cf_log(llevError, "Map %s called for trigger on connected %d but there ain't any button list for that map!\n", cf_map_get_sstring_property(map, CFAPI_MAP_PROP_PATH), id);
+        return mr_finished;
+    }
+    /* run the object link */
+    cf_map_trigger_connected(ol, NULL, 1);
+
+    return mr_finished;
+}
+
 /** Available animation commands. */
 CFanimationHook animationbox[] = {
     { "north", initmovement, runmovement },
@@ -507,7 +555,8 @@ CFanimationHook animationbox[] = {
     { "notice", initnotice, runnotice },
     { "stop", initstop, runstop },
     { "moveto", initmoveto, runmoveto },
-    { "message", initmessage, runmessage }
+    { "message", initmessage, runmessage },
+    { "trigger", inittrigger, runtrigger }
 };
 
 int animationcount = sizeof(animationbox)/sizeof(CFanimationHook);
@@ -586,11 +635,10 @@ static CFmovement *parse_animation_block(char *buffer, size_t buffer_size, FILE
                 parameters = NULL;
         }
         animationhook = get_command(name);
-        if (parent->verbose) {
-            if (!animationhook)
-                cf_log(llevError, "CFAnim: %s - Unknown animation command\n", name);
-            else
-                cf_log(llevDebug, "CFAnim: Parsed %s -> %p\n", name, animationhook);
+        if (!animationhook)
+            cf_log(llevError, "CFAnim: %s - Unknown animation command\n", name);
+        else if (parent->verbose) {
+            cf_log(llevDebug, "CFAnim: Parsed %s -> %p\n", name, animationhook);
         }
         if (!animationhook) {
             if (parent->errors_allowed)
@@ -658,17 +706,25 @@ static int equality_split(char *buffer, char **variable, char **value) {
  * @return 1 if strg was processed, 0 else.
  */
 static int get_boolean(const char *strg, int *bl) {
-    if (!strncmp(strg, "y", 1))
+    /*
+     * We're only parsing the first character, so we
+     * really don't need to call strncmp().
+     * Should future checks need to check multiple characters,
+     * strncmp() would be a good choice. For that reason,
+     * I won't optimize this down to a switch statement.
+     * It makes it clearer how to handle multi-character checks.
+     */
+    if (*strg == 'y')
         *bl = 1;
-    else if (!strncmp(strg, "n", 1))
+    else if (*strg == 'n')
         *bl = 0;
-    else if (!strncmp(strg, "Y", 1))
+    else if (*strg == 'Y')
         *bl = 1;
-    else if (!strncmp(strg, "N", 1))
+    else if (*strg == 'N')
         *bl = 0;
-    else if (!strncmp(strg, "1", 1))
+    else if (*strg == '1')
         *bl = 1;
-    else if (!strncmp(strg, "0", 1))
+    else if (*strg == '0')
         *bl = 0;
     else
         return 0;
@@ -774,7 +830,7 @@ static int start_animation(object *who, object *activator, object *event, const
     enum    time_enum timetype;
     int     errors_allowed = 0;
     int     verbose = 0;
-    const char *animationitem;
+    const char *animationitem = NULL;
     char    buffer[HUGE_BUF];
     char    *variable;
     char    *value;
@@ -796,10 +852,13 @@ static int start_animation(object *who, object *activator, object *event, const
         errors_found = 1;
         cf_log(llevError, "CFAnim: '%s' has an invalid syntax.\n", buffer);
     }
-    if (feof(fichier))
+    if (feof(fichier)) {
+        fclose(fichier);
         return 0;
+    }
     if (strncmp(buffer, "[Config]", 8)) {
         cf_log(llevError, "CFAnim: Fatal error in %s: [Config] must be the first group defined.\n", file);
+        fclose(fichier);
         return 0;
     }
     while (fgets(buffer, HUGE_BUF, fichier)) {
@@ -901,18 +960,26 @@ static int start_animation(object *who, object *activator, object *event, const
         if (animationitem)
             cf_free_string(animationitem);
         cf_log(llevError, "CFAnim: Errors occurred during the parsing of %s\n", file);
+        fclose(fichier);
+        return 0;
+    }
+    if (!animationitem) {
+        cf_log(llevError, "CFAnim: no animation specified when using %s\n", file);
+        fclose(fichier);
         return 0;
     }
     if (!victim) {
         if (animationitem)
             cf_free_string(animationitem);
         cf_log(llevError,  "CFAnim: Fatal error - victim is NULL");
+        fclose(fichier);
         return 0;
     }
     if (!(current_anim = create_animation())) {
         if (animationitem)
             cf_free_string(animationitem);
         cf_log(llevError, "CFAnim: Fatal error - Not enough memory.\n");
+        fclose(fichier);
         return 0;
     }
     if (always_delete) {
@@ -927,6 +994,8 @@ static int start_animation(object *who, object *activator, object *event, const
             cf_log(llevError, "CFAnim: No correct victim found or errors found, aborting.\n");
         if (animationitem)
             cf_free_string(animationitem);
+        free(current_anim);
+        fclose(fichier);
         return 0;
     }
     if (unique && !always_delete) {
@@ -957,6 +1026,7 @@ static int start_animation(object *who, object *activator, object *event, const
             if (value == NULL) {
                 cf_log(llevError, "CFAnim: no matching animation %s in file.\n", animationitem);
                 cf_free_string(animationitem);
+                fclose(fichier);
                 return 0;
             }
         }
@@ -1022,6 +1092,16 @@ static void animate_one(CFanimation *animation, long int milliseconds) {
     cf_object_set_flag(animation->victim, FLAG_WIZ, 0);
 }
 
+/**
+ * Return the number of microseconds between two timespec structures.
+ * This function was copied from common/time.c since linking is too hard.
+ */
+long usec_elapsed(struct timespec first, struct timespec second) {
+    time_t sec_elapsed = second.tv_sec - first.tv_sec;
+    long nsec_elapsed = second.tv_nsec - first.tv_nsec;
+    return (sec_elapsed * 1e6) + (nsec_elapsed / 1e3);
+}
+
 /**
  * Animates all currently running animations.
  */
@@ -1029,20 +1109,20 @@ static void animate(void) {
     CFanimation *current;
     CFanimation *next;
     CFanimation *previous_anim = NULL;
-    struct timeval now;
-    static struct timeval yesterday;
+    struct timespec now;
+    static struct timespec yesterday;
     static int already_passed = 0;
     long int delta_milli;
 
-    (void)GETTIMEOFDAY(&now);
+    clock_gettime(CLOCK_MONOTONIC, &now);
     if (!already_passed) {
         already_passed = 1;
-        memcpy(&yesterday, &now, sizeof(struct timeval));
+        yesterday = now;
         return;
     }
-    delta_milli = (now.tv_sec-yesterday.tv_sec)*1000+(now.tv_usec/1000-yesterday.tv_usec/1000);
+    delta_milli = usec_elapsed(yesterday, now) / 1e3;
     /*printf("Working for %ld milli seconds\n", delta_milli);*/
-    memcpy(&yesterday, &now, sizeof(struct timeval));
+    yesterday = now;
     for (current = first_animation; current; current = current->nextanimation)
         animate_one(current, delta_milli);
     current = first_animation;
@@ -1149,16 +1229,18 @@ CF_PLUGIN int eventListener(int *type, ...) {
     /*third =*/ va_arg(args, object *);
     buf = va_arg(args, char *);
 
-    if (buf != NULL)
-        strcpy(message, buf);
-    else
+    if (buf != NULL) {
+        snprintf(message, sizeof(message), "%s", buf);
+    } else {
         message[0] = '\0';
+    }
 
     query = va_arg(args, int); /* 'fix', ignored */
     event = va_arg(args, object *);
 
     if (query == 1 && strcmp(message, "query_object_is_animated") == 0) {
         rv = is_animated_object(who);
+        va_end(args);
         return rv;
     }
 
