@@ -33,9 +33,7 @@
 /** Full name. */
 #define PLUGIN_VERSION "Sqlite Logger Plugin 1.0 alpha"
 
-#ifndef __CEXTRACT__
 #include <plugin.h>
-#endif
 
 #undef MODULEAPI
 #ifdef WIN32
@@ -44,11 +42,13 @@
 # else
 #  define MODULEAPI __declspec(dllimport)
 # endif
-#elif HAVE_VISIBILITY
+#else
+#ifdef HAVE_VISIBILITY
 # define MODULEAPI __attribute__((visibility("default")))
 #else
 # define MODULEAPI
 #endif
+#endif
 
 #include <plugin_common.h>
 
