@@ -58,6 +58,7 @@
 /* First let's include the header file needed                                */
 
 #include <cfpython.h>
+#include <fcntl.h>
 #include <stdarg.h>
 #include <node.h>
 #include <svnversion.h>
@@ -93,56 +94,7 @@ static PythonCmd CustomCommand[NR_CUSTOM_CMD];
 /** Cached compiled scripts. */
 static pycode_cache_entry pycode_cache[PYTHON_CACHE_SIZE];
 
-static void set_exception(const char *fmt, ...);
-static PyObject *createCFObject(PyObject *self, PyObject *args);
-static PyObject *createCFObjectByName(PyObject *self, PyObject *args);
-static PyObject *getCFPythonVersion(PyObject *self, PyObject *args);
-static PyObject *getReturnValue(PyObject *self, PyObject *args);
-static PyObject *setReturnValue(PyObject *self, PyObject *args);
-static PyObject *matchString(PyObject *self, PyObject *args);
-static PyObject *findPlayer(PyObject *self, PyObject *args);
-static PyObject *readyMap(PyObject *self, PyObject *args);
-static PyObject *createMap(PyObject *self, PyObject *args);
-static PyObject *getMapDirectory(PyObject *self, PyObject *args);
-static PyObject *getUniqueDirectory(PyObject *self, PyObject *args);
-static PyObject *getTempDirectory(PyObject *self, PyObject *args);
-static PyObject *getConfigDirectory(PyObject *self, PyObject *args);
-static PyObject *getLocalDirectory(PyObject *self, PyObject *args);
-static PyObject *getPlayerDirectory(PyObject *self, PyObject *args);
-static PyObject *getDataDirectory(PyObject *self, PyObject *args);
-static PyObject *getWhoAmI(PyObject *self, PyObject *args);
-static PyObject *getWhoIsActivator(PyObject *self, PyObject *args);
-static PyObject *getWhoIsThird(PyObject *self, PyObject *args);
-static PyObject *getWhatIsMessage(PyObject *self, PyObject *args);
-static PyObject *getScriptName(PyObject *self, PyObject *args);
-static PyObject *getScriptParameters(PyObject *self, PyObject *args);
-static PyObject *getEvent(PyObject *self, PyObject *args);
-static PyObject *getPrivateDictionary(PyObject *self, PyObject *args);
-static PyObject *getSharedDictionary(PyObject *self, PyObject *args);
-static PyObject *getArchetypes(PyObject *self, PyObject *args);
-static PyObject *getPlayers(PyObject *self, PyObject *args);
-static PyObject *getMaps(PyObject *self, PyObject *args);
-static PyObject *getParties(PyObject *self, PyObject *args);
-static PyObject *getRegions(PyObject *self, PyObject *args);
-static PyObject *getFriendlyList(PyObject *self, PyObject *args);
-static PyObject *registerCommand(PyObject *self, PyObject *args);
-static PyObject *registerGEvent(PyObject *self, PyObject *args);
-static PyObject *unregisterGEvent(PyObject *self, PyObject *args);
 static PyObject *CFPythonError;
-static PyObject *getTime(PyObject *self, PyObject *args);
-static PyObject *destroyTimer(PyObject *self, PyObject *args);
-static PyObject *getMapHasBeenLoaded(PyObject *self, PyObject *args);
-static PyObject *findAnimation(PyObject *self, PyObject *args);
-static PyObject *log_message(PyObject *self, PyObject *args);
-static PyObject *findFace(PyObject *self, PyObject *args);
-static PyObject *getSeasonName(PyObject *self, PyObject *args);
-static PyObject *getMonthName(PyObject *self, PyObject *args);
-static PyObject *getWeekdayName(PyObject *self, PyObject *args);
-static PyObject *getPeriodofdayName(PyObject *self, PyObject *args);
-static PyObject *addReply(PyObject *self, PyObject *args);
-static PyObject *setPlayerMessage(PyObject *self, PyObject *args);
-static PyObject *npcSay(PyObject *self, PyObject *args);
-static PyObject *costStringFromValue(PyObject *self, PyObject *args);
 
 /** Set up an Python exception object. */
 static void set_exception(const char *fmt, ...) {
@@ -156,58 +108,6 @@ static void set_exception(const char *fmt, ...) {
     PyErr_SetString(PyExc_ValueError, buf);
 }
 
-static PyMethodDef CFPythonMethods[] = {
-    { "WhoAmI",              getWhoAmI,              METH_NOARGS,  NULL },
-    { "WhoIsActivator",      getWhoIsActivator,      METH_NOARGS,  NULL },
-    { "WhoIsOther",          getWhoIsThird,          METH_NOARGS,  NULL },
-    { "WhatIsMessage",       getWhatIsMessage,       METH_NOARGS,  NULL },
-    { "ScriptName",          getScriptName,          METH_NOARGS,  NULL },
-    { "ScriptParameters",    getScriptParameters,    METH_NOARGS,  NULL },
-    { "WhatIsEvent",         getEvent,               METH_NOARGS,  NULL },
-    { "MapDirectory",        getMapDirectory,        METH_NOARGS,  NULL },
-    { "UniqueDirectory",     getUniqueDirectory,     METH_NOARGS,  NULL },
-    { "TempDirectory",       getTempDirectory,       METH_NOARGS,  NULL },
-    { "ConfigDirectory",     getConfigDirectory,     METH_NOARGS,  NULL },
-    { "LocalDirectory",      getLocalDirectory,      METH_NOARGS,  NULL },
-    { "PlayerDirectory",     getPlayerDirectory,     METH_NOARGS,  NULL },
-    { "DataDirectory",       getDataDirectory,       METH_NOARGS,  NULL },
-    { "ReadyMap",            readyMap,               METH_VARARGS, NULL },
-    { "CreateMap",           createMap,              METH_VARARGS, NULL },
-    { "FindPlayer",          findPlayer,             METH_VARARGS, NULL },
-    { "MatchString",         matchString,            METH_VARARGS, NULL },
-    { "GetReturnValue",      getReturnValue,         METH_NOARGS,  NULL },
-    { "SetReturnValue",      setReturnValue,         METH_VARARGS, NULL },
-    { "PluginVersion",       getCFPythonVersion,     METH_NOARGS,  NULL },
-    { "CreateObject",        createCFObject,         METH_NOARGS,  NULL },
-    { "CreateObjectByName",  createCFObjectByName,   METH_VARARGS, NULL },
-    { "GetPrivateDictionary", getPrivateDictionary,  METH_NOARGS,  NULL },
-    { "GetSharedDictionary", getSharedDictionary,    METH_NOARGS,  NULL },
-    { "GetPlayers",          getPlayers,             METH_NOARGS,  NULL },
-    { "GetArchetypes",       getArchetypes,          METH_NOARGS,  NULL },
-    { "GetMaps",             getMaps,                METH_NOARGS,  NULL },
-    { "GetParties",          getParties,             METH_NOARGS,  NULL },
-    { "GetRegions",          getRegions,             METH_NOARGS,  NULL },
-    { "GetFriendlyList",     getFriendlyList,        METH_NOARGS,  NULL },
-    { "RegisterCommand",     registerCommand,        METH_VARARGS, NULL },
-    { "RegisterGlobalEvent", registerGEvent,         METH_VARARGS, NULL },
-    { "UnregisterGlobalEvent", unregisterGEvent,     METH_VARARGS, NULL },
-    { "GetTime",             getTime,                METH_NOARGS,  NULL },
-    { "DestroyTimer",        destroyTimer,           METH_VARARGS, NULL },
-    { "MapHasBeenLoaded",    getMapHasBeenLoaded,    METH_VARARGS, NULL },
-    { "Log",                 log_message,            METH_VARARGS, NULL },
-    { "FindFace",            findFace,               METH_VARARGS, NULL },
-    { "FindAnimation",       findAnimation,          METH_VARARGS, NULL },
-    { "GetSeasonName",       getSeasonName,          METH_VARARGS, NULL },
-    { "GetMonthName",        getMonthName,           METH_VARARGS, NULL },
-    { "GetWeekdayName",      getWeekdayName,         METH_VARARGS, NULL },
-    { "GetPeriodofdayName",  getPeriodofdayName,     METH_VARARGS, NULL },
-    { "AddReply",            addReply,               METH_VARARGS, NULL },
-    { "SetPlayerMessage",    setPlayerMessage,       METH_VARARGS, NULL },
-    { "NPCSay",              npcSay,                 METH_VARARGS, NULL },
-    { "CostStringFromValue", costStringFromValue,    METH_VARARGS, NULL },
-    { NULL, NULL, 0, NULL }
-};
-
 CFPContext *context_stack;
 
 CFPContext *current_context;
@@ -752,7 +652,7 @@ static PyObject *npcSay(PyObject *self, PyObject *args) {
 }
 
 static PyObject *costStringFromValue(PyObject *self, PyObject *args) {
-    uint64 value;
+    uint64_t value;
     char buf[2048];
     int largest_coin = 0;
 
@@ -763,6 +663,58 @@ static PyObject *costStringFromValue(PyObject *self, PyObject *args) {
     return Py_BuildValue("s", buf);
 }
 
+static PyMethodDef CFPythonMethods[] = {
+    { "WhoAmI",              getWhoAmI,              METH_NOARGS,  NULL },
+    { "WhoIsActivator",      getWhoIsActivator,      METH_NOARGS,  NULL },
+    { "WhoIsOther",          getWhoIsThird,          METH_NOARGS,  NULL },
+    { "WhatIsMessage",       getWhatIsMessage,       METH_NOARGS,  NULL },
+    { "ScriptName",          getScriptName,          METH_NOARGS,  NULL },
+    { "ScriptParameters",    getScriptParameters,    METH_NOARGS,  NULL },
+    { "WhatIsEvent",         getEvent,               METH_NOARGS,  NULL },
+    { "MapDirectory",        getMapDirectory,        METH_NOARGS,  NULL },
+    { "UniqueDirectory",     getUniqueDirectory,     METH_NOARGS,  NULL },
+    { "TempDirectory",       getTempDirectory,       METH_NOARGS,  NULL },
+    { "ConfigDirectory",     getConfigDirectory,     METH_NOARGS,  NULL },
+    { "LocalDirectory",      getLocalDirectory,      METH_NOARGS,  NULL },
+    { "PlayerDirectory",     getPlayerDirectory,     METH_NOARGS,  NULL },
+    { "DataDirectory",       getDataDirectory,       METH_NOARGS,  NULL },
+    { "ReadyMap",            readyMap,               METH_VARARGS, NULL },
+    { "CreateMap",           createMap,              METH_VARARGS, NULL },
+    { "FindPlayer",          findPlayer,             METH_VARARGS, NULL },
+    { "MatchString",         matchString,            METH_VARARGS, NULL },
+    { "GetReturnValue",      getReturnValue,         METH_NOARGS,  NULL },
+    { "SetReturnValue",      setReturnValue,         METH_VARARGS, NULL },
+    { "PluginVersion",       getCFPythonVersion,     METH_NOARGS,  NULL },
+    { "CreateObject",        createCFObject,         METH_NOARGS,  NULL },
+    { "CreateObjectByName",  createCFObjectByName,   METH_VARARGS, NULL },
+    { "GetPrivateDictionary", getPrivateDictionary,  METH_NOARGS,  NULL },
+    { "GetSharedDictionary", getSharedDictionary,    METH_NOARGS,  NULL },
+    { "GetPlayers",          getPlayers,             METH_NOARGS,  NULL },
+    { "GetArchetypes",       getArchetypes,          METH_NOARGS,  NULL },
+    { "GetMaps",             getMaps,                METH_NOARGS,  NULL },
+    { "GetParties",          getParties,             METH_NOARGS,  NULL },
+    { "GetRegions",          getRegions,             METH_NOARGS,  NULL },
+    { "GetFriendlyList",     getFriendlyList,        METH_NOARGS,  NULL },
+    { "RegisterCommand",     registerCommand,        METH_VARARGS, NULL },
+    { "RegisterGlobalEvent", registerGEvent,         METH_VARARGS, NULL },
+    { "UnregisterGlobalEvent", unregisterGEvent,     METH_VARARGS, NULL },
+    { "GetTime",             getTime,                METH_NOARGS,  NULL },
+    { "DestroyTimer",        destroyTimer,           METH_VARARGS, NULL },
+    { "MapHasBeenLoaded",    getMapHasBeenLoaded,    METH_VARARGS, NULL },
+    { "Log",                 log_message,            METH_VARARGS, NULL },
+    { "FindFace",            findFace,               METH_VARARGS, NULL },
+    { "FindAnimation",       findAnimation,          METH_VARARGS, NULL },
+    { "GetSeasonName",       getSeasonName,          METH_VARARGS, NULL },
+    { "GetMonthName",        getMonthName,           METH_VARARGS, NULL },
+    { "GetWeekdayName",      getWeekdayName,         METH_VARARGS, NULL },
+    { "GetPeriodofdayName",  getPeriodofdayName,     METH_VARARGS, NULL },
+    { "AddReply",            addReply,               METH_VARARGS, NULL },
+    { "SetPlayerMessage",    setPlayerMessage,       METH_VARARGS, NULL },
+    { "NPCSay",              npcSay,                 METH_VARARGS, NULL },
+    { "CostStringFromValue", costStringFromValue,    METH_VARARGS, NULL },
+    { NULL, NULL, 0, NULL }
+};
+
 static void initContextStack(void) {
     current_context = NULL;
     context_stack = NULL;
@@ -849,9 +801,17 @@ static void log_python_error(void) {
 
     if (catcher != NULL) {
         PyObject *output = PyObject_GetAttrString(catcher, "value"); //get the stdout and stderr from our catchOutErr object
+#ifdef IS_PY3K
+        PyObject* empty = PyUnicode_FromString("");
+#else
         PyObject* empty = PyString_FromString("");
+#endif
 
+#ifdef IS_PY3K
+        cf_log_plain(llevError, PyUnicode_AsUTF8(output));
+#else
         cf_log_plain(llevError, PyString_AsString(output));
+#endif
         Py_DECREF(output);
 
         PyObject_SetAttrString(catcher, "value", empty);
@@ -947,11 +907,9 @@ static PyCodeObject *compilePython(char *filename) {
         Py_DECREF(scriptfile);
     }
 
-    if (run) {
-        run->used_time = time(NULL);
-        return run->code;
-    } else
-        return NULL;
+    assert(run != NULL);
+    run->used_time = time(NULL);
+    return run->code;
 }
 
 static int do_script(CFPContext *context, int silent) {
@@ -968,7 +926,11 @@ static int do_script(CFPContext *context, int silent) {
         pushContext(context);
         dict = PyDict_New();
         PyDict_SetItemString(dict, "__builtins__", PyEval_GetBuiltins());
+#ifdef IS_PY3K
+        ret = PyEval_EvalCode((PyObject *)pycode, dict, NULL);
+#else
         ret = PyEval_EvalCode(pycode, dict, NULL);
+#endif
         if (PyErr_Occurred()) {
             log_python_error();
         }
@@ -1001,8 +963,7 @@ static void addConstants(PyObject *module, const char *name, const CFConstant *c
     PyObject *new;
     PyObject *dict;
 
-    strncpy(tmp, "Crossfire_", sizeof(tmp));
-    strncat(tmp, name, sizeof(tmp)-strlen(tmp));
+    snprintf(tmp, sizeof(tmp), "Crossfire_%s", name);
 
     new = PyModule_New(tmp);
     dict = PyDict_New();
@@ -1018,8 +979,7 @@ static void addConstants(PyObject *module, const char *name, const CFConstant *c
     }
     PyDict_SetItemString(PyModule_GetDict(module), name, new);
 
-    strncpy(tmp, name, sizeof(tmp));
-    strncat(tmp, "Name", sizeof(tmp)-strlen(tmp));
+    snprintf(tmp, sizeof(tmp), "%sName", name);
     PyDict_SetItemString(PyModule_GetDict(module), tmp, dict);
     Py_DECREF(dict);
 }
@@ -1034,8 +994,7 @@ static void addSimpleConstants(PyObject *module, const char *name, const CFConst
     char tmp[1024];
     PyObject *new;
 
-    strncpy(tmp, "Crossfire_", sizeof(tmp));
-    strncat(tmp, name, sizeof(tmp)-strlen(tmp));
+    snprintf(tmp, sizeof(tmp), "Crossfire_%s", name);
 
     new = PyModule_New(tmp);
 
@@ -1203,16 +1162,6 @@ static void initConstants(PyObject *module) {
         { NULL, 0 }
     };
 
-    static const CFConstant cstCostFlag[] = {
-        { "TRUE", F_TRUE },
-        { "BUY", F_BUY },
-        { "SELL", F_SELL },
-        { "NOBARGAIN", F_NO_BARGAIN },
-        { "IDENTIFIED", F_IDENTIFIED },
-        { "NOTCURSED", F_NOT_CURSED },
-        { NULL, 0 }
-    };
-
     static const CFConstant cstAttackType[] = {
         { "PHYSICAL", AT_PHYSICAL },
         { "MAGIC", AT_MAGIC },
@@ -1274,37 +1223,43 @@ static void initConstants(PyObject *module) {
     };
 
     static const CFConstant cstEventType[] = {
+        /** Object_specific events. */
         { "APPLY", EVENT_APPLY },
-        { "ATTACK", EVENT_ATTACK },
+        { "ATTACK", EVENT_ATTACKED },
+        { "ATTACKS", EVENT_ATTACKS },
+        { "BOUGHT", EVENT_BOUGHT },
+        { "CLOSE", EVENT_CLOSE },
         { "DEATH", EVENT_DEATH },
+        { "DESTROY", EVENT_DESTROY },
         { "DROP", EVENT_DROP },
         { "PICKUP", EVENT_PICKUP },
         { "SAY", EVENT_SAY },
+        { "SELLING", EVENT_SELLING },
         { "STOP", EVENT_STOP },
         { "TIME", EVENT_TIME },
         { "THROW", EVENT_THROW },
         { "TRIGGER", EVENT_TRIGGER },
-        { "CLOSE", EVENT_CLOSE },
         { "TIMER", EVENT_TIMER },
-        { "DESTROY", EVENT_DESTROY },
+        { "USER", EVENT_USER },
+
+        /** Global events. */
         { "BORN", EVENT_BORN },
         { "CLOCK", EVENT_CLOCK },
         { "CRASH", EVENT_CRASH },
-        { "PLAYER_DEATH", EVENT_PLAYER_DEATH },
         { "GKILL", EVENT_GKILL },
+        { "KICK", EVENT_KICK },
         { "LOGIN", EVENT_LOGIN },
         { "LOGOUT", EVENT_LOGOUT },
         { "MAPENTER", EVENT_MAPENTER },
         { "MAPLEAVE", EVENT_MAPLEAVE },
+        { "MAPLOAD", EVENT_MAPLOAD },
         { "MAPRESET", EVENT_MAPRESET },
+        { "MAPUNLOAD", EVENT_MAPUNLOAD },
+        { "MUZZLE", EVENT_MUZZLE },
+        { "PLAYER_DEATH", EVENT_PLAYER_DEATH },
         { "REMOVE", EVENT_REMOVE },
         { "SHOUT", EVENT_SHOUT },
         { "TELL", EVENT_TELL },
-        { "MUZZLE", EVENT_MUZZLE },
-        { "KICK", EVENT_KICK },
-        { "MAPUNLOAD", EVENT_MAPUNLOAD },
-        { "MAPLOAD", EVENT_MAPLOAD },
-        { "USER", EVENT_USER },
         { NULL, 0 }
     };
 
@@ -1325,16 +1280,37 @@ static void initConstants(PyObject *module) {
         { NULL, 0 }
     };
 
+    static const CFConstant cstAttackMovement[] = {
+        { "DISTATT", DISTATT },
+        { "RUNATT", RUNATT },
+        { "HITRUN", HITRUN },
+        { "WAITATT", WAITATT },
+        { "RUSH", RUSH },
+        { "ALLRUN", ALLRUN },
+        { "DISTHIT", DISTHIT },
+        { "WAIT2", WAIT2 },
+        { "PETMOVE", PETMOVE },
+        { "CIRCLE1", CIRCLE1 },
+        { "CIRCLE2", CIRCLE2 },
+        { "PACEH", PACEH },
+        { "PACEH2", PACEH2 },
+        { "RANDO", RANDO },
+        { "RANDO2", RANDO2 },
+        { "PACEV", PACEV },
+        { "PACEV2", PACEV2 },
+        { NULL, 0 }
+    };
+
     addConstants(module, "Direction", cstDirection);
     addConstants(module, "Type", cstType);
     addConstants(module, "Move", cstMove);
     addConstants(module, "MessageFlag", cstMessageFlag);
-    addConstants(module, "CostFlag", cstCostFlag);
     addConstants(module, "AttackType", cstAttackType);
     addConstants(module, "AttackTypeNumber", cstAttackTypeNumber);
     addConstants(module, "EventType", cstEventType);
     addSimpleConstants(module, "Time", cstTime);
     addSimpleConstants(module, "ReplyType", cstReplyTypes);
+    addSimpleConstants(module, "AttackMovement", cstAttackMovement);
 }
 
 /*
@@ -1536,6 +1512,8 @@ CF_PLUGIN void cfpython_runPluginCommand(object *op, const char *params) {
     context->activator   = NULL;
     context->third       = NULL;
     context->fix         = 0;
+    /* We are not running from an event, so set it to NULL to avoid segfaults. */
+    context->event       = NULL;
     snprintf(context->script, sizeof(context->script), "%s", buf);
     if (params)
         snprintf(context->options, sizeof(context->options), "%s", params);
@@ -1554,6 +1532,46 @@ CF_PLUGIN void cfpython_runPluginCommand(object *op, const char *params) {
 /*    printf("Execution complete"); */
 }
 
+static int GECodes[] = {
+    EVENT_BORN,
+    EVENT_CLOCK,
+    EVENT_PLAYER_DEATH,
+    EVENT_GKILL,
+    EVENT_LOGIN,
+    EVENT_LOGOUT,
+    EVENT_MAPENTER,
+    EVENT_MAPLEAVE,
+    EVENT_MAPRESET,
+    EVENT_REMOVE,
+    EVENT_SHOUT,
+    EVENT_TELL,
+    EVENT_MUZZLE,
+    EVENT_KICK,
+    EVENT_MAPUNLOAD,
+    EVENT_MAPLOAD,
+    0  
+};
+
+static const char* GEPaths[] = {
+    "born",
+    "clock",
+    "death",
+    "gkill",
+    "login",
+    "logout",
+    "mapenter",
+    "mapleave",
+    "mapreset",
+    "remove",
+    "shout",
+    "tell",
+    "muzzle",
+    "kick",
+    "mapunload",
+    "mapload",
+    NULL  
+};
+
 CF_PLUGIN int postInitPlugin(void) {
     PyObject *scriptfile;
     char path[1024];
@@ -1561,23 +1579,8 @@ CF_PLUGIN int postInitPlugin(void) {
 
     cf_log(llevDebug, "CFPython 2.0a post init\n");
     initContextStack();
-    cf_system_register_global_event(EVENT_BORN, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_CLOCK, PLUGIN_NAME, cfpython_globalEventListener);
-    /*registerGlobalEvent(NULL, EVENT_CRASH, PLUGIN_NAME, cfpython_globalEventListener);*/
-    cf_system_register_global_event(EVENT_PLAYER_DEATH, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_GKILL, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_LOGIN, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_LOGOUT, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_MAPENTER, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_MAPLEAVE, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_MAPRESET, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_REMOVE, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_SHOUT, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_TELL, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_MUZZLE, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_KICK, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_MAPUNLOAD, PLUGIN_NAME, cfpython_globalEventListener);
-    cf_system_register_global_event(EVENT_MAPLOAD, PLUGIN_NAME, cfpython_globalEventListener);
+    for (i = 0; GECodes[i] != 0; i++)
+        cf_system_register_global_event(GECodes[i], PLUGIN_NAME, cfpython_globalEventListener);
 
     scriptfile = cfpython_openpyfile(cf_get_maps_directory("python/events/python_init.py", path, sizeof(path)));
     if (scriptfile != NULL) {
@@ -1596,6 +1599,14 @@ CF_PLUGIN int postInitPlugin(void) {
     return 0;
 }
 
+static const char *getGlobalEventPath(int code) {
+    for (int i = 0; GECodes[i] != 0; i++) {
+        if (GECodes[i] == code)
+            return GEPaths[i];
+    }
+    return "";
+}
+
 CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
     va_list args;
     int rv = 0;
@@ -1617,7 +1628,7 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
     context->talk        = NULL;
     rv = context->returnvalue = 0;
     cf_get_maps_directory("python/events/python_event.py", context->script, sizeof(context->script));
-    strcpy(context->options, "");
+    snprintf(context->options, sizeof(context->options), "%s", getGlobalEventPath(context->event_code));
     switch (context->event_code) {
     case EVENT_CRASH:
         cf_log(llevDebug, "Unimplemented for now\n");
@@ -1626,7 +1637,6 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
     case EVENT_BORN:
         op = va_arg(args, object *);
         context->activator = Crossfire_Object_wrap(op);
-        snprintf(context->options, sizeof(context->options), "born");
         break;
 
     case EVENT_PLAYER_DEATH:
@@ -1634,14 +1644,13 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
         context->who = Crossfire_Object_wrap(op);
         op = va_arg(args, object *);
         context->activator = Crossfire_Object_wrap(op);
-        snprintf(context->options, sizeof(context->options), "death");
         break;
 
     case EVENT_GKILL:
         op = va_arg(args, object *);
+        object* hitter = va_arg(args, object *);
         context->who = Crossfire_Object_wrap(op);
-        context->activator = Crossfire_Object_wrap(op);
-        snprintf(context->options, sizeof(context->options), "gkill");
+        context->activator = Crossfire_Object_wrap(hitter);
         break;
 
     case EVENT_LOGIN:
@@ -1650,7 +1659,6 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
         buf = va_arg(args, char *);
         if (buf != NULL)
             snprintf(context->message, sizeof(context->message), "%s", buf);
-        snprintf(context->options, sizeof(context->options), "login");
         break;
 
     case EVENT_LOGOUT:
@@ -1659,13 +1667,11 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
         buf = va_arg(args, char *);
         if (buf != NULL)
             snprintf(context->message, sizeof(context->message), "%s", buf);
-        snprintf(context->options, sizeof(context->options), "logout");
         break;
 
     case EVENT_REMOVE:
         op = va_arg(args, object *);
         context->activator = Crossfire_Object_wrap(op);
-        snprintf(context->options, sizeof(context->options), "remove");
         break;
 
     case EVENT_SHOUT:
@@ -1674,7 +1680,6 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
         buf = va_arg(args, char *);
         if (buf != NULL)
             snprintf(context->message, sizeof(context->message), "%s", buf);
-        snprintf(context->options, sizeof(context->options), "shout");
         break;
 
     case EVENT_MUZZLE:
@@ -1683,7 +1688,6 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
         buf = va_arg(args, char *);
         if (buf != NULL)
             snprintf(context->message, sizeof(context->message), "%s", buf);
-        snprintf(context->options, sizeof(context->options), "muzzle");
         break;
 
     case EVENT_KICK:
@@ -1692,30 +1696,25 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
         buf = va_arg(args, char *);
         if (buf != NULL)
             snprintf(context->message, sizeof(context->message), "%s", buf);
-        snprintf(context->options, sizeof(context->options), "kick");
         break;
 
     case EVENT_MAPENTER:
         op = va_arg(args, object *);
         context->activator = Crossfire_Object_wrap(op);
         context->who = Crossfire_Map_wrap(va_arg(args, mapstruct *));
-        snprintf(context->options, sizeof(context->options), "mapenter");
         break;
 
     case EVENT_MAPLEAVE:
         op = va_arg(args, object *);
         context->activator = Crossfire_Object_wrap(op);
         context->who = Crossfire_Map_wrap(va_arg(args, mapstruct *));
-        snprintf(context->options, sizeof(context->options), "mapleave");
         break;
 
     case EVENT_CLOCK:
-        snprintf(context->options, sizeof(context->options), "clock");
         break;
 
     case EVENT_MAPRESET:
         context->who = Crossfire_Map_wrap(va_arg(args, mapstruct *));
-        snprintf(context->options, sizeof(context->options), "mapreset");
         break;
 
     case EVENT_TELL:
@@ -1726,22 +1725,26 @@ CF_PLUGIN int cfpython_globalEventListener(int *type, ...) {
             snprintf(context->message, sizeof(context->message), "%s", buf);
         op = va_arg(args, object *);
         context->third = Crossfire_Object_wrap(op);
-        snprintf(context->options, sizeof(context->options), "tell");
         break;
 
     case EVENT_MAPUNLOAD:
         context->who = Crossfire_Map_wrap(va_arg(args, mapstruct *));
-        snprintf(context->options, sizeof(context->options), "mapunload");
         break;
 
     case EVENT_MAPLOAD:
         context->who = Crossfire_Map_wrap(va_arg(args, mapstruct *));
-        snprintf(context->options, sizeof(context->options), "mapload");
         break;
     }
     va_end(args);
     context->returnvalue = 0;
 
+    if (context->event_code == EVENT_CLOCK) {
+        // Ignore EVENT_CLOCK. It is not being used in maps, but nevertheless
+        // runs python_init.py several times per second even while idling.
+        freeContext(context);
+        return rv;
+    }
+
     if (!do_script(context, 1)) {
         freeContext(context);
         return rv;
