@@ -28,7 +28,6 @@
 /*****************************************************************************/
 
 #include <cfpython.h>
-#include <cfpython_map_private.h>
 #include <hashtable.h>
 
 /* Table for keeping track of which PyObject goes with with Crossfire object */
@@ -129,12 +128,12 @@ static PyObject *Map_GetHeight(Crossfire_Map *whoptr, void *closure) {
 
 static PyObject *Map_GetEnterX(Crossfire_Map *whoptr, void *closure) {
     MAPEXISTCHECK(whoptr);
-    return Py_BuildValue("i", cf_map_get_int_property(whoptr->map, CFAPI_MAP_PROP_ENTER_X));
+    return Py_BuildValue("i", cf_map_get_enter_x(whoptr->map));
 }
 
 static PyObject *Map_GetEnterY(Crossfire_Map *whoptr, void *closure) {
     MAPEXISTCHECK(whoptr);
-    return Py_BuildValue("i", cf_map_get_enter_x(whoptr->map));
+    return Py_BuildValue("i", cf_map_get_enter_y(whoptr->map));
 }
 
 static PyObject *Map_GetMessage(Crossfire_Map *whoptr, void *closure) {
@@ -219,7 +218,7 @@ static PyObject *Map_Check(Crossfire_Map *map, PyObject *args) {
     char *what;
     int x, y;
     object *foundob;
-    sint16 nx, ny;
+    int16_t nx, ny;
     int mflags;
 
     if (!PyArg_ParseTuple(args, "s(ii)", &what, &x, &y))
@@ -230,7 +229,7 @@ static PyObject *Map_Check(Crossfire_Map *map, PyObject *args) {
     /* make sure the map is swapped in */
     ensure_map_in_memory(map);
 
-    mflags = cf_map_get_flags(map->map, &(map->map), (sint16)x, (sint16)y, &nx, &ny);
+    mflags = cf_map_get_flags(map->map, &(map->map), (int16_t)x, (int16_t)y, &nx, &ny);
     if (mflags&P_OUT_OF_MAP) {
         Py_INCREF(Py_None);
         return Py_None;
@@ -448,3 +447,162 @@ PyObject *Crossfire_Map_wrap(mapstruct *what) {
 
     return (PyObject *)wrapper;
 }
+
+/* Python binding */
+static PyGetSetDef Map_getseters[] = {
+    { "Difficulty",      (getter)Map_GetDifficulty,  NULL, NULL, NULL },
+    { "Path",            (getter)Map_GetPath,        (setter)Map_SetPath, NULL, NULL },
+    { "TempName",        (getter)Map_GetTempName,    NULL, NULL, NULL },
+    { "Name",            (getter)Map_GetName,        NULL, NULL, NULL },
+    { "ResetTime",       (getter)Map_GetResetTime,   NULL, NULL, NULL },
+    { "ResetTimeout",    (getter)Map_GetResetTimeout, NULL, NULL, NULL },
+    { "Players",         (getter)Map_GetPlayers,     NULL, NULL, NULL },
+    { "Light",           (getter)Map_GetDarkness,    NULL, NULL, NULL },
+    { "Darkness",        (getter)Map_GetDarkness,    NULL, NULL, NULL },
+    { "Width",           (getter)Map_GetWidth,       NULL, NULL, NULL },
+    { "Height",          (getter)Map_GetHeight,      NULL, NULL, NULL },
+    { "EnterX",          (getter)Map_GetEnterX,      NULL, NULL, NULL },
+    { "EnterY",          (getter)Map_GetEnterY,      NULL, NULL, NULL },
+    { "Message",         (getter)Map_GetMessage,     NULL, NULL, NULL },
+    { "Region",          (getter)Map_GetRegion,      NULL, NULL, NULL },
+    { "Unique",          (getter)Map_GetUnique,      NULL, NULL, NULL },
+    {  NULL, NULL, NULL, NULL, NULL }
+};
+
+static PyMethodDef MapMethods[] = {
+    { "Print",            (PyCFunction)Map_Message,          METH_VARARGS, NULL },
+    { "ObjectAt",         (PyCFunction)Map_GetFirstObjectAt, METH_VARARGS, NULL },
+    { "CreateObject",     (PyCFunction)Map_CreateObject,     METH_VARARGS, NULL },
+    { "Check",            (PyCFunction)Map_Check,            METH_VARARGS, NULL },
+    { "Next",             (PyCFunction)Map_Next,             METH_NOARGS,  NULL },
+    { "Insert",           (PyCFunction)Map_Insert,           METH_VARARGS, NULL },
+    { "InsertAround",     (PyCFunction)Map_InsertAround,     METH_VARARGS, NULL },
+    { "ChangeLight",      (PyCFunction)Map_ChangeLight,      METH_VARARGS, NULL },
+    { "TriggerConnected", (PyCFunction)Map_TriggerConnected, METH_VARARGS, NULL },
+    { NULL, NULL, 0, NULL }
+};
+
+static PyNumberMethods MapConvert = {
+    NULL,            /* binaryfunc nb_add; */        /* __add__ */
+    NULL,            /* binaryfunc nb_subtract; */   /* __sub__ */
+    NULL,            /* binaryfunc nb_multiply; */   /* __mul__ */
+#ifndef IS_PY3K
+    NULL,            /* binaryfunc nb_divide; */     /* __div__ */
+#endif
+    NULL,            /* binaryfunc nb_remainder; */  /* __mod__ */
+    NULL,            /* binaryfunc nb_divmod; */     /* __divmod__ */
+    NULL,            /* ternaryfunc nb_power; */     /* __pow__ */
+    NULL,            /* unaryfunc nb_negative; */    /* __neg__ */
+    NULL,            /* unaryfunc nb_positive; */    /* __pos__ */
+    NULL,            /* unaryfunc nb_absolute; */    /* __abs__ */
+#ifdef IS_PY3K
+    NULL,            /* inquiry nb_bool; */          /* __bool__ */
+#else
+    NULL,            /* inquiry nb_nonzero; */       /* __nonzero__ */
+#endif
+    NULL,            /* unaryfunc nb_invert; */      /* __invert__ */
+    NULL,            /* binaryfunc nb_lshift; */     /* __lshift__ */
+    NULL,            /* binaryfunc nb_rshift; */     /* __rshift__ */
+    NULL,            /* binaryfunc nb_and; */        /* __and__ */
+    NULL,            /* binaryfunc nb_xor; */        /* __xor__ */
+    NULL,            /* binaryfunc nb_or; */         /* __or__ */
+#ifndef IS_PY3K
+    NULL,            /* coercion nb_coerce; */       /* __coerce__ */
+#endif
+#ifdef IS_PY3K
+    /* This is not a typo. For Py3k it should be Crossfire_Map_Long
+     * and NOT Crossfire_Map_Int.
+     */
+    Crossfire_Map_Long, /* unaryfunc nb_int; */      /* __int__ */
+    NULL,               /* void *nb_reserved; */
+#else
+    Crossfire_Map_Int,  /* unaryfunc nb_int; */      /* __int__ */
+    Crossfire_Map_Long, /* unaryfunc nb_long; */     /* __long__ */
+#endif
+    NULL,            /* unaryfunc nb_float; */       /* __float__ */
+#ifndef IS_PY3K
+    NULL,            /* unaryfunc nb_oct; */         /* __oct__ */
+    NULL,            /* unaryfunc nb_hex; */         /* __hex__ */
+#endif
+    NULL,            /* binaryfunc nb_inplace_add; */
+    NULL,            /* binaryfunc nb_inplace_subtract; */
+    NULL,            /* binaryfunc nb_inplace_multiply; */
+#ifndef IS_PY3K
+    NULL,            /* binaryfunc nb_inplace_divide; */
+#endif
+    NULL,            /* binaryfunc nb_inplace_remainder; */
+    NULL,            /* ternaryfunc nb_inplace_power; */
+    NULL,            /* binaryfunc nb_inplace_lshift; */
+    NULL,            /* binaryfunc nb_inplace_rshift; */
+    NULL,            /* binaryfunc nb_inplace_and; */
+    NULL,            /* binaryfunc nb_inplace_xor; */
+    NULL,            /* binaryfunc nb_inplace_or; */
+
+    NULL,            /* binaryfunc nb_floor_divide; */
+    NULL,            /* binaryfunc nb_true_divide; */
+    NULL,            /* binaryfunc nb_inplace_floor_divide; */
+    NULL,            /* binaryfunc nb_inplace_true_divide; */
+#if defined(IS_PY25) || defined(IS_PY3K)
+    NULL             /* unaryfunc nb_index; */
+#endif
+};
+
+/* Our actual Python MapType */
+PyTypeObject Crossfire_MapType = {
+#ifdef IS_PY3K
+    /* See http://bugs.python.org/issue4385 */
+    PyVarObject_HEAD_INIT(NULL, 0)
+#else
+    PyObject_HEAD_INIT(NULL)
+    0,                         /* ob_size*/
+#endif
+    "Crossfire.Map",           /* tp_name*/
+    sizeof(Crossfire_Map),     /* tp_basicsize*/
+    0,                         /* tp_itemsize*/
+    Crossfire_Map_dealloc,     /* tp_dealloc*/
+    NULL,                      /* tp_print*/
+    NULL,                      /* tp_getattr*/
+    NULL,                      /* tp_setattr*/
+#ifdef IS_PY3K
+    NULL,                      /* tp_reserved */
+#else
+    (cmpfunc)Map_InternalCompare, /* tp_compare*/
+#endif
+    NULL,                      /* tp_repr*/
+    &MapConvert,               /* tp_as_number*/
+    NULL,                      /* tp_as_sequence*/
+    NULL,                      /* tp_as_mapping*/
+    PyObject_HashNotImplemented, /* tp_hash */
+    NULL,                      /* tp_call*/
+    NULL,                      /* tp_str*/
+    PyObject_GenericGetAttr,   /* tp_getattro*/
+    PyObject_GenericSetAttr,   /* tp_setattro*/
+    NULL,                      /* tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE, /* tp_flags*/
+    "Crossfire maps",          /* tp_doc */
+    NULL,                      /* tp_traverse */
+    NULL,                      /* tp_clear */
+    (richcmpfunc)Crossfire_Map_RichCompare, /* tp_richcompare */
+    0,                         /* tp_weaklistoffset */
+    NULL,                      /* tp_iter */
+    NULL,                      /* tp_iternext */
+    MapMethods,                /* tp_methods */
+    NULL,                      /* tp_members */
+    Map_getseters,             /* tp_getset */
+    NULL,                      /* tp_base */
+    NULL,                      /* tp_dict */
+    NULL,                      /* tp_descr_get */
+    NULL,                      /* tp_descr_set */
+    0,                         /* tp_dictoffset */
+    NULL,                      /* tp_init */
+    NULL,                      /* tp_alloc */
+    Crossfire_Map_new,         /* tp_new */
+    NULL,                      /* tp_free */
+    NULL,                      /* tp_is_gc */
+    NULL,                      /* tp_bases */
+    NULL,                      /* tp_mro */
+    NULL,                      /* tp_cache */
+    NULL,                      /* tp_subclasses */
+    NULL,                      /* tp_weaklist */
+    NULL,                      /* tp_del */
+};
