@@ -49,11 +49,13 @@
 
 #include <stdarg.h>
 #include <assert.h>
+#include <string.h>
 
 #include <cfrhg.h>
-#ifndef __CEXTRACT__
 #include <cfrhg_proto.h>
-#endif
+#include <svnversion.h>
+
+CF_PLUGIN char SvnRevPlugin[] = SVN_REV;
 
 /** Link between a map and the exits to generate for it. */
 typedef struct house_zone_struct {
