@@ -31,9 +31,7 @@
 #define PLUGIN_NAME "citylife"
 #define PLUGIN_VERSION "City life version 0.2"
 
-#ifndef __CEXTRACT__
 #include <plugin.h>
-#endif
 
 #undef MODULEAPI
 #ifdef WIN32
@@ -42,11 +40,13 @@
 # else
 #  define MODULEAPI __declspec(dllimport)
 # endif
-#elif HAVE_VISIBILITY
+#else
+#ifdef HAVE_VISIBILITY
 # define MODULEAPI __attribute__((visibility("default")))
 #else
 # define MODULEAPI
 #endif
+#endif
 
 #include <plugin_common.h>
 
