@@ -66,7 +66,7 @@ extern const char  *cf_get_season_name(int index);
 extern const char  *cf_get_month_name(int index);
 extern const char  *cf_get_weekday_name(int index);
 extern const char  *cf_get_periodofday_name(int index);
-extern void         cf_cost_string_from_value(uint64 cost, int largest_coin, char *buffer, int length);
+extern void         cf_cost_string_from_value(uint64_t cost, int largest_coin, char *buffer, int length);
 
 /* Objects */
 extern void         cf_object_set_int_property(object *op, int propcode, int value);
@@ -81,8 +81,8 @@ extern void         cf_object_set_float_property(object *op, int propcode, float
 extern mapstruct   *cf_object_get_map_property(object *op, int propcode);
 extern archetype   *cf_object_get_archetype_property(object *op, int propcode);
 extern partylist   *cf_object_get_partylist_property(object *op, int propcode);
-extern sint64       cf_object_get_int64_property(object *op, int propcode);
-extern void         cf_object_set_int64_property(object *op, int propcode, sint64 value);
+extern int64_t       cf_object_get_int64_property(object *op, int propcode);
+extern void         cf_object_set_int64_property(object *op, int propcode, int64_t value);
 extern double       cf_object_get_double_property(object *op, int propcode);
 extern sstring      cf_object_get_sstring_property(object *op, int propcode);
 extern char        *cf_object_get_string_property(object *op, int propcode, char *buf, int size);
@@ -110,8 +110,7 @@ extern int          cf_object_cast_spell(object *op, object *caster, int dir, ob
 extern void         cf_object_learn_spell(object *op, object *spell, int special_prayer);
 extern void         cf_object_forget_spell(object *op, object *sp);
 extern object      *cf_object_check_for_spell(object *op, const char *name);
-extern int          cf_object_cast_ability(object *caster, object *ctoo, int dir, object *sp, char *flags);
-extern int          cf_object_pay_amount(object *pl, uint64 to_pay);
+extern int          cf_object_pay_amount(object *pl, uint64_t to_pay);
 extern int          cf_object_pay_item(object *op, object *pl);
 extern void         cf_object_set_long_property(object *op, int propcode, long value);
 extern int          cf_object_transfer(object *op, int x, int y, int randomly, object *originator);
@@ -132,22 +131,23 @@ extern void         cf_object_update(object *op, int flags);
 extern void         cf_object_pickup(object *op, object *what);
 extern const char  *cf_object_get_key(object *op, const char *keyname);
 extern int          cf_object_set_key(object *op, const char *keyname, const char *value, int add_key);
-extern sint16       cf_object_get_resistance(object *op, int rtype);
-extern void         cf_object_set_resistance(object *op, int rtype, sint16 value);
+extern int16_t       cf_object_get_resistance(object *op, int rtype);
+extern void         cf_object_set_resistance(object *op, int rtype, int16_t value);
 extern int          cf_object_move(object *op, int dir, object*originator);
 extern void         cf_object_apply_below(object *pl);
 extern object      *cf_object_clone(object *op, int clonetype);
-extern void         cf_object_change_exp(object *op, sint64 exp, const char *skill_name, int flag);
+extern void         cf_object_change_exp(object *op, int64_t exp, const char *skill_name, int flag);
 extern int          cf_object_change_abil(object *op, object *tmp);
 extern int          cf_object_user_event(object *op, object *activator, object *third, const char *message, int fix);
 extern int          cf_object_remove_depletion(object *op, int level);
 extern object      *cf_object_find_by_arch_name(const object *who, const char *name);
 extern object      *cf_object_find_by_name(const object *who, const char *name);
-extern object      *cf_object_split(object *orig_ob, uint32 nr, char *err, size_t size);
+extern object      *cf_object_split(object *orig_ob, uint32_t nr, char *err, size_t size);
 extern object      *cf_object_clone(object *op, int clonetype);
 extern int          cf_object_set_face(object *op, const char *face);
 extern int          cf_object_set_animation(object *op, const char *animation);
 extern object      *cf_identify(object *op);
+extern int64_t      cf_object_perm_exp(object *op);
 /* Maps */
 /*extern void        *cf_map_get_property(mapstruct *map, int propcode);*/
 extern sstring cf_map_get_sstring_property(mapstruct *map, int propcode);
@@ -166,7 +166,7 @@ extern object      *cf_map_get_object_at(mapstruct *m, int x, int y);
 extern object      *cf_map_insert_object(mapstruct *where, object *op, int x, int y);
 extern object      *cf_map_insert_object_around(mapstruct *where, object *op, int x, int y);
 extern object      *cf_map_find_by_archetype_name(const char *str, mapstruct *map, int nx, int ny);
-extern int          cf_map_get_flags(mapstruct *oldmap, mapstruct **newmap, sint16 x, sint16 y, sint16 *nx, sint16 *ny);
+extern int          cf_map_get_flags(mapstruct *oldmap, mapstruct **newmap, int16_t x, int16_t y, int16_t *nx, int16_t *ny);
 extern object      *cf_map_insert_object_there(object *op, mapstruct *m, object *originator, int flag);
 extern int          cf_map_get_difficulty(mapstruct *map);
 extern int          cf_map_get_reset_time(mapstruct *map);
@@ -198,6 +198,7 @@ extern partylist   *cf_player_get_party(object *op);
 extern void         cf_player_set_party(object *op, partylist *party);
 extern int          cf_player_can_pay(object *op);
 extern int          cf_player_knowledge_has(object *op, const char *knowledge);
+extern void         cf_player_knowledge_give(object *op, const char *knowledge);
 extern int          cf_player_arrest(object *who);
 
 /* Archetypes */
@@ -237,16 +238,4 @@ extern void         cf_quest_start(object *pl, sstring quest_code, int state);
 extern void         cf_quest_set_player_state(object *pl, sstring quest_code, int state);
 extern int          cf_quest_was_completed(object *pl, sstring quest_code);
 
-
-#ifdef WIN32
-
-struct timezone {
-    int tz_minuteswest;
-    int tz_dsttime;
-};
-
-int gettimeofday(struct timeval *time_Info, struct timezone *timezone_Info);
-
-#endif
-
 #endif /* PLUGIN_COMMON_H */
