@@ -34,8 +34,10 @@
 /*                                                                           */
 /*****************************************************************************/
 
-#include <plugin_common.h>
 #include <assert.h>
+#include <string.h>
+
+#include "plugin_common.h"
 
 static f_plug_api cfapiSystem_add_string = NULL;
 static f_plug_api cfapiSystem_find_string = NULL;
@@ -75,8 +77,6 @@ static f_plug_api cfapiObject_reset = NULL;
 static f_plug_api cfapiObject_activate_rune = NULL;
 static f_plug_api cfapiObject_check_trigger = NULL;
 static f_plug_api cfapiObject_query_money = NULL;
-static f_plug_api cfapiObject_query_cost = NULL;
-static f_plug_api cfapiObject_query_cost_string = NULL;
 static f_plug_api cfapiObject_cast = NULL;
 static f_plug_api cfapiObject_learn_spell = NULL;
 static f_plug_api cfapiObject_forget_spell = NULL;
@@ -126,6 +126,7 @@ static f_plug_api cfapiCost_string_from_value = NULL;
 static f_plug_api cfapiPlayer_quest = NULL;
 static f_plug_api cfapiObject_remove_depletion = NULL;
 static f_plug_api cfapiPlayer_knowledge = NULL;
+static f_plug_api cfapiObject_perm_exp = NULL;
 
 #define GET_HOOK(x, y, z) { \
     getHooks(&z, 1, y, &x); \
@@ -170,8 +171,6 @@ int cf_init_plugin(f_plug_api getHooks) {
     GET_HOOK(cfapiObject_activate_rune, "cfapi_object_spring_trap", z);
     GET_HOOK(cfapiObject_check_trigger, "cfapi_object_check_trigger", z);
     GET_HOOK(cfapiObject_query_money, "cfapi_object_query_money", z);
-    GET_HOOK(cfapiObject_query_cost, "cfapi_object_query_cost", z);
-    GET_HOOK(cfapiObject_query_cost_string, "cfapi_object_query_cost_string", z);
     GET_HOOK(cfapiObject_cast, "cfapi_object_cast", z);
     GET_HOOK(cfapiObject_learn_spell, "cfapi_object_learn_spell", z);
     GET_HOOK(cfapiObject_forget_spell, "cfapi_object_forget_spell", z);
@@ -228,6 +227,7 @@ int cf_init_plugin(f_plug_api getHooks) {
     GET_HOOK(cfapiObject_find_by_arch_name, "cfapi_object_find_by_arch_name", z);
     GET_HOOK(cfapiObject_find_by_name, "cfapi_object_find_by_name", z);
     GET_HOOK(cfapiPlayer_knowledge, "cfapi_player_knowledge", z);
+    GET_HOOK(cfapiObject_perm_exp, "cfapi_object_perm_exp", z);
     return 1;
 }
 
@@ -296,15 +296,15 @@ void cf_map_set_string_property(mapstruct *map, int propcode, const char *value)
 }
 
 /* Should get replaced by tons of more explicit wrappers */
-sint16 cf_object_get_resistance(object *op, int rtype) {
+int16_t cf_object_get_resistance(object *op, int rtype) {
     int type;
-    sint16 resist;
+    int16_t resist;
 
     cfapiObject_get_property(&type, op, CFAPI_OBJECT_PROP_RESIST, rtype, &resist);
     assert(type == CFAPI_INT16);
     return resist;
 }
-void cf_object_set_resistance(object *op, int rtype, sint16 value) {
+void cf_object_set_resistance(object *op, int rtype, int16_t value) {
     int type;
 
     cfapiObject_set_property(&type, op, CFAPI_OBJECT_PROP_RESIST, rtype, value);
@@ -336,7 +336,8 @@ long cf_object_get_long_property(object *op, long propcode) {
 void cf_object_set_movetype_property(object *op, int propcode, MoveType value) {
     int type;
 
-    cfapiObject_set_property(&type, op, propcode, value);
+    /* note: MoveType can't be used through va_arg, so use MoveType * */
+    cfapiObject_set_property(&type, op, propcode, &value);
     assert(type == CFAPI_MOVETYPE);
 }
 MoveType cf_object_get_movetype_property(object *op, int propcode) {
@@ -363,9 +364,9 @@ mapstruct *cf_object_get_map_property(object *op, int propcode) {
     assert(type == CFAPI_PMAP);
     return value;
 }
-sint64 cf_object_get_int64_property(object *op, int propcode) {
+int64_t cf_object_get_int64_property(object *op, int propcode) {
     int type;
-    sint64 value;
+    int64_t value;
 
     cfapiObject_get_property(&type, op, propcode, &value);
     assert(type == CFAPI_SINT64);
@@ -384,7 +385,7 @@ void cf_object_set_float_property(object *op, int propcode, float value) {
     cfapiObject_set_property(&type, op, propcode, value);
     assert(type == CFAPI_FLOAT);
 }
-void cf_object_set_int64_property(object *op, int propcode, sint64 value) {
+void cf_object_set_int64_property(object *op, int propcode, int64_t value) {
     int type;
 
     cfapiObject_set_property(&type, op, propcode, value);
@@ -486,12 +487,25 @@ void cf_object_set_object_property(object *op, int propcode, object *value) {
  * Wrapper for change_exp().
  * @copydoc change_exp().
  */
-void cf_object_change_exp(object *op, sint64 exp, const char *skill_name, int flag) {
+void cf_object_change_exp(object *op, int64_t exp, const char *skill_name, int flag) {
     int type;
 
     cfapiObject_change_exp(&type, op, exp, skill_name && strlen(skill_name) > 0 ? skill_name : NULL, flag);
     assert(type == CFAPI_NONE);
 }
+
+/**
+ * Wrapper for PERM_EXP macro.
+ * @return The amount of total experience that is permanent experience
+ */
+int64_t cf_object_perm_exp(object *op) {
+    int type, ret;
+
+    cfapiObject_perm_exp(&type, op, &ret);
+    assert(type == CFAPI_SINT64);
+    return ret;
+}
+
 int cf_player_move(player *pl, int dir) {
     int type, ret;
 
@@ -665,7 +679,7 @@ object *cf_object_clone(object *op, int clonetype) {
  * Wrapper for object_split().
  * @copydoc object_split().
  */
-object *cf_object_split(object *orig_ob, uint32 nr, char *err, size_t size) {
+object *cf_object_split(object *orig_ob, uint32_t nr, char *err, size_t size) {
     int type;
     object *value;
 
@@ -700,7 +714,7 @@ int cf_object_pay_item(object *op, object *pl) {
  * Wrapper for pay_for_amount().
  * @copydoc pay_for_amount().
  */
-int cf_object_pay_amount(object *pl, uint64 to_pay) {
+int cf_object_pay_amount(object *pl, uint64_t to_pay) {
     int type, value;
 
     cfapiObject_pay_amount(&type, pl, to_pay, &value);
@@ -719,18 +733,6 @@ int cf_object_cast_spell(object *op, object *caster, int dir, object *spell_ob,
     assert(type == CFAPI_INT);
     return value;
 }
-/**
- * Should there be a difference nowadays between that and cast_spell ?
- * @todo
- * either totally remove or replace by cf_object_cast_spell().
- */
-int cf_object_cast_ability(object *caster, object *ctoo, int dir, object *sp, char *flags) {
-    int type, value;
-
-    cfapiObject_cast(&type, caster, ctoo, dir, sp, flags, &value);
-    assert(type == CFAPI_INT);
-    return value;
-}
 
 /**
  * Wrapper for do_learn_spell().
@@ -790,7 +792,7 @@ int cf_object_remove_depletion(object *op, int level) {
  * Wrapper for identify().
  * @copydoc identify()
  * @param op
- * @return 
+ * @return
  */
 object *cf_identify(object *op) {
     int type;
@@ -888,6 +890,18 @@ int cf_player_knowledge_has(object *op, const char *knowledge) {
     return value;
 }
 
+/**
+ * Wrapper for knowledge_give();
+ * @param op who to give knowledge to.
+ * @param knowledge knowledge to give, internal value.
+ */
+void cf_player_knowledge_give(object *op, const char *knowledge) {
+    int type;
+
+    cfapiPlayer_knowledge(&type, 2, op, knowledge);
+    assert(type == CFAPI_NONE);
+}
+
 /**
  * Wrapper for player_arrest().
  * @copydoc player_arrest()
@@ -959,32 +973,10 @@ int cf_object_query_money(const object *op) {
     return value;
 }
 
-/**
- * Wrapper for query_cost().
- * @copydoc query_cost().
- */
-int cf_object_query_cost(const object *tmp, object *who, int flag) {
-    int type, value;
-
-    cfapiObject_query_cost(&type, tmp, who, flag, &value);
-    assert(type == CFAPI_INT);
-    return value;
-}
-
-/**
- * Wrapper for query_cost_string modifier to take a char* and length instead of StringBuffer.
- */
-void cf_object_query_cost_string(const object *tmp, object *who, int flag, char *buffer, int length) {
-    int type;
-
-    cfapiObject_query_cost_string(&type, tmp, who, flag, buffer, length);
-    assert(type == CFAPI_NONE);
-}
-
 /**
  * Wrapper for cost_string_from_value modified to take a char* and length instead of a StringBuffer.
  */
-void cf_cost_string_from_value(uint64 cost, int largest_coin, char *buffer, int length) {
+void cf_cost_string_from_value(uint64_t cost, int largest_coin, char *buffer, int length) {
     int type;
 
     cfapiCost_string_from_value(&type, cost, largest_coin, buffer, length);
@@ -1459,7 +1451,7 @@ char *cf_strdup_local(const char *str) {
  * Wrapper for get_map_flags().
  * @copydoc get_map_flags()
  */
-int cf_map_get_flags(mapstruct *oldmap, mapstruct **newmap, sint16 x, sint16 y, sint16 *nx, sint16 *ny) {
+int cf_map_get_flags(mapstruct *oldmap, mapstruct **newmap, int16_t x, int16_t y, int16_t *nx, int16_t *ny) {
     int type, value;
 
     cfapiMap_get_property(&type, oldmap, CFAPI_MAP_PROP_FLAGS, newmap, x, y, nx, ny, &value);
@@ -2092,21 +2084,3 @@ int cf_quest_was_completed(object *pl, sstring quest_code) {
 
     return ret;
 }
-
-#ifdef WIN32
-int gettimeofday(struct timeval *time_Info, struct timezone *timezone_Info) {
-    /* Get the time, if they want it */
-    if (time_Info != NULL) {
-        time_Info->tv_sec = time(NULL);
-        time_Info->tv_usec = timeGetTime()*1000;
-    }
-    /* Get the timezone, if they want it */
-    if (timezone_Info != NULL) {
-        _tzset();
-        timezone_Info->tz_minuteswest = _timezone;
-        timezone_Info->tz_dsttime = _daylight;
-    }
-    /* And return */
-    return 0;
-}
-#endif
