@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_floor_c =
- *   "$Id: floor.c 14925 2011-07-30 21:01:42Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2013 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Floor handling.
@@ -42,7 +27,8 @@
  * @return
  * 1 if this tile should propagate, 0 else.
  */
-static int can_propagate(char item) {
+static int can_propagate(char item)
+{
     return (item == '\0' || item == '<' || item == '>') ? 1 : 0;
 }
 
@@ -58,7 +44,8 @@ static int can_propagate(char item) {
  * @param floor_arch
  * what floor to use.
  */
-static void put_floor(mapstruct *map, char **layout, int x, int y, object *floor_arch) {
+static void put_floor(mapstruct *map, char **layout, int x, int y, object *floor_arch)
+{
     int dx, dy;
     object *floor;
 
@@ -67,8 +54,9 @@ static void put_floor(mapstruct *map, char **layout, int x, int y, object *floor
 
     for (dx = -1; dx < 2; dx++) {
         for (dy = -1; dy < 2; dy++) {
-            if (GET_MAP_OB(map, x+dx, y+dy) == NULL && can_propagate(layout[x+dx][y+dy]))
+            if (GET_MAP_OB(map, x+dx, y+dy) == NULL && can_propagate(layout[x+dx][y+dy])) {
                 put_floor(map, layout, x+dx, y+dy, floor_arch);
+            }
         }
     }
 }
@@ -84,8 +72,8 @@ static void put_floor(mapstruct *map, char **layout, int x, int y, object *floor
  * @return
  * Crossfire map.
  */
-mapstruct *make_map_floor(char **layout, char *floorstyle, RMParms *RP) {
-    char styledirname[256];
+mapstruct *make_map_floor(char **layout, char *floorstyle, RMParms *RP)
+{
     mapstruct *style_map = NULL;
     object *the_floor;
     mapstruct *newMap = NULL;
@@ -95,16 +83,18 @@ mapstruct *make_map_floor(char **layout, char *floorstyle, RMParms *RP) {
     newMap = get_empty_map(RP->Xsize, RP->Ysize);
 
     /* get the style map */
-    snprintf(styledirname, sizeof(styledirname), "%s", "/styles/floorstyles");
+    const char *styledirname = "/styles/floorstyles";
     style_map = find_style(styledirname, floorstyle, -1);
-    if (style_map == NULL)
+    if (style_map == NULL) {
         return newMap;
+    }
 
     if (RP->multiple_floors) {
         for (x = 0; x < RP->Xsize; x++) {
             for (y = 0; y < RP->Ysize; y++) {
-                if (GET_MAP_OB(newMap, x, y) == NULL && layout[x][y] == '\0')
+                if (GET_MAP_OB(newMap, x, y) == NULL && layout[x][y] == '\0') {
                     put_floor(newMap, layout, x, y, pick_random_object(style_map));
+                }
             }
         }
     }
@@ -115,8 +105,9 @@ mapstruct *make_map_floor(char **layout, char *floorstyle, RMParms *RP) {
 
         for (x = 0; x < RP->Xsize; x++)
             for (y = 0; y < RP->Ysize; y++) {
-                if (GET_MAP_OB(newMap, x, y) != NULL)
+                if (GET_MAP_OB(newMap, x, y) != NULL) {
                     continue;
+                }
                 thisfloor = arch_to_object(the_floor->arch);
                 object_insert_in_map_at(thisfloor, newMap, thisfloor, INS_NO_MERGE|INS_NO_WALK_ON, x, y);
             }
