@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_account_c =
- *   "$Id: account.c 14485 2011-05-23 17:55:28Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2010 Mark Wedel & Crossfire Development Team
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
-
 /**
  * @file
  * This file contains account management logic - creation, deletion,
@@ -55,17 +40,24 @@
  * printable characters.
  * The accounts file is designed to be human readable, even if it is not expected that
  * a human will read it.
- * The passwords are stored in whatever crypt_string() returns.  On some systems, this
- * may be plaintext.
+ * Passwords are hashed using crypt(3) in traditional mode, unless you're on
+ * FreeBSD or Windows in which case passwords are stored in plaintext due to
+ * legacy reasons.
  * expansion (last field) is there for possible expansion.  There may be desire to store
  * bits of information there.  An example might be dm=1.  But it is there for expansion.
  */
 
-#include <global.h>
-#include <object.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <ctype.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "object.h"
+#include "sproto.h"
+
+#include "output_file.h"
 
 /** Number of fields in the accounts file.  These are colon seperated */
 #define NUM_ACCOUNT_FIELDS 6
@@ -85,7 +77,7 @@ typedef struct account_struct {
     char  *name;                    /**< Account name */
     char  *password;                /**< Password for this account */
     time_t last_login;              /**< Last time this account was logged in */
-    int	  num_characters;           /**< Number of characters on this account */
+    int   num_characters;           /**< Number of characters on this account */
     char  *character_names[MAX_CHARACTERS_PER_ACCOUNT+1];
                                     /**< Character names associated with this account
                                      +1 added to allow for NULL termination */
@@ -98,6 +90,14 @@ typedef struct account_struct {
  */
 static account_struct *accounts=NULL;
 
+/**
+ * Whether the account information was loaded or not.
+ * Because accounts_save() is called in cleanup(), we don't want
+ * programs using this library to erase the server's account information,
+ * since it is NOT loaded by default.
+ */
+static int accounts_loaded = 0;
+
 /**
  * Name of the accounts file.  I can not ever see a reason why this
  * name would not work, but may as well still make it easy to change it.
@@ -109,16 +109,16 @@ static account_struct *accounts=NULL;
  * it can then verify that the data is added is loaded back into memory
  * properly.  As such, we don't worry about memory cleanup, etc.
  */
-void clear_accounts(void) {
+void accounts_clear(void) {
     accounts = NULL;
+    accounts_loaded = 0;
 }
 
 /**
  * This loads all the account entries into memory.  It is presumed to only
  * be called once during the program startup.
  */
-void account_load_entries(void)
-{
+void accounts_load(void) {
     char fname[MAX_BUF], buf[VERY_BIG_BUF];
     FILE *fp;
     account_struct *ac, *last=NULL;
@@ -134,8 +134,7 @@ void account_load_entries(void)
         /* This may not in fact be a critical error - on a new server,
          * the accounts file may not yet exist.
          */
-        char err[MAX_BUF];
-        LOG(llevInfo,"Warning: Unable to open %s [%s]\n", fname, strerror_local(errno, err, sizeof(err)));
+        LOG(llevInfo,"Warning: Unable to open %s [%s]\n", fname, strerror(errno));
         return;
     }
     while (fgets(buf, VERY_BIG_BUF, fp)) {
@@ -232,6 +231,7 @@ void account_load_entries(void)
 
     }
     fclose(fp);
+    accounts_loaded = 1;
 }
 
 /**
@@ -247,14 +247,14 @@ static void account_write_entry(FILE *fp, account_struct *ac)
 {
     int i;
 
-    fprintf(fp,"%s:%s:%u:", ac->name, ac->password, (uint32)ac->last_login);
+    fprintf(fp,"%s:%s:%u:", ac->name, ac->password, (uint32_t)ac->last_login);
     for (i=0; i<ac->num_characters; i++) {
         if (i != 0)
             fprintf(fp,";%s", ac->character_names[i]);
         else
             fprintf(fp,"%s", ac->character_names[i]);
     }
-    fprintf(fp,":%u:\n", (uint32) ac->created);
+    fprintf(fp,":%u:\n", (uint32_t) ac->created);
 }
 
 
@@ -266,41 +266,35 @@ static void account_write_entry(FILE *fp, account_struct *ac)
  */
 void accounts_save(void)
 {
-    char fname[MAX_BUF], fname1[MAX_BUF];
+    char fname[MAX_BUF];
     FILE *fp;
+    OutputFile of;
     account_struct *ac;
 
-    snprintf(fname, MAX_BUF,"%s/%s.new", settings.localdir, ACCOUNT_FILE);
-
-    fp = fopen(fname,"w");
-    if (fp == NULL) {
-        char err[MAX_BUF];
-
-        LOG(llevError, "Cannot open accounts file %s: %s\n", fname,
-            strerror_local(errno, err, sizeof(err)));
+    if (accounts_loaded == 0)
         return;
-    }
 
-    fprintf(fp, "# This file should not be edited while the server is running.\n");
-    fprintf(fp, "# Otherwise, any changes made may be overwritten by the server\n");
+    snprintf(fname, MAX_BUF,"%s/%s", settings.localdir, ACCOUNT_FILE);
+
+    fp = of_open(&of, fname);
+    if (fp == NULL)
+        return;
+
+    fprintf(fp, "# IMPORTANT: Do not edit this file while the server is running. This file is\n"
+                "# only read when the server starts, and any changes will be overwritten when\n"
+                "# the server exits.\n");
     fprintf(fp, "# Format:\n");
     fprintf(fp, "# Account name:Password:Account last used:Characters (semicolon separated):created:expansion\n");
     for (ac=accounts; ac; ac=ac->next) {
         /* Don't write out accounts with no characters associated unless the
          * account is at least a day old.
          */
-        if (ac->num_characters || (ac->created > (time(NULL) - 24*60*60)))
+        // 86400 seconds in a day, so no reason to make the game have to recalculate this all the time
+        // SilverNexus 2014-06-12
+        if (ac->num_characters || (ac->created > (time(NULL) - 86400)))
             account_write_entry(fp, ac);
     }
-    fclose(fp);
-
-    /* We write to a new file name - in that way, if we crash while writing the file,
-     * we are not left with a corrupted file.  So now we need to rename it to the
-     * file to use.
-     */
-    snprintf(fname1, MAX_BUF,"%s/%s", settings.localdir, ACCOUNT_FILE);
-    unlink(fname1);
-    rename(fname, fname1);
+    of_close(&of);
 }
 
 /**
@@ -323,9 +317,8 @@ const char *account_exists(const char *account_name)
 }
 
 /**
- * Checks if the name and password are valid.  Note that on the return modes,
- * we do not differentiate on non existing account and wrong password -
- * one could use account_exists() if one is checking just for the name.
+ * Check if the given account exists, and whether the password is correct.
+ *
  * Note - if we do get a match, we update the last_login value - it is
  * presumed that if someone knows the right accountname/password, that
  * the account is effectively getting logged in.
@@ -337,23 +330,18 @@ const char *account_exists(const char *account_name)
  * @return
  * 0 if no match/wrong password, 1 if a match is found and password matches.
  */
-int account_check_name_password(const char *account_name, const char *account_password)
-{
+int account_login(const char *account_name, const char *account_password) {
     account_struct *ac;
 
     for (ac=accounts; ac; ac=ac->next) {
-        /* account names must be unique.  So if we get a matching name,
-         * we want to check the password.  If the password is correct,
-         * we have a good match.  If the password doesn't match, there is
-         * no reason to do anymore searching on the list, because we won't
-         * find another matching name.
-         */
+        /* Look for a matching account name and check the password. */
         if (!strcasecmp(ac->name, account_name)) {
-            if (!strcmp(ac->password, crypt_string(account_password, ac->password))) {
-                ac->last_login=time(NULL);
+            if (check_password(account_password, ac->password)) {
+                ac->last_login = time(NULL);
                 return 1;
+            } else {
+                return 0;
             }
-            else return 0;
         }
     }
     return 0;
@@ -363,13 +351,13 @@ int account_check_name_password(const char *account_name, const char *account_pa
  * Checks a string to make sure it does not have any invalid characters.
  * We are fairly permissive on character here.
  * String must start with alphanumeric
- * String must not contain :;/'
+ * String must not contain :;/'[
  * String can not end if a space
  * This string will get used for file/directory names, but so long
  * as as characters are not included that are not illegal, one can still
  * manipulate the file/directory by putting it in quotes.  If one starts
  * to block all characters that may get interperted by shells, a large
- * number of characters now get blocked (|*[]()!, etc), and deciding
+ * number of characters now get blocked (|*]()!, etc), and deciding
  * which should be allowed and not just becomes a judgement call, so easier
  * to just allow all and have the user put it in quotes.
  *
@@ -384,9 +372,16 @@ int account_check_string(const char *str)
 
     /* Require first character to be letter or number */
     if (!isalnum(*str)) return 1;
-    for (; *str != '\0'; str++) {
+    for (; *str != '\0'; ++str) {
         if (!isprint(*str)) return 1;
-        if (*str == ':' || *str==';' || *str=='/' || *str=='\'') return 1;
+		switch (*str){
+            case ':':
+            case ';':
+            case '/':
+            case '\'':
+            case '[':
+                return 1;
+        }
     }
     /* Don't allow space characters at end of string. */
     if (isspace(*(str-1))) return 1;
@@ -394,6 +389,7 @@ int account_check_string(const char *str)
     return 0;
 }
 
+
 /**
  * Adds an account.  It does error checking, but the caller might
  * want to do checking before getting here.
@@ -409,16 +405,11 @@ int account_check_string(const char *str)
  * @retval 2
  * account already exists.
  */
-int account_add_account(const char *account_name, const char *account_password)
-{
+int account_new(const char *account_name, const char *account_password) {
     account_struct *ac;
 
-    /* We need to check the password because we don't know what crypt_string() will do -
-     * it may just return the string we pass in.  We should probably check the results
-     * returned from crypt_string(), but the problem there is that if we have a faulty
-     * algorithm which in fact is putting in invalid characters, there isn not much
-     * the players can do about that.
-     */
+    // Check password for invalid characters because newhash() may just
+    // return the string in plaintext.
     if (account_check_string(account_name) || account_check_string(account_password))
         return 1;
 
@@ -426,7 +417,7 @@ int account_add_account(const char *account_name, const char *account_password)
 
     ac = malloc(sizeof(account_struct));
     ac->name = strdup_local(account_name);
-    ac->password = strdup_local(crypt_string(account_password, NULL));
+    ac->password = strdup_local(newhash(account_password));
     ac->last_login = time(NULL);
     ac->created = ac->last_login;
     ac->num_characters = 0;
@@ -443,6 +434,9 @@ int account_add_account(const char *account_name, const char *account_password)
     ac->next = accounts;
     accounts = ac;
 
+    /* mark that accounts should be saved through accounts_save(). */
+    accounts_loaded = 1;
+
     return 0;
 }
 
@@ -464,8 +458,7 @@ int account_add_account(const char *account_name, const char *account_password)
  * number of characters on this account has reached a maximum.
  */
 
-int account_add_player_to_account(const char *account_name, const char *player_name)
-{
+int account_link(const char *account_name, const char *player_name) {
     account_struct *ac;
 
     for (ac=accounts; ac; ac=ac->next) {
@@ -484,13 +477,13 @@ int account_add_player_to_account(const char *account_name, const char *player_n
 }
 
 /**
- * Adds a player name to an account.  Player corresponds to the names used
+ * Removes a player name from an account.  Player corresponds to the names used
  * in the pl object, not the person sitting at the computer.  This function
  * presumes that the caller has done the work to verify that the player
  * does in fact exist, and does any related work to update the player.  What this
  * function does is simply update the account structure.
  * @param account_name
- * name of the account we are adding this player to.
+ * name of the account we are removing this player from.
  * @param player_name
  * name of this player.
  * @retval 0
@@ -501,11 +494,13 @@ int account_add_player_to_account(const char *account_name, const char *player_n
  * player of this name not on account.
  */
 
-int account_remove_player_from_account(const char *account_name, const char *player_name)
-{
+int account_remove_player(const char *account_name, const char *player_name) {
     account_struct *ac;
     int i, match=0;
 
+    if (account_name == NULL)
+        return 0;
+
     for (ac=accounts; ac; ac=ac->next) {
         if (!strcasecmp(ac->name, account_name)) break;
     }
@@ -516,7 +511,7 @@ int account_remove_player_from_account(const char *account_name, const char *pla
      * always null terminated, so this also makes sure we copy the null down.
      */
     for (i=0; i<ac->num_characters; i++) {
-        if (!strcasecmp(ac->character_names[i], player_name)) {
+        if (!strcmp(ac->character_names[i], player_name)) {
             free(ac->character_names[i]);
             match=1;
         }
@@ -571,7 +566,7 @@ const char *account_get_account_for_char(const char *charname)
 
     for (ac=accounts; ac; ac=ac->next) {
         for (i=0; i<ac->num_characters; i++) {
-            if (!strcasecmp(ac->character_names[i], charname)) {
+            if (!strcmp(ac->character_names[i], charname)) {
                 return ac->name;
             }
         }
@@ -660,29 +655,35 @@ int account_is_logged_in(const char *name)
  * @retval 3
  * current password is invalid.
  */
-int account_change_password(const char *account_name, const char *current_password, const char *new_password)
-{
+int account_change_password(const char *account_name,
+        const char *current_password, const char *new_password) {
     account_struct *ac;
 
-    /* We need to check the password because we don't know what crypt_string() will do -
-     * it may just return the string we pass in.  We should probably check the results
-     * returned from crypt_string(), but the problem there is that if we have a faulty
-     * algorithm which in fact is putting in invalid characters, there isn not much
-     * the players can do about that.
-     */
-    if (account_check_string(account_name) || account_check_string(current_password) || account_check_string(new_password))
+    // Check password for invalid characters as in account_new().
+    if (account_check_string(account_name) || account_check_string(current_password) ||
+            account_check_string(new_password)) {
         return 1;
-
-    for (ac=accounts; ac; ac=ac->next) {
-        if (!strcasecmp(ac->name, account_name)) break;
     }
-    if (ac == NULL) return 2;
 
-    if (strcmp(crypt_string(current_password, ac->password), ac->password))
+    // Iterate through accounts list until a matching name is found.
+    for (ac = accounts; ac; ac = ac->next) {
+        if (!strcasecmp(ac->name, account_name)) {
+            break;
+        }
+    }
+
+    // Check if the given account actually exists.
+    if (ac == NULL) {
+        return 2;
+    }
+
+    // Return an error if the current password does not match.
+    if (!check_password(current_password, ac->password)) {
         return 3;
+    }
 
     free(ac->password);
-    ac->password = strdup_local(crypt_string(new_password, NULL));
+    ac->password = strdup_local(newhash(new_password));
 
     return 0;
 }
