@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_alchemy_c =
- *   "$Id: alchemy.c 14921 2011-07-30 18:11:43Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /* March 96 - Laid down original code. -b.t. thomas@astro.psu.edu */
 
 /**
@@ -33,14 +18,18 @@
  * This contains all alchemy-related functions.
  */
 
-#include <global.h>
-#include <object.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <skills.h>
-#include <spells.h>
+#include "global.h"
+
 #include <assert.h>
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "object.h"
+#include "shop.h"
+#include "skills.h"
+#include "spells.h"
+#include "sproto.h"
 
 /** define this for some helpful debuging information */
 #if 0
@@ -91,10 +80,38 @@ static const char *cauldron_sound(void) {
     return cauldron_effect[rndm(0, size-1)];
 }
 
+/**
+ * Compute a success probability, between .01 and .95, based on the
+ * level difference.
+ * @param diff level difference.
+ * @return success probability.
+ */
+static float chance_fn(int diff) {
+    if (diff > 10)
+        return MAX(.01, .3 - (diff - 10) * .03);
+
+    if (diff > -10)
+        return .5 + .02 * (float)(-diff);
+
+    return MIN(.95, .70 + (-diff - 10) * .01);
+}
+
 /**
  * Compute the success probability of a recipe.
  *
- * The scale is kind of linear, with 3 steps depending on the level difference between recipe and skill.
+ * Probability of success is a function of the difference between the recipe
+ * difficulty and the player's skill level, adjusted by the cauldron bonus.
+ *
+ * The shape is roughly 1-exp(-x):
+ * ------------+-------------
+ *  Difference | Probability
+ * ------------+-------------
+ *      10     |    0.31
+ *       0     |    0.51
+ *     -10     |    0.71
+ *     -30     |    0.91
+ *  <= -35     |    0.95
+ * ------------+-------------
  *
  * @param rp recipe to attempt.
  * @param skill skill being used.
@@ -102,21 +119,13 @@ static const char *cauldron_sound(void) {
  * @return chance between 0.01 and .95.
  */
 static float recipe_chance(const recipe *rp, const object *skill, const object *cauldron) {
-    int cauldron_add_skill;
-
     assert(rp);
     assert(skill);
     assert(cauldron);
 
-    cauldron_add_skill = (cauldron->magic + 1) / 2;
-
-    if (skill->level + cauldron_add_skill < rp->diff - 10)
-        return MAX(.01, .3 - (rp->diff - 10 - skill->level - cauldron_add_skill) * .03);
-
-    if (skill->level + cauldron_add_skill <= rp->diff + 10)
-        return .5 + .02 * (float)(skill->level + cauldron_add_skill - rp->diff);
-
-    return MIN(.95, .70 + (skill->level + cauldron_add_skill - rp->diff - 10) * .01);
+    const int cauldron_add_skill = (cauldron->magic + 1) / 2;
+    const int eff_skill = skill->level + cauldron_add_skill;
+    return chance_fn(rp->diff - eff_skill);
 }
 
 /**
@@ -186,8 +195,8 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
         /* find the recipe */
         rp = find_recipe(fl, formula, cauldron->inv);
         if (rp != NULL) {
-            uint64 value_ingredients;
-            uint64 value_item;
+            uint64_t value_ingredients;
+            uint64_t value_item;
             int attempt_shadow_alchemy;
 
             /* the caster gets an increase in ability based on thier skill lvl */
@@ -218,7 +227,7 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
             /* determine value of ingredients */
             value_ingredients = 0;
             FOR_INV_PREPARE(cauldron, tmp)
-                value_ingredients += query_cost(tmp, NULL, F_TRUE);
+                value_ingredients += price_base(tmp);
             FOR_INV_FINISH();
 
             attempt_shadow_alchemy = !is_defined_recipe(rp, cauldron, caster);
@@ -232,7 +241,7 @@ static void attempt_do_alchemy(object *caster, object *cauldron) {
                 LOG(llevDebug, "percent success chance =  %f ab%d / diff%d*lev%d\n", success_chance, ability, rp->diff, item->level);
 #endif
 
-                value_item = query_cost(item, NULL, F_TRUE|F_IDENTIFIED|F_NOT_CURSED);
+                value_item = price_base(item);
                 if (attempt_shadow_alchemy && value_item > value_ingredients) {
 #ifdef ALCHEMY_DEBUG
 #ifndef WIN32
@@ -269,12 +278,12 @@ static int content_recipe_value(object *op) {
 
     FOR_INV_PREPARE(op, tmp) {
         tval = 0;
-        strcpy(name, tmp->name);
+        safe_strncpy(name, tmp->name, sizeof(name));
         if (tmp->title)
             snprintf(name, sizeof(name), "%s %s", tmp->name, tmp->title);
-        tval = (strtoint(name)*(tmp->nrof ? tmp->nrof : 1));
+        tval = (strtoint(name)*NROF(tmp));
 #ifdef ALCHEMY_DEBUG
-        LOG(llevDebug, "Got ingredient %d %s(%d)\n", tmp->nrof ? tmp->nrof : 1, name, tval);
+        LOG(llevDebug, "Got ingredient %d %s(%d)\n", NROF(tmp), name, tval);
 #endif
         formula += tval;
     } FOR_INV_FINISH();
@@ -435,7 +444,7 @@ static object *make_item_from_recipe(object *cauldron, const recipe *rp) {
 
     /* If item is already in container, we need to remove its weight, since it can change later on. */
     if (item->env != NULL)
-        object_sub_weight(cauldron, item->weight*(item->nrof != 0 ? item->nrof : 1));
+        object_sub_weight(cauldron, item->weight*NROF(item));
 
     /* Find the appropriate artifact template...*/
     if (strcmp(rp->title, "NONE")) {
@@ -446,9 +455,13 @@ static object *make_item_from_recipe(object *cauldron, const recipe *rp) {
         }
         transmute_materialname(item, art->item);
         give_artifact_abilities(item, art->item);
+        if (is_identified(item)) {
+            /* artifacts are generated unidentified, so if the item is identified we must give it its real properties. */
+            object_give_identified_properties(item);
+        }
     }
     if (item->env != NULL)
-        object_add_weight(cauldron, item->weight*(item->nrof != 0 ? item->nrof : 1));
+        object_add_weight(cauldron, item->weight*NROF(item));
 
     if (QUERY_FLAG(cauldron, FLAG_CURSED))
         SET_FLAG(item, FLAG_CURSED);
@@ -482,7 +495,8 @@ static object *find_transmution_ob(object *first_ingred, const recipe *rp, size_
             size_t i;
 
             for (i = 0; i < rp->arch_names; i++) {
-                if (strcmp(item->arch->name, rp->arch_name[i]) == 0) {
+                /* Items with titles are skipped to avoid shadow alchemy ability stacking. */
+                if (strcmp(item->arch->name, rp->arch_name[i]) == 0 && !(item->title)) {
                     *rp_arch_index = i;
                     break;
                 }
@@ -562,7 +576,7 @@ static void alchemy_failure_effect(object *op, object *cauldron, const recipe *r
         if (rndm(0, 2)) { /* slag created */
             object *tmp;
             int weight = 0;
-            uint16 material = M_STONE;
+            uint16_t material = M_STONE;
 
             FOR_INV_PREPARE(cauldron, tmp) { /* slag has coadded ingredient properties */
                 weight += tmp->weight;
@@ -820,7 +834,9 @@ static int calc_alch_danger(object *caster, object *cauldron, const recipe *rp)
 
     /* Ingredients. */
     FOR_INV_PREPARE(cauldron, item) {
-        danger += item->weight / 100;
+        /* Nicolas Weeger: no reason why this is the case.
+         danger += item->weight / 100;
+         */
         danger++;
         if (QUERY_FLAG(item, FLAG_CURSED) || QUERY_FLAG(item, FLAG_DAMNED))
             danger += 5;
@@ -864,7 +880,7 @@ static int calc_alch_danger(object *caster, object *cauldron, const recipe *rp)
  * 1 if the ingredients match the recipe, 0 if not.
  */
 static int is_defined_recipe(const recipe *rp, const object *cauldron, object *caster) {
-    uint32 batches_in_cauldron;
+    uint32_t batches_in_cauldron;
     const linked_char *ingredient;
     int number;
 
@@ -881,7 +897,7 @@ static int is_defined_recipe(const recipe *rp, const object *cauldron, object *c
     /* check for matching ingredients */
     batches_in_cauldron = 0;
     for (ingredient = rp->ingred; ingredient != NULL; ingredient = ingredient->next) {
-        uint32 nrof;
+        uint32_t nrof;
         const char *name;
         int ok;
 
@@ -912,7 +928,7 @@ static int is_defined_recipe(const recipe *rp, const object *cauldron, object *c
 
             if (strcmp(name2, name) == 0) {
                 if (ob->nrof%nrof == 0) {
-                    uint32 batches;
+                    uint32_t batches;
 
                     batches = ob->nrof/nrof;
                     if (batches_in_cauldron == 0) {
