@@ -1,50 +1,34 @@
 /*
- * static char *rcsid_apply_c =
- *   "$Id: apply.c 14943 2011-08-04 17:48:15Z akirschbaum $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
-/*
-  CrossFire, A Multiplayer game for X-windows
-
-  Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-  Copyright (C) 1992 Frank Tore Johansen
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-  The authors can be reached via e-mail to crossfire-devel@real-time.com
-*/
 
 /**
  * @file
  * Handles objects being applied, and their effect.
  */
 
-#include <global.h>
-#include <living.h>
-#include <spells.h>
-#include <skills.h>
-#include <tod.h>
-
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-
-/* Want this regardless of rplay. */
-#include <sounds.h>
+#include "global.h"
 
 /* need math lib for double-precision and pow() in dragon_eat_flesh() */
 #include <math.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "living.h"
+#include "skills.h"
+#include "sounds.h"
+#include "spells.h"
+#include "sproto.h"
+#include "tod.h"
 
 static int apply_check_apply_restrictions(object *who, object *op, int aflags);
 static int apply_check_personalized_blessings(object *who, const object *op);
@@ -134,9 +118,6 @@ int should_director_abort(const object *op, const object *victim) {
  *
  * @param tmp
  * item that was applied.
- *
- * @note
- * handle_apply_yield() has been renamed to apply_handle_yield()
  */
 void apply_handle_yield(object *tmp) {
     const char *yield;
@@ -163,13 +144,16 @@ void apply_handle_yield(object *tmp) {
  * @return TRUE if face changed
  */
 int set_object_face_main(object *op) {
-    int newface = op->arch->clone.face->number;
+    const Face *newface = op->arch->clone.face;
     sstring saved = object_get_value(op, "face_closed");
 
+    if (op->more)
+        set_object_face_main(op->more);
+
     if (saved)
         newface = find_face(saved, newface);
-    if (newface && op->face != &new_faces[newface]) {
-        op->face = &new_faces[newface];
+    if (newface && op->face != newface) {
+        op->face = newface;
         return TRUE;
     }
     return FALSE;
@@ -188,21 +172,25 @@ int set_object_face_main(object *op) {
  */
 static int set_object_face_other(object *op) {
     sstring custom;
-    int newface = 0;
+    const Face *newface = NULL;
+    object *head = op->head ? op->head : op;
 
-    if (op->face && op->other_arch && op->other_arch->clone.face)
-        newface = op->other_arch->clone.face->number;
+    if (op->more)
+        set_object_face_other(op->more);
+
+    if (head->face && head->other_arch && head->other_arch->clone.face)
+        newface = head->other_arch->clone.face;
 
     if (op->face != op->arch->clone.face) {
         /* object has a custom face, save it so it gets correctly restored later. */
         object_set_value(op, "face_closed", op->face->name, 1);
     }
 
-    custom = object_get_value(op, "face_opened");
+    custom = object_get_value(head, "face_opened");
     if (custom)
         newface = find_face(custom, newface);
-    if (newface && op->face->number != newface) {
-        op->face = &new_faces[newface];
+    if (newface && op->face != newface) {
+        op->face = newface;
         return TRUE;
     }
     return FALSE;
@@ -224,12 +212,14 @@ static int set_object_face_other(object *op) {
  * player.
  * @param sack
  * container the player is opening or closing.
+ * @param aflags
+ * combination of @ref AP_xxx flags.
  * @return
  * 1 if an object is applied somehow or another, 0 if error/no apply
  *
  * @author Eneq(at)(csd.uu.se)
  */
-int apply_container(object *op, object *sack) {
+int apply_container(object *op, object *sack, int aflags) {
     char name_sack[MAX_BUF], name_tmp[MAX_BUF];
     object *tmp = op->container;
 
@@ -237,20 +227,104 @@ int apply_container(object *op, object *sack) {
         return 0; /* This might change */
 
     if (sack == NULL || sack->type != CONTAINER) {
-        LOG(llevError, "apply_container: %s is not container!\n", sack ? sack->name : "NULL");
+        LOG(llevError, "apply_container: '%s' tried to apply %s, which is not a container\n", op->name, sack ? sack->name : "(null)");
         return 0;
     }
 
+    if (sack->head)
+        sack = sack->head;
+
+    query_name(sack, name_sack, MAX_BUF);
+
+    if ( aflags == AP_APPLY || aflags == AP_UNAPPLY || aflags == AP_OPEN ) {
+        // What if the container isn't in the player's inventory?
+        if ( sack->env != op ) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
+                                 "Not in your inventory: %s",
+                                 name_sack);
+            return 0;
+        }        
+    }
+    if ( aflags == AP_APPLY ) {
+        // What if the container is open?  Make it just ready!
+        if ( op->container == sack ) {
+            op->container = NULL;
+            if (op->contr != NULL)
+                op->contr->socket.container_position = 0;
+            CLEAR_FLAG(sack, FLAG_APPLIED);
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                                 "You readied %s.",
+                                 name_sack);
+            SET_FLAG(sack, FLAG_APPLIED);
+            // FIXME: This is not flipping the face!
+            if (set_object_face_main(sack)) // change image to closed
+                esrv_update_item(UPD_FLAGS|UPD_FACE, op, sack);
+            else
+                esrv_update_item(UPD_FLAGS, op, sack);
+            return 0;
+        }
+        // What if the container is already applied?  Do nothing!
+        if (QUERY_FLAG(sack, FLAG_APPLIED)) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                                 "Already readied %s.",
+                                 name_sack);
+            return 0;
+        }
+        // What if the container is closed?  Same as no aflags.
+        aflags = AP_NULL;
+    }
+
+    if ( aflags == AP_OPEN ) {
+        // What if the container is already open?
+        if ( op->container == sack ) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                                 "Already opened %s.",
+                                 name_sack);
+            return 0;
+        }
+        // Set the container as applied and then proceed as if no special flags
+        SET_FLAG(sack, FLAG_APPLIED);
+        aflags = AP_NULL;
+    }
+
+    if ( aflags == AP_UNAPPLY ) {
+        // If not open, two cases:
+        if ( op->container != sack ) {
+            if (QUERY_FLAG(sack, FLAG_APPLIED)) {
+                CLEAR_FLAG(sack, FLAG_APPLIED);
+                draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                                     "You closed %s.",
+                                     name_sack);
+                esrv_update_item(UPD_FLAGS, op, sack);
+                return 0;
+            }
+            else {
+                draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                                     "Already closed %s.",
+                                     name_sack);
+                return 0;
+            }
+        }
+        // open; same as no special flags
+        aflags = AP_NULL;
+    }
+    
     /* If we have a currently open container, then it needs
      * to be closed in all cases if we are opening this one up.
      * We then fall through if appropriate for opening the new
      * container.
      */
-    if (op->container && QUERY_FLAG(sack, FLAG_APPLIED)) {
+    if (op->container && QUERY_FLAG(op->container, FLAG_APPLIED) &&
+        (QUERY_FLAG(sack, FLAG_APPLIED) || sack->env != op) )
+    {
         tag_t tmp_tag = op->container->count;
-
+   
         if (op->container->env != op) { /* if container is on the ground */
-            op->container->move_off = 0;
+            object *part = op->container->head ? op->container->head : op->container;
+            while (part) {
+                part->move_off = 0;
+                part = part->more;
+            }
         }
 
         /* Query name before the close event, as the container could be destroyed. */
@@ -281,8 +355,6 @@ int apply_container(object *op, object *sack) {
             return 1;
     }
 
-    query_name(sack, name_sack, MAX_BUF);
-
     /* If the player is trying to open it (which he must be doing
      * if we got here), and it is locked, check to see if player
      * has the equipment to open it.
@@ -339,7 +411,13 @@ int apply_container(object *op, object *sack) {
         }
 
         /* set it so when the player walks off, we can unapply the sack */
-        sack->move_off = MOVE_ALL;      /* trying force closing it */
+        {
+            object *part = sack->head ? sack->head : sack;
+            while (part) {
+                part->move_off = MOVE_ALL;
+                part = part->more;
+            }
+        }
 
         CLEAR_FLAG(sack, FLAG_APPLIED);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
@@ -464,7 +542,7 @@ void do_forget_spell(object *op, const char *spell) {
     player_unready_range_ob(op->contr, spob);
     esrv_remove_spell(op->contr, spob);
     object_remove(spob);
-    object_free2(spob, 0);
+    object_free(spob, 0);
 }
 
 /**
@@ -476,9 +554,6 @@ void do_forget_spell(object *op, const char *spell) {
  * item being applied.
  * @return
  * 0 if item can't be applied, 1 else.
- *
- * @note
- * check_race_restrictions() has been renamed to apply_check_race_restrictions()
  */
 static int apply_check_race_restrictions(object *who, object *item) {
     char buf[MAX_BUF];
@@ -519,9 +594,6 @@ static int apply_check_race_restrictions(object *who, object *item) {
  * - 0: player or monster can't apply objects of that type
  * - 1: has been applied, or there was an error applying the object
  * - 2: objects of that type can't be applied if not in inventory
- *
- * @note
- * manual_apply() has been renamed to apply_manual()
  */
 int apply_manual(object *op, object *tmp, int aflag) {
     tmp = HEAD(tmp);
@@ -538,10 +610,6 @@ int apply_manual(object *op, object *tmp, int aflag) {
     if (!apply_check_race_restrictions(op, tmp))
         return METHOD_SILENT_ERROR;
 
-    /* Lauwenmark: Handle for plugin apply event */
-    if (execute_event(tmp, EVENT_APPLY, op, NULL, NULL, SCRIPT_FIX_ALL) != 0)
-        return METHOD_OK;
-
     if (op->contr)
         play_sound_player_only(op->contr, SOUND_TYPE_ITEM, tmp, 0, "apply");
 
@@ -566,9 +634,6 @@ int apply_manual(object *op, object *tmp, int aflag) {
  * - 0: player or monster can't apply objects of that type
  * - 1: has been applied, or there was an error applying the object
  * - 2: objects of that type can't be applied if not in inventory
- *
- * @note
- * player_apply() has been renamed to apply_by_living()
  */
 int apply_by_living(object *pl, object *op, int aflag, int quiet) {
     int tmp;
@@ -594,13 +659,10 @@ int apply_by_living(object *pl, object *op, int aflag, int quiet) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
                       "It must have been an illusion.");
         object_remove(op);
-        object_free2(op, 0);
+        object_free(op, 0);
         return 1;
     }
 
-    pl->contr->last_used = op;
-    pl->contr->last_used_id = op->count;
-
     tmp = apply_manual(pl, op, aflag);
     if (!quiet) {
         if (tmp == METHOD_UNHANDLED) {
@@ -630,9 +692,6 @@ int apply_by_living(object *pl, object *op, int aflag, int quiet) {
  *
  * @param pl
  * player.
- *
- * @note
- * player_apply_below() has been renamed to apply_by_living_below()
  */
 void apply_by_living_below(object *pl) {
     object *tmp;
@@ -954,9 +1013,6 @@ static int unapply_for_ob(object *who, object *op, int aflags) {
  * is set, do we really are what the other flags may be?)
  * See include/define.h for detailed description of the meaning of
  * these return values.
- *
- * @note
- * can_apply_object() has been renamed to apply_can_apply_object()
  */
 int apply_can_apply_object(const object *who, const object *op) {
     int i, retval = 0;
@@ -1328,7 +1384,7 @@ int apply_special(object *who, object *op, int aflags) {
         }
     }
     if (who->type == PLAYER)
-        esrv_update_item(UPD_NROF|UPD_FLAGS|UPD_WEIGHT, who, op);
+        esrv_update_item(UPD_NAME|UPD_NROF|UPD_FLAGS|UPD_WEIGHT, who, op);
     return 0;
 }
 
@@ -1341,9 +1397,6 @@ int apply_special(object *who, object *op, int aflags) {
  * object to initialize.
  * @return
  * 1 if object was initialized, 0 else.
- *
- * @note
- * auto_apply() has been renamed to apply_auto()
  */
 int apply_auto(object *op) {
     object *tmp;
@@ -1362,7 +1415,7 @@ int apply_auto(object *op) {
             if (tmp == NULL)
                 return 0;
             if (QUERY_FLAG(tmp, FLAG_CURSED) || QUERY_FLAG(tmp, FLAG_DAMNED)) {
-                object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+                object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
                 tmp = NULL;
             }
         } while (!tmp);
@@ -1392,7 +1445,7 @@ int apply_auto(object *op) {
                 object_free_drop_inventory(tmp);
         } FOR_INV_FINISH();
         object_remove(op);
-        object_free2(op, FREE_OBJ_NO_DESTROY_CALLBACK);
+        object_free(op, FREE_OBJ_NO_DESTROY_CALLBACK);
         break;
     }
     return 0;
@@ -1406,9 +1459,6 @@ int apply_auto(object *op) {
  *
  * @param m
  * map to fix.
- *
- * @note
- * fix_auto_apply() has been renamed to apply_auto_fix()
  */
 
 void apply_auto_fix(mapstruct *m) {
@@ -1539,7 +1589,7 @@ void scroll_failure(object *op, int failure, int power) {
         if (op->stats.sp < 0)
             /* For some reason the sp can become negative here. */
             op->stats.sp = 0;
-        object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+        object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
         return;
     }
 
@@ -1572,7 +1622,7 @@ void scroll_failure(object *op, int failure, int power) {
             cast_magic_storm(op, tmp, power);
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
                           "You unlease uncontrolled mana!");
-            object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+            object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
             return;
         }
     }
@@ -1612,7 +1662,7 @@ void apply_changes_to_player(object *pl, object *change, int limit_stats) {
      */
     if (! (limit_stats & AC_PLAYER_STAT_NO_CHANGE)) {
         for (i = 0; i < NUM_STATS; i++) {
-            sint8 stat = get_attr_value(&pl->contr->orig_stats, i);
+            int8_t stat = get_attr_value(&pl->contr->orig_stats, i);
             int race_bonus = get_attr_value(&pl->arch->clone.stats, i);
 
             stat += get_attr_value(&change->stats, i);
@@ -1650,8 +1700,10 @@ void apply_changes_to_player(object *pl, object *change, int limit_stats) {
             int stat = get_attr_value(&pl->contr->orig_stats, i);
             int race_bonus = get_attr_value(&pl->arch->clone.stats, i);
 
-            if (i == CHA)
+            if (i == CHARISMA) {
                 continue; /* exclude cha from this */
+            }
+
             if (stat < 20+race_bonus) {
                 change_attr_value(&pl->contr->orig_stats, i, 1);
                 excess_stat--;
@@ -1672,8 +1724,28 @@ void apply_changes_to_player(object *pl, object *change, int limit_stats) {
     /* first, look for the force object banning changing the
      * face.  Certain races never change face with class.
      */
-    if (object_find_by_name(pl, "NOCLASSFACECHANGE") == NULL) {
-        pl->animation_id = GET_ANIM_ID(change);
+    int has_noclassfacechange = (object_find_by_name(pl, "NOCLASSFACECHANGE") != NULL);
+    const Animations *anim = NULL;
+
+    if (change->anim_suffix) {
+        char buf[MAX_BUF];
+
+        snprintf(buf, MAX_BUF, "%s_%s", pl->animation->name, change->anim_suffix);
+        anim = try_find_animation(buf);
+        if (anim) {
+            pl->animation = anim;
+            pl->anim_speed = -1;
+            CLEAR_FLAG(pl, FLAG_ANIMATE);
+            animate_object(pl, pl->facing);
+        }
+    }
+    /* Check for anim == -1 so that we can override specific class faces for races.
+     * This allows us to have custom class faces on the races that lack noclassfacechange
+     *
+     * Daniel Hawkins 2020-09-08
+     */
+    if ((!has_noclassfacechange) && anim == 0) {
+        pl->animation = GET_ANIM(change);
         pl->face = change->face;
 
         if (QUERY_FLAG(change, FLAG_ANIMATE))
@@ -1682,19 +1754,6 @@ void apply_changes_to_player(object *pl, object *change, int limit_stats) {
             CLEAR_FLAG(pl, FLAG_ANIMATE);
     }
 
-    if (change->anim_suffix) {
-        char buf[MAX_BUF];
-        int anim;
-
-        snprintf(buf, MAX_BUF, "%s_%s", animations[pl->animation_id].name, change->anim_suffix);
-        anim = try_find_animation(buf);
-        if (anim) {
-            pl->animation_id = anim;
-            pl->anim_speed = -1;
-            CLEAR_FLAG(pl, FLAG_ANIMATE);
-            animate_object(pl, pl->facing);
-        }
-    }
     /* Hard coding in class name is a horrible idea - lets
      * use the supported mechanism for this
      */
@@ -1703,10 +1762,6 @@ void apply_changes_to_player(object *pl, object *change, int limit_stats) {
 
 }
 
-void legacy_apply_container(object *op, object *sack) {
-    apply_container(op, sack);
-}
-
 /**
  * Checks for general apply restrictions (no body, prohibited by god, conflicts
  * with other items, etc.)
