@@ -1,29 +1,15 @@
 /*
- * static char *rcsid_attack_c =
- *   "$Id: attack.c 15075 2011-09-10 12:47:41Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail to crossfire-devel@real-time.com
-*/
 
 /**
  * @file
@@ -32,17 +18,17 @@
  * clean functions. Are all parameters required? Seems quite a mess to send damage/wc etc in attack_ob_simple().
  */
 
+#include "global.h"
+
 #include <assert.h>
-#include <global.h>
-#include <living.h>
-#include <material.h>
-#include <skills.h>
+#include <stdlib.h>
+#include <string.h>
 
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-
-#include <sounds.h>
+#include "living.h"
+#include "material.h"
+#include "skills.h"
+#include "sounds.h"
+#include "sproto.h"
 
 /*#define ATTACK_DEBUG*/
 
@@ -84,6 +70,16 @@ static void cancellation(object *op) {
     }
 }
 
+static void object_get_materialtype(object* op, materialtype_t** mt) {
+    if (op->materialname == NULL) {
+        for (*mt = materialt; *mt != NULL && (*mt)->next != NULL; *mt = (*mt)->next) {
+            if (op->material & (*mt)->material)
+                break;
+        }
+    } else
+        *mt = name_to_material(op->materialname);
+}
+
 /**
  * Checks to make sure the item actually
  * made its saving throw based on the tables.  It does not take
@@ -101,15 +97,8 @@ static void cancellation(object *op) {
  */
 static int did_make_save_item(object *op, int type, object *originator) {
     int i, roll, saves = 0, attacks = 0, number;
-    materialtype_t *mt;
-
-    if (op->materialname == NULL) {
-        for (mt = materialt; mt != NULL && mt->next != NULL; mt = mt->next) {
-            if (op->material&mt->material)
-                break;
-        }
-    } else
-        mt = name_to_material(op->materialname);
+    materialtype_t* mt;
+    object_get_materialtype(op, &mt);
     if (mt == NULL)
         return TRUE;
     roll = rndm(1, 20);
@@ -132,8 +121,6 @@ static int did_make_save_item(object *op, int type, object *originator) {
         return TRUE;
     if (roll == 20)
         return TRUE;
-    if (roll == 1)
-        return FALSE;
 
     for (number = 0; number < NROFATTACKS; number++) {
         i = 1<<number;
@@ -155,6 +142,31 @@ static int did_make_save_item(object *op, int type, object *originator) {
     return TRUE;
 }
 
+static void put_in_icecube(object* op, object* originator) {
+    archetype* at = find_archetype("icecube");
+    if (at == NULL)
+        return;
+    op = stop_item(op);
+    if (op == NULL)
+        return;
+
+    // Put in existing icecube if one exists, otherwise make one
+    object* tmp = map_find_by_archetype(op->map, op->x, op->y, at);
+    if (tmp == NULL) {
+        tmp = arch_to_object(at);
+        /* This was in the old (pre new movement code) -
+         * icecubes have slow_move set to 1 - don't want
+         * that for ones we create.
+         */
+        tmp->move_slow_penalty = 0;
+        tmp->move_slow = 0;
+        object_insert_in_map_at(tmp, op->map, originator, 0, op->x, op->y);
+    }
+    if (!QUERY_FLAG(op, FLAG_REMOVED))
+        object_remove(op);
+    (void)object_insert_in_ob(op, tmp);
+}
+
 /**
  * Object is attacked with some attacktype (fire, ice, ...).
  * Calls did_make_save_item().  It then performs the
@@ -167,16 +179,42 @@ static int did_make_save_item(object *op, int type, object *originator) {
  * @param originator
  * what is attacking.
  */
-void save_throw_object(object *op, uint32 type, object *originator) {
+void save_throw_object(object *op, uint32_t type, object *originator) {
+    int dam;
+
     if (!did_make_save_item(op, type, originator)) {
         object *env = op->env;
         int x = op->x, y = op->y;
         mapstruct *m = op->map;
+        /* For use with burning off equipped items */
+        int weight = op->weight;
 
         op = stop_item(op);
         if (op == NULL)
             return;
 
+        /*
+         * If this object is a transport and has players in it, make them disembark.
+         */
+        if (op->type == TRANSPORT && op->inv) {
+            if (op->map == NULL) {
+                LOG(llevError, "Transport %s not on a map but with an item %s in it?\n", op->name, op->inv->name);
+            } else {
+                char name[MAX_BUF];
+                query_name(op, name, sizeof(name));
+                FOR_INV_PREPARE(op, inv) {
+                    if (inv->contr) {
+                        draw_ext_info_format(NDI_UNIQUE, 0, inv, MSG_TYPE_APPLY, MSG_TYPE_APPLY_UNAPPLY,
+                            "You are expelled from the %s during its destruction.",
+                            name);
+                        inv->contr->transport = NULL;
+                    }
+                }
+                FOR_INV_FINISH();
+            }
+        }
+
+
         /* Set off runes in the inventory of the object being destroyed. */
         FOR_INV_PREPARE(op, inv)
             if (inv->type == RUNE)
@@ -226,6 +264,29 @@ void save_throw_object(object *op, uint32 type, object *originator) {
         }
         if (type&(AT_FIRE|AT_ELECTRICITY)) {
             if (env) {
+                /* Check to see if the item being burnt is being worn */
+                if (QUERY_FLAG(op, FLAG_APPLIED)) {
+                    /* if the object is applied, it should be safe to assume env is a player or creature. */
+                    if (env->resist[ATNR_FIRE] < 100)
+                        /* Should the message type be something different? */
+                        draw_ext_info_format(NDI_RED, 0, env, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                            "OUCH! It burns!");
+                    else
+                        draw_ext_info_format(NDI_UNIQUE, 0, env, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
+                            "Despite the flame, you feel nothing.");
+                    /* burning off an item causes 1 point of fire damage for every kilogram of mass the item has */
+                    dam = weight / 1000 * (100 - env->resist[ATNR_FIRE]) / 100;
+                    /* Double the damage on cursed items */
+                    if (QUERY_FLAG(op, FLAG_CURSED))
+                        dam *= 2;
+                    /* Triple the damage on damned items. A cursed and damned item would thus inflict 6x damage. */
+                    if (QUERY_FLAG(op, FLAG_DAMNED))
+                        dam *= 3;
+                    env->stats.hp -= dam;
+                    /* You die at -1, not 0 */
+                    if (env->stats.hp < 0)
+                        kill_player(env, NULL);
+                }
                 op = create_archetype("burnout");
                 op->x = env->x,
                 op->y = env->y;
@@ -237,33 +298,9 @@ void save_throw_object(object *op, uint32 type, object *originator) {
         return;
     }
     /* The value of 50 is arbitrary. */
-    if (type&AT_COLD
-    && (op->resist[ATNR_COLD] < 50)
-    && !QUERY_FLAG(op, FLAG_NO_PICK)
-    && (RANDOM()&2)) {
-        object *tmp;
-        archetype *at = find_archetype("icecube");
-
-        if (at == NULL)
-            return;
-        op = stop_item(op);
-        if (op == NULL)
-            return;
-        tmp = map_find_by_archetype(op->map, op->x, op->y, at);
-        if (tmp == NULL) {
-            tmp = arch_to_object(at);
-            /* This was in the old (pre new movement code) -
-             * icecubes have slow_move set to 1 - don't want
-             * that for ones we create.
-             */
-            tmp->move_slow_penalty = 0;
-            tmp->move_slow = 0;
-            object_insert_in_map_at(tmp, op->map, originator, 0, op->x, op->y);
-        }
-        if (!QUERY_FLAG(op, FLAG_REMOVED))
-            object_remove(op);
-        (void)object_insert_in_ob(op, tmp);
-        return;
+    if (type & AT_COLD && (op->resist[ATNR_COLD] < 50) &&
+        !QUERY_FLAG(op, FLAG_NO_PICK) && (RANDOM() & 2)) {
+        put_in_icecube(op, originator);
     }
 }
 
@@ -282,9 +319,9 @@ void save_throw_object(object *op, uint32 type, object *originator) {
  * @return
  * 1 if it hits something, 0 otherwise.
  */
-int hit_map(object *op, int dir, uint32 type, int full_hit) {
+int hit_map(object *op, int dir, uint32_t type, int full_hit) {
     mapstruct *map;
-    sint16 x, y;
+    int16_t x, y;
     int retflag = 0;  /* added this flag..  will return 1 if it hits a monster */
     tag_t op_tag;
 
@@ -564,6 +601,11 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
         snprintf(buf2, sizeof(buf2), "hits");
     }
 
+    if (dam > 0) {
+        if (hitter->chosen_skill)
+            play_sound_map(SOUND_TYPE_HIT, hitter, 0, hitter->chosen_skill->name);
+    }
+
     /* bail out if a monster is casting spells */
     owner = object_get_owner(hitter);
     if (hitter->type != PLAYER && (owner == NULL || owner->type != PLAYER))
@@ -582,16 +624,6 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
             snprintf(buf, sizeof(buf), "%s's %s %s you.", owner->name, hitter->name, buf2);
         else {
             snprintf(buf, sizeof(buf), "%s%s you.", hitter->name, buf2);
-            if (dam != 0) {
-                if (hitter->chosen_skill)
-                    play_sound_player_only(op->contr, SOUND_TYPE_HIT_BY, op, 0, hitter->chosen_skill->name);
-                else if (dam < 10)
-                    play_sound_player_only(op->contr, SOUND_TYPE_HIT_BY, op, 0, "low");
-                else if (dam < 20)
-                    play_sound_player_only(op->contr, SOUND_TYPE_HIT_BY, op, 0, "medium");
-                else
-                    play_sound_player_only(op->contr, SOUND_TYPE_HIT_BY, op, 0, "high");
-            }
         }
         draw_ext_info(NDI_BLACK, 0, op, MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT, buf);
     } /* end of player hitting player */
@@ -600,16 +632,6 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
     /*if(hitter->type == PLAYER && rndm(0, 2) == 0) {*/
     if (hitter->type == PLAYER) {
         snprintf(buf, sizeof(buf), "You %s.", buf1);
-        if (dam != 0) {
-            if (hitter->chosen_skill)
-                play_sound_player_only(hitter->contr, SOUND_TYPE_HIT, hitter, 0, hitter->chosen_skill->name);
-            else if (dam < 10)
-                play_sound_player_only(hitter->contr, SOUND_TYPE_HIT, hitter, 0, "low");
-            else if (dam < 20)
-                play_sound_player_only(hitter->contr, SOUND_TYPE_HIT, hitter, 0, "medium");
-            else
-                play_sound_player_only(hitter->contr, SOUND_TYPE_HIT, hitter, 0, "high");
-        }
         draw_ext_info(NDI_BLACK, 0, hitter, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
                       buf);
     } else if (owner != NULL && owner->type == PLAYER) {
@@ -618,12 +640,16 @@ static void attack_message(int dam, int type, object *op, object *hitter) {
         && (hitter->subtype == SP_EXPLOSION || hitter->subtype == SP_BULLET || hitter->subtype == SP_CONE)) {
             i = 4;
             map = hitter->map;
-            if (out_of_map(map, hitter->x, hitter->y))
+            if (OUT_OF_REAL_MAP(map, hitter->x, hitter->y))
                 return;
             FOR_MAP_PREPARE(map, hitter->x, hitter->y, next)
                 if (next->type == SPELL_EFFECT
-                && (next->subtype == SP_EXPLOSION || next->subtype == SP_BULLET || next->subtype == SP_CONE))
+                && (next->subtype == SP_EXPLOSION || next->subtype == SP_BULLET || next->subtype == SP_CONE)) {
                     i *= 3;
+                    if (i > 10000)
+                        /* no need to test more, and avoid overflows */
+                        break;
+                }
             FOR_MAP_FINISH();
             if (i < 0)
                 return;
@@ -718,31 +744,37 @@ static void thrown_item_effect(object *, object *);
  */
 static int attack_ob_simple(object *op, object *hitter, int base_dam, int base_wc) {
     int simple_attack, roll, dam;
-    uint32 type;
+    uint32_t type;
     tag_t op_tag, hitter_tag;
+    const char *anim_suffix = NULL;
 
     if (get_attack_mode(&op, &hitter, &simple_attack))
         return 1;
 
-    /* Lauwenmark: Handle for plugin attack event */
-    if (execute_event(op, EVENT_ATTACK, hitter, hitter->current_weapon ? hitter->current_weapon : hitter, NULL, SCRIPT_FIX_ALL) != 0)
-        return 0;
-
     /* Lauwenmark: This is used to handle script_weapons with weapons.
      * Only used for players.
      */
     if (hitter->type == PLAYER) {
         if (hitter->current_weapon != NULL) {
             /* Lauwenmark: Handle for plugin attack event */
-            if (execute_event(hitter->current_weapon, EVENT_ATTACK,
+            if (execute_event(hitter->current_weapon, EVENT_ATTACKS,
                               hitter, op, NULL, SCRIPT_FIX_ALL) != 0)
                 return 0;
-            if (hitter->current_weapon->anim_suffix)
-                apply_anim_suffix(hitter, hitter->current_weapon->anim_suffix);
-        } else if (hitter->chosen_skill &&  hitter->chosen_skill->anim_suffix)
-            /* if no weapon, then skill (karate, wraith feed) attack */
-            apply_anim_suffix(hitter, hitter->chosen_skill->anim_suffix);
+        }
     }
+
+    if (hitter->current_weapon) {
+        anim_suffix = hitter->current_weapon->anim_suffix;
+    } else if (hitter->chosen_skill) {
+        anim_suffix = hitter->chosen_skill->anim_suffix;
+    }
+
+    if (!anim_suffix) {
+        anim_suffix = "attack";
+    }
+    apply_anim_suffix(hitter, anim_suffix);
+    
+
     op_tag = op->count;
     hitter_tag = hitter->count;
     /*
@@ -857,6 +889,7 @@ static int attack_ob_simple(object *op, object *hitter, int base_dam, int base_w
             return 0;
         }
     } else {/* end of if hitter hit op */
+        play_sound_map(SOUND_TYPE_HIT, hitter, 0, "miss");
         dam = 0; /* if we missed, dam=0 */
     }
 
@@ -919,8 +952,8 @@ static int stick_arrow(object *op, object *tmp) {
 object *hit_with_arrow(object *op, object *victim) {
     object *container, *hitter;
     int hit_something = 0;
-    tag_t victim_tag, hitter_tag;
-    sint16 victim_x, victim_y;
+    tag_t victim_tag, hitter_tag, container_tag;
+    int16_t victim_x, victim_y;
     mapstruct *victim_map;
     const char *old_skill = NULL;
 
@@ -947,6 +980,7 @@ object *hit_with_arrow(object *op, object *victim) {
          * might be called until this THROWN_OBJ is either reassembled or
          * removed at the end of this function must be able to deal with empty
          * THROWN_OBJs. */
+        container_tag = container->count;
     }
 
     /* Try to hit victim */
@@ -960,7 +994,7 @@ object *hit_with_arrow(object *op, object *victim) {
        items, but there is no information about the fact it was
        thrown
     */
-    if (execute_event(op, EVENT_ATTACK, hitter, victim, NULL, SCRIPT_FIX_ALL) == 0) {
+    if (execute_event(op, EVENT_ATTACKS, hitter, victim, NULL, SCRIPT_FIX_ALL) == 0) {
         /*
          * temporary set the hitter's skill to the one associated with the
          * throw wrapper. This is needed to that thrower gets it's xp at the
@@ -1001,14 +1035,18 @@ object *hit_with_arrow(object *op, object *victim) {
             if (hitter == NULL)
                 return NULL;
         } else {
-            object_remove(container);
-            object_free_drop_inventory(container);
+            if(!object_was_destroyed(container, container_tag)) {
+                object_remove(container);
+                object_free_drop_inventory(container);
+            }
         }
 
         /* Try to stick arrow into victim */
         if (!object_was_destroyed(victim, victim_tag)
-        && stick_arrow(hitter, victim))
+                && stick_arrow(hitter, victim)) {
+            object_set_owner(hitter, NULL);
             return NULL;
+        }
 
         /* Else try to put arrow on victim's map square
         * remove check for P_WALL here.  If the arrow got to this
@@ -1019,10 +1057,12 @@ object *hit_with_arrow(object *op, object *victim) {
         */
         if (victim_x != hitter->x || victim_y != hitter->y) {
             object_remove(hitter);
+            object_set_owner(hitter, NULL);
             object_insert_in_map_at(hitter, victim_map, hitter, 0, victim_x, victim_y);
         } else {
             /* Else leave arrow where it is */
             object_merge(hitter, NULL);
+            object_set_owner(hitter, NULL);
         }
         return NULL;
     }
@@ -1060,7 +1100,9 @@ static void tear_down_wall(object *op) {
         }
         return; /* no animations, so nothing more to do */
     }
+    assert(op->stats.maxhp > 0);
     perc = NUM_ANIMATIONS(op)-((int)NUM_ANIMATIONS(op)*op->stats.hp)/op->stats.maxhp;
+
     if (perc >= (int)NUM_ANIMATIONS(op))
         perc = NUM_ANIMATIONS(op)-1;
     else if (perc < 1)
@@ -1118,7 +1160,7 @@ static void scare_creature(object *target, object *hitter) {
  * @return
  * damage to actually do.
  */
-static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 attacknum) {
+static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32_t attacknum) {
     int doesnt_slay = 1;
     char name_hitter[MAX_BUF], name_op[MAX_BUF];
 
@@ -1133,26 +1175,32 @@ static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 a
         return 0;
     }
 
-    if (hitter->current_weapon && hitter->current_weapon->discrete_damage != NULL)
-        dam = hitter->current_weapon->discrete_damage[attacknum];
-    else if (hitter->discrete_damage != NULL)
-        dam = hitter->discrete_damage[attacknum];
-
     /* AT_INTERNAL is supposed to do exactly dam.  Put a case here so
      * people can't mess with that or it otherwise get confused.  */
     if (attacknum == ATNR_INTERNAL)
         return dam;
 
     if (hitter->slaying) {
-        if ((op->race != NULL && strstr(hitter->slaying, op->race))
-        || (op->arch && op->arch->name != NULL && strstr(op->arch->name, hitter->slaying))) {
+        // Look for the race as one of many that can be listed.
+        if ((op->race != NULL && strstr(op->race, hitter->slaying))
+        /*
+         * Since we can do multiple races, we should be able to avoid the arch check.
+         * This gives a more flexible race system, so skeletal mages can be a type of skeleton and such
+        || (op->arch && op->arch->name != NULL && strstr(op->arch->name, hitter->slaying))
+        */) {
             doesnt_slay = 0;
             dam *= 3;
         }
     }
 
     /* Adjust the damage for resistance. Note that neg. values increase damage. */
-    if (op->resist[attacknum]) {
+    /*
+     * Skip lifestealing here, because it undergoes a more specific resistance scaling
+     * in its own section that involves the better of drain/life stealing resistance
+     *
+     * Daniel Hawkins 2018-05-31
+     */
+    if (attacknum != ATNR_LIFE_STEALING && op->resist[attacknum]) {
         /* basically:  dam = dam*(100-op->resist[attacknum])/100;
          * in case 0>dam>1, we try to "simulate" a float value-effect */
         dam *= (100-op->resist[attacknum]);
@@ -1319,7 +1367,7 @@ static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 a
                  */
                 if (!op_on_battleground(hitter, NULL, NULL, NULL) && !QUERY_FLAG(op, FLAG_WAS_WIZ)) {
                     object *owner = object_get_owner(hitter);
-                    sint64 orig_exp = op->stats.exp;
+                    int64_t orig_exp = op->stats.exp;
 
                     change_exp(op, -op->stats.exp/rate, NULL, 0);
 
@@ -1378,14 +1426,11 @@ static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 a
         break;
 
     case ATNR_COUNTERSPELL:
-        query_name(op, name_op, MAX_BUF);
-        query_name(hitter, name_hitter, MAX_BUF);
-        LOG(llevError, "%s was hit by %s with counterspell attack.\n", name_op, name_hitter);
         dam = 0;
-        /* This should never happen.  Counterspell is handled
-         * seperately and filtered out.  If this does happen,
-         * Counterspell has no effect on anything but spells, so it
-         * does no damage. */
+        /* While counterspell is handled separately and filtered out when it
+         * moves, players can still step on a square that has an active
+         * counterspell. When this happens, do no damage because counterspell
+         * has no effect on anything but spells. */
         break;
 
     case ATNR_HOLYWORD: {
@@ -1437,8 +1482,8 @@ static int hit_with_one_attacktype(object *op, object *hitter, int dam, uint32 a
 
             /* Wraith also get food through life stealing */
             if (is_wraith_pl(hitter)) {
-                if (hitter->stats.food+dam >= 999)
-                    hitter->stats.food = 999;
+                if (hitter->stats.food+dam >= MAX_FOOD)
+                    hitter->stats.food = MAX_FOOD;
                 else
                     hitter->stats.food += dam;
                 fix_object(hitter);
@@ -1544,8 +1589,13 @@ static int kill_object(object *op, int dam, object *hitter) {
             draw_ext_info_format(NDI_UNIQUE, 0, owner, MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_PET_DIED,
                                  "Your pet, the %s, is killed by %s.",
                                  op->name, hitter->name);
-        } else
+        }
+        /*
+         * there can be items both friendly and without any owner, for instance
+         * in various maps, so this isn't an error.
+        else
             LOG(llevError, "BUG: hit_player(): Encountered golem without owner.\n");
+         */
 
         object_remove(op);
         object_free_drop_inventory(op);
@@ -1562,8 +1612,8 @@ static int kill_object(object *op, int dam, object *hitter) {
     if (op_on_battleground(op, NULL, NULL, NULL))
         battleg = 1;
 
-    /* is this player killing?*/
-    if (op->type == PLAYER && owner->type == PLAYER)
+    /* is this player killing? -- Don't count it if you suicide, though. */
+    if (op->type == PLAYER && owner->type == PLAYER && owner != op)
         pk = 1;
 
     /* Player killed something */
@@ -1607,9 +1657,6 @@ static int kill_object(object *op, int dam, object *hitter) {
                                      "You killed %s.",
                                      killed);
             }
-            /* Only play sounds for melee kills */
-            if (hitter->type == PLAYER)
-                play_sound_map(SOUND_TYPE_HIT, owner, 0, "kill");
         }
 
         /* If a player kills another player, not on
@@ -1619,8 +1666,11 @@ static int kill_object(object *op, int dam, object *hitter) {
          * player that the object belonged to - so if you killed another player
          * with spells, pets, whatever, there was no penalty.
          * Changed to make luck penalty configurable in settings.
+		 *
+		 * Simplified comparison since pk is no longer set to 1 if self-kill
+		 *  -- SilverNexus 2017-06-17+
          */
-        if (op->type == PLAYER && owner != op && !battleg)
+        if (pk == 1 && !battleg)
             change_luck(owner, -settings.pk_luck_penalty);
 
         /* This code below deals with finding the appropriate skill
@@ -1642,14 +1692,7 @@ static int kill_object(object *op, int dam, object *hitter) {
          * to.  Make sure skop is an actual skill, and not a skill tool!
          */
         if ((!skop || skop->type != SKILL) && skill) {
-            int i;
-
-            for (i = 0; i < NUM_SKILLS; i++)
-                if (owner->contr->last_skill_ob[i]
-                && !strcmp(owner->contr->last_skill_ob[i]->skill, skill)) {
-                    skop = owner->contr->last_skill_ob[i];
-                    break;
-                }
+            skop = find_applied_skill_by_name(owner, skill);
         }
     } /* Was it a player that hit somethign */
     else {
@@ -1687,11 +1730,12 @@ static int kill_object(object *op, int dam, object *hitter) {
 
     draw_ext_info(NDI_ALL, op->type == PLAYER ? 1 : 10, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
                   kill_message);
+    play_sound_map(SOUND_TYPE_HIT, op, 0, "kill");
 
 
     /* If you didn't kill yourself, and your not the wizard */
     if (owner != op && !QUERY_FLAG(op, FLAG_WAS_WIZ)) {
-        sint64 exp;
+        int64_t exp;
 
         exp = calc_skill_exp(owner, op, skop);
 
@@ -1815,7 +1859,7 @@ int friendly_fire(object *op, object *hitter) {
  * @todo
  * rename to something more meaningful.
  */
-int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit) {
+int hit_player(object *op, int dam, object *hitter, uint32_t type, int full_hit) {
     int maxdam = 0, ndam = 0, magic = (type&AT_MAGIC);
     int maxattacktype, attacknum;
     int body_attack = op->head != NULL;   /* Did we hit op's head? */
@@ -1825,6 +1869,14 @@ int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit) {
     int friendlyfire;
     object *owner;
 
+    /* Lauwenmark: Handle for plugin attack event */
+    if (execute_event(op, EVENT_ATTACKED, hitter, hitter->current_weapon ? hitter->current_weapon : hitter, NULL, SCRIPT_FIX_ALL) != 0)
+        return 0;
+    FOR_INV_PREPARE(op, inv)
+        if (execute_event(inv, EVENT_ATTACKED, hitter, hitter->current_weapon ? hitter->current_weapon : hitter, NULL, SCRIPT_FIX_ALL) != 0)
+            return 0;
+    FOR_INV_FINISH();
+
     if (get_attack_mode(&op, &hitter, &simple_attack))
         return 0;
 
@@ -1970,17 +2022,13 @@ int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit) {
 
     if (!full_hit) {
         archetype *at;
-        int area;
-        int remainder;
-
-        area = 0;
+        unsigned int area = 0;
         for (at = op->arch; at != NULL; at = at->more)
             area++;
-        assert(area > 0);
 
         /* basically: maxdam /= area; we try to "simulate" a float
            value-effect */
-        remainder = 100*(maxdam%area)/area;
+        unsigned int remainder = 100*(maxdam%area)/area;
         maxdam /= area;
         if (RANDOM()%100 < remainder)
             maxdam++;
@@ -2014,7 +2062,7 @@ int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit) {
     /* Eneq(@csd.uu.se): Check to see if monster runs away. */
     if (op->stats.hp >= 0
     && (QUERY_FLAG(op, FLAG_MONSTER) || op->type == PLAYER)
-    && op->stats.hp < (signed short)(((float)op->run_away/(float)100)*(float)op->stats.maxhp)) {
+    && op->stats.hp < (int16_t)((int32_t)op->run_away * op->stats.maxhp / 100)) {
         if (QUERY_FLAG(op, FLAG_MONSTER))
             SET_FLAG(op, FLAG_RUN_AWAY);
         else
@@ -2063,6 +2111,9 @@ int hit_player(object *op, int dam, object *hitter, uint32 type, int full_hit) {
  */
 static void poison_living(object *op, object *hitter, int dam) {
     archetype *at = find_archetype("poisoning");
+    if (at == NULL) {
+        return;
+    }
     object *tmp = arch_present_in_ob(at, op);
     const char *skill;
 
@@ -2131,6 +2182,34 @@ static void poison_living(object *op, object *hitter, int dam) {
         tmp->stats.food++;
 }
 
+int slow_living_by(object *op, const int speed_penalty) {
+    archetype *at = find_archetype("slowness");
+    if (at == NULL) {
+        return 0;
+    }
+    object* tmp = arch_present_in_ob(at, op);
+    int ret;
+    if (tmp == NULL) {
+        tmp = arch_to_object(at);
+        tmp->stats.exp = -speed_penalty;
+        tmp = object_insert_in_ob(tmp, op);
+        ret = 1;
+    }
+    // If we are hitting for more intense slowness, override the old one.
+    else if (tmp->stats.exp > -speed_penalty) {
+        tmp->stats.exp = -speed_penalty;
+        tmp->stats.food -= 3; // But also reduce the duration to compensate.
+        ret = 2;
+    } else {
+        tmp->stats.food++;
+        ret = 3;
+    }
+    SET_FLAG(tmp, FLAG_APPLIED);
+    tmp->speed_left = 0;
+    fix_object(op);
+    return ret;
+}
+
 /**
  * Slow a living thing.
  *
@@ -2142,23 +2221,36 @@ static void poison_living(object *op, object *hitter, int dam) {
  * damage to deal.
  */
 static void slow_living(object *op, object *hitter, int dam) {
-    archetype *at = find_archetype("slowness");
-    object *tmp;
+    // Used to calculate the speed penalty of the slow attempt
+    int speed_penalty;
 
-    if (at == NULL) {
-        LOG(llevError, "Can't find slowness archetype.\n");
-    }
-    tmp = arch_present_in_ob(at, op);
-    if (tmp == NULL) {
-        tmp = arch_to_object(at);
-        tmp = object_insert_in_ob(tmp, op);
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
+    /**
+     * Give slow attacks some oomph -- always speed -1 was REALLY weak
+     *
+     * Daniel Hawkins 2018-05-31
+     */
+    // Higher level slow effects make you slower.
+    speed_penalty = hitter->level - op->level + random_roll(1, 5, hitter, PREFER_LOW);
+    // Resistance to slow will also affect how much you are slowed by.
+    speed_penalty = speed_penalty * (100-op->resist[ATNR_SLOW]) / 100;
+    // Make sure we actually have a penalty amount. We can assume that op is not immune in this method.
+    if (speed_penalty < 1)
+        speed_penalty = 1;
+    else if (speed_penalty > 30) // Data barrier for stats.exp is 127, but that is huge slowness. Pick something less than that.
+        speed_penalty = 30;
+    switch (slow_living_by(op, speed_penalty)) {
+    case 1:
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE,
+                      MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
                       "The world suddenly moves very fast!");
-    } else
-        tmp->stats.food++;
-    SET_FLAG(tmp, FLAG_APPLIED);
-    tmp->speed_left = 0;
-    fix_object(op);
+        break;
+    case 2:
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE,
+                      MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
+                      "The world moves even faster!");
+        break;
+
+    }
 }
 
 /**
@@ -2269,6 +2361,20 @@ void paralyze_living(object *op, int dam) {
     max = (100-op->resist[ATNR_PARALYZE])/2;
     if (op->speed_left < -(FABS(op->speed)*max))
         op->speed_left = (float)-(FABS(op->speed)*max);
+    // Set a paralyze flag and print a message to a player if the flag isn't set;
+    // this tells the player that he/she has been hit by a paralysis attack.
+    if (!QUERY_FLAG(op, FLAG_PARALYZED)) {
+        SET_FLAG(op, FLAG_PARALYZED);
+        if (op->type == PLAYER)
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_BAD_EFFECT_START,
+                      "You limbs stop moving!");
+    }
+    /* If the flag is already set, then the paralysis is merely extended.
+     * At this point, we do nothing.
+     * It may be worthwhile to give players another message on paralysis extensions.
+     *
+     * Daniel Hawkins 2017-08-22
+     */
 }
 
 /**
@@ -2292,41 +2398,53 @@ void paralyze_living(object *op, int dam) {
  * damage to deal, will contain computed damage or 0 if strike failed.
  */
 static void deathstrike_living(object *op, object *hitter, int *dam) {
-    int atk_lev, def_lev, kill_lev;
+    int atk_lev, def_lev, kill_lev, roll;
 
     if (hitter->slaying) {
         if (!((QUERY_FLAG(op, FLAG_UNDEAD) && strstr(hitter->slaying, undead_name))
-        || (op->race && strstr(hitter->slaying, op->race))))
+        || (op->race && strstr(op->race, hitter->slaying))))
+        {
+            *dam = 0; // Don't do damage if we aren't deathstriking them.
             return;
+        }
     } else
         if (QUERY_FLAG(op, FLAG_UNDEAD))
+        {
+            *dam = 0; // Don't do damage if we aren't deathstriking them.
             return;
+        }
 
     def_lev = op->level;
     if (def_lev < 1) {
-        LOG(llevError, "BUG: arch %s, name %s with level < 1\n", op->arch->name, op->name);
+        LOG(llevError, "deathstrike_living: arch %s (%s in %s at %d, %d) has level < 1\n", op->arch->name, op->name, op->map->name, op->x, op->y);
         def_lev = 1;
     }
-    atk_lev = (hitter->chosen_skill ? hitter->chosen_skill->level : hitter->level)/2;
+    /*
+     * Redo this calculation -- you could essentially only kill creatures less than half your level,
+     * making death extremely weak at high levels.
+     * Refactoring to use a d20 roll with a fairly high DC (still dependent on level)
+     * Also, toss in a resistance-based hit modifier.
+     * Higher resistance requires higher levels in order to kill with a death attack.
+     *
+     * Daniel Hawkins 2018-05-21
+     */
+    atk_lev = (hitter->chosen_skill ? hitter->chosen_skill->level : hitter->level);
     /* LOG(llevDebug, "Deathstrike - attack level %d, defender level %d\n", atk_lev, def_lev); */
 
-    if (atk_lev >= def_lev) {
-        kill_lev = random_roll(0, atk_lev-1, hitter, PREFER_HIGH);
+    roll = random_roll(1, 20, hitter, PREFER_HIGH);
+    kill_lev = roll - 18 + atk_lev; // Use 19+ as a kill for same level and no resistance;
+    kill_lev = kill_lev * (100 - op->resist[ATNR_DEATH]) / 100; // Do not compress to *= for roundoff reasons.
 
-        /* Note that the below effectively means the ratio of the atk vs
-         * defener level is important - if level 52 character has very little
-         * chance of killing a level 50 monster.  This should probably be
-         * redone.
+    // If we hit, then kill them. Otherwise, damage is 0.
+    if (kill_lev > def_lev) {
+        *dam = op->stats.hp+10; /* take all hp. they can still save for 1/2 */
+        /* I think this doesn't really do much.  Because of
+         * integer rounding, this only makes any difference if the
+         * attack level is double the defender level.
          */
-        if (kill_lev >= def_lev) {
-            *dam = op->stats.hp+10; /* take all hp. they can still save for 1/2 */
-            /* I think this doesn't really do much.  Because of
-             * integer rounding, this only makes any difference if the
-             * attack level is double the defender level.
-             */
-            *dam *= kill_lev/def_lev;
-        }
-    } else {
+        *dam *= kill_lev/def_lev;
+    }
+    else {
         *dam = 0;  /* no harm done */
     }
 }
