@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_c_misc_c =
- *   "$Id: c_misc.c 15823 2011-11-20 10:29:06Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Various functions. Handles misc. input request - things like hash table, malloc, maps,
@@ -34,17 +19,20 @@
 
 #define WANT_UNARMED_SKILLS
 
-#include <global.h>
-#include <loader.h>
+#include "global.h"
+
+#include <assert.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
 
 #undef SS_STATISTICS
-#include <shstr.h>
-
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <assert.h>
-#include <sounds.h>
+#include "loader.h"
+#include "shstr.h"
+#include "sounds.h"
+#include "sproto.h"
+#include "version.h"
 
 /**
  * This is the 'maps' command.
@@ -55,34 +43,75 @@
  * optional substring to search for.
  */
 void map_info(object *op, const char *search) {
-    mapstruct *m;
-    char map_path[MAX_BUF];
-    long sec = seconds();
-
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-                         i18n(op, "Current time is: %02ld:%02ld:%02ld."),
-                         (sec%86400)/3600, (sec%3600)/60, sec%60);
-
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-                  i18n(op, "[fixed]Path               Pl PlM IM   TO Dif Reset"));
-
-    for (m = first_map; m != NULL; m = m->next) {
-        if (*search != '\0' && strstr(m->path, search) == NULL)
-            continue;   /* Skip unwanted maps */
-
-        /* Print out the last 18 characters of the map name... */
-        if (strlen(m->path) <= 18)
-            strcpy(map_path, m->path);
-        else
-            strcpy(map_path, m->path+strlen(m->path)-18);
-
+    if ( *search != 0 ) {
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-                             i18n(op, "[fixed]%-18.18s %2d %2d   %1d %4d %2d  %02d:%02d:%02d"),
-                             map_path, m->players, players_on_map(m, FALSE),
-                             m->in_memory, m->timeout, m->difficulty,
-                             (MAP_WHEN_RESET(m)%86400)/3600, (MAP_WHEN_RESET(m)%3600)/60,
-                             MAP_WHEN_RESET(m)%60);
+                      i18n(op, "Maps matching search: '%s'"), search);
     }
+    if (QUERY_FLAG(op, FLAG_WIZ)) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+                      i18n(op, "[fixed]Path             Reset In (HH:MM:SS) Pl IM   TO"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+                      i18n(op, "[fixed]Path             Reset In (HH:MM)"));
+    }
+
+    /* Allow a comma-separate list of search strings; more complicated because of the const */
+    char *to_be_freed = NULL;
+    char *search_array[64];
+    int search_array_count = 0;
+    if ( search[0] ) {
+        to_be_freed = strdup(search);
+        if ( !to_be_freed ) {
+            search_array[0] = to_be_freed;
+            search_array_count = 1;
+        } else {
+            search_array_count = split_string(to_be_freed,search_array,64,',');
+        }
+    }
+
+    for (mapstruct *m = first_map; m != NULL; m = m->next) {
+        bool match = TRUE;
+        if ( search_array_count ) {
+            match = FALSE;
+            for (int i=0; i<search_array_count; ++i) {
+                if ( strstr(m->path,search_array[i]) ) {
+                    match=TRUE;
+                    break;
+                }
+            }
+        }
+        if ( !match ) continue;   /* Skip unwanted maps */
+
+        /* Print out the last 26 characters of the map name... */
+        char map_path[MAX_BUF];
+        if (strlen(m->path) <= 26) {
+            strcpy(map_path, m->path);
+        } else {
+            safe_strncpy(map_path, m->path + strlen(m->path) - 26, sizeof(map_path));
+        }
+
+        uint32_t ttr = MAP_WHEN_RESET(m) - seconds() > 0 ? MAP_WHEN_RESET(m) - seconds() : 0;
+        if ( !m->reset_timeout && !m->unique ) set_map_reset_time(m);
+        if (m->players) {
+            ttr = (m->unique ? 0 : m->reset_timeout) + 300;
+        }
+        else if (m->timeout) {
+            ttr = (m->unique ? 0 : m->reset_timeout) + m->timeout;
+        }
+        const uint32_t hh = ttr/3600, mm = (ttr%3600)/60, ss = ttr%60;
+        if (QUERY_FLAG(op, FLAG_WIZ)) {
+            draw_ext_info_format(
+                NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+                i18n(op, "[fixed]%-26.26s %2d:%02d:%02d  %2d %2d %4d"),
+                map_path, hh, mm, ss, m->players, m->in_memory, m->timeout);
+        } else {
+            draw_ext_info_format(
+                NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
+                i18n(op, "[fixed]%-26.26s %2d:%02d%s"), map_path, hh, mm,
+                m->players ? " (in use)" : "");
+        }
+    }
+    if ( to_be_freed ) free( to_be_freed );
 }
 
 /**
@@ -236,7 +265,7 @@ static void malloc_info(object *op) {
 
     for (m = first_map, nrofmaps = 0; m != NULL; m = m->next, nrofmaps++)
         if (m->in_memory == MAP_IN_MEMORY) {
-            mapmem += MAP_WIDTH(m)*MAP_HEIGHT(m)*(sizeof(object *)+sizeof(MapSpace));
+            mapmem += map_size(m) * (sizeof(object *) + sizeof(MapSpace));
             nrm++;
         }
 
@@ -245,20 +274,41 @@ static void malloc_info(object *op) {
                          sizeof(object), sizeof(player), sizeof(mapstruct));
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
-                         i18n(op, "[fixed]%4d used objects:    %8d"),
-                         ob_used, i = (ob_used*sizeof(object)));
+                         i18n(op, "[fixed]Objects:"));
 
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
+                         i18n(op, "[fixed]%6d used"), ob_used);
+
+    if (ob_used != nrofallocobjects - nroffreeobjects) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
+                             i18n(op, "[fixed]      (used list mismatch: %d)"),
+                             nrofallocobjects - nroffreeobjects);
+    }
+
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
+                         i18n(op, "[fixed]%6d free (%.2f%% of %d allocated)"),
+                         ob_free, (float)ob_free / nrofallocobjects * 100, nrofallocobjects);
+
+    if (ob_free != nroffreeobjects) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
+                             i18n(op, "[fixed]      (free list mismatch: %d)"),
+                             nroffreeobjects);
+    }
+
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
+                         i18n(op, "[fixed]%6d on active list"),
+                         object_count_active());
+
+    i = (ob_used*sizeof(object));
     sum_used += i;
     sum_alloc += i;
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
-                         i18n(op, "[fixed]%4d free objects:    %8d"),
-                         ob_free, i = (ob_free*sizeof(object)));
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
-                         i18n(op, "[fixed]%4d active objects:  %8d"),
-                         object_count_active(), 0);
+                         i18n(op, "[fixed]     object total: %11d"), i);
 
+    i = (ob_free*sizeof(object));
     sum_alloc += i;
+
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
                          i18n(op, "[fixed]%4d players:         %8d"),
                          players, i = (players*sizeof(player)));
@@ -289,7 +339,7 @@ static void malloc_info(object *op) {
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
                          i18n(op, "[fixed]%4d animations:      %8d"),
-                         anims, i = (anims*sizeof(uint16)));
+                         anims, i = (anims*sizeof(uint16_t)));
 
     sum_alloc += i;
     sum_used += i;
@@ -361,7 +411,7 @@ void current_region_info(object *op) {
         return;
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-                         i18n(op, "You are in %s. \n %s"),
+                         i18n(op, "You are in %s.\n%s"),
                          get_region_longname(r), get_region_msg(r));
 }
 
@@ -494,7 +544,7 @@ static int name_cmp(const chars_names *c1, const chars_names *c2) {
  */
 void list_players(object *op, region *reg, partylist *party) {
     player *pl;
-    uint16 i;
+    uint16_t i;
     char *format;
     int num_players = 0, num_wiz = 0, num_afk = 0, num_bot = 0;
     chars_names *chars = NULL;
@@ -641,7 +691,7 @@ void display_who_entry(object *op, player *pl, const char *format) {
 void get_who_escape_code_value(char *return_val, int size, const char letter, player *pl) {
     switch (letter) {
     case 'N':
-        snprintf(return_val, size, "%s", pl->ob->name);
+        strlcpy(return_val, pl->ob->name, size);
         break;
 
     case 't':
@@ -657,7 +707,7 @@ void get_who_escape_code_value(char *return_val, int size, const char letter, pl
         break;
 
     case 'h':
-        snprintf(return_val, size, "%s", pl->peaceful ? "" : " <Hostile>");
+        strlcpy(return_val, pl->peaceful ? "" : " <Hostile>", size);
         break;
 
     case 'l':
@@ -665,35 +715,35 @@ void get_who_escape_code_value(char *return_val, int size, const char letter, pl
         break;
 
     case 'd':
-        snprintf(return_val, size, "%s", (QUERY_FLAG(pl->ob, FLAG_WIZ) ? " <WIZ>" : ""));
+        strlcpy(return_val, (QUERY_FLAG(pl->ob, FLAG_WIZ) ? " <WIZ>" : ""), size);
         break;
 
     case 'a':
-        snprintf(return_val, size, "%s", (QUERY_FLAG(pl->ob, FLAG_AFK) ? " <AFK>" : ""));
+        strlcpy(return_val, (QUERY_FLAG(pl->ob, FLAG_AFK) ? " <AFK>" : ""), size);
         break;
 
     case 'b':
-        snprintf(return_val, size, "%s", (pl->socket.is_bot == 1) ? " <BOT>" : "");
+        strlcpy(return_val, (pl->socket.is_bot == 1) ? " <BOT>" : "", size);
         break;
 
     case 'm':
-        snprintf(return_val, size, "%s", pl->ob->map->path);
+        strlcpy(return_val, pl->ob->map->path, size);
         break;
 
     case 'M':
-        snprintf(return_val, size, "%s", pl->ob->map->name ? pl->ob->map->name : "Untitled");
+        strlcpy(return_val, pl->ob->map->name ? pl->ob->map->name : "Untitled", size);
         break;
 
     case 'r':
-        snprintf(return_val, size, "%s", get_name_of_region_for_map(pl->ob->map));
+        strlcpy(return_val, get_name_of_region_for_map(pl->ob->map), size);
             break;
 
     case 'R':
-        snprintf(return_val, size, "%s", get_region_longname(get_region_by_map(pl->ob->map)));
+        strlcpy(return_val, get_region_longname(get_region_by_map(pl->ob->map)), size);
         break;
 
     case 'i':
-        snprintf(return_val, size, "%s", pl->socket.host);
+        strlcpy(return_val, pl->socket.host, size);
         break;
 
     case '%':
@@ -715,7 +765,7 @@ void get_who_escape_code_value(char *return_val, int size, const char letter, pl
  * unused.
  */
 void command_afk(object *op, const char *params) {
-    if QUERY_FLAG(op, FLAG_AFK) {
+    if (QUERY_FLAG(op, FLAG_AFK)) {
         CLEAR_FLAG(op, FLAG_AFK);
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
                       i18n(op, "You are no longer AFK"));
@@ -1010,7 +1060,8 @@ void command_printlos(object *op, const char *params) {
  * unused.
  */
 void command_version(object *op, const char *params) {
-    version(op);
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN,
+            MSG_TYPE_ADMIN_VERSION, "Crossfire "FULL_VERSION);
 }
 
 /**
@@ -1054,17 +1105,17 @@ void command_listen(object *op, const char *params) {
  */
 void command_statistics(object *pl, const char *params) {
     char buf[MAX_BUF];
-    uint32 hours, minutes;
-    uint64 seconds; /* 64 bit to prevent overflows an intermediate results */
+    uint32_t hours, minutes;
+    uint64_t seconds; /* 64 bit to prevent overflows an intermediate results */
 
     if (!pl->contr)
         return;
-    strcpy(buf, i18n(pl, "[fixed]  Experience: %"));
+    safe_strncpy(buf, i18n(pl, "[fixed]  Experience: %"), sizeof(buf));
     strcat(buf, FMT64);
     draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_STATISTICS,
                          buf,
                          pl->stats.exp);
-    strcpy(buf, i18n(pl, "[fixed]  Next Level: %"));
+    safe_strncpy(buf, i18n(pl, "[fixed]  Next Level: %"), sizeof(buf));
     strcat(buf, FMT64);
     draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_STATISTICS,
                          buf,
@@ -1097,6 +1148,21 @@ void command_statistics(object *pl, const char *params) {
     draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_STATISTICS,
                          i18n(pl, "\nAttack Mode: %s"),
                          i18n(pl, pl->contr->peaceful ?  "Peaceful" : "Hostile"));
+    /**
+     * Add preliminary damage per second calculations here
+     */
+    float weap_speed = pl->weapon_speed; // This is the number of attacks per tick.
+    if (weap_speed < 0.0f)
+        weap_speed = 0.0f;
+    if (weap_speed > 1.0f)
+        weap_speed = 1.0f;
+    // We will initially calculate the damage if every attack you perform hits.
+    // This will serve as a baseline for future calculations
+    float dps = (1000000.0f / max_time) * weap_speed * pl->stats.dam;
+    // TODO: Account for opposing AC in calculations, make some sort of table/chart.
+    // Then we round the floating-point.
+    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_STATISTICS,
+                        i18n(pl, "\n\nDam/Sec: %4d"), (int)(dps + 0.5f));
 
     /* max_time is in microseconds - thus divide by 1000000.
      * Need 64 bit values, as otherwise ticks_played * max_time
@@ -1106,8 +1172,8 @@ void command_statistics(object *pl, const char *params) {
      * the time estimates use the current value.  But I'm presuming that
      * max_time won't change very often.  MSW 2009-12-01
      */
-    seconds = (uint64)pl->contr->ticks_played * (uint64)max_time / 1000000;
-    minutes = (uint32)seconds / 60;
+    seconds = (uint64_t)pl->contr->ticks_played * (uint64_t)max_time / 1000000;
+    minutes = (uint32_t)seconds / 60;
     hours = minutes / 60;
     minutes = minutes % 60;
 
@@ -1247,7 +1313,6 @@ void command_bowmode(object *op, const char *params) {
         "firenw",
         "bestarrow"
     };
-    char buf[MAX_BUF];
     int i, found;
 
     if (*params == '\0') {
@@ -1265,16 +1330,19 @@ void command_bowmode(object *op, const char *params) {
         }
     }
     if (!found) {
-        snprintf(buf, sizeof(buf), "bowmode: Unknown options %s, valid options are:", params);
+        StringBuffer *buf = stringbuffer_new();
+        stringbuffer_append_printf(buf, "bowmode: Unknown options %s, valid options are:", params);
         for (i = 0; i <= bow_bestarrow; i++) {
-            strcat(buf, " ");
-            strcat(buf, types[i]);
+            stringbuffer_append_string(buf, " ");
+            stringbuffer_append_string(buf, types[i]);
             if (i < bow_nw)
-                strcat(buf, ",");
+                stringbuffer_append_string(buf, ",");
             else
-                strcat(buf, ".");
+                stringbuffer_append_string(buf, ".");
         }
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG, buf);
+        char *result = stringbuffer_finish(buf);
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG, result);
+        free(result);
         return;
     }
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
@@ -1647,6 +1715,49 @@ static void show_commands(object *op, int what) {
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, line);
 }
 
+/**
+ * Find an appropriate help file. Will search regular commands, and wizard ones
+ * if asked for. Specified language is tried, as well as English.
+ * 'path' is altered whatever the return value.
+ * @param name command name.
+ * @param language player language.
+ * @param wiz if 1 the wizard-related files are searched, else no.
+ * @param path buffer to contain the found help file.
+ * @param length length of path.
+ * @return 1 if a file was found in which case path contains the path, else 0.
+ */
+static int find_help_file(const char *name, const char *language, int wiz, char *path, int length) {
+    struct stat st;
+
+    snprintf(path, length, "%s/help/%s.%s", settings.datadir, name, language);
+    if (stat(path, &st) == 0 && S_ISREG(st.st_mode)) {
+        return 1;
+    }
+
+    if (strcmp(language, "en")) {
+        snprintf(path, length, "%s/help/%s.en", settings.datadir, name);
+        if (stat(path, &st) == 0 && S_ISREG(st.st_mode)) {
+            return 1;
+        }
+    }
+
+    if (!wiz)
+        return 0;
+
+    snprintf(path, length, "%s/wizhelp/%s.%s", settings.datadir, name, language);
+    if (stat(path, &st) == 0 && S_ISREG(st.st_mode)) {
+        return 1;
+    }
+    if (strcmp(language, "en")) {
+        snprintf(path, length, "%s/wizhelp/%s.en", settings.datadir, name);
+        if (stat(path, &st) == 0 && S_ISREG(st.st_mode)) {
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
 /**
  * Player is asking for some help.
  *
@@ -1656,21 +1767,18 @@ static void show_commands(object *op, int what) {
  * what kind of help to ask for.
  */
 void command_help(object *op, const char *params) {
-    struct stat st;
     FILE *fp;
     char filename[MAX_BUF], line[MAX_BUF];
     int len;
     const char *language;
 
-    language = i18n_get_language_code(op->contr->language);
-
     /*
      * Main help page?
      */
     if (*params == '\0') {
         snprintf(filename, sizeof(filename), "%s/def_help", settings.datadir);
         if ((fp = fopen(filename, "r")) == NULL) {
-            LOG(llevError, "Cannot open help file %s: %s\n", filename, strerror_local(errno, line, sizeof(line)));
+            LOG(llevError, "Cannot open help file %s: %s\n", filename, strerror(errno));
             return;
         }
         while (fgets(line, MAX_BUF, fp)) {
@@ -1719,26 +1827,20 @@ void command_help(object *op, const char *params) {
         return;
     }
 
-    snprintf(filename, sizeof(filename), "%s/mischelp/%s.%s", settings.datadir, params, language);
-    if (stat(filename, &st) || !S_ISREG(st.st_mode)) {
-        if (op) {
-            snprintf(filename, sizeof(filename), "%s/help/%s.%s", settings.datadir, params, language);
-            if (stat(filename, &st) || !S_ISREG(st.st_mode)) {
-                if (QUERY_FLAG(op, FLAG_WIZ)) {
-                    snprintf(filename, sizeof(filename), "%s/wizhelp/%s.%s", settings.datadir, params, language);
-                    if (stat(filename, &st) || !S_ISREG(st.st_mode))
-                        goto nohelp;
-                } else
-                    goto nohelp;
-            }
-        }
+    language = i18n_get_language_code(op->contr->language);
+
+    if (!find_help_file(params, language, QUERY_FLAG(op, FLAG_WIZ), filename, sizeof(filename))) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+                            i18n(op, "No help available on '%s'"),
+                            params);
+        return;
     }
 
     /*
      * Found that. Just cat it to screen.
      */
     if ((fp = fopen(filename, "r")) == NULL) {
-        LOG(llevError, "Cannot open help file %s: %s\n", filename, strerror_local(errno, line, sizeof(line)));
+        LOG(llevError, "Cannot open help file %s: %s\n", filename, strerror(errno));
         return;
     }
 
@@ -1754,16 +1856,6 @@ void command_help(object *op, const char *params) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, line);
     }
     fclose(fp);
-    return;
-
-    /*
-     * No_help -escape
-     */
-
-nohelp:
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                         i18n(op, "No help available on '%s'"),
-                         params);
 }
 
 /**
@@ -1805,6 +1897,19 @@ int onoff_value(const char *line) {
     }
 }
 
+/**
+ * Tell players to use the 'delete' command. Too many new players have
+ * accidentally deleted their characters because they didn't carefully read
+ * the warning that the original 'quit' command gave.
+ */
+void command_quit(object* op, const char* params) {
+    draw_ext_info(
+        NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+        i18n(op,
+             "To leave the game, sleep in (apply) a bed to reality. To "
+             "permenantly delete your character, use the 'delete' command."));
+}
+
 /**
  * Player wants to totally delete her character.
  *
@@ -1813,7 +1918,7 @@ int onoff_value(const char *line) {
  * @param params
  * unused.
  */
-void command_quit(object *op, const char *params) {
+void command_delete(object *op, const char *params) {
     if (QUERY_FLAG(op, FLAG_WAS_WIZ)) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_ADMIN_DM, "Can't quit when in DM mode.");
         return;
@@ -1928,14 +2033,16 @@ void receive_player_password(object *op) {
     }
 
     if (op->contr->state == ST_CHANGE_PASSWORD_NEW) {
-        strcpy(op->contr->new_password, crypt_string(op->contr->write_buf+1, NULL));
-        send_query(&op->contr->socket, CS_QUERY_HIDEINPUT, i18n(op, "Please confirm your new password, or blank to cancel:"));
+        safe_strncpy(op->contr->new_password, newhash(op->contr->write_buf+1),
+                sizeof(op->contr->new_password));
+        send_query(&op->contr->socket, CS_QUERY_HIDEINPUT,
+                i18n(op, "Please confirm your new password, or blank to cancel:"));
         player_set_state(op->contr, ST_CHANGE_PASSWORD_CONFIRM);
         return;
     }
 
     if (op->contr->state == ST_CHANGE_PASSWORD_CONFIRM) {
-        if (strcmp(crypt_string(op->contr->write_buf+1, op->contr->new_password), op->contr->new_password)) {
+        if (!check_password(op->contr->write_buf+1, op->contr->new_password)) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
                           i18n(op, "The new passwords don't match!"));
         } else {
@@ -1947,7 +2054,8 @@ void receive_player_password(object *op) {
         return;
     }
 
-    strcpy(op->contr->password, crypt_string(op->contr->write_buf+1, NULL));
+    safe_strncpy(op->contr->password, newhash(op->contr->write_buf+1),
+            sizeof(op->contr->password));
     player_set_state(op->contr, ST_ROLL_STAT);
     check_login(op, TRUE);
 }
@@ -2065,6 +2173,13 @@ void command_wimpy(object *op, const char *params) {
         return;
     }
 
+    if (i < 0 || i > 100) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
+                            i18n(op, "Wimpy level should be between 1 and 100."),
+                            i);
+        return;
+    }
+
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
                          i18n(op, "Your new wimpy level is %d."),
                          i);
@@ -2144,9 +2259,17 @@ void command_kill_pets(object *op, const char *params) {
  * unused.
  */
 void command_passwd(object *pl, const char *params) {
-    send_query(&pl->contr->socket, CS_QUERY_HIDEINPUT, i18n(pl, "Password change.\nPlease enter your current password, or empty string to cancel."));
+    /* If old client, this is the way you change your password. */
+    if (pl->contr->socket.login_method < 1){
+        send_query(&pl->contr->socket, CS_QUERY_HIDEINPUT, i18n(pl, "Password change.\nPlease enter your current password, or empty string to cancel."));
 
-    player_set_state(pl->contr, ST_CHANGE_PASSWORD_OLD);
+        player_set_state(pl->contr, ST_CHANGE_PASSWORD_OLD);
+    }
+    /* If new client (login_method = 2) or jxclient (login_method = 1), changing the password does nothing anyway, so error out */
+    else{
+        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+                          i18n(pl, "passwd is maintained for older clients that do not support the account system. Please use the 'Password' button in your character selection screen to change your password."));
+    }
 }
 
 /**
@@ -2159,12 +2282,12 @@ void command_passwd(object *pl, const char *params) {
  * skill being used.
  */
 void do_harvest(object *pl, int dir, object *skill) {
-    sint16 x, y;
+    int16_t x, y;
     int count = 0, proba; /* Probability to get the item, 100 based. */
-    int level, exp;
+    int level, exp, check_exhaust = 0;
     object *found[10]; /* Found items that can be harvested. */
     mapstruct *map;
-    object *item, *inv;
+    object *item, *inv, *harvested;
     sstring trace, ttool, tspeed, race, tool, slevel, sexp;
     float speed;
 
@@ -2172,7 +2295,7 @@ void do_harvest(object *pl, int dir, object *skill) {
     y = pl->y+freearr_y[dir];
     map = pl->map;
 
-    if (!pl->type == PLAYER)
+    if (!IS_PLAYER(pl))
         return;
 
     if (!map)
@@ -2220,6 +2343,8 @@ void do_harvest(object *pl, int dir, object *skill) {
 
     inv = found[rndm(0, count-1)];
     assert(inv);
+    item = inv->env;
+    assert(item);
 
     slevel = object_get_value(inv, "harvest_level");
     sexp = object_get_value(inv, "harvest_exp");
@@ -2253,32 +2378,54 @@ void do_harvest(object *pl, int dir, object *skill) {
 
     /* Check the new item can fit into inventory.
      * Fixes bug #3060474: fishing puts more fishes into inventory than you can carry. */
-    if (((uint32)(pl->weight + pl->carrying + inv->weight)) > get_weight_limit(pl->stats.Str)) {
+    if (((uint32_t)(pl->weight + pl->carrying + inv->weight)) > get_weight_limit(pl->stats.Str)) {
         draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You are carrying too much to %s a %s.", skill->slaying, inv->name);
         return;
     }
 
     /* Ok, got it. */
-    item = object_new();
-    object_copy_with_inv(inv, item);
-    object_set_value(item, "harvestable", NULL, 0);
-    if (QUERY_FLAG(item, FLAG_MONSTER)) {
-        int spot = object_find_free_spot(item, pl->map, pl->x, pl->y, 0, SIZEOFFREE);
+    if (inv->nrof == 0) {
+        harvested = object_new();
+        object_copy_with_inv(inv, harvested);
+    } else {
+        if (count == 1 && inv->nrof == 1) {
+            check_exhaust = 1;
+        }
+        harvested = object_split(inv, 1, NULL, 0);
+    }
+    object_set_value(harvested, "harvestable", NULL, 0);
+    if (QUERY_FLAG(harvested, FLAG_MONSTER)) {
+        int spot = object_find_free_spot(harvested, pl->map, pl->x, pl->y, 0, SIZEOFFREE);
         if (spot == -1) {
             /* Better luck next time...*/
-            object_remove(item);
+            object_remove(harvested);
             draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You fail to %s anything.", skill->slaying);
             return;
         }
-        object_insert_in_map_at(item, pl->map, NULL, 0, pl->x+freearr_x[spot], pl->y+freearr_y[spot]);
-        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You %s a %s!", skill->slaying, item->name);
+        object_insert_in_map_at(harvested, pl->map, NULL, 0, pl->x+freearr_x[spot], pl->y+freearr_y[spot]);
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You %s a %s!", skill->slaying, harvested->name);
     } else {
-        item = object_insert_in_ob(item, pl);
-        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You %s some %s", skill->slaying, item->name);
+        harvested = object_insert_in_ob(harvested, pl);
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You %s some %s", skill->slaying, harvested->name);
     }
 
     /* Get exp */
     change_exp(pl, exp, skill->name, SK_EXP_ADD_SKILL);
 
+    if (check_exhaust) {
+        sstring replacement = object_get_value(item, "harvest_exhaust_replacement");
+        if (replacement) {
+            if (replacement[0] != '-') {
+                archetype *other = find_archetype(replacement);
+                if (other) {
+                    object *final = object_create_arch(other);
+                    object_insert_in_map_at(final, map, item, INS_BELOW_ORIGINATOR, item->x, item->y);
+                }
+            }
+            object_remove(item);
+            object_free(item, FREE_OBJ_FREE_INVENTORY);
+        }
+    }
+
     return;
 }
