@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_c_move_c =
- *   "$Id: c_move.c 13558 2010-08-05 20:53:39Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Move-related (north, east, ...) commands.
@@ -162,3 +147,16 @@ void command_stay(object *op, const char *params) {
         return;
     fire(op, 0);
 }
+
+void do_goto(object *op, const char *name, int x, int y) {
+    object *dummy;
+    dummy = object_new();
+    dummy->map = op->map;
+    EXIT_PATH(dummy) = add_string(name);
+    EXIT_X(dummy) = x;
+    EXIT_Y(dummy) = y;
+    dummy->name = add_string(name);
+
+    enter_exit(op, dummy);
+    object_free(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
+}
