@@ -1,41 +1,29 @@
 /*
- * static char *rcsid_c_new_c =
- *   "$Id: c_new.c 13558 2010-08-05 20:53:39Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-  CrossFire, A Multiplayer game for X-windows
-
-  Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-  Copyright (C) 1992 Frank Tore Johansen
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-  The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * This file deals with administrative commands from the client.
  */
 
-#include <global.h>
-#include <commands.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+#include <ctype.h>
+
+#include "commands.h"
+#include "sproto.h"
 
 #ifndef tolower
 /** Simple macro to convert a letter to lowercase. */
@@ -221,3 +209,39 @@ void command_fire(object *op, const char *params) {
 void command_fire_stop(object *op, const char *params) {
     op->contr->fire_on = 0;
 }
+
+/**
+ * Player wants to face a given direction.
+ *
+ * @param op
+ * player.
+ * @param params
+ * additional parameters.
+ */
+void command_face(object *op, const char *params) {
+    int dir;
+
+    if ( !isdigit(*params) ) {
+        if ( strcmp(params,"stay") == 0 || strcmp(params,"down") == 0 ) dir=0;
+        else if ( strcmp(params,"north") == 0 || strcmp(params,"n") == 0 ) dir=1;
+        else if ( strcmp(params,"northeast") == 0 || strcmp(params,"ne") == 0 ) dir=2;
+        else if ( strcmp(params,"east") == 0 || strcmp(params,"e") == 0 ) dir=3;
+        else if ( strcmp(params,"southeast") == 0 || strcmp(params,"se") == 0 ) dir=4;
+        else if ( strcmp(params,"south") == 0 || strcmp(params,"s") == 0 ) dir=5;
+        else if ( strcmp(params,"southwest") == 0 || strcmp(params,"sw") == 0 ) dir=6;
+        else if ( strcmp(params,"west") == 0 || strcmp(params,"w") == 0 ) dir=7;
+        else if ( strcmp(params,"northwest") == 0 || strcmp(params,"nw") == 0 ) dir=8;
+        else {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+                                 "Unknown direction to face: %s",params);
+            return;
+        }
+    }
+    else dir = atoi(params);
+    if (dir < 0 || dir >= 9) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+                      "Can't face to a non adjacent square.");
+        return;
+    }
+    face_player(op, dir);
+}
