@@ -1,44 +1,33 @@
 /*
- * static char *rcsid_c_object_c =
- *   "$Id: c_object.c 16343 2011-12-28 22:45:32Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2007 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Object commands, including picking/dropping, locking, etc.
  * @todo clean multiple variations of same stuff (pickup and such), or rename for less confusion.
  */
 
-#include <global.h>
-#include <loader.h>
-#include <skills.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <living.h>
+#include "global.h"
+
+#include <ctype.h>
 #include <math.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "living.h"
+#include "loader.h"
+#include "shop.h"
+#include "skills.h"
+#include "sproto.h"
 
 static void set_pickup_mode(const object *op, int i);
 
@@ -46,10 +35,6 @@ static void set_pickup_mode(const object *op, int i);
  * Object id parsing functions
  */
 
-/** Simple ::objectlink allocation, fail-safe. */
-#define OBLINKMALLOC(p) if (!((p) = (objectlink *)malloc(sizeof(objectlink)))) \
-                            fatal(OUT_OF_MEMORY);
-
 /**
  * Search from start and through below for what matches best with params.
  * we use object_matches_string above - this gives us consistent behaviour
@@ -146,6 +131,25 @@ void command_rskill(object *pl, const char *params) {
     change_skill(pl, skill, 0);
 }
 
+/**
+ * Attempt to use a skill from its subtype. If multiple skills share the same,
+ * the first found is used.
+ * @param op who is applying a skill.
+ * @param skill_subtype skill subtype.
+ * @param params additional parameters to the skill.
+ * @param missing_message message to display if op doesn't know the skill.
+ */
+static void do_skill_by_number(object *op, int skill_subtype, const char *params,
+                               const char *missing_message) {
+    object *skop = find_skill_by_number(op, skill_subtype);
+    if (skop) {
+        do_skill(op, op, skop, op->facing, *params == '\0' ? NULL : params);
+        return;
+    }
+
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
+                  missing_message);
+}
 
 /* These functions (command_search, command_disarm) are really juse wrappers for
  * things like 'use_skill ...').  In fact, they should really be obsoleted
@@ -160,7 +164,7 @@ void command_rskill(object *pl, const char *params) {
  * unused.
  */
 void command_search(object *op, const char *params) {
-    use_skill(op, skill_names[SK_FIND_TRAPS]);
+    do_skill_by_number(op, SK_FIND_TRAPS, params, "You don't know how to search for unusual things.");
 }
 
 /**
@@ -172,7 +176,7 @@ void command_search(object *op, const char *params) {
  * unused.
  */
 void command_disarm(object *op, const char *params) {
-    use_skill(op, skill_names[SK_DISARM_TRAPS]);
+    do_skill_by_number(op, SK_DISARM_TRAPS, params, "You don't know how to disarm traps.");
 }
 
 /**
@@ -187,16 +191,7 @@ void command_disarm(object *op, const char *params) {
  * what to throw.
  */
 void command_throw(object *op, const char *params) {
-    object *skop;
-
-    skop = find_skill_by_name(op, skill_names[SK_THROWING]);
-    if (skop) {
-        do_skill(op, op, skop, op->facing, *params == '\0' ? NULL : params);
-        return;
-    }
-
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
-                  "You have no knowledge of the skill throwing.");
+    do_skill_by_number(op, SK_THROWING, params, "You have no knowledge of the skill throwing.");
 }
 
 /**
@@ -227,6 +222,10 @@ void command_apply(object *op, const char *params) {
         aflag = AP_UNAPPLY;
         params += 3;
     }
+    if (!strncmp(params, "-o ", 3)) {
+        aflag = AP_OPEN;
+        params += 3;
+    }
     if (!strncmp(params, "-b ", 3)) {
         params += 3;
         if (op->container)
@@ -269,7 +268,7 @@ void command_apply(object *op, const char *params) {
  * @return
  * 1 if it will fit, 0 if it will not.
  */
-int sack_can_hold(const object *pl, const object *sack, const object *op, uint32 nrof) {
+int sack_can_hold(const object *pl, const object *sack, const object *op, uint32_t nrof) {
     char name[MAX_BUF];
     query_name(sack, name, MAX_BUF);
 
@@ -294,12 +293,12 @@ int sack_can_hold(const object *pl, const object *sack, const object *op, uint32
     }
     if (op->type == SPECIAL_KEY && sack->slaying && op->slaying) {
         draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                             "You can't want put the key into %s.",
+                             "You can't put the key into %s.",
                              name);
         return 0;
     }
     if (sack->weight_limit) {
-        sint32 new_weight;
+        int32_t new_weight;
 
         new_weight = sack->carrying+(nrof ? nrof : 1)
             *(op->weight+(op->type == CONTAINER ? op->carrying*op->stats.Str : 0))
@@ -333,8 +332,11 @@ static void pick_up_object(object *pl, object *op, object *tmp, int nrof) {
      */
     char buf[HUGE_BUF], name[MAX_BUF];
     object *env = tmp->env;
-    uint32 weight, effective_weight_limit;
-    int tmp_nrof = tmp->nrof ? tmp->nrof : 1;
+    uint32_t weight, effective_weight_limit;
+    const int tmp_nrof = NROF(tmp);
+    tag_t tag;
+    mapstruct* map = tmp->map;
+    int16_t x = tmp->x, y = tmp->y;
 
     /* IF the player is flying & trying to take the item out of a container
      * that is in his inventory, let him.  tmp->env points to the container
@@ -400,15 +402,25 @@ static void pick_up_object(object *pl, object *op, object *tmp, int nrof) {
     query_name(tmp, name, MAX_BUF);
 
     if (QUERY_FLAG(tmp, FLAG_UNPAID)) {
-        char *value = stringbuffer_finish(query_cost_string(tmp, pl, F_BUY|F_SHOP, NULL));
+        char *value = cost_str(shop_price_buy(tmp, pl));
         snprintf(buf, sizeof(buf), "%s will cost you %s.", name, value);
         free(value);
     } else
         snprintf(buf, sizeof(buf), "You pick up the %s.", name);
 
     /* Now item is about to be picked. */
-    if (execute_event(tmp, EVENT_PICKUP, pl, op, NULL, SCRIPT_FIX_ALL) != 0)
+    tag = tmp->count;
+    if (execute_event(tmp, EVENT_PICKUP, pl, op, NULL, SCRIPT_FIX_ALL) != 0) {
+        /* put item back, if it still exists */
+        if (tmp->count == tag && !QUERY_FLAG(tmp, FLAG_FREED)) {
+          if (env != NULL) {
+            object_insert_in_ob(tmp, env);
+          } else {
+            object_insert_in_map_at(tmp, map, NULL, 0, x, y);
+          }
+        }
         return;
+    }
 
     draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
                   buf);
@@ -677,7 +689,7 @@ void command_take(object *op, const char *params) {
  * @param nrof
  * if non zero, then nrof objects is tried to put into sack, else everything is put.
  */
-void put_object_in_sack(object *op, object *sack, object *tmp, uint32 nrof) {
+void put_object_in_sack(object *op, object *sack, object *tmp, uint32_t nrof) {
     object *sack2, *orig = sack;
     char name_sack[MAX_BUF], name_tmp[MAX_BUF];
 
@@ -813,7 +825,7 @@ void put_object_in_sack(object *op, object *sack, object *tmp, uint32 nrof) {
  * object dropped, NULL if it was destroyed.
  * @todo shouldn't tmp be NULL if object_was_destroyed returns true?
  */
-object *drop_object(object *op, object *tmp, uint32 nrof) {
+object *drop_object(object *op, object *tmp, uint32_t nrof) {
     tag_t tmp_tag;
 
     if (QUERY_FLAG(tmp, FLAG_NO_DROP)) {
@@ -853,7 +865,7 @@ object *drop_object(object *op, object *tmp, uint32 nrof) {
 
         query_name(tmp, name, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                             "You drop the %s. The gods who lent it to you retrieves it.",
+                             "You drop the %s. The gods who lent it to you retrieve it.",
                              name);
         object_free_drop_inventory(tmp);
 
@@ -883,7 +895,7 @@ object *drop_object(object *op, object *tmp, uint32 nrof) {
 
     tmp_tag = tmp->count;
     object_insert_in_map_at(tmp, op->map, op, INS_BELOW_ORIGINATOR, op->x, op->y);
-    if (!object_was_destroyed(tmp, tmp_tag) && !QUERY_FLAG(tmp, FLAG_UNPAID) && tmp->type != MONEY && is_in_shop(op)) {
+    if (!object_was_destroyed(tmp, tmp_tag) && !QUERY_FLAG(tmp, FLAG_UNPAID) && tmp->type != MONEY && shop_contains(op)) {
         sell_item(tmp, op);
     }
 
@@ -946,22 +958,6 @@ void drop(object *op, object *tmp) {
         return;
     }
 
-    if (op->type == PLAYER) {
-        if (op->contr->last_used == tmp && op->contr->last_used_id == tmp->count) {
-            object *n = NULL;
-
-            if (tmp->below != NULL)
-                n = tmp->below;
-            else if (tmp->above != NULL)
-                n = tmp->above;
-            op->contr->last_used = n;
-            if (n != NULL)
-                op->contr->last_used_id = n->count;
-            else
-                op->contr->last_used_id = 0;
-        }
-    };
-
     if (op->container) {
         if (op->type == PLAYER) {
             put_object_in_sack(op, op->container, tmp, op->contr->count);
@@ -1279,6 +1275,7 @@ object *find_marked_object(object *op) {
      * player hasn't dropped the item.  We use count on the off chance that
      * an item got reincarnated at some point.
      */
+     /*
     FOR_INV_PREPARE(op, tmp) {
         if (tmp->invisible)
             continue;
@@ -1292,6 +1289,19 @@ object *find_marked_object(object *op) {
             }
         }
     } FOR_INV_FINISH();
+    */
+    /* Try a different way of doing this
+     * We check the environment of the marked object
+     * to make sure it is still in the player's inventory.
+     * In addition, we ensure there is the correct tag for that item.
+     *
+     * Daniel Hawkins 2018-10-23
+     */
+    if (op->contr->mark->env == op && op->contr->mark->count == op->contr->mark_count)
+        return op->contr->mark;
+    // Otherwise reset the mark, since it is no longer valid.
+    op->contr->mark = NULL;
+    op->contr->mark_count = 0;
     return NULL;
 }
 
@@ -1405,6 +1415,10 @@ void examine_monster(object *op, object *tmp, int level) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                       "It looks very ill.");
 
+    if (level < 10)
+        return;
+    knowledge_show_monster_detail(op, mon->arch->name);
+
     if (level < 15)
         return;
 
@@ -1438,7 +1452,7 @@ void examine_monster(object *op, object *tmp, int level) {
 void examine(object *op, object *tmp) {
     char buf[VERY_BIG_BUF];
     int in_shop;
-    int i, exp = 0;
+    int i, exp = 0, conn;
 
     /* we use this to track how far along we got with trying to identify an item,
      * so that we can give the appropriate message to the player */
@@ -1535,7 +1549,7 @@ void examine(object *op, object *tmp) {
         } else snprintf(prefix, MAX_BUF, "%s:",  tmp->nrof <= 1?"That is":"Those are");
     }
     /* now we need to get the rest of the object description */
-    ob_describe(tmp, op, buf, sizeof(buf));
+    ob_describe(tmp, op, 1, buf, sizeof(buf));
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                                             "%s %s", prefix, buf);
@@ -1547,12 +1561,57 @@ void examine(object *op, object *tmp) {
     }
 
     switch (tmp->type) {
-    case SPELLBOOK:
-        if (QUERY_FLAG(tmp, FLAG_IDENTIFIED) && tmp->inv) {
-            char level[100];
+    case SKILLSCROLL:
+    case SKILL_TOOL:
+        // Embedded skills are stored as an archetype name and don't get reified
+        // until the player actually reads/equips the object, so we need to
+        // handle it differently from spells, which are stored in the inv.
+        if (!tmp->skill) break;  // Blank skill scroll, somehow.
+        archetype *skill = get_archetype_by_skill_name(tmp->skill, SKILL);
+        if (!skill) {
+            // Skill name doesn't correspond to any actual skill.
+            draw_ext_info(NDI_UNIQUE|NDI_BLUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_INFO,
+                "Unfortunately the scroll is damaged and unreadable.");
+            break;
+        }
+        // Only print the flavor text once we have identified.
+        if (is_identified(tmp) && skill->clone.msg) {
+            StringBuffer *sb = stringbuffer_new();
+            stringbuffer_append_string(sb, skill->clone.msg);
+            stringbuffer_trim_whitespace(sb);
+            char *const fluff = stringbuffer_finish(sb);
+            // SPELL_INFO is not a perfect match here, but it should display in the
+            // same manner as the spell descriptions below and there's no SKILL_INFO
+            // message type.
+            draw_ext_info(NDI_UNIQUE|NDI_BLUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_INFO, fluff);
+            free(fluff);
+        }
+        break;
 
-            get_levelnumber(tmp->inv->level, level, 100);
-            snprintf(buf, sizeof(buf), "%s is a %s level %s spell", tmp->inv->name, level, tmp->inv->skill);
+    case SPELLBOOK:
+    case SCROLL:
+    case WAND:
+    case ROD:
+    case POTION:
+        // Only print the flavor text once we have identified.
+        if (is_identified(tmp) && tmp->inv && tmp->inv->msg) {
+            // If the embedded spell has a msg, display it here so that the
+            // player knows what it does before they actually read/use the item.
+            // Strip trailing newlines so that the output of examine() is
+            // contiguous.
+            // TODO: It might be better to strip the newlines in object_set_msg
+            // and append them at print time, rather than ensuring that the msg
+            // is newline-terminated and stripping off the newlines when we don't
+            // want them; C string handling makes the latter a lot less convenient.
+            StringBuffer *sb = stringbuffer_new();
+            stringbuffer_append_string(sb, tmp->inv->msg);
+            stringbuffer_trim_whitespace(sb);
+            char *const fluff = stringbuffer_finish(sb);
+            draw_ext_info(NDI_UNIQUE|NDI_BLUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_INFO, fluff);
+            free(fluff);
+        }
+        if (tmp->type == WAND && is_identified(tmp)) {
+            snprintf(buf, sizeof(buf), "It has %d charges left.", tmp->stats.food);
         }
         break;
 
@@ -1570,11 +1629,6 @@ void examine(object *op, object *tmp) {
         } else
             if (tmp->weight_limit && tmp->stats.Str < 100)
                 snprintf(buf, sizeof(buf), "Its weight limit is %.1f kg.", tmp->weight_limit/(10.0*(100-tmp->stats.Str)));
-            break;
-
-    case WAND:
-        if (QUERY_FLAG(tmp, FLAG_IDENTIFIED))
-            snprintf(buf, sizeof(buf), "It has %d charges left.", tmp->stats.food);
         break;
     }
 
@@ -1582,6 +1636,9 @@ void examine(object *op, object *tmp) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                       buf);
 
+    // TODO: combine with weight and properly pluralize, so you get:
+    // It is made of stone and weighs 15.0 kg.
+    // They are made of paper and weigh 3 kg.
     if (tmp->materialname != NULL && !tmp->msg) {
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                              "It is made of: %s.",
@@ -1589,24 +1646,22 @@ void examine(object *op, object *tmp) {
     }
     /* Where to wear this item */
     for (i = 0; i < NUM_BODY_LOCATIONS; i++) {
-        if (tmp->body_info[i] < -1) {
-            if (op->body_info[i])
-                draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                                     "It goes %s (%d)",
+        if (tmp->body_info[i]) {
+            if (op->body_info[i]) {
+                if (tmp->body_info[i] < -1) {
+                    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
+                                     "%s %s (%d)", tmp->nrof > 1 ? "They go" : "It goes",
                                      body_locations[i].use_name, -tmp->body_info[i]);
-            else
-                draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                                     "It goes %s",
-                                     body_locations[i].nonuse_name);
-        } else if (tmp->body_info[i]) {
-            if (op->body_info[i])
-                draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                                     "It goes %s",
+                } else {
+                    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
+                                     "%s %s", tmp->nrof > 1 ? "They go" : "It goes",
                                      body_locations[i].use_name);
-            else
+                }
+            } else {
                 draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                                     "It goes %s",
+                                     "%s %s", tmp->nrof > 1 ? "They go" : "It goes",
                                      body_locations[i].nonuse_name);
+            }
         }
     }
 
@@ -1616,21 +1671,21 @@ void examine(object *op, object *tmp) {
                       buf);
     }
 
-    in_shop = is_in_shop(op);
+    in_shop = shop_contains(op);
 
     if (tmp->value && !QUERY_FLAG(tmp, FLAG_STARTEQUIP) && !QUERY_FLAG(tmp, FLAG_NO_PICK)) {
-        char *value = stringbuffer_finish(query_cost_string(tmp, op, F_SELL|F_APPROX, NULL));
+        char *value = cost_approx_str(tmp, op);
         snprintf(buf, sizeof(buf), "You reckon %s worth %s.", tmp->nrof > 1 ? "they are" : "it is", value);
         free(value);
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                       buf);
         if (in_shop) {
             if (QUERY_FLAG(tmp, FLAG_UNPAID)) {
-                value = stringbuffer_finish(query_cost_string(tmp, op, F_BUY|F_SHOP, NULL));
+                value = cost_str(shop_price_buy(tmp, op));
                 snprintf(buf, sizeof(buf), "%s would cost you %s.", tmp->nrof > 1 ? "They" : "It", value);
                 free(value);
             } else {
-                value = stringbuffer_finish(query_cost_string(tmp, op, F_SELL|F_SHOP, NULL));
+                value = cost_str(shop_price_sell(tmp, op));
                 snprintf(buf, sizeof(buf), "You are offered %s for %s.", value, tmp->nrof > 1 ? "them" : "it");
                 free(value);
             }
@@ -1643,9 +1698,28 @@ void examine(object *op, object *tmp) {
         examine_monster(op, tmp, 0);
 
     /* Is this item buildable? */
-    if (QUERY_FLAG(tmp, FLAG_IS_BUILDABLE))
+    if (QUERY_FLAG(tmp, FLAG_IS_BUILDABLE)) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                       "This is a buildable item.");
+        if (QUERY_FLAG(tmp, FLAG_IS_LINKED)){
+            conn = get_button_value(tmp);
+            if (conn) {
+                FOR_INV_PREPARE(op, tmp_inv) {
+                    if (tmp_inv->type == FORCE && tmp_inv->slaying != NULL
+                        && strcmp(tmp_inv->slaying, op->map->path) == 0
+                        && tmp_inv->msg != NULL
+                        && tmp_inv->path_attuned == (uint32_t) conn) {
+
+                        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                                  MSG_TYPE_COMMAND_EXAMINE, "Connected with: %s",
+                                  tmp_inv->msg);
+
+                        break;
+                    }
+                } FOR_INV_FINISH();
+            }
+        }
+    }
 
     /* Does the object have a message?  Don't show message for all object
      * types - especially if the first entry is a match
@@ -1656,20 +1730,29 @@ void examine(object *op, object *tmp) {
     && tmp->type != CORPSE
     && !tmp->move_on
     && strncasecmp(tmp->msg, "@match", 6)) {
-        /* This is just a hack so when identifying hte items, we print
-         * out the extra message
+        /* This is just a hack so when identifying the items, we print
+         * out the extra message. Do this only for "identifiable types", e.g.
+         * we don't want to print this message when the player looks at a
+         * locked door (where msg is used to store the message they get when
+         * trying to force it open).
+         *
+         * Also, don't print the message for the object unless it has been identified
+         * 		-- SilverNexus 2015-05-20
          */
-        if (need_identify(tmp) && QUERY_FLAG(tmp, FLAG_IDENTIFIED))
+        if (is_identifiable_type(tmp) && is_identified(tmp)) {
             draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                           "The object has a story:");
 
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
-                      tmp->msg);
+		    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
+					      tmp->msg);
+        }
     }
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE,
                   " "); /* Blank line */
 
-    knowledge_item_can_be_used_alchemy(op, tmp);
+    if (is_identified(tmp)) {
+        knowledge_item_can_be_used_alchemy(op, tmp);
+    }
 }
 
 /**
@@ -1742,14 +1825,21 @@ void inventory(object *op, object *inv) {
  *
  * @param op
  * must be a player.
+ * @param old
+ * previous pickup mode.
  */
-static void display_new_pickup(const object *op) {
+static void display_new_pickup(const object *op, int old) {
     int i = op->contr->mode;
 
     esrv_send_pickup(op->contr);
 
-    if (!(i&PU_NEWMODE) || !(i&PU_DEBUG))
+    if (!(i&PU_NEWMODE) || !(i&PU_DEBUG)) {
+        if ((old & PU_INHIBIT) != (op->contr->mode & PU_INHIBIT)) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+                "Pickup is %s.", (old & PU_INHIBIT) ? "active" : "inhibited");
+        }
         return;
+    }
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
                          "%d NEWMODE",
@@ -1864,7 +1954,7 @@ static void display_new_pickup(const object *op) {
  * @todo trash old pickup mode, merge with new pickup.
  */
 void command_pickup(object *op, const char *params) {
-    uint32 i;
+    uint32_t i;
     static const char *names[] = {
         "debug", "inhibit", "stop", "food", "drink",
         "valuables", "bow", "arrow", "helmet", "shield",
@@ -1873,7 +1963,7 @@ void command_pickup(object *op, const char *params) {
         "skillscroll", "readables", "magicdevice", "notcursed", "jewels",
         "flesh", "container", NULL
     };
-    static const uint32 modes[] = {
+    static const uint32_t modes[] = {
         PU_DEBUG, PU_INHIBIT, PU_STOP, PU_FOOD, PU_DRINK, PU_VALUABLES, PU_BOW, PU_ARROW, PU_HELMET,
         PU_SHIELD, PU_ARMOUR, PU_BOOTS, PU_GLOVES, PU_CLOAK, PU_KEY, PU_MISSILEWEAPON, PU_MELEEWEAPON,
         PU_MAGICAL, PU_POTION, PU_SPELLBOOK, PU_SKILLSCROLL, PU_READABLES, PU_MAGIC_DEVICE,
@@ -1883,7 +1973,7 @@ void command_pickup(object *op, const char *params) {
     if (*params == '\0') {
         /* if the new mode is used, just print the settings */
         if (op->contr->mode&PU_NEWMODE) {
-            display_new_pickup(op);
+            display_new_pickup(op, op->contr->mode);
             return;
         }
         if (1)
@@ -1900,6 +1990,7 @@ void command_pickup(object *op, const char *params) {
 
         for (mode = 0; names[mode]; mode++) {
             if (!strcmp(names[mode], params+1)) {
+                int old = op->contr->mode;
                 i = op->contr->mode;
                 if (!(i&PU_NEWMODE))
                     i = PU_NEWMODE;
@@ -1914,7 +2005,7 @@ void command_pickup(object *op, const char *params) {
                         i = i|modes[mode];
                 }
                 op->contr->mode = i;
-                display_new_pickup(op);
+                display_new_pickup(op, old);
                 return;
             }
         }
@@ -1932,7 +2023,7 @@ void command_pickup(object *op, const char *params) {
         return;
     }
     set_pickup_mode(op, i);
-    display_new_pickup(op);
+    display_new_pickup(op, op->contr->mode);
 }
 
 /**
@@ -2278,10 +2369,10 @@ void command_use(object *op, const char *params) {
     /*sstring data;*/
     recipe *transformation;
 
-    if (!op->type == PLAYER)
+    if (!IS_PLAYER(op))
         return;
 
-    snprintf(copy, sizeof(copy), "%s", params);
+    strlcpy(copy, params, sizeof(copy));
     with = strstr(copy, " with ");
     if (!with) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE, "Syntax is use <item> with <item>.");
