@@ -1,41 +1,27 @@
 /*
- * static char *rcsid_c_party_c =
- *   "$Id: c_party.c 15075 2011-09-10 12:47:41Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Party-related functions and variables.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
+#include "global.h"
+
+#include <string.h>
+
+#include "sproto.h"
+#include "spells.h"
 
 /**
  * Is the password the player entered to join a party the right one?
@@ -311,13 +297,6 @@ void command_party(object *op, const char *params) {
             return;
         }
 
-        if (op->contr->party == party) {
-            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                                 "You are already a member of party: %s",
-                                 party->partyname);
-            return;
-        }
-
         if (get_party_password(op, party)) {
             return;
         }
@@ -329,7 +308,7 @@ void command_party(object *op, const char *params) {
     party_help(op);
 }
 
-/** Valid modes for 'party_rejoin'. @todo document that */
+/** Valid modes for 'party_rejoin', indexed by ::party_rejoin_mode. */
 static const char *rejoin_modes[] = {
     "no",
     "if_exists",
