@@ -1,44 +1,31 @@
 /*
- * static char *rcsid_c_range_c =
- *   "$Id: c_range.c 13889 2010-09-26 10:02:47Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Range related commands (casting, shooting, throwing, etc.).
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
-#include <skills.h>
-#include <shared/newclient.h>
-#include <commands.h>
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "commands.h"
+#include "shared/newclient.h"
+#include "skills.h"
+#include "spells.h"
+#include "sproto.h"
 
 /**
  * 'invoke' command, fires a spell immediately.
@@ -105,11 +92,11 @@ static void show_matching_spells(object *op, const char *params) {
         && (*params == '\0' || !strncmp(params, spell->name, strlen(params)))) {
             if (spell->path_attuned&op->path_denied) {
                 snprintf(spell_sort[num_found++], sizeof(spell_sort[0]),
-                         "%s:%-22s %3s %3s", spell->skill ? spell->skill : "generic",
+                         "%s:%-30s %3s %3s", spell->skill ? spell->skill : "generic",
                          spell->name, "den", "den");
             } else {
                 snprintf(spell_sort[num_found++], sizeof(spell_sort[0]),
-                         "%s:%-22s %3d %3d", spell->skill ? spell->skill : "generic",
+                         "%s:%-30s %3d %3d", spell->skill ? spell->skill : "generic",
                          spell->name, spell->level,
                          SP_level_spellpoint_cost(op, spell, SPELL_HIGHEST));
             }
@@ -149,7 +136,7 @@ static void show_matching_spells(object *op, const char *params) {
 
             draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
                                  "\n[fixed]%s spells %.*s <lvl> <sp>",
-                                 tmp, (int)(12-strlen(tmp)), "              ");
+                                 tmp, (int)(20-strlen(tmp)), "              ");
         }
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
                              "[fixed]%s",
@@ -174,7 +161,7 @@ void command_cast_spell(object *op, const char *params, char command) {
     char *cp, cpy[MAX_BUF];
     object *spob;
 
-    strncpy(cpy, params, sizeof(cpy));
+    safe_strncpy(cpy, params, sizeof(cpy));
 
     if (command == 'i')
         castnow = 1;
@@ -312,7 +299,7 @@ int legal_range(object *op, int r) {
  * @param k
  * '+' selects next range, other values previous range.
  */
-void change_spell(object *op, char k) {
+static void change_spell(object *op, char k) {
     char name[MAX_BUF];
 
     do {
