@@ -1,44 +1,33 @@
 /*
- * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c 16870 2012-01-28 11:46:31Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Those functions are used by DMs.
  * @todo explain item stack, item specifier for commands.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
-#include <treasure.h>
-#include <skills.h>
+#include "global.h"
+
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+
+#include "sproto.h"
+#include "spells.h"
+#include "treasure.h"
+#include "skills.h"
 
 /* Defines for DM item stack **/
 #define STACK_SIZE         50   /**< Stack size, static */
@@ -50,6 +39,9 @@ enum {
     STACK_FROM_NUMBER  = 3    /**< Item is a number (may be top) */
 };
 
+/** Time, in seconds from epoch, of server shutdown. */
+int cmd_shutdown_time = 0;
+
 /**
  * Enough of the DM functions seem to need this that I broke
  * it out to a seperate function.  name is the person
@@ -311,7 +303,7 @@ static object *get_dm_object(player *pl, const char **params, int *from) {
  * option, must be "TRUE" for the test to happen.
  */
 void command_loadtest(object *op, const char *params) {
-    uint32 x, y;
+    uint32_t x, y;
     char buf[1024];
 
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DEBUG,
@@ -337,6 +329,17 @@ void command_loadtest(object *op, const char *params) {
     }
 }
 
+static void unhide(object* op) {
+    op->contr->hidden = 0;
+    op->invisible = 1;
+    op->map->players++;
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
+                  "You are no longer hidden from other players");
+    draw_ext_info_format(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL,
+                         MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
+                         "%s has entered the game.", op->name);
+}
+
 /**
  * Actually hides or unhides specified player (obviously a DM).
  *
@@ -347,15 +350,7 @@ void command_loadtest(object *op, const char *params) {
  */
 static void do_wizard_hide(object *op, int silent_dm) {
     if (op->contr->hidden) {
-        op->contr->hidden = 0;
-        op->invisible = 1;
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
-                      "You are no longer hidden from other players");
-        op->map->players++;
-        draw_ext_info_format(NDI_UNIQUE|NDI_ALL|NDI_DK_ORANGE, 5, NULL,
-                             MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
-                             "%s has entered the game.",
-                             op->name);
+        unhide(op);
         if (!silent_dm) {
             draw_ext_info(NDI_UNIQUE|NDI_ALL|NDI_LT_GREEN, 1, NULL,
                           MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM,
@@ -425,7 +420,7 @@ void command_setgod(object *op, const char *params) {
 
     if (*params == '\0' || !(str = strchr(params, ' '))) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                      "Usage: setgod object god");
+                      "Usage: set_god player god");
         return;
     }
 
@@ -677,19 +672,50 @@ void command_toggle_shout(object *op, const char *params) {
  * @param op
  * wizard shutting down the server.
  * @param params
- * ignored.
+ * When to shut down the server.
  */
 void command_shutdown(object *op, const char *params) {
-    /*
-     * We need to give op - command_kick expects it.  however, this means
-     * the op won't get kicked off, so we do it ourselves
-     */
-    command_kick2(op, "");
-    hiscore_check(op, 0); /* Always check score */
-    (void)save_player(op, 0);
-    play_again(op);
-    cleanup();
-    /* not reached */
+    if (strlen(params) == 0) {
+        /* Give DM command help and display current shutdown status. */
+        command_help(op, "shutdown");
+
+        if (cmd_shutdown_time != 0) {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                    MSG_TYPE_COMMAND_DM, "Server is shutting down soon.");
+        }
+    } else if (strcmp(params, "cancel") == 0) {
+        /* Tell everyone that the server is no longer shutting down. */
+        if (cmd_shutdown_time != 0) {
+            draw_ext_info(NDI_UNIQUE | NDI_ALL, 0, op, MSG_TYPE_ADMIN,
+                    MSG_TYPE_ADMIN_DM, "Server shutdown cancelled.");
+            cmd_shutdown_time = 0;
+            LOG(llevInfo, "Server shutdown cancelled by %s.\n", op->name);
+        } else {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                    MSG_TYPE_COMMAND_ERROR, "No shutdown is pending.");
+        }
+    } else if (strncmp(params, "now", 3) == 0) {
+        /* Announce and shut down immediately. */
+        draw_ext_info(NDI_UNIQUE | NDI_ALL, 0, op, MSG_TYPE_ADMIN,
+                    MSG_TYPE_ADMIN_DM, "Server is shutting down now!");
+        cmd_shutdown_time = time(NULL);
+        LOG(llevInfo, "Server shutdown initiated by %s.\n", op->name);
+    } else {
+        /* Schedule (but don't announce) a shutdown. */
+        int minutes = atoi(params);
+
+        if (minutes > 0 && minutes <= 720) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                    MSG_TYPE_COMMAND_SUCCESS,
+                    "Server will shut down in %d minutes.", minutes);
+            cmd_shutdown_time = time(NULL) + minutes * 60;
+            LOG(llevInfo, "Server shutdown initiated by %s in %d minutes.\n", op->name, minutes);
+        } else {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                    MSG_TYPE_COMMAND_ERROR,
+                    "Please specify a reasonable time in minutes.");
+        }
+    }
 }
 
 /**
@@ -701,9 +727,6 @@ void command_shutdown(object *op, const char *params) {
  * map to teleport to. Can be absolute or relative path.
  */
 void command_goto(object *op, const char *params) {
-    const char *name;
-    object *dummy;
-
     if (!op)
         return ;
 
@@ -713,19 +736,7 @@ void command_goto(object *op, const char *params) {
         return;
     }
 
-    name = params;
-    dummy = object_new();
-    dummy->map = op->map;
-    EXIT_PATH(dummy) = add_string(name);
-    EXIT_X(dummy) = -1;
-    EXIT_Y(dummy) = -1;
-    dummy->name = add_string(name);
-
-    enter_exit(op, dummy);
-    object_free2(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM,
-                         "Difficulty: %d.",
-                         op->map->difficulty);
+    do_goto(op, params, -1, -1);
 }
 
 /**
@@ -796,7 +807,7 @@ int player_arrest(object *who) {
     x = who->x;
     y = who->y;
     enter_exit(who, dummy);
-    object_free2(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
+    object_free(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
 
     if (cur == who->map && x == who->x && y == who->y)
         return -2;
@@ -887,7 +898,7 @@ void command_summon(object *op, const char *params) {
     EXIT_X(dummy) = op->x+freearr_x[i];
     EXIT_Y(dummy) = op->y+freearr_y[i];
     enter_exit(pl->ob, dummy);
-    object_free2(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
+    object_free(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
     draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM,
                   "You are summoned.");
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM,
@@ -937,7 +948,7 @@ void command_teleport(object *op, const char *params) {
     EXIT_X(dummy) = pl->ob->x+freearr_x[i];
     EXIT_Y(dummy) = pl->ob->y+freearr_y[i];
     enter_exit(op, dummy);
-    object_free2(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
+    object_free(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
     if (!op->contr->hidden)
         draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM,
                       "You see a portal open.");
@@ -972,9 +983,9 @@ void command_teleport(object *op, const char *params) {
  */
 void command_create(object *op, const char *params) {
     object *tmp = NULL;
-    uint32 i;
+    uint32_t i;
     int magic, set_magic = 0, set_nrof = 0, gotquote, gotspace;
-    uint32 nrof;
+    uint32_t nrof;
     char *cp, *bp, *bp2, *bp3, *endline, cpy[MAX_BUF];
     archetype *at, *at_spell = NULL;
     const artifact *art = NULL;
@@ -987,7 +998,7 @@ void command_create(object *op, const char *params) {
                       "Usage: create [nr] [magic] <archetype> [ of <artifact>] [variable_to_patch setting]");
         return;
     }
-    strncpy(cpy, params, sizeof(cpy));
+    safe_strncpy(cpy, params, sizeof(cpy));
     bp = cpy;
 
     /* We need to know where the line ends */
@@ -1044,7 +1055,7 @@ void command_create(object *op, const char *params) {
         if (!at_spell || at_spell->clone.type != SPELL)
             at_spell = find_archetype_by_object_name(cp);
         if (!at_spell || at_spell->clone.type != SPELL) {
-            strcpy(spell_name, cp);
+            safe_strncpy(spell_name, cp, sizeof(spell_name));
             fsp = strchr(spell_name, ' ');
             if (fsp) {
                 *fsp = 0;
@@ -1086,7 +1097,14 @@ void command_create(object *op, const char *params) {
         }
     } /* if cp */
 
-    if ((at->clone.type == ROD || at->clone.type == WAND || at->clone.type == SCROLL || at->clone.type == SPELLBOOK)
+    /* rods and potions can get their spell from the artifact */
+    if ((at->clone.type == ROD || at->clone.type == POTION) && !at_spell && (!art || !art->item->other_arch)) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+                             "Unable to find spell %s for object that needs it, or it is of wrong type",
+                             cp);
+        return;
+    }
+    if ((at->clone.type == WAND || at->clone.type == SCROLL || at->clone.type == SPELLBOOK)
     && !at_spell) {
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
                              "Unable to find spell %s for object that needs it, or it is of wrong type",
@@ -1106,7 +1124,7 @@ void command_create(object *op, const char *params) {
         set_abs_magic(tmp, magic);
     if (art)
         give_artifact_abilities(tmp, art->item);
-    if (need_identify(tmp)) {
+    if (!is_identifiable_type(tmp)) {
         SET_FLAG(tmp, FLAG_IDENTIFIED);
         CLEAR_FLAG(tmp, FLAG_KNOWN_MAGICAL);
         object_give_identified_properties(tmp);
@@ -1253,10 +1271,10 @@ void command_create(object *op, const char *params) {
             if (out_of_map(op->map, head->x+size_x, head->y+size_y)) {
                 if (head->x < size_x || head->y < size_y) {
                     dm_stack_pop(op->contr);
-                    object_free2(head, FREE_OBJ_NO_DESTROY_CALLBACK);
+                    object_free(head, FREE_OBJ_NO_DESTROY_CALLBACK);
                     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
                                   "Object too big to insert in map, or wrong position.");
-                    object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+                    object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
                     return;
                 }
 
@@ -1287,7 +1305,7 @@ void command_create(object *op, const char *params) {
     }
 
     /* free the one we used to copy */
-    object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+    object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
 }
 
 /*
@@ -1797,8 +1815,8 @@ void command_reset(object *op, const char *params) {
     int res = 0;
 
     if (*params == '\0') {
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                      "Reset what map [name]?");
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                MSG_TYPE_COMMAND_ERROR, "Which map should be reset?");
         return;
     }
 
@@ -1808,28 +1826,33 @@ void command_reset(object *op, const char *params) {
         params = space + 1;
     }
 
-    if (strcmp(params, ".") == 0)
-        snprintf(path, sizeof(path), "%s", op->map->path);
-    else
+    /* Use the DM's map if the current map was given. */
+    if (strcmp(params, ".") == 0) {
+        strlcpy(path, op->map->path, sizeof(path));
+    } else {
         path_combine_and_normalize(op->map->path, params, path, sizeof(path));
+    }
+
     m = has_been_loaded(path);
     if (m == NULL) {
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                      "No such map.");
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                MSG_TYPE_COMMAND_ERROR, "No such map.");
         return;
     }
 
     if (confirmation) {
-        if (strcmp(params, ".") == 0 && m->unique) {
-            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                                 "Can't reset a player unique map while on it, use 'reset full-reset %s' while not on it.",
-                                 m->path);
+        if (m->unique && (op->map == m)) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                    MSG_TYPE_COMMAND_ERROR,
+                    "Cannot reset a unique player map while on it. Use "
+                    "'reset full-reset %s' while standing somewhere else.",
+                    m->path);
             return;
         }
 
         if (strncmp("full-reset", confirmation, strlen("full-reset"))) {
-            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
-                          "Invalid confirmation, must be 'full-reset'.");
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                    MSG_TYPE_COMMAND_ERROR, "Confirm using 'full-reset'.");
             return;
         }
     }
@@ -1843,7 +1866,7 @@ void command_reset(object *op, const char *params) {
         return;
     }
 
-    snprintf(path, sizeof(path), "%s", m->path);
+    strlcpy(path, m->path, sizeof(path));
 
     if (m->in_memory != MAP_SWAPPED) {
         if (m->in_memory != MAP_IN_MEMORY) {
@@ -1889,7 +1912,7 @@ void command_reset(object *op, const char *params) {
         /* Need to re-insert player if swap failed for some reason */
         if (tmp) {
             object_insert_in_map_at(op, m, NULL, 0, op->x, op->y);
-            object_free2(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
+            object_free(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
         }
 
         if (res < 0 && res != SAVE_ERROR_PLAYER)
@@ -1916,7 +1939,6 @@ void command_reset(object *op, const char *params) {
     /* Here, map reset succeeded. */
 
     if (m && m->in_memory == MAP_SWAPPED) {
-
         if (confirmation) {
             map_remove_unique_files(m);
             LOG(llevDebug, "DM %s fully resetting map %s.\n", op->name, m->path);
@@ -1928,23 +1950,25 @@ void command_reset(object *op, const char *params) {
         flush_old_maps();
     }
 
-    if (confirmation)
-        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM,
-                             "Fully resetting map %s.",
-                             path);
-    else
-        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM,
-                             "Resetting map %s.",
-                             path);
+    /* Display the appropriate success message. */
+    if (confirmation) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                MSG_TYPE_COMMAND_DM, "Fully resetting map %s.", path);
+    } else {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                MSG_TYPE_COMMAND_DM, "Resetting map %s.", path);
+    }
 
     if (tmp) {
         enter_exit(tmp, dummy);
-        object_free2(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
+        object_free(dummy, FREE_OBJ_NO_DESTROY_CALLBACK);
     }
 
+    /* Remind the DM how to fully reset the map. */
     if (confirmation == NULL) {
-        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM,
-                         "Use 'reset full-reset %s' to fully reset the map.", params);
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND,
+                MSG_TYPE_COMMAND_DM,
+                "Use 'reset full-reset %s' to fully reset the map.", params);
     }
 }
 
@@ -1966,14 +1990,7 @@ void command_nowiz(object *op, const char *params) { /* 'noadm' is alias */
     if (settings.real_wiz == TRUE)
         CLEAR_FLAG(op, FLAG_WAS_WIZ);
     if (op->contr->hidden) {
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_DM,
-                      "You are no longer hidden from other players");
-        op->map->players++;
-        draw_ext_info_format(NDI_UNIQUE|NDI_ALL|NDI_DK_ORANGE, 5, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
-                             "%s has entered the game.",
-                             op->name);
-        op->contr->hidden = 0;
-        op->invisible = 1;
+        unhide(op);
     } else
         draw_ext_info(NDI_UNIQUE|NDI_ALL|NDI_LT_GREEN, 1, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM,
                       "The Dungeon Master is gone..");
@@ -2017,7 +2034,8 @@ static int checkdm(object *op, const char *pl_name, const char *pl_passwd, const
     }
 
     while (fgets(line_buf, 160, dmfile) != NULL) {
-        if (line_buf[0] == '#')
+        // Skip empty lines as well as commented ones.
+        if (line_buf[0] == '#' || line_buf[0] == '\n')
             continue;
         if (sscanf(line_buf, "%[^:]:%[^:]:%s\n", name, passwd, host) != 3) {
             LOG(llevError, "Warning - malformed dm file entry: %s\n", line_buf);
@@ -2069,7 +2087,7 @@ static int do_wizard_dm(object *op, const char *params, int silent) {
          * is probably something like a wiz_force which gives that and any
          * other desired abilities.
          */
-        clear_los(op);
+        clear_los(op->contr);
         op->contr->write_buf[0] = '\0';
 
         if (!silent)
@@ -2261,7 +2279,7 @@ static void command_learn_spell_or_prayer(object *op, const char *params, int sp
     }
 
     do_learn_spell(op, tmp, special_prayer);
-    object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+    object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
 }
 
 /**
@@ -2289,10 +2307,11 @@ void command_learn_special_prayer(object *op, const char *params) {
 }
 
 /**
- * Wizard wishes to forget a spell.
+ * Command for players to forget a spell. This command used to be available
+ * only to DMs.
  *
  * @param op
- * wizard.
+ * player
  * @param params
  * spell name to forget.
  */
@@ -2347,9 +2366,7 @@ void command_loadplugin(object *op, const char *params) {
         return;
     }
 
-    strcpy(buf, LIBDIR);
-    strcat(buf, "/plugins/");
-    strcat(buf, params);
+    snprintf(buf, sizeof(buf), LIBDIR"/plugins/%s", params);
     LOG(llevDebug, "Requested plugin file is %s\n", buf);
     if (plugins_init_plugin(buf) == 0) {
         LOG(llevInfo, "DM %s loaded plugin %s\n", op->name, params);
@@ -2648,7 +2665,7 @@ void command_style_map_info(object *op, const char *params) {
 
     for (mp = styles; mp != NULL; mp = mp->next) {
         maps_used++;
-        mapmem += MAP_WIDTH(mp)*MAP_HEIGHT(mp)*(sizeof(object *)+sizeof(MapSpace))+sizeof(mapstruct);
+        mapmem += map_size(mp) * (sizeof(object *)+sizeof(MapSpace))+sizeof(mapstruct);
         for (x = 0; x < MAP_WIDTH(mp); x++) {
             for (y = 0; y < MAP_HEIGHT(mp); y++) {
                 FOR_MAP_PREPARE(mp, x, y, tmp)
@@ -2719,6 +2736,20 @@ void command_purge_quest_definitions(object *op, const char * param) {
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, "Purged quests definitions.");
 }
 
+static void do_dump(object *who, object *what) {
+    StringBuffer *sb;
+    char *diff;
+
+    sb = stringbuffer_new();
+    object_dump(what, sb);
+    diff = stringbuffer_finish(sb);
+    draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, diff);
+    free(diff);
+
+    /* Let's push that item on the dm's stack */
+    dm_stack_push(who->contr, what->count);
+}
+
 /**
  * Player wants to dump object below her.
  *
@@ -2729,16 +2760,84 @@ void command_purge_quest_definitions(object *op, const char * param) {
  */
 void command_dumpbelow(object *op, const char *params) {
     if (op && op->below) {
-        StringBuffer *sb;
-        char *diff;
+        do_dump(op, op->below);
+    }
+}
 
-        sb = stringbuffer_new();
-        object_dump(op->below, sb);
-        diff = stringbuffer_finish(sb);
-        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, diff);
-        free(diff);
+/**
+ * Player wants to dump object above her.
+ *
+ * @param op
+ * player asking for information.
+ * @param params
+ * unused.
+ */
+void command_dumpabove(object *op, const char *params) {
+    if (op && op->above) {
+        do_dump(op, op->above);
+    }
+}
 
-        /* Let's push that item on the dm's stack */
-        dm_stack_push(op->contr, op->below->count);
+/**
+ * Wizard wants to know some server settings, so display.
+ * @param op wizard asking for settings.
+ * @param ignored ignored additional text.
+ */
+void command_settings(object *op, const char *ignored) {
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, "Server settings:"));
+
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * item power factor: %2f"), settings.item_power_factor);
+
+    if (settings.not_permadeth) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * death is not permanent"));
+    } else if (settings.resurrection) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * permanent death, resurrection is enabled"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * permanent death, resurrection is NOT enabled"));
+    }
+
+    if (settings.set_title) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can set their title"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can't set their title"));
+    }
+
+    if (settings.spell_encumbrance) {
+        if (settings.spell_failure_effects) {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * too much equipment can lead to spell failure and ill effects"));
+        } else {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * too much equipment can lead to spell failure but no ill effects"));
+        }
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * too much equipment can't lead to spell failure"));
+    }
+
+    if (settings.casting_time) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * casting takes time"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * casting is immediate"));
+    }
+
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * permanent experience: %d%%"), settings.permanent_exp_ratio);
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * death penalty %d%% or %d levels"), settings.death_penalty_ratio, settings.death_penalty_level);
+
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * friendly fire: %d%%"), settings.set_friendly_fire);
+
+    if (settings.no_player_stealing) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can't steal from other players"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can steal from other players"));
+    }
+
+    if (settings.create_home_portals) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can create portals from their apartments"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can't create portals from their apartments"));
+    }
+
+    if (settings.allow_denied_spells_writing) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can write spells they are denied"));
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, i18n(op, " * players can't write spells they are denied"));
     }
 }
