@@ -1,42 +1,29 @@
 /*
- * static char *rcsid_commands_c =
- *   "$Id: commands.c 13414 2010-06-19 15:28:21Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Contains the definition for all in-game commands a player can issue.
  */
 
-#include <global.h>
-#include <commands.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
 #include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "commands.h"
+#include "sproto.h"
 
 /* Added times to all the commands.  However, this was quickly done,
  * and probably needs more refinements.  All socket and DM commands
@@ -54,6 +41,7 @@ command_array_struct Commands[] = {
     { "bowmode", command_bowmode,        0.0 },
     { "brace", command_brace,            0.0 },
     { "cast", command_cast,              0.2 },   /* Is this right? */
+    { "delete", command_delete,          0.0 },
     { "disarm", command_disarm,          1.0 },
     { "dm", command_dm,                  0.0 },
     { "dmhide", command_dmhide,          0.0 }, /* Like dm, but don't tell a dm arrived, hide player */
@@ -62,6 +50,7 @@ command_array_struct Commands[] = {
     { "empty", command_empty,            1.0 },
     { "examine", command_examine,        0.5 },
     { "fix_me", command_fix_me,          0.0 },
+    { "forget_spell", command_forget_spell, 0.0 },
     { "get", command_take,               1.0 },
     { "help", command_help,              0.0 },
     { "hiscore", command_hiscore,        0.0 },
@@ -130,6 +119,7 @@ command_array_struct Commands[] = {
     { "run_stop", command_run_stop,      0.0 },
     { "fire", command_fire,              1.0 },
     { "fire_stop", command_fire_stop,    0.0 },
+    { "face", command_face,              0.0 },
     { "quest", command_quest,            0.0 },
     { "knowledge", command_knowledge,    0.0 }
 };
@@ -222,6 +212,7 @@ command_array_struct WizCommands [] = {
     { "diff", command_diff,                      0.0 },
     { "dmtell", command_dmtell,                  0.0 },
     { "dump", command_dump,                      0.0 },
+    { "dumpabove", command_dumpabove,            0.0 },
     { "dumpbelow", command_dumpbelow,            0.0 },
     { "dumpfriendlyobjects", command_dumpfriendlyobjects, 0.0 },
     { "dumpallarchetypes", command_dumpallarchetypes, 0.0 },
@@ -229,7 +220,6 @@ command_array_struct WizCommands [] = {
     { "dumpallobjects", command_dumpallobjects,  0.0 },
     { "dumpmap", command_dumpmap,                0.0 },
     { "follow", command_follow,                  0.0 },
-    { "forget_spell", command_forget_spell,      0.0 },
     { "free", command_free,                      0.0 },
     { "freeze", command_freeze,                  0.0 },
     { "goto", command_goto,                      0.0 },
@@ -252,6 +242,7 @@ command_array_struct WizCommands [] = {
     { "remove", command_remove,                  0.0 },
     { "reset", command_reset,                    0.0 },
     { "set_god", command_setgod,                 0.0 },
+    { "settings", command_settings,              0.0 },
     { "server_speed", command_speed,             0.0 },
     { "shutdown", command_shutdown,              0.0 },
     { "ssdumptable", command_ssdumptable,        0.0 },
