@@ -1,29 +1,15 @@
 /*
- * static char *rcsid_disease_c =
- *   "$Id: disease.c 16358 2011-12-29 08:56:15Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail to crossfire-devel@real-time.com
-*/
 
 /**
  * @file
@@ -124,16 +110,18 @@
  * speed            speed of movement, from DISEASE
  */
 
-#include <global.h>
-#include <object.h>
-#include <living.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
-#include <sounds.h>
-#include <skills.h>
+#include "global.h"
+
 #include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "living.h"
+#include "object.h"
+#include "skills.h"
+#include "sounds.h"
+#include "spells.h"
+#include "sproto.h"
 
 static void remove_symptoms(object *disease);
 static object *find_symptom(object *disease);
@@ -280,7 +268,7 @@ static object *find_symptom(object *disease) {
 static void check_infection(object *disease) {
     int x, y, range, mflags;
     mapstruct *map, *map2;
-    sint16 i, j, i2, j2;
+    int16_t i, j, i2, j2;
 
     range = abs(disease->magic);
     if (disease->env) {
@@ -540,7 +528,7 @@ static void do_symptoms(object *disease) {
     if (disease->stats.ac != 0) {
         float scale;
         int i;
-        sint8 cur_stat;
+        int8_t cur_stat;
 
         symptom->value += disease->stats.ac;
         scale = 1.0+symptom->value/100.0;
