@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_gods_c =
- *   "$Id: gods.c 14970 2011-08-11 20:44:45Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * All this functions handle gods: give presents, punish, and so on.
@@ -36,18 +21,20 @@
  * Sept 1996 - moved code over to object -oriented gods -b.t.
  */
 
-#include <global.h>
-#include <living.h>
-#include <object.h>
-#include <spells.h>
-#include <sounds.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
 
-static int worship_forbids_use(object *op, object *exp_obj, uint32 flag, const char *string);
+#include <stdlib.h>
+#include <string.h>
+
+#include "living.h"
+#include "object.h"
+#include "sounds.h"
+#include "spells.h"
+#include "sproto.h"
+
+static int worship_forbids_use(object *op, object *exp_obj, uint32_t flag, const char *string);
 static void stop_using_item(object *op, int type, int number);
-static void update_priest_flag(const object *god, object *exp_ob, uint32 flag);
+static void update_priest_flag(const object *god, object *exp_ob, uint32_t flag);
 static void god_intervention(object *op, const object *god, object *skill, object *altar);
 static int god_examines_priest(object *op, const object *god);
 static int god_examines_item(const object *god, object *item);
@@ -145,10 +132,17 @@ const char *determine_god(object *op) {
             godnr = rndm(1, gl->id);
             while (gl) {
                 if (gl->id == godnr)
+                {
+                    // This used to be after the loop, but if we do not find a god,
+                    // it would have made a null pointer dereference. Moved here
+                    // for better clarity and (hopefully) fewer bugs.
+                    // SilverNexus 2018-01-17
+                    op->title = add_string(gl->name);
+
                     break;
+                }
                 gl = gl->next;
             }
-            op->title = add_string(gl->name);
         }
 
         return op->title;
@@ -279,7 +273,7 @@ static int god_gives_present(object *op, const object *god, treasure *tr) {
 
     /* And just in case nothing was put and something is needed, bail out. */
     if ((tmp->type == ROD || tmp->type == WAND) && (tmp->inv == NULL)) {
-        object_free2(tmp, 0);
+        object_free(tmp, 0);
         return 0;
     }
 
@@ -295,6 +289,26 @@ static int god_gives_present(object *op, const object *god, treasure *tr) {
     return 1;
 }
 
+/**
+ * Try to leave a cult. Deducts experience from 'skill' proportional to 'angry'.
+ * Returns true if successful (only when 'angry' is 1) or false otherwise.
+ */
+static bool try_leave_cult(object* pl, object* skill, int angry) {
+    const uint64_t loss = angry * (skill->stats.exp / 10);
+    if (loss)
+        change_exp(pl, -random_roll64(0, loss, pl, PREFER_LOW),
+                   skill ? skill->skill : "none", SK_SUBTRACT_SKILL_EXP);
+
+    /* random chance based on our current level
+     * note it gets harder the higher we get
+     */
+    if ((angry == 1) && !(random_roll(0, skill->level, pl, PREFER_LOW))) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
 /**
  * Player prays at altar.
  * Checks for god changing, divine intervention, and so on.
@@ -346,7 +360,6 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
         if (((random_roll(0, 399, pl, PREFER_LOW))-bonus) < 0)
             god_intervention(pl, pl_god, skill, altar);
     } else { /* praying to another god! */
-        uint64 loss = 0;
         int angry = 1;
 
         /* I believe the logic for detecting opposing gods was completely
@@ -378,20 +391,8 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
                                  "Heretic! %s is angered!",
                                  pl_god->name);
 
-        /* whether we will be successfull in defecting or not -
-         * we lose experience from the clerical experience obj
-         */
-
-        loss = angry*(skill->stats.exp/10);
-        if (loss)
-            change_exp(pl, -random_roll64(0, loss, pl, PREFER_LOW), skill ? skill->skill : "none", SK_SUBTRACT_SKILL_EXP);
-
-        /* May switch Gods, but its random chance based on our current level
-         * note it gets harder to swap gods the higher we get
-         */
-        if ((angry == 1) && !(random_roll(0, skill->level, pl, PREFER_LOW))) {
-            if (become_follower(pl, &altar->other_arch->clone))
-                remove_special_prayers(pl, pl_god);
+        if (try_leave_cult(pl, skill, angry)) {
+            become_follower(pl, &altar->other_arch->clone);
         } else {
             /* toss this player off the altar.  He can try again. */
             draw_ext_info(NDI_UNIQUE|NDI_NAVY, 0, pl, MSG_TYPE_ATTRIBUTE, MSG_TYPE_ATTRIBUTE_GOD,
@@ -473,18 +474,9 @@ static void remove_special_prayers(object *op, const object *god) {
  * new god to worship.
  * @return
  * 1 if successfully converted, 0 if the god doesn't like the race, or something else.
- * @todo isn't there duplication with remove_special_prayers() for spell removing?
  * @todo split the check to make this function only actually become follower
  */
 int become_follower(object *op, const object *new_god) {
-    const object *old_god = NULL;                      /* old god */
-    treasure *tr;
-    object *skop;
-    int i, sk_applied;
-    int undeadified = 0; /* Turns to true if changing god can changes the undead
-                          * status of the player.*/
-    old_god = find_god(determine_god(op));
-
     /* take away any special god-characteristic items. */
     FOR_INV_PREPARE(op, item) {
         /* remove all invisible startequip items which are
@@ -502,6 +494,7 @@ int become_follower(object *op, const object *new_god) {
     } FOR_INV_FINISH();
 
     /* remove any items given by the old god */
+    const object *old_god = find_god(determine_god(op));
     if (old_god) {
         /* Changed to use the new "divine_giver_name" key_value
          *   so it can reliably delete enchanted items.  Now it loops
@@ -509,6 +502,7 @@ int become_follower(object *op, const object *new_god) {
          *   treasure list.
          */
         follower_remove_given_items(op, op, old_god);
+        remove_special_prayers(op, old_god);
     }
 
     if (!op || !new_god)
@@ -526,7 +520,7 @@ int become_follower(object *op, const object *new_god) {
     }
 
     /* give the player any special god-characteristic-items. */
-    for (tr = new_god->randomitems->items; tr != NULL; tr = tr->next) {
+    for (treasure *tr = new_god->randomitems->items; tr != NULL; tr = tr->next) {
         if (tr->item
         && tr->item->clone.invisible
         && tr->item->clone.type != SPELLBOOK
@@ -539,7 +533,7 @@ int become_follower(object *op, const object *new_god) {
                          "You become a follower of %s!",
                          new_god->name);
 
-    skop = object_find_by_type_subtype(op, SKILL, SK_PRAYING);
+    object *skop = object_find_by_type_subtype(op, SKILL, SK_PRAYING);
     /* Player has no skill - give them the skill */
     if (!skop) {
         /* The archetype should always be defined - if we crash here because it doesn't,
@@ -549,8 +543,10 @@ int become_follower(object *op, const object *new_god) {
         link_player_skills(op);
     }
 
-    sk_applied = QUERY_FLAG(skop, FLAG_APPLIED); /* save skill status */
+    int sk_applied = QUERY_FLAG(skop, FLAG_APPLIED); /* save skill status */
 
+    int undeadified = 0; /* Turns to true if changing god can changes the undead
+                          * status of the player.*/
     /* Clear the "undead" status. We also need to force a call to change_abil,
      * so I set undeadified for that.
      * - gros, 21th July 2006.
@@ -584,16 +580,16 @@ int become_follower(object *op, const object *new_god) {
     /* make sure that certain immunities do NOT get passed
      * to the follower!
      */
-    for (i = 0; i < NROFATTACKS; i++)
+    for (int i = 0; i < NROFATTACKS; i++)
         if (skop->resist[i] > 30
         && (i == ATNR_FIRE || i == ATNR_COLD || i == ATNR_ELECTRICITY || i == ATNR_POISON))
             skop->resist[i] = 30;
 
-    skop->stats.hp = (sint16)new_god->last_heal;
-    skop->stats.sp = (sint16)new_god->last_sp;
-    skop->stats.grace = (sint16)new_god->last_grace;
-    skop->stats.food = (sint16)new_god->last_eat;
-    skop->stats.luck = (sint8)new_god->stats.luck;
+    skop->stats.hp = (int16_t)new_god->last_heal;
+    skop->stats.sp = (int16_t)new_god->last_sp;
+    skop->stats.grace = (int16_t)new_god->last_grace;
+    skop->stats.food = (int16_t)new_god->last_eat;
+    skop->stats.luck = (int8_t)new_god->stats.luck;
     /* gods may pass on certain flag properties */
     update_priest_flag(new_god, skop, FLAG_SEE_IN_DARK);
     update_priest_flag(new_god, skop, FLAG_REFL_SPELL);
@@ -641,9 +637,6 @@ int become_follower(object *op, const object *new_god) {
     if (!sk_applied)
         CLEAR_FLAG(skop, FLAG_APPLIED);
 
-    // check is now done after converting
-    //remove_special_prayers(op, new_god);
-
     return 1;
 }
 
@@ -660,7 +653,7 @@ int become_follower(object *op, const object *new_god) {
  * @return
  * 1 if player was changed, 0 if no change.
  */
-static int worship_forbids_use(object *op, object *exp_obj, uint32 flag, const char *string) {
+static int worship_forbids_use(object *op, object *exp_obj, uint32_t flag, const char *string) {
     if (QUERY_FLAG(&op->arch->clone, flag)) {
         if (QUERY_FLAG(op, flag) != QUERY_FLAG(exp_obj, flag)) {
             update_priest_flag(exp_obj, op, flag);
@@ -713,7 +706,7 @@ static void stop_using_item(object *op, int type, int number) {
  * @param flag
  * flag to consider.
  */
-static void update_priest_flag(const object *god, object *exp_ob, uint32 flag) {
+static void update_priest_flag(const object *god, object *exp_ob, uint32_t flag) {
     if (QUERY_FLAG(god, flag) && !QUERY_FLAG(exp_ob, flag))
         SET_FLAG(exp_ob, flag);
     else if (QUERY_FLAG(exp_ob, flag) && !QUERY_FLAG(god, flag)) {
@@ -893,7 +886,7 @@ static int improve_weapon_magic(object *op, object *tr, object *weapon, object *
 static int god_enchants_weapon(object *op, const object *god, object *tr, object *skill) {
     char buf[MAX_BUF];
     object *weapon;
-    uint32 attacktype;
+    uint32_t attacktype;
 
     weapon = object_find_by_type_applied(op, WEAPON);
     if (weapon == NULL)
@@ -1004,9 +997,6 @@ static void god_intervention(object *op, const object *god, object *skill, objec
         return;
     }
 
-    // removed on 2009-12-12 because the function now removes prayers NOT from god.
-    //remove_special_prayers(op, god);
-
     /* lets do some checks of whether we are kosher with our god */
     if (god_examines_priest(op, god) < 0)
         return;
@@ -1133,28 +1123,10 @@ static void god_intervention(object *op, const object *god, object *skill, objec
         /* Heal depletion */
         if (item->type == BOOK && item->invisible
             && strcmp(item->name, "heal depletion") == 0) {
-            object *depl;
-            archetype *at;
-            int i;
-
-            if ((at = find_archetype(ARCH_DEPLETION)) == NULL) {
-                LOG(llevError, "Could not find archetype depletion.\n");
-                continue;
+            if (remove_depletion(op, -1)) {
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_PRAY,
+                              "Shimmering light surrounds and restores you!");
             }
-            depl = arch_present_in_ob(at, op);
-            if (depl == NULL)
-                continue;
-            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_PRAY,
-                          "Shimmering light surrounds and restores you!");
-            for (i = 0; i < NUM_STATS; i++)
-                if (get_attr_value(&depl->stats, i))
-                    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ATTRIBUTE,
-                                  MSG_TYPE_ATTRIBUTE_BAD_EFFECT_END,
-                                  restore_msg[i]);
-            object_remove(depl);
-            object_free_drop_inventory(depl);
-            fix_object(op);
-            return;
         }
 
         /* Voices */
