@@ -1,177 +1,214 @@
 /*
- * static char *rcsid_init_c =
- *   "$Id: init.c 16018 2011-12-03 15:13:09Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2010-2011 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
- * Server initialisation, settings loading, command-line handling and such.
+ * Server initialization, settings, and command-line handling.
  */
 
-#include <global.h>
-#include <loader.h>
-#include <version.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <errno.h>
+#include <signal.h>
+#include <stdlib.h>
+#include <string.h>
 
 /* Needed for strcasecmp(). */
 #ifndef WIN32
 #include <strings.h>
 #endif
 
+#include "loader.h"
+#include "version.h"
+#include "server.h"
+#include "sproto.h"
+
 static void help(void);
-static void usage(void);
 static void init_beforeplay(void);
 static void init_startup(void);
-static void compile_info(void);
 static void init_signals(void);
 static void init_races(void);
 static void dump_races(void);
 static void add_to_racelist(const char *race_name, object *op);
 static racelink *get_racelist(void);
-static void fatal_signal(int make_core);
-
-/** Filename when in daemon mode. */
-static char default_daemon_log[] = "logfile";
 
+/**
+ * Command line option: set logfile name.
+ * @param val new name.
+ */
 static void set_logfile(char *val) {
     settings.logfilename = val;
 }
 
+/** Command line option: show version. */
 static void call_version(void) {
-    version(NULL);
-    exit(0);
-}
-
-static void showscores(void) {
-    hiscore_display(NULL, 9999, "");
-    exit(0);
+    puts(FULL_VERSION);
+    exit(EXIT_SUCCESS);
 }
 
+/** Command line option: debug flag. */
 static void set_debug(void) {
     settings.debug = llevDebug;
 }
 
+/** Command line option: unset debug flag. */
 static void unset_debug(void) {
     settings.debug = llevInfo;
 }
 
+/** Command line option: monster debug flag. */
 static void set_mondebug(void) {
     settings.debug = llevMonster;
 }
 
+/** Command line option: dump monsters. */
 static void set_dumpmon1(void) {
     settings.dumpvalues = 1;
 }
 
+/** Command line option: dump abilities. */
 static void set_dumpmon2(void) {
     settings.dumpvalues = 2;
 }
 
+/** Command line option: dump artifacts. */
 static void set_dumpmon3(void) {
     settings.dumpvalues = 3;
 }
 
+/** Command line option: dump spells. */
 static void set_dumpmon4(void) {
     settings.dumpvalues = 4;
 }
 
+/** Command line option: ? */
 static void set_dumpmon5(void) {
     settings.dumpvalues = 5;
 }
 
+/** Command line option: dump races. */
 static void set_dumpmon6(void) {
     settings.dumpvalues = 6;
 }
 
+/** Command line option: dump alchemy. */
 static void set_dumpmon7(void) {
     settings.dumpvalues = 7;
 }
 
+/** Command line option: dump gods. */
 static void set_dumpmon8(void) {
     settings.dumpvalues = 8;
 }
 
+/** Command line option: dump alchemy costs. */
 static void set_dumpmon9(void) {
     settings.dumpvalues = 9;
 }
 
+/**
+ * Command line option: dump monster treasures.
+ * @param name monster's name to dump treasure for.
+ */
 static void set_dumpmont(const char *name) {
     settings.dumpvalues = 10;
     settings.dumparg = name;
 }
 
-static void set_daemon(void) {
-    settings.daemonmode = 1;
-    if (settings.logfilename[0] == '\0') {
-        settings.logfilename = default_daemon_log;
-    }
-}
-
+/**
+ * Command line option: set data path.
+ * @param path new path.
+ */
 static void set_datadir(const char *path) {
     settings.datadir = path;
 }
 
+/**
+ * Command line option: set configuration path.
+ * @param path new path.
+ */
 static void set_confdir(const char *path) {
     settings.confdir = path;
 }
 
+/**
+ * Command line option: set local path.
+ * @param path new path.
+ */
 static void set_localdir(const char *path) {
     settings.localdir = path;
 }
 
+/**
+ * Command line option: set map path.
+ * @param path new path.
+ */
 static void set_mapdir(const char *path) {
     settings.mapdir = path;
 }
 
+/**
+ * Command line option: set archetypes file name.
+ * @param path new name.
+ */
 static void set_archetypes(const char *path) {
     settings.archetypes = path;
 }
 
+/**
+ * Command line option: set regions file name.
+ * @param path new name.
+ */
 static void set_regions(const char *path) {
     settings.regions = path;
 }
 
+/**
+ * Command line option: set treasures file name.
+ * @param path new name.
+ */
 static void set_treasures(const char *path) {
     settings.treasures = path;
 }
 
+/**
+ * Command line option: set unique path.
+ * @param path new path.
+ */
 static void set_uniquedir(const char *path) {
     settings.uniquedir = path;
 }
 
+/**
+ * Command line option: set template path.
+ * @param path new path.
+ */
 static void set_templatedir(const char *path) {
     settings.templatedir = path;
 }
 
+/**
+ * Command line option: set player path.
+ * @param path new path.
+ */
 static void set_playerdir(const char *path) {
     settings.playerdir = path;
 }
 
+/**
+ * Command line option: set temporary file path.
+ * @param path new path.
+ */
 static void set_tmpdir(const char *path) {
     settings.tmpdir = path;
 }
@@ -180,11 +217,6 @@ static void free_races(void);
 
 static void free_materials(void);
 
-static void showscoresparm(const char *data) {
-    hiscore_display(NULL, 9999, data);
-    exit(0);
-}
-
 /**
  * Change the server's port. Will exit() if invalid value.
  *
@@ -211,6 +243,14 @@ static void set_disable_plugin(const char *name) {
     settings.disabled_plugins = disable;
 }
 
+/**
+ * Dump all animations, then exit.
+ */
+static void server_dump_animations(void) {
+    dump_animations();
+    cleanup();
+}
+
 /** Typedefs used when calling option handlers. */
 /*@{*/
 typedef void (*cmdlinefunc_args0)(void);
@@ -225,8 +265,8 @@ typedef void (*cmdlinefunc_args2)(const char* arg1, const char* arg2);
  */
 struct Command_Line_Options {
     const char *cmd_option; /**< How it is called on the command line. */
-    uint8 num_args;         /**< Number or args it takes. */
-    uint8 pass;             /**< What pass this should be processed on. @todo describe passes :) */
+    uint8_t num_args;         /**< Number or args it takes. */
+    uint8_t pass;             /**< What pass this should be processed on. @todo describe passes :) */
     void (*func)();         /**< function to call when we match this.
                              * if num_args is true, than that gets passed
                              * to the function, otherwise nothing is passed
@@ -245,27 +285,24 @@ static struct Command_Line_Options options[] = {
     /** Pass 1 functions - Stuff that can/should be called before we actually
      * initialize any data.
      */
-    { "-h", 0, 1, help },
-    /* Honor -help also, since it is somewhat common */
-    { "-help", 0, 1, help },
-    { "-v", 0, 1, call_version },
-    { "-d", 0, 1, set_debug },
-    { "+d", 0, 1, unset_debug },
-    { "-mon", 0, 1, set_mondebug },
-    { "-data", 1, 1, set_datadir },
-    { "-conf", 1, 1, set_confdir },
-    { "-local", 1, 1, set_localdir },
-    { "-maps", 1, 1, set_mapdir },
     { "-arch", 1, 1, set_archetypes },
-    { "-regions", 1, 1, set_regions },
+    { "-conf", 1, 1, set_confdir },
+    { "-d", 0, 1, set_debug },
+    { "-data", 1, 1, set_datadir },
+    { "-disable-plugin", 1, 1, set_disable_plugin },
+    { "-h", 0, 1, help },
+    { "-local", 1, 1, set_localdir },
+    { "-log", 1, 1, set_logfile },
+    { "-maps", 1, 1, set_mapdir },
+    { "-mon", 0, 1, set_mondebug },
+    { "-n", 0, 1, unset_debug },
     { "-playerdir", 1, 1, set_playerdir },
-    { "-treasures", 1, 1, set_treasures },
-    { "-uniquedir", 1, 1, set_uniquedir },
+    { "-regions", 1, 1, set_regions },
     { "-templatedir", 1, 1, set_templatedir },
     { "-tmpdir", 1, 1, set_tmpdir },
-    { "-log", 1, 1, set_logfile },
-    { "-detach", 0, 1, set_daemon },
-    { "-disable-plugin", 1, 1, set_disable_plugin },
+    { "-treasures", 1, 1, set_treasures },
+    { "-uniquedir", 1, 1, set_uniquedir },
+    { "-v", 0, 1, call_version },
 
 #ifdef WIN32
     /* Windows service stuff */
@@ -277,12 +314,11 @@ static struct Command_Line_Options options[] = {
     /** Pass 2 functions.  Most of these could probably be in pass 1,
      * as they don't require much of anything to bet set up.
      */
-    { "-csport", 1, 2, set_csport },
+    { "-p", 1, 2, set_csport },
 
     /** Start of pass 3 information. In theory, by pass 3, all data paths
      * and defaults should have been set up.
      */
-    { "-o", 0, 3, compile_info },
     { "-m", 0, 3, set_dumpmon1 },
     { "-m2", 0, 3, set_dumpmon2 },
     { "-m3", 0, 3, set_dumpmon3 },
@@ -295,8 +331,7 @@ static struct Command_Line_Options options[] = {
     { "-mt", 1, 3, set_dumpmont },
     { "-mexp", 0, 3, dump_experience },
     { "-mq", 0, 3, dump_quests },
-    { "-s", 0, 3, showscores },
-    { "-score", 1, 3, showscoresparm }
+    { "-dump-anims", 0, 3, server_dump_animations },
 };
 
 /**
@@ -347,7 +382,7 @@ static void parse_args(int argc, char *argv[], int pass) {
         }
         if (i == sizeof(options)/sizeof(struct Command_Line_Options)) {
             fprintf(stderr, "Unknown option: %s\n", argv[on_arg]);
-            usage();
+            fprintf(stderr, "Type '%s -h' for usage.\n", argv[0]);
             exit(1);
         }
     }
@@ -389,7 +424,6 @@ static void load_materials(void) {
     int i, value;
 
     snprintf(filename, sizeof(filename), "%s/materials", settings.datadir);
-    LOG(llevDebug, "Reading material type data from %s...\n", filename);
     if ((fp = fopen(filename, "r")) == NULL) {
         LOG(llevError, "Cannot open %s for reading\n", filename);
         mt = get_empty_mat();
@@ -430,7 +464,7 @@ static void load_materials(void) {
                 if ((next = strchr(cp, ',')) != NULL)
                     *(next++) = '\0';
                 sscanf(cp, "%d", &value);
-                mt->save[i] = (sint8)value;
+                mt->save[i] = (int8_t)value;
                 cp = next;
             }
         } else if (!strncmp(cp, "mods", 4)) {
@@ -443,14 +477,14 @@ static void load_materials(void) {
                 if ((next = strchr(cp, ',')) != NULL)
                     *(next++) = '\0';
                 sscanf(cp, "%d", &value);
-                mt->mod[i] = (sint8)value;
+                mt->mod[i] = (int8_t)value;
                 cp = next;
             }
         }
     }
     free(mt->next);
     mt->next = NULL;
-    LOG(llevDebug, "Done.\n");
+    LOG(llevDebug, "loaded material type data\n");
     fclose(fp);
 }
 
@@ -522,22 +556,22 @@ static void load_settings(void) {
             }
         } else if (!strcasecmp(buf, "metaserver_server")) {
             if (has_val)
-                strcpy(settings.meta_server, cp);
+                safe_strncpy(settings.meta_server, cp, sizeof(settings.meta_server));
             else
                 LOG(llevError, "load_settings: metaserver_server must have a value.\n");
         } else if (!strcasecmp(buf, "motd")) {
             if (has_val)
-                strcpy(settings.motd, cp);
+                safe_strncpy(settings.motd, cp, sizeof(settings.motd));
             else
                 LOG(llevError, "load_settings: motd must have a value.\n");
         } else if (!strcasecmp(buf, "dm_mail")) {
             if (has_val)
-                strcpy(settings.dm_mail, cp);
+                safe_strncpy(settings.dm_mail, cp, sizeof(settings.dm_mail));
             else
                 LOG(llevError, "load_settings: dm_mail must have a value.\n");
         } else if (!strcasecmp(buf, "metaserver_host")) {
             if (has_val)
-                strcpy(settings.meta_host, cp);
+                safe_strncpy(settings.meta_host, cp, sizeof(settings.meta_host));
             else
                 LOG(llevError, "load_settings: metaserver_host must have a value.\n");
         } else if (!strcasecmp(buf, "port")) {
@@ -550,7 +584,7 @@ static void load_settings(void) {
             else
                 settings.meta_port = port;
         } else if (!strcasecmp(buf, "metaserver_comment")) {
-            strcpy(settings.meta_comment, cp);
+            safe_strncpy(settings.meta_comment, cp, sizeof(settings.meta_comment));
         } else if (!strcasecmp(buf, "worldmapstartx")) {
             int size = atoi(cp);
 
@@ -672,12 +706,25 @@ static void load_settings(void) {
             } else {
                 LOG(llevError, "load_settings: Unknown value for recycle_tmp_maps: %s\n", cp);
             }
+        } else if (!strcasecmp(buf, "always_show_hp")) {
+            if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+                settings.always_show_hp = 2;
+            } else if (!strcasecmp(cp, "damaged")) {
+                settings.always_show_hp = 1;
+            } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+                settings.always_show_hp = 0;
+            } else {
+                LOG(llevError, "load_settings: Unknown value for always_show_hp: %s\n", cp);
+            }
         } else if (!strcasecmp(buf, "who_format")) {
             if (has_val)
-                strcpy(settings.who_format, cp);
+                safe_strncpy(settings.who_format, cp,
+                        sizeof(settings.who_format));
         } else if (!strcasecmp(buf, "who_wiz_format")) {
-            if (has_val)
-                strcpy(settings.who_wiz_format, cp);
+            if (has_val) {
+                safe_strncpy(settings.who_wiz_format, cp,
+                        sizeof(settings.who_wiz_format));
+            }
         } else if (!strcasecmp(buf, "spellpoint_level_depend")) {
             if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
                 settings.spellpoint_level_depend = TRUE;
@@ -737,7 +784,7 @@ static void load_settings(void) {
             else
                 settings.item_power_factor = tmp;
         } else if (!strcasecmp(buf, "pk_luck_penalty")) {
-            sint16 val = atoi(cp);
+            int16_t val = atoi(cp);
 
             if (val < -100 || val > 100)
                 LOG(llevError, "load_settings: pk_luck_penalty must be between -100 and 100, %d is invalid\n", val);
@@ -809,7 +856,7 @@ static void load_settings(void) {
                 LOG(llevError, "load_settings: unknown value for personalized_blessings: %s\n", cp);
             }
         } else if (!strcasecmp(buf, "pk_max_experience")) {
-            sint64 pkme = atoll(cp);
+            int64_t pkme = atoll(cp);
             if (pkme < 0)
                 pkme = -1;
             settings.pk_max_experience = pkme;
@@ -900,6 +947,32 @@ static void load_settings(void) {
             } else {
                 LOG(llevError, "load_settings: unknown value for ignore_plugin_compatibility: %s\n", cp);
             }
+        } else if (!strcasecmp(buf, "account_block_create")) {
+            if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+                settings.account_block_create = TRUE;
+            } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+                settings.account_block_create = FALSE;
+            } else {
+                LOG(llevError, "load_settings: unknown value for account_block_create: %s\n", cp);
+            }
+        } else if (!strcasecmp(buf, "account_trusted_host")) {
+            free(settings.account_trusted_host);
+            settings.account_trusted_host = strdup_local(cp);
+        } else if (!strcasecmp(buf, "crypt_mode")) {
+            int val = atoi(cp);
+            if (val != 0 && val != 1) {
+                LOG(llevError, "load_settings: crypt_mode must be 0 or 1\n");
+            } else {
+                settings.crypt_mode = val;
+            }
+        } else if (!strcasecmp(buf, "min_name")) {
+            int val = atoi(cp);
+
+            if (val < 1 || val > MAX_NAME )
+                LOG(llevError, "load_settings: min_name (%d) need to be within %d-%d\n",
+                    val, 1, MAX_NAME);
+            else
+                settings.min_name = val;
         } else {
             LOG(llevError, "Unknown value in settings file: %s\n", buf);
         }
@@ -921,35 +994,32 @@ static void load_settings(void) {
 }
 
 /**
- * This is the main server initialisation function.
+ * This is the main server initialization function.
  *
  * Called only once, when starting the program.
+ * @param argc argument count.
+ * @param argv arguments on the command line.
  */
 void init(int argc, char **argv) {
-    init_done = 0;  /* Must be done before init_signal() */
+    init_done = 0;      /* Must be done before init_signal() */
     logfile = stderr;
-    parse_args(argc, argv, 1); /* First arg pass - right now it does
-     * nothing, but in future specifying the
-     * LibDir in this pass would be reasonable*/
 
-    init_library(); /* Must be called early */
-    load_settings(); /* Load the settings file */
+    /* First argument pass - right now it does nothing, but in the future specifying
+     * the LibDir in this pass would be reasonable. */
+    parse_args(argc, argv, 1);
+
+    init_library();     /* Must be called early */
+    load_settings();    /* Load the settings file */
     load_materials();
     parse_args(argc, argv, 2);
-    fprintf(logfile, "Welcome to CrossFire, v%s\n", FULL_VERSION);
-    fprintf(logfile, "Copyright (C) 1994,2011 Mark Wedel & Crossfire Development Team\n");
-    fprintf(logfile, "Copyright (C) 1992 Frank Tore Johansen.\n");
 
-    if (strcmp(settings.dm_mail, "") != 0) {
-        fprintf(logfile, "Maintained locally by: %s\n", settings.dm_mail);
-        fprintf(logfile, "Questions and bugs should be mailed to above address.\n");
-    }
+    LOG(llevInfo, "Crossfire %s\n", FULL_VERSION);
     SRANDOM(time(NULL));
 
-    init_startup(); /* Write (C), check shutdown/forbid files */
-    init_signals(); /* Sets up signal interceptions */
-    init_commands(); /* Sort command tables */
-    read_map_log(); /* Load up the old temp map files */
+    init_startup();     /* Check shutdown/forbid files */
+    init_signals();     /* Sets up signal interceptions */
+    init_commands();    /* Sort command tables */
+    read_map_log();     /* Load up the old temp map files */
     init_skills();
     init_ob_methods();
     cftimer_init();
@@ -957,15 +1027,10 @@ void init(int argc, char **argv) {
 
     parse_args(argc, argv, 3);
 
-#ifndef WIN32 /* ***win32: no become_daemon in windows */
-    if (settings.daemonmode)
-        become_daemon();
-#endif
-
     init_beforeplay();
     init_server();
     metaserver2_init();
-    account_load_entries();
+    accounts_load();
     reset_sleep();
     init_done = 1;
 }
@@ -982,63 +1047,60 @@ void free_server(void) {
     free_quest_definitions();
     while (settings.disabled_plugins) {
         linked_char *next = settings.disabled_plugins->next;
-        free(settings.disabled_plugins->name);
+        free((void *)settings.disabled_plugins->name);
         free(settings.disabled_plugins);
         settings.disabled_plugins = next;
     }
 }
 
-static void usage(void) {
-    (void)fprintf(logfile, "Usage: crossfire [-h] [-<flags>]...\n");
-}
-
+/**
+ * Display the command line options and exits.
+ */
 static void help(void) {
-    /* The information in usage is redundant with what is given below, so why call it? */
-    /*    usage();*/
-    printf("Flags:\n");
-    printf(" -csport <port> Specifies the port to use for the new client/server code.\n");
-    printf(" -d           Turns on some debugging.\n");
-    printf(" +d           Turns off debugging (useful if server compiled with debugging\n");
-    printf("              as default).\n");
-    printf(" -detach      The server will go in the background, closing all\n");
-    printf("              connections to the tty.\n");
-    printf(" -h           Display this information.\n");
-    printf(" -log <file>  Specifies which file to send output to.\n");
-    printf("              Only has meaning if -detach is specified.\n");
-    printf(" -mon         Turns on monster debugging.\n");
-    printf(" -o           Prints out info on what was defined at compile time.\n");
-    printf(" -s           Display the high-score list.\n");
-    printf(" -score <name or class> Displays all high scores with matching name/class.\n");
-    printf(" -v           Print version and developer contact information.\n");
-    printf(" -conf        Sets the configuration dir (settings, motd, etc.)\n");
-    printf(" -data        Sets the lib dir (archetypes, treasures, etc.)\n");
-    printf(" -local       Read/write local data (hiscore, unique items, etc.)\n");
-    printf(" -maps        Sets the directory for maps.\n");
-    printf(" -arch        Sets the archetype file to use.\n");
-    printf(" -regions     Sets the regions file to use.\n");
-    printf(" -playerdir   Sets the directory for the player files.\n");
-    printf(" -templatedir Sets the directory for template generate maps.\n");
-    printf(" -treasures   Sets the treasures file to use.\n");
-    printf(" -uniquedir   Sets the unique items/maps directory.\n");
-    printf(" -tmpdir      Sets the directory for temporary files (mostly maps.)\n");
-    printf(" -m           Lists out suggested experience for all monsters.\n");
-    printf(" -m2          Dumps out abilities.\n");
-    printf(" -m3          Dumps out artifact information.\n");
-    printf(" -m4          Dumps out spell information.\n");
-    printf(" -m5          Dumps out skill information.\n");
-    printf(" -m6          Dumps out race information.\n");
-    printf(" -m7          Dumps out alchemy information.\n");
-    printf(" -m8          Dumps out gods information.\n");
-    printf(" -m9          Dumps out more alchemy information (formula checking).\n");
-    printf(" -mt <name>   Dumps out list of treasures for a monster.\n");
-    printf(" -mexp        Dumps out the experience table.\n");
-    printf(" -mq          Dumps out the list of defined quests.\n");
+    printf("Usage: crossfire-server [options]\n\n");
+
+    printf("Options:\n");
+    printf(" -arch        Set the file with archetype definitions.\n");
+    printf(" -conf        Set the directory to find configuration files.\n");
+    printf(" -d           Turn on extra debugging messages.\n");
+    printf(" -data        Set the data (share/) directory (archetypes, treasures, etc).\n");
     printf(" -disable-plugin\n"
            "              Disables specified plugin. Use the name without the extension.\n"
            "              Can be specified multiple times. 'All' disables all plugins.\n");
-    exit(0);
+    printf(" -dump-anims  Dump animations.\n");
+    printf(" -h           Print this help message.\n");
+    printf(" -local       Set the local data (var/) directory.\n");
+    printf(" -log <file>  Write logging information to the given file.\n");
+    printf(" -m           List suggested experience for all monsters.\n");
+    printf(" -m2          Dump monster abilities.\n");
+    printf(" -m3          Dump artifact information.\n");
+    printf(" -m4          Dump spell information.\n");
+    printf(" -m5          Dump skill information.\n");
+    printf(" -m6          Dump race information.\n");
+    printf(" -m7          Dump alchemy information.\n");
+    printf(" -m8          Dump gods information.\n");
+    printf(" -m9          Dump more alchemy information (formula checking).\n");
+    printf(" -maps        Set the map directory.\n");
+    printf(" -mexp        Dump the experience table.\n");
+    printf(" -mon         Turn on monster debugging.\n");
+    printf(" -mq          Dump the quest list.\n");
+    printf(" -mt <name>   Dump a list of treasures for a monster.\n");
+    printf(" -n           Turn off debugging messages if on by default.\n");
+    printf(" -p <port>    Specifies the port to listen on for incoming connections.\n");
+    printf(" -playerdir   Set the player files directory.\n");
+    printf(" -regions     Set the region file.\n");
+    printf(" -templatedir Set the template map directory.\n");
+    printf(" -tmpdir      Set the directory for temporary files (mostly maps.)\n");
+    printf(" -treasures   Set the treasures file.\n");
+    printf(" -uniquedir   Set the unique items/maps directory.\n");
+    printf(" -v           Print version information.\n");
+    exit(EXIT_SUCCESS);
 }
 
+/**
+ * Called before the server starts listening to connections, processes various
+ * dump-related options.
+ */
 static void init_beforeplay(void) {
     init_archetypes(); /* If not called before, reads all archetypes from file */
     init_artifacts();  /* If not called before, reads all artifacts from file */
@@ -1118,81 +1180,11 @@ static void init_startup(void) {
 }
 
 /**
- * Dump compilation information, activated with the -o flag.
- *
- * It writes out information on how Imakefile and config.h was configured
- * at compile time.
+ * Signal handler that begins a normal server shutdown.
  */
-static void compile_info(void) {
-    int i = 0;
-    char err[MAX_BUF];
-
-    printf("Non-standard include files:\n");
-#ifdef HAVE_MALLOC_H
-    printf("<malloc.h>\n");
-    i = 1;
-#endif
-#ifdef HAVE_MEMORY_H
-    printf("<memory.h\n");
-    i = 1;
-#endif
-    if (!i)
-        printf("(none)\n");
-    printf("Datadir:\t\t%s\n", settings.datadir);
-    printf("Localdir:\t\t%s\n", settings.localdir);
-#ifdef PERM_FILE
-    printf("Perm file:\t<ETC>/%s\n", PERM_FILE);
-#endif
-#ifdef SHUTDOWN_FILE
-    printf("Shutdown file:\t<ETC>/%s\n", SHUTDOWN_FILE);
-#endif
-    printf("Save player:\t<true>\n");
-    printf("Save mode:\t%4.4o\n", SAVE_MODE);
-    printf("Playerdir:\t<VAR>/%s\n", settings.playerdir);
-    printf("Itemsdir:\t<VAR>/%s\n", settings.uniquedir);
-    printf("Tmpdir:\t\t%s\n", settings.tmpdir);
-    printf("Map max timeout:\t%d\n", MAP_MAXTIMEOUT);
-    printf("Max objects:\t%d\n", MAX_OBJECTS);
-#ifdef USE_CALLOC
-    printf("Use_calloc:\t<true>\n");
-#else
-    printf("Use_calloc:\t<false>\n");
-#endif
-
-    printf("Max_time:\t%d\n", MAX_TIME);
-
-#ifdef WIN32 /* ***win32 compile_info(): remove execl... */
-    printf("Logfilename:\t%s\n", settings.logfilename);
-    exit(0);
-#else
-    execl("/bin/uname", "uname", "-a", NULL);
-    LOG(llevError, "Oops, shouldn't have gotten here: execl(/bin/uname) failed: %s\n", strerror_local(errno, err, sizeof(err)));
-    exit(-1);
-#endif
-}
-
-/* Signal handlers: */
-
-#ifndef DEBUG
-/**
- * SIGSERV handler.
- * @param i
- * unused.
- */
-static void rec_sigsegv(int i) {
-    LOG(llevError, "\nSIGSEGV received.\n");
-    fatal_signal(1);
-}
-#endif
-
-/**
- * SIGINT handler.
- * @param i
- * unused.
- */
-static void rec_sigint(int i) {
-    LOG(llevInfo, "\nSIGINT received.\n");
-    fatal_signal(0);
+static void signal_shutdown(int signum_unused) {
+    (void) signum_unused; /* avoid unused warning if enambled */
+    shutdown_flag += 1;
 }
 
 /**
@@ -1215,85 +1207,6 @@ static void rec_sighup(int i) {
     }
 }
 
-#ifndef DEBUG
-/**
- * SIGQUIT handler.
- *
- * @param i
- * unused.
- */
-static void rec_sigquit(int i) {
-    LOG(llevInfo, "\nSIGQUIT received\n");
-    fatal_signal(1);
-}
-#endif
-
-#ifndef DEBUG
-/**
- * SIGPIPE handler.
- *
- * Keep running if we receive a sigpipe.  Crossfire should really be able
- * to handle this signal (at least at some point in the future if not
- * right now).  By causing a dump right when it is received, it is not
- * doing much good.  However, if it core dumps later on, at least it can
- * be looked at later on, and maybe fix the problem that caused it to
- * dump core.  There is no reason that SIGPIPES should be fatal
- *
- * @param i
- * unused.
- */
-static void rec_sigpipe(int i) {
-    LOG(llevError, "\nSIGPIPE--------------\n------------\n--------\n---\n");
-#if 1 && !defined(WIN32) /* ***win32: we don't want send SIGPIPE */
-    LOG(llevInfo, "\nReceived SIGPIPE, ignoring...\n");
-    signal(SIGPIPE, rec_sigpipe);/* hocky-pux clears signal handlers */
-#else
-    LOG(llevError, "\nSIGPIPE received, not ignoring...\n");
-    fatal_signal(1); /*Might consider to uncomment this line */
-#endif
-}
-
-#ifdef SIGBUS
-/**
- * SIGBUS handler.
- *
- * @param i
- * unused.
- */
-static void rec_sigbus(int i) {
-    LOG(llevError, "\nSIGBUS received\n");
-    fatal_signal(1);
-}
-#endif
-
-/**
- * SIGTERM handler.
- *
- * @param i
- * unused.
- */
-static void rec_sigterm(int i) {
-    LOG(llevInfo, "\nSIGTERM received\n");
-    fatal_signal(0);
-}
-#endif
-
-/**
- * General signal handling. Will exit() in any case.
- *
- * @param make_core
- * if set abort() instead of exit() to generate a core dump.
- */
-static void fatal_signal(int make_core) {
-    if (init_done) {
-        emergency_save(0);
-        clean_tmp_files();
-    }
-    if (make_core)
-        abort();
-    exit(0);
-}
-
 /**
  * Setup our signal handlers.
  */
@@ -1306,17 +1219,8 @@ static void init_signals(void) {
     sa.sa_flags = 0;
     sa.sa_handler = rec_sighup;
     sigaction(SIGHUP, &sa, NULL);
-    signal(SIGINT, rec_sigint);
-#ifndef DEBUG
-    signal(SIGQUIT, rec_sigquit);
-    signal(SIGSEGV, rec_sigsegv);
-    LOG(llevInfo, "\n---------registering SIGPIPE\n");
-    signal(SIGPIPE, rec_sigpipe);
-#ifdef SIGBUS
-    signal(SIGBUS, rec_sigbus);
-#endif
-    signal(SIGTERM, rec_sigterm);
-#endif
+    signal(SIGINT, signal_shutdown);
+    signal(SIGPIPE, SIG_IGN);
 #endif /* win32 */
 }
 
@@ -1338,9 +1242,8 @@ static void init_races(void) {
     first_race = NULL;
 
     snprintf(fname, sizeof(fname), "%s/races", settings.datadir);
-    LOG(llevDebug, "Reading races from %s...\n", fname);
     if (!(file = fopen(fname, "r"))) {
-        LOG(llevError, "Cannot open races file %s: %s\n", fname, strerror_local(errno, buf, sizeof(buf)));
+        LOG(llevError, "Cannot open races file %s: %s\n", fname, strerror(errno));
         return;
     }
 
@@ -1379,9 +1282,15 @@ static void init_races(void) {
             if ((mon = find_archetype(cp)) == NULL)
                 LOG(llevError, "Creature %s in race file lacks archetype\n", cp);
             else {
-                if (set_race && (!mon->clone.race || strcmp(mon->clone.race, race))) {
+                // If the race is specified somewhere in the race field, we are good.
+                // This allows us to specify multiple races on arches that are in the races file.
+                if (set_race && (!mon->clone.race || !strstr(mon->clone.race, race))) {
                     if (mon->clone.race) {
-                        LOG(llevDebug, " Resetting race to %s from %s for archetype %s\n", race, mon->clone.race, mon->name);
+                        LOG(llevDebug, "races: Appending to %s from %s for archetype %s\n", race, mon->clone.race, mon->name);
+                        // Copy the existing race into the race total list.
+                        strcpy(race+strlen(race), ","); // Arbitrary separator
+                        strncpy(race+strlen(race), mon->clone.race, MAX_BUF - 1 - strlen(race));
+                        // Clear the existing race string so we can add the new one
                         free_string(mon->clone.race);
                     }
                     mon->clone.race = add_string(race);
@@ -1393,7 +1302,7 @@ static void init_races(void) {
         }
     }
     fclose(file);
-    LOG(llevDebug, "done races.\n");
+    LOG(llevDebug, "loaded races\n");
 }
 
 /**
