@@ -1,24 +1,15 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2010 Crossfire Development Team
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire@metalforge.org
-*/
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
 /**
 @file
@@ -64,11 +55,15 @@ is sent as needed, incrementally as to not freeze the server.
 - more things to keep trace of
 */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
 #include <assert.h>
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "sproto.h"
+#include "output_file.h"
 
 struct knowledge_player;
 struct knowledge_type;
@@ -117,9 +112,9 @@ typedef StringBuffer* (*knowledge_can_use_alchemy)(sstring code, const char *ite
 /**
  * Get the face for a knowledge item.
  * @param code knowledge internal code.
- * @return face number, -1 (as unsigned) for invalid.
+ * @return face number, NULL for invalid.
  */
-typedef unsigned (*knowledge_face)(sstring code);
+typedef const Face *(*knowledge_face)(sstring code);
 
 struct knowledge_item;
 
@@ -351,7 +346,7 @@ static void knowledge_alchemy_attempt(player *pl, const knowledge_item *item) {
     object *cauldron = NULL, *inv;
     object *ingredients[50];
     int index, x, y;
-    uint32 count, counts[50];
+    uint32_t count, counts[50];
     char name[MAX_BUF];
     const char *ingname;
     linked_char *ing;
@@ -388,11 +383,11 @@ static void knowledge_alchemy_attempt(player *pl, const knowledge_item *item) {
     }
     for (inv = pl->ob->inv; inv != NULL; inv = inv->below) {
 
-        if (QUERY_FLAG(inv, FLAG_INV_LOCKED) || QUERY_FLAG(inv, FLAG_STARTEQUIP) /*|| (need_identify(inv) && !QUERY_FLAG(inv, FLAG_IDENTIFIED))*/)
+        if (QUERY_FLAG(inv, FLAG_INV_LOCKED) || QUERY_FLAG(inv, FLAG_STARTEQUIP))
             continue;
 
         if (inv->title == NULL)
-            strncpy(name, inv->name, sizeof(name));
+            safe_strncpy(name, inv->name, sizeof(name));
         else
             snprintf(name, sizeof(name), "%s %s", inv->name, inv->title);
 
@@ -462,7 +457,7 @@ static void knowledge_alchemy_attempt(player *pl, const knowledge_item *item) {
         if (object_was_destroyed(ingredients[index], tag)) {
             draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Hum, some item disappeared, stopping the attempt.");
             return;
-            
+
         }
         if (count == ingredients[index]->nrof && ingredients[index]->env == pl->ob) {
             draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Hum, couldn't drop some items, stopping the attempt.");
@@ -500,34 +495,44 @@ static void knowledge_alchemy_attempt(player *pl, const knowledge_item *item) {
 /**
  * Try to get a face for an alchemy recipe.
  * @param code alchemy code.
- * @return face, -1 as unsigned if none could be found.
+ * @return face, NULL if none could be found.
  */
-static unsigned knowledge_alchemy_face(sstring code) {
+static const Face *knowledge_alchemy_face(sstring code) {
     const recipe *rp = knowledge_alchemy_get_recipe(code);
     const artifact *art;
-    const archetype *arch;
+    archetype *arch;
+    object *item;
+    const Face *face;
 
     if (!rp) {
         LOG(llevError, "knowledge: couldn't find recipe for %s", code);
-        return (unsigned)-1;
+        return NULL;
     }
 
     if (rp->arch_names == 0)
-        return (unsigned)-1;
+        return NULL;
 
     arch = find_archetype(rp->arch_name[0]);
+    if (arch == NULL) {
+        return NULL;
+    }
     if (strcmp(rp->title, "NONE") == 0) {
-        return arch->clone.face->number;
+        return arch->clone.face;
     }
 
     art = locate_recipe_artifact(rp, 0);
     if (art == NULL)
-        return arch->clone.face->number;
+        return arch->clone.face;
 
-    if (art->item->face != blank_face && art->item->face != NULL)
-        return art->item->face->number;
+    face = arch->clone.face;
+    item = arch_to_object(arch);
+    give_artifact_abilities(item, art->item);
+    object_give_identified_properties(item);
+    if (item->face != NULL && item->face != blank_face)
+        face = item->face;
+    object_free(item, FREE_OBJ_FREE_INVENTORY | FREE_OBJ_NO_DESTROY_CALLBACK);
 
-    return arch->clone.face->number;
+    return face;
 }
 
 /**
@@ -564,7 +569,11 @@ static int knowledge_add(knowledge_player *current, const char *item, const know
     }
 
     /* keep the knowledge */
-    check = calloc(1, sizeof(knowledge_item));
+    /**
+     * Use malloc instead of calloc here.
+     * We're setting all relevant values, anyway.
+     */
+    check = malloc(sizeof(knowledge_item));
     check->item = add_string(item);
     check->handler = kt;
     if (current->item_count >= current->item_allocated) {
@@ -604,7 +613,7 @@ static void knowledge_monster_detail(const char *item, StringBuffer *buf) {
         return;
 
     stringbuffer_append_printf(buf, " *** %s ***\n", monster->clone.name);
-    describe_item(&monster->clone, NULL, buf);
+    describe_item(&monster->clone, NULL, 1, buf);
 }
 
 /**
@@ -647,15 +656,15 @@ static int knowledge_monster_add(struct knowledge_player *current, const char *i
 /**
  * Get the face for a monster.
  * @param code monster's code.
- * @return face, -1 as unsigned if invalid.
+ * @return face, NULL if invalid.
  */
-static unsigned knowledge_monster_face(sstring code) {
+static const Face *knowledge_monster_face(sstring code) {
     const archetype *monster = find_archetype(code);
 
     if (!monster || monster->clone.face == blank_face || monster->clone.face == NULL)
-        return (unsigned)-1;
+        return NULL;
 
-    return monster->clone.face->number;
+    return monster->clone.face;
 }
 
 /**
@@ -776,9 +785,9 @@ static int knowledge_god_add(struct knowledge_player *current, const char *item,
 /**
  * Get the face for a god.
  * @param code god's code.
- * @return face, -1 as unsigned if invalid.
+ * @return face, NULL if invalid.
  */
-static unsigned knowledge_god_face(sstring code) {
+static const Face *knowledge_god_face(sstring code) {
     char buf[MAX_BUF];
     size_t letter;
     const archetype *altar_arch;
@@ -795,9 +804,9 @@ static unsigned knowledge_god_face(sstring code) {
     }
     altar_arch = find_archetype(buf);
     if (altar_arch == NULL)
-        return (unsigned)-1;
+        return NULL;
 
-    return altar_arch->clone.face->number;
+    return altar_arch->clone.face;
 }
 
 /**
@@ -841,13 +850,27 @@ static int knowledge_message_validate(const char *item) {
     return get_message_from_identifier(item) != NULL;
 }
 
+/**
+ * Get the face for a message.
+ * @param code message's code.
+ * @return face, NULL if invalid.
+ */
+static const Face *knowledge_message_face(sstring code) {
+    const GeneralMessage *msg = get_message_from_identifier(code);
+
+    if (!msg)
+        /* warn? */
+        return NULL;
+
+    return get_message_face(msg);
+}
 
 /** All handled knowledge items. */
-static const knowledge_type const knowledges[] = {
+static const knowledge_type knowledges[] = {
     { "alchemy", knowledge_alchemy_summary, knowledge_alchemy_detail, knowledge_alchemy_validate, knowledge_add, "recipes", knowledge_alchemy_can_use_item, knowledge_alchemy_attempt, "knowledge_recipes.111", knowledge_alchemy_face },
     { "monster", knowledge_monster_summary, knowledge_monster_detail, knowledge_monster_validate, knowledge_monster_add, "monsters", NULL, NULL, "knowledge_monsters.111", knowledge_monster_face },
     { "god", knowledge_god_summary, knowledge_god_detail, knowledge_god_validate, knowledge_god_add, "gods", NULL, NULL, "knowledge_gods.111", knowledge_god_face },
-    { "message", knowledge_message_summary, knowledge_message_detail, knowledge_message_validate, knowledge_add, "messages", NULL, NULL, "knowledge_messages.111", NULL },
+    { "message", knowledge_message_summary, knowledge_message_detail, knowledge_message_validate, knowledge_add, "messages", NULL, NULL, "knowledge_messages.111", knowledge_message_face },
     { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL }
 };
 
@@ -874,16 +897,15 @@ static const knowledge_type *knowledge_find(const char *type) {
  */
 static void knowledge_write_player_data(const knowledge_player *kp) {
     FILE *file;
-    char write[MAX_BUF], final[MAX_BUF];
+    OutputFile of;
+    char fname[MAX_BUF];
     const knowledge_item *item;
     int i;
 
-    snprintf(final, sizeof(final), "%s/%s/%s/%s.knowledge", settings.localdir, settings.playerdir, kp->player_name, kp->player_name);
-    snprintf(write, sizeof(write), "%s.new", final);
+    snprintf(fname, sizeof(fname), "%s/%s/%s/%s.knowledge", settings.localdir, settings.playerdir, kp->player_name, kp->player_name);
 
-    file = fopen(write, "w+");
+    file = of_open(&of, fname);
     if (!file) {
-        LOG(llevError, "knowledge: couldn't open player knowledge file %s!", write);
         draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!");
         return;
     }
@@ -893,10 +915,11 @@ static void knowledge_write_player_data(const knowledge_player *kp) {
         fprintf(file, "%s:%s\n", item->handler->type, item->item);
     }
 
-    fclose(file);
+    if (!of_close(&of)) {
+        draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!");
+        return;
+    }
     /** @todo rename/backup, stuff like that */
-    unlink(final);
-    rename(write, final);
 }
 
 /**
@@ -939,7 +962,7 @@ static void knowledge_read_player_data(knowledge_player *kp) {
             continue;
         }
 
-        item = calloc(1, sizeof(knowledge_item));
+        item = malloc(sizeof(knowledge_item));
         item->item = add_string(dot + 1);
         item->handler = type;
         if (kp->item_count == kp->item_allocated) {
@@ -981,27 +1004,25 @@ static knowledge_player *knowledge_get_or_create(const player *pl) {
 }
 
 /**
- * Player is reading a book, give knowledge if needed, warn player, and such.
- * @param pl who is reading.
- * @param book what is read.
+ * Give a knowledge item from its code.
+ * @param pl who to give the knowldge to.
+ * @param marker knowledge's code.
+ * @param book optional item containing the knowledge code.
  */
-void knowledge_read(player *pl, object *book) {
-    sstring marker = object_get_value(book, "knowledge_marker");
-    char *dot, *copy;
+void knowledge_give(player *pl, const char *marker, const object *book) {
+    char *dot, *copy, *code;
     const knowledge_type *type;
     int none, added = 0;
     knowledge_player *current = knowledge_get_or_create(pl);
 
-    if (!marker)
-        return;
-
     /* process marker, find if already known */
     dot = strchr(marker, ':');
     if (dot == NULL)
         return;
 
-    copy = calloc(1, strlen(marker) + 1);
-    strncpy(copy, marker, strlen(marker) + 1);
+    copy = strdup(marker);
+    if (!copy)
+	fatal(OUT_OF_MEMORY);
 
     dot = strchr(copy, ':');
     *dot = '\0';
@@ -1009,15 +1030,26 @@ void knowledge_read(player *pl, object *book) {
 
     type = knowledge_find(copy);
     if (!type) {
-        LOG(llevError, "knowledge: invalid marker type %s in %s\n", copy, book->name);
+        LOG(llevError, "knowledge: invalid marker type %s in %s\n", copy, book == NULL ? "(null)" : book->name);
         free(copy);
         return;
     }
 
     none = (current->items == NULL);
-    if (type->validate(dot)) {
-        added = type->add(current, dot, type, pl);
+
+    code = dot;
+    while (code && code[0] != '\0') {
+        dot = strchr(code, '/');
+        if (dot) {
+            *dot = '\0';
+            dot++;
+        }
+        if (type->validate(code)) {
+            added += type->add(current, code, type, pl);
+        }
+        code = dot;
     }
+
     free(copy);
 
     if (added) {
@@ -1032,6 +1064,17 @@ void knowledge_read(player *pl, object *book) {
         knowledge_write_player_data(current);
 }
 
+/**
+ * Player is reading a book, give knowledge if needed, warn player, and such.
+ * @param pl who is reading.
+ * @param book what is read.
+ */
+void knowledge_read(player *pl, object *book) {
+    sstring marker = object_get_value(book, "knowledge_marker");
+    if (marker != NULL)
+        knowledge_give(pl, marker, book);
+}
+
 /**
  * Actually display knowledge list.
  * @param pl who to display to.
@@ -1082,7 +1125,7 @@ static void knowledge_do_display(object *pl, const knowledge_type *show_only, co
                 header = 1;
             }
 
-            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s", i, final);
+            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_MISC, MSG_TYPE_CLIENT_NOTICE, "(%3d) %s", i + 1, final);
         }
 
         free(final);
@@ -1278,7 +1321,7 @@ int knowledge_player_knows(const player *pl, const char *knowledge) {
         return 0;
     }
 
-    strncpy(copy, knowledge, MAX_BUF);
+    strlcpy(copy, knowledge, sizeof(copy));
     pos = strchr(copy, ':');
     if (pos == NULL) {
         LOG(llevError, "knowledge_player_knows: invalid knowledge item %s\n", knowledge);
@@ -1346,23 +1389,23 @@ void knowledge_item_can_be_used_alchemy(object *op, const object *item) {
  */
 void knowledge_send_info(socket_struct *ns) {
     int i;
-    unsigned face;
+    const Face *face;
     SockList sl;
 
-    face = find_face("knowledge_generic.111", (unsigned)-1);
-    if (face != (unsigned)-1 && (!ns->faces_sent[face] & NS_FACESENT_FACE))
+    face = find_face("knowledge_generic.111", NULL);
+    if (face != NULL && (!(ns->faces_sent[face->number] & NS_FACESENT_FACE)))
         esrv_send_face(ns, face, 0);
 
     SockList_Init(&sl);
     SockList_AddString(&sl, "replyinfo knowledge_info\n");
-    SockList_AddPrintf(&sl, "::%u:0\n", face);
+    SockList_AddPrintf(&sl, "::%u:0\n", face ? face->number : 0);
 
     for (i = 0; knowledges[i].type != NULL; i++) {
-        face = find_face(knowledges[i].face, (unsigned)-1);
-        if (face != (unsigned)-1 && (!ns->faces_sent[face] & NS_FACESENT_FACE))
+        face = find_face(knowledges[i].face, NULL);
+        if (face != NULL && (!(ns->faces_sent[face->number] & NS_FACESENT_FACE)))
             esrv_send_face(ns, face, 0);
 
-        SockList_AddPrintf(&sl, "%s:%s:%u:%s\n", knowledges[i].type, knowledges[i].name, face, knowledges[i].attempt_alchemy != NULL ? "1" : "0");
+        SockList_AddPrintf(&sl, "%s:%s:%u:%s\n", knowledges[i].type, knowledges[i].name, face ? face->number : 0, knowledges[i].attempt_alchemy != NULL ? "1" : "0");
     }
 
     Send_With_Handling(ns, &sl);
@@ -1416,7 +1459,7 @@ void knowledge_process_incremental(void) {
     const knowledge_item *item;
     StringBuffer *buf;
     char *title;
-    unsigned face;
+    const Face *face;
     knowledge_player *cur = knowledge_global, *prev = NULL;
     player *pl;
 
@@ -1458,12 +1501,12 @@ void knowledge_process_incremental(void) {
             item->handler->summary(item->item, buf);
             title = stringbuffer_finish(buf);
 
-            face = (unsigned)-1;
+            face = NULL;
             if (item->handler->item_face != NULL)
                 face = item->handler->item_face(item->item);
 
-            if (face == (unsigned)-1)
-                face = find_face(item->handler->face, (unsigned)-1);
+            if (face == NULL)
+                face = find_face(item->handler->face, NULL);
 
             size = 4 + (2 + strlen(item->handler->type)) + (2 + strlen(title)) + 4;
 
@@ -1475,10 +1518,10 @@ void knowledge_process_incremental(void) {
 
             SockList_AddInt(&sl, i + 1);
             SockList_AddLen16Data(&sl, item->handler->type, strlen(item->handler->type));
-            if ((face != (unsigned)-1) && !(pl->socket.faces_sent[face]&NS_FACESENT_FACE))
+            if (face != NULL && !(pl->socket.faces_sent[face->number]&NS_FACESENT_FACE))
                 esrv_send_face(&pl->socket, face, 0);
             SockList_AddLen16Data(&sl, title, strlen(title));
-            SockList_AddInt(&sl, face);
+            SockList_AddInt(&sl, face ? face->number : 0);
 
             free(title);
         }
@@ -1492,3 +1535,28 @@ void knowledge_process_incremental(void) {
         break;
     }
 }
+
+/**
+ * Display the details of a monster if the player knows them.
+ * @param op player asking for details.
+ * @param name monster's archetype name.
+ */
+void knowledge_show_monster_detail(object *op, const char *name) {
+    knowledge_player *kp;
+    StringBuffer *buf = NULL;
+    char *result;
+
+    if (op->contr == NULL)
+        return;
+
+    kp = knowledge_get_or_create(op->contr);
+
+    if (!knowledge_known(kp, name, &knowledges[1]))
+        return;
+
+    buf = stringbuffer_new();
+    knowledge_monster_detail(name, buf);
+    result = stringbuffer_finish(buf);
+    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_EXAMINE, result);
+    free(result);
+}
