@@ -1,43 +1,33 @@
 /*
- * static char *rcsid_login_c =
- *   "$Id: login.c 16766 2012-01-21 11:30:34Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006-2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Player login/logout/save functions.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
-#include <loader.h>
-#include <define.h>
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
+
+#include "define.h"
+#include "loader.h"
+#include "output_file.h"
+#include "spells.h"
+#include "sproto.h"
+#include "server.h"
 
 static void copy_file(const char *filename, FILE *fpout);
 
@@ -221,13 +211,14 @@ void destroy_object(object *op) {
  */
 int save_player(object *op, int flag) {
     FILE *fp;
-    char filename[MAX_BUF], *tmpfilename, backupfile[MAX_BUF];
+    OutputFile of;
+    char filename[MAX_BUF], *tmpfilename;
     object *container = NULL;
     player *pl = op->contr;
     int i, wiz = QUERY_FLAG(op, FLAG_WIZ);
     long checksum;
 #ifdef BACKUP_SAVE_AT_HOME
-    sint16 backup_x, backup_y;
+    int16_t backup_x, backup_y;
 #endif
 
     if (!op->stats.exp)
@@ -268,7 +259,7 @@ int save_player(object *op, int flag) {
          * terminated.
          */
         if (!account_get_account_for_char(pl->ob->name))
-            account_add_player_to_account(pl->socket.account_name, pl->ob->name);
+            account_link(pl->socket.account_name, pl->ob->name);
     }
 
 
@@ -338,7 +329,7 @@ int save_player(object *op, int flag) {
     fprintf(fp, "Cha %d\n", pl->orig_stats.Cha);
 
     fprintf(fp, "lev_array %d\n", MIN(op->level, 10));
-    for (i = 1; i <= pl->last_level && i <= 10; i++) {
+    for (i = 1; i <= MIN(op->level, 10) && i <= 10; i++) {
         fprintf(fp, "%d\n", pl->levhp[i]);
         fprintf(fp, "%d\n", pl->levsp[i]);
         fprintf(fp, "%d\n", pl->levgrace[i]);
@@ -388,6 +379,12 @@ int save_player(object *op, int flag) {
     CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
 
     if (!flag) {
+        // Clear last_skill_ob before removing inventory. This prevents us
+        // from accessing removed skill objects during cleanup.
+        for (int i = 0; i < MAX_SKILLS; i++) {
+            op->contr->last_skill_ob[i] = NULL;
+        }
+
         while (op->inv != NULL)
             destroy_object(op->inv);
 
@@ -398,10 +395,8 @@ int save_player(object *op, int flag) {
     }
 
     checksum = 0;
-    snprintf(backupfile, sizeof(backupfile), "%s.tmp", filename);
-    rename(filename, backupfile);
-    fp = fopen(filename, "w");
-    if (!fp) {
+    fp = of_open(&of, filename);
+    if (fp == NULL) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE,
                       "Can't open file for save.");
         unlink(tmpfilename);
@@ -412,13 +407,11 @@ int save_player(object *op, int flag) {
     copy_file(tmpfilename, fp);
     unlink(tmpfilename);
     free(tmpfilename);
-    if (fclose(fp) == EOF) { /* got write error */
+    if (!of_close(&of)) {
         draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE,
                       "Can't close file for save.");
-        rename(backupfile, filename); /* Restore the original */
         return 0;
-    } else
-        unlink(backupfile);
+    }
 
     /* Eneq(@csd.uu.se): Reveal the container if we have one. */
     if (flag && container != NULL)
@@ -427,7 +420,9 @@ int save_player(object *op, int flag) {
     if (!flag)
         esrv_send_inventory(op, op);
 
-    chmod(filename, SAVE_MODE);
+    if (chmod(filename, SAVE_MODE) != 0) {
+        LOG(llevError, "Could not set permissions on '%s'\n", filename);
+    }
 
     /* if this is the first player save, quest or knowledge states can be unsaved */
     if (!op->contr->has_directory) {
@@ -508,7 +503,7 @@ void check_login(object *op, int check_pass) {
     char filename[MAX_BUF];
     char buf[MAX_BUF], bufall[MAX_BUF];
     int i, value;
-    uint32 uvalue;
+    uint32_t uvalue;
     player *pl = op->contr, *pltmp;
     int correct = 0;
     time_t elapsed_save_time = 0;
@@ -568,7 +563,9 @@ void check_login(object *op, int check_pass) {
 
     if (fgets(bufall, MAX_BUF, fp) != NULL) {
         if (!strncmp(bufall, "checksum ", 9)) {
-            (void)fgets(bufall, MAX_BUF, fp);
+            if ( fgets(bufall, MAX_BUF, fp) == NULL ) {
+                bufall[0]=0; /* should never happen */
+            }
         }
         if (sscanf(bufall, "password %s\n", buf)) {
             /* New password scheme: */
@@ -663,9 +660,9 @@ void check_login(object *op, int check_pass) {
             pl->mode = uvalue;
         }
         else if (!strcmp(buf, "map"))
-            snprintf(pl->maplevel, sizeof(pl->maplevel), "%s", val_string);
+            strlcpy(pl->maplevel, val_string, sizeof(pl->maplevel));
         else if (!strcmp(buf, "savebed_map"))
-            snprintf(pl->savebed_map, sizeof(pl->savebed_map), "%s", val_string);
+            strlcpy(pl->savebed_map, val_string, sizeof(pl->savebed_map));
         else if (!strcmp(buf, "bed_x"))
             pl->bed_x = value;
         else if (!strcmp(buf, "bed_y"))
@@ -705,16 +702,32 @@ void check_login(object *op, int check_pass) {
         } else if (!strcmp(buf, "lev_array")) {
             for (i = 1; i <= value; i++) {
                 int j;
+                int count=0;
 
-                fscanf(fp, "%d\n", &j);
+                count = fscanf(fp, "%d\n", &j);
+                if ( !count ) j=0; // sanity; should never happen
+                if (j < 3)
+                    j = 3;
+                else if (j > 9)
+                    j = 9;
                 pl->levhp[i] = j;
-                fscanf(fp, "%d\n", &j);
+                count = fscanf(fp, "%d\n", &j);
+                if ( !count ) j=0; // sanity; should never happen
+                if (j < 2)
+                    j = 2;
+                else if (j > 6)
+                    j = 6;
                 pl->levsp[i] = j;
-                fscanf(fp, "%d\n", &j);
+                count = fscanf(fp, "%d\n", &j);
+                if ( !count ) j=0; // sanity; should never happen
+                if (j < 1)
+                    j = 1;
+                else if (j > 3)
+                    j = 3;
                 pl->levgrace[i] = j;
             }
         } else if (!strcmp(buf, "party_rejoin_mode"))
-            pl->rejoin_party = value;
+            pl->rejoin_party = (enum party_rejoin_mode)value;
         else if (!strcmp(buf, "party_rejoin_name"))
             party_name = strdup_local(val_string);
         else if (!strcmp(buf, "party_rejoin_password")) {
@@ -745,6 +758,7 @@ void check_login(object *op, int check_pass) {
 
     CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
 
+    LOG(llevInfo, "login: %s from %s\n", op->name, op->contr->socket.host);
     strncpy(pl->title, op->arch->clone.name, sizeof(pl->title)-1);
     pl->title[sizeof(pl->title)-1] = '\0';
 
@@ -776,8 +790,7 @@ void check_login(object *op, int check_pass) {
 
     /* make sure he's a player--needed because of class change. */
     op->type = PLAYER;
-
-    enter_exit(op, NULL);
+    enter_player_maplevel(op);
 
     pl->name_changed = 1;
     player_set_state(pl, ST_PLAYING);
@@ -806,6 +819,7 @@ void check_login(object *op, int check_pass) {
                          MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
                          "%s has entered the game.",
                          pl->ob->name);
+    login_check_shutdown(op);
 
     /* Lauwenmark : Here we handle the LOGIN global event */
     execute_global_event(EVENT_LOGIN, pl, pl->socket.host);
@@ -821,10 +835,13 @@ void check_login(object *op, int check_pass) {
                       "Your character was dead last time you played.");
         kill_player(op, NULL);
         if (pl->state != ST_PLAYING)
+        {
+            // Prevent memory leak from strdup-ed party_name.
+            if (party_name)
+                free(party_name);
             return;
+        }
     }
-    LOG(llevInfo, "LOGIN: Player named %s from ip %s\n", op->name,
-        op->contr->socket.host);
 
     /* Do this after checking for death - no reason sucking up bandwidth if
      * the data isn't needed.
