@@ -1,40 +1,26 @@
 /*
- * static char *rcsid_move_c =
- *    "$Id: move.c 13308 2010-05-28 06:12:12Z kbulgrien $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Those functions handle object moving and pushing.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <stdlib.h>
+
+#include "sproto.h"
 
 static int roll_ob(object *op, int dir, object *pusher);
 
@@ -80,7 +66,7 @@ int move_ob(object *op, int dir, object *originator) {
 
         op_tag = op->count;
         for (part = op; part != NULL; part = part->more) {
-            sint16 x, y;
+            int16_t x, y;
 
             if (get_map_flags(part->map, &m, part->x+freearr_x[dir], part->y+freearr_y[dir], &x, &y)&P_OUT_OF_MAP)
                 continue;
@@ -100,15 +86,21 @@ int move_ob(object *op, int dir, object *originator) {
     }
 
     for (part = op; part != NULL; part = part->more) {
-        sint16 x, y;
+        int16_t x, y;
 
         if (get_map_flags(part->map, &m, part->x+freearr_x[dir], part->y+freearr_y[dir], &x, &y)&P_OUT_OF_MAP)
             return 0;
 
-        if (!QUERY_FLAG(op, FLAG_WIZPASS) && blocked_link(op, m, x, y))
+        if (!QUERY_FLAG(op, FLAG_WIZPASS) && blocked_link(op->type == TRANSPORT ? part : op, m, x, y))
             return 0;
     }
 
+    if (QUERY_FLAG(op, FLAG_ALIVE) && op->move_type == MOVE_WALK && op->weight >= 100) {
+        if (!op->hide) {
+            play_sound_map(SOUND_TYPE_GROUND, op, 0, "step");
+        }
+    }
+
     object_remove(op);
     object_insert_in_map_at(op, op->map, originator, 0, op->x+freearr_x[dir], op->y+freearr_y[dir]);
 
@@ -170,8 +162,10 @@ int transfer_ob(object *op, int x, int y, int randomly, object *originator) {
     op = HEAD(op);
     object_remove(op);
     tmp = object_insert_in_map_at(op, op->map, originator, 0, x+freearr_x[i], y+freearr_y[i]);
-    if (op && op->type == PLAYER)
+    if (op && op->type == PLAYER) {
         map_newmap_cmd(&op->contr->socket);
+        player_update_bg_music(op);
+    }
     if (tmp)
         return 0;
     else
@@ -197,12 +191,12 @@ int transfer_ob(object *op, int x, int y, int randomly, object *originator) {
  * user is still valid, but may have moved or not.
  * @todo fix weird return values.
  */
-int teleport(object *teleporter, uint8 tele_type, object *user) {
+int teleport(object *teleporter, uint8_t tele_type, object *user) {
     object *altern[120]; /* Better use c/malloc here in the future */
     int i, j, k, nrofalt = 0;
     object *other_teleporter, *tmp;
     mapstruct *m;
-    sint16 sx, sy;
+    int16_t sx, sy;
 
     if (user == NULL)
         return 0;
@@ -268,8 +262,10 @@ int teleport(object *teleporter, uint8 tele_type, object *user) {
     object_remove(user);
 
     tmp = object_insert_in_map_at(user, other_teleporter->map, NULL, 0, other_teleporter->x+freearr_x[k], other_teleporter->y+freearr_y[k]);
-    if (tmp && tmp->type == PLAYER)
+    if (tmp && tmp->type == PLAYER) {
         map_newmap_cmd(&tmp->contr->socket);
+        player_update_bg_music(tmp);
+    }
     return (tmp == NULL);
 }
 
@@ -323,7 +319,7 @@ void recursive_roll(object *op, int dir, object *pusher) {
 
 static int try_fit(object *op, mapstruct *m, int x, int y) {
     object *more;
-    sint16 tx, ty;
+    int16_t tx, ty;
     int mflags;
     mapstruct *m2;
 
@@ -371,7 +367,7 @@ static int try_fit(object *op, mapstruct *m, int x, int y) {
  */
 
 static int roll_ob(object *op, int dir, object *pusher) {
-    sint16 x, y;
+    int16_t x, y;
     int flags;
     mapstruct *m;
     MoveType move_block;
@@ -482,7 +478,7 @@ int push_ob(object *who, int dir, object *pusher) {
         if (pusher->contr->run_on) { /* only when we run */
             draw_ext_info_format(NDI_UNIQUE, 0, pusher,
                                  MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
-                                 "You start to attack %s !!",
+                                 "You start to attack %s!",
                                  who->name);
             CLEAR_FLAG(who, FLAG_UNAGGRESSIVE); /* the sucker don't like you anymore */
             object_set_enemy(who, pusher);
@@ -558,18 +554,22 @@ int move_to(object *op, int x, int y) {
     if (op->x == x && op->y == y)
         return 0;
 
-    if (GET_MAP_FLAGS(op->map, x, y)&P_OUT_OF_MAP)
+    if (get_map_flags(op->map, NULL, x, y, NULL, NULL)&P_OUT_OF_MAP)
         return 2;
 
     direction = monster_compute_path(op, GET_MAP_OB(op->map, x, y), -1);
     if (direction == -1)
         return 2;
 
+    op->direction = direction;
     op->facing = direction;
-    if (op->animation_id)
+    if (op->animation)
         animate_object(op, op->direction);
 
-    /* this shouldn't fail, as the direction computing takes into account the blocked state... */
-    move_ob(op, direction, op);
+    /* can fail, as the direction computing takes into account the blocked state,
+     * except for the final spot... */
+    if (move_ob(op, direction, op) == 0)
+        return 2;
+
     return 1;
 }
