@@ -1,29 +1,15 @@
 /*
- * static char *rcsid_ob_types =
- *   "$Id: build_map.c 5057 2006-10-29 07:50:09Z mwedel $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail to crossfire-devel@real-time.com
-*/
 
 /**
  * @file
@@ -70,6 +56,7 @@ void register_all_ob_types(void) {
     init_type_lamp();
     init_type_lighter();
     init_type_marker();
+    init_type_mimic();
     init_type_mood_floor();
     init_type_peacemaker();
     init_type_pedestal();
