@@ -1,40 +1,29 @@
 /*
- * static char *rcsid_pets_c =
- *    "$Id: pets.c 14545 2011-06-12 14:44:40Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Those functions deal with pets.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <assert.h>
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "sproto.h"
 
 /**
  * Mark all inventory items as FLAG_NO_DROP.
@@ -61,14 +50,11 @@ static void mark_inventory_as_no_drop(object *ob) {
  * will contain the path to the enemy.
  * @return
  * enemy, or NULL if nothing suitable.
- *
- * @note
- * pet_get_enemy() has been renamed to pets_get_enemy()
  */
 object *pets_get_enemy(object *pet, rv_vector *rv) {
     object *owner, *tmp, *attacker, *tmp3;
     int i;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *nm;
     int search_arr[SIZEOFFREE];
     int mflags;
@@ -144,7 +130,7 @@ object *pets_get_enemy(object *pet, rv_vector *rv) {
                 && tmp2 != owner
                 && monster_can_detect_enemy(pet, tmp2, rv)) {
                     if (!monster_can_see_enemy(pet, tmp2)) {
-                        if (tmp3 != NULL)
+                        if (tmp3 == NULL)
                             tmp3 = tmp2;
                     } else {
                         object_set_enemy(pet, tmp2);
@@ -204,8 +190,7 @@ object *pets_get_enemy(object *pet, rv_vector *rv) {
                     && tmp2 != owner
                     && monster_can_detect_enemy(pet, tmp2, rv)) {
                         if (!monster_can_see_enemy(pet, tmp2)) {
-                            if (tmp3 != NULL)
-                                tmp3 = tmp2;
+                            assert(tmp3 == NULL);
                         } else {
                             object_set_enemy(pet, tmp2);
                             if (monster_check_enemy(pet, rv) != NULL)
@@ -236,9 +221,6 @@ object *pets_get_enemy(object *pet, rv_vector *rv) {
  *
  * @param owner
  * player we wish to remove all pets of.
- *
- * @note
- * terminate_all_pets() has been renamed to pets_terminate_all()
  */
 void pets_terminate_all(object *owner) {
     objectlink *obl, *next;
@@ -262,9 +244,6 @@ void pets_terminate_all(object *owner) {
  * process of entering a new map when this is called.
  * Thus the map isn't loaded yet, and we have to remove
  * the pet...
- *
- * @note
- * remove_all_pets() has been renamed to pets_remove_all()
  */
 void pets_remove_all(void) {
     objectlink *obl, *next;
@@ -298,9 +277,6 @@ void pets_remove_all(void) {
  * pet trying to follow. Will be object_remove()'d if can't follow.
  * @param owner
  * owner of ob.
- *
- * @note
- * follow_owner() has been renamed to pets_follow_owner()
  */
 void pets_follow_owner(object *ob, object *owner) {
     int dir;
@@ -334,14 +310,11 @@ void pets_follow_owner(object *ob, object *owner) {
  *
  * @param ob
  * pet to move.
- *
- * @note
- * pet_move() has been renamed to pets_move()
  */
 void pets_move(object *ob) {
     int dir, i;
     tag_t tag;
-    sint16 dx, dy;
+    int16_t dx, dy;
     object *owner;
     mapstruct *m;
 
@@ -539,9 +512,6 @@ static object *fix_summon_pet(archetype *at, object *op, int dir) {
  * Updated this to allow more than the golem 'head' to attack.
  * @param op
  * golem to be moved.
- *
- * @note
- * move_golem() has been renamed to pets_move_golem()
  */
 void pets_move_golem(object *op) {
     int made_attack = 0;
@@ -588,7 +558,7 @@ void pets_move_golem(object *op) {
         return;
 
     for (tmp = op; tmp; tmp = tmp->more) {
-        sint16 x = tmp->x+DIRX(op), y = tmp->y+DIRY(op);
+        int16_t x = tmp->x+DIRX(op), y = tmp->y+DIRY(op);
         object *victim;
         mapstruct *m;
         int mflags;
@@ -653,9 +623,6 @@ void pets_move_golem(object *op) {
  * @param dir
  * desired direction.
  * @todo trash.
- *
- * @note
- * control_golem() has been renamed to pets_control_golem()
  */
 void pets_control_golem(object *op, int dir) {
     op->direction = dir;
@@ -677,9 +644,6 @@ void pets_control_golem(object *op, int dir) {
  * failed to summon something.
  * @retval 1
  * summoned correctly something.
- *
- * @note
- * summon_golem() has been renamed to pets_summon_golem()
  */
 int pets_summon_golem(object *op, object *caster, int dir, object *spob) {
     object *tmp;
@@ -762,6 +726,7 @@ int pets_summon_golem(object *op, object *caster, int dir, object *spob) {
                 object_set_owner(tmp, owner);
                 tmp->attack_movement = PETMOVE;
                 add_friendly_object(tmp);
+                set_spell_skill(op, caster, spob, tmp);
                 SET_FLAG(tmp, FLAG_FRIENDLY);
             }
         }
@@ -858,7 +823,7 @@ static object *choose_cult_monster(object *pl, const object *god, int summon_lev
 
     /* Determine the number of races available */
     racenr = 0;
-    strcpy(buf, god->race);
+    safe_strncpy(buf, god->race, sizeof(buf));
     race = strtok(buf, ",");
     while (race) {
         racenr++;
@@ -868,7 +833,7 @@ static object *choose_cult_monster(object *pl, const object *god, int summon_lev
     /* next, randomly select a race from the aligned_races string */
     if (racenr > 1) {
         racenr = rndm(0, racenr-1);
-        strcpy(buf, god->race);
+        safe_strncpy(buf, god->race, sizeof(buf));
         race = strtok(buf, ",");
         for (i = 0; i < racenr; i++)
             race = strtok(NULL, ",");
@@ -939,12 +904,9 @@ static object *choose_cult_monster(object *pl, const object *god, int summon_lev
  * nothing was summoned.
  * @retval 1
  * something was summoned.
- *
- * @note
- * summon_object() has been renamed to pets_summon_object()
  */
 int pets_summon_object(object *op, object *caster, object *spell_ob, int dir, const char *stringarg) {
-    sint16 x, y, nrof = 1, i;
+    int16_t x, y, nrof = 1, i;
     archetype *summon_arch;
     int ndir, mult;
 
@@ -1156,9 +1118,6 @@ static object *get_real_owner(object *ob) {
  * pet shouldn't attack target.
  * @retval 1
  * target is a suitable victim for the pet.
- *
- * @note
- * should_arena_attack() has been renamed to pets_should_arena_attack()
  */
 int pets_should_arena_attack(object *pet, object *owner, object *target) {
     object *rowner, *towner;
