@@ -1,46 +1,42 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2001-2010 Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire@metalforge.org
-*/
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
 /**
-@file
-Quest-related low-level mechanisms.
+ * @file
+ * Quest-related low-level mechanisms.
+ *
+ * You should only need to call the public functions, all that are not static.
+ *
+ * Data is loaded on a need-only basis - when a player quest state is queried
+ * or modified, data is read. Also, free_quest() can be called to release
+ * memory without preventing quest operations after.
+ *
+ * Write is done for each player whenever the state changes, to ensure data
+ * integrity.
+ */
 
-You should only need to call the public functions, all that are not static.
+#include "global.h"
 
-Data is loaded on a need-only basis - when a player quest state is queried or modified, data is read.
-Also, free_quest() can be called to release memory without preventing quest operations after.
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
 
-Write is done for each player whenever the state changes, to ensure data integrity.
+#include "output_file.h"
+#include "sproto.h"
 
-@todo
-- add protocol commands to send notifications instead of using draw_ext_info()
-
-*/
-
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+/* Media tags for information messages prefix. */
+#define TAG_START   "[color=#aa55ff]"
+#define TAG_END     "[/color]"
 
 /** Quest status that indicates a quest was completed and may be restarted. */
 #define QC_CAN_RESTART -1
@@ -65,6 +61,7 @@ typedef struct quest_player {
 /** Player quest state. */
 static quest_player *player_states = NULL;
 
+/** One condition to automatically move to a quest step. */
 typedef struct quest_condition {
     sstring quest_code;          /**< The quest that triggers the condition */
     int minstep;                  /**< The earliest step in the quest that triggers the condition,
@@ -89,14 +86,14 @@ typedef struct quest_definition {
     sstring quest_title;            /**< Quest title for player. */
     sstring quest_description;      /**< Quest longer description. */
     int quest_restart;              /**< If non zero, can be restarted. */
-    int face;                       /**< Face associated with this quest. */
-    uint32 client_code;             /**< The code used to communicate with the client, merely a unique index. */
+    const Face *face;           /**< Face associated with this quest. */
+    uint32_t client_code;             /**< The code used to communicate with the client, merely a unique index. */
     quest_step_definition *steps;   /**< Quest steps. */
     struct quest_definition *parent;/**< Parent for this quest, NULL if it is a 'top-level' quest */
     struct quest_definition *next;  /**< Next quest in the definition list. */
 } quest_definition;
 
-static int quests_loaded = 0;           /**< Did we already read the 'default.quests' file? */
+static int quests_loaded = 0;           /**< Number of quests loaded. If zero, quests not yet loaded. */
 static quest_definition *quests = NULL; /**< All known quests. */
 
 /**
@@ -152,12 +149,19 @@ static quest_definition *quest_get_by_code(sstring code) {
     return NULL;
 }
 
-#define QUESTFILE_NEXTQUEST 0
-#define QUESTFILE_QUEST 1
-#define QUESTFILE_QUESTDESC 2
-#define QUESTFILE_STEP 3
-#define QUESTFILE_STEPDESC 4
-#define QUESTFILE_STEPCOND 5
+/**
+ * @defgroup QUESTFILE_xxx Quest file parsing state.
+ *
+ * This is the parsing state when loading a file through load_quests_from_file().
+ */
+/*@{*/
+#define QUESTFILE_NEXTQUEST 0   /**< Waiting for next quest definition. */
+#define QUESTFILE_QUEST 1       /**< In a quest definition. */
+#define QUESTFILE_QUESTDESC 2   /**< In a quest description. */
+#define QUESTFILE_STEP 3        /**< In a quest step. */
+#define QUESTFILE_STEPDESC 4    /**< In a quest step description. */
+#define QUESTFILE_STEPCOND 5    /**< In a quest step conditions. */
+/*@}*/
 
 /**
  * Loads all of the quests which are found in the given file, any global states
@@ -176,13 +180,13 @@ static int load_quests_from_file(const char *filename) {
     quest_step_definition *step = NULL;
     char final[MAX_BUF], read[MAX_BUF];
     FILE *file;
-    StringBuffer *buf;
+    StringBuffer *buf=NULL;
 
     int loaded_quests =0, found =0;
     snprintf(final, sizeof(final), "%s/%s/%s", settings.datadir, settings.mapdir, filename);
     file = fopen(final, "r");
     if (!file) {
-        LOG(llevError, "Can't open %s for reading quests", filename);
+        LOG(llevError, "Can't open %s for reading quests\n", filename);
         return -1;
     }
 
@@ -221,7 +225,7 @@ static int load_quests_from_file(const char *filename) {
             } else
                 condition_parsed =1;
             if (!condition_parsed) {
-                LOG(llevError, "Invalid line '%s' in setwhen block for quest %s", read, quest->quest_code);
+                LOG(llevError, "Invalid line '%s' in setwhen block for quest %s=n", read, quest->quest_code);
                 continue;
             }
 
@@ -341,10 +345,10 @@ static int load_quests_from_file(const char *filename) {
             }
 
             if (strncmp(read, "face ", 5) == 0) {
-                int face;
+                const Face *face;
                 read[strlen(read) - 1] = '\0';
-                face = find_face(read + 5, 0);
-                if (face == 0) {
+                face = find_face(read + 5, NULL);
+                if (face == NULL) {
                     LOG(llevError, "Quest %s has invalid face %s.\n", quest->quest_code, read + 5);
                 } else {
                     quest->face = face;
@@ -396,23 +400,28 @@ static int load_quests_from_file(const char *filename) {
 
     fclose(file);
 
-    if (in != 0)
+    if (in != 0) {
         LOG(llevError, "quest: quest definition file %s read in, ends with state %d\n", final, in);
 
+        /* The buffer may not have been freed. */
+        if (buf != NULL) {
+            stringbuffer_finish(buf);
+        }
+    }
+
     return loaded_quests;
 }
 
 /** Load all quest definitions. Can be called multiple times, will be ignored. */
 static void quest_load_definitions(void) {
-    int found = 0;
     if (quests_loaded)
         return;
-    quests_loaded = 1;
-    found = load_quests_from_file("world.quests");
+    int found = load_quests_from_file("world.quests");
     if (found >= 0) {
         LOG(llevInfo, "%d quests found.\n", found);
+        quests_loaded = found;
     } else {
-        LOG(llevError, "Quest Loading Failed");
+        LOG(llevError, "Quest Loading Failed\n");
     }
 }
 
@@ -472,7 +481,6 @@ static quest_state *get_new_quest_state(void) {
 static void quest_read_player_data(quest_player *pq) {
     FILE *file;
     char final[MAX_BUF], read[MAX_BUF], data[MAX_BUF];
-    StringBuffer *buf = NULL;
     quest_state *qs = NULL, *prev = NULL;
     int warned = 0, state;
     quest_definition *quest = NULL;
@@ -492,16 +500,7 @@ static void quest_read_player_data(quest_player *pq) {
         if (sscanf(read, "quest %s\n", data)) {
             qs = get_new_quest_state();
             qs->code = add_string(data);
-            if (prev == NULL) {
-                pq->quests = qs;
-            } else {
-                prev->next = qs;
-            }
-            prev = qs;
             quest = quest_get_by_code(qs->code);
-            if (quest == NULL) {
-                LOG(llevDebug, "Unknown quest %s in quest file %s", qs->code, final);
-            }
             continue;
         }
 
@@ -517,7 +516,7 @@ static void quest_read_player_data(quest_player *pq) {
             if (quest != NULL && state != -1) {
                 quest_step_definition *step = quest_get_step(quest, state);
                 if (step == NULL) {
-                    LOG(llevError, "invalid quest step %d for %s in %s", state, quest->quest_code, final);
+                    LOG(llevError, "invalid quest step %d for %s in %s\n", state, quest->quest_code, final);
                 }
                 else if (step->is_completion_step)
                     qs->is_complete = 1;
@@ -525,6 +524,17 @@ static void quest_read_player_data(quest_player *pq) {
             continue;
         }
         if (strcmp(read, "end_quest\n") == 0) {
+            if (quest == NULL) {
+                LOG(llevDebug, "Unknown quest %s in quest file %s\n", qs->code, final);
+                free(qs);
+            } else {
+                if (prev == NULL) {
+                    pq->quests = qs;
+                } else {
+                    prev->next = qs;
+                }
+                prev = qs;
+            }
             qs = NULL;
             continue;
         }
@@ -538,8 +548,6 @@ static void quest_read_player_data(quest_player *pq) {
 
     if (qs)
         LOG(llevError, "quest: missing end_quest in %s\n", final);
-    if (buf)
-        free(stringbuffer_finish(buf));
 
     fclose(file);
 }
@@ -550,15 +558,14 @@ static void quest_read_player_data(quest_player *pq) {
  */
 static void quest_write_player_data(const quest_player *pq) {
     FILE *file;
-    char write[MAX_BUF], final[MAX_BUF];
+    OutputFile of;
+    char fname[MAX_BUF];
     const quest_state *state;
 
-    snprintf(final, sizeof(final), "%s/%s/%s/%s.quest", settings.localdir, settings.playerdir, pq->player_name, pq->player_name);
-    snprintf(write, sizeof(write), "%s.new", final);
+    snprintf(fname, sizeof(fname), "%s/%s/%s/%s.quest", settings.localdir, settings.playerdir, pq->player_name, pq->player_name);
 
-    file = fopen(write, "w+");
-    if (!file) {
-        LOG(llevError, "quest: couldn't open player quest file %s!", write);
+    file = of_open(&of, fname);
+    if (file == NULL) {
         draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!");
         return;
     }
@@ -573,10 +580,11 @@ static void quest_write_player_data(const quest_player *pq) {
         state = state->next;
     }
 
-    fclose(file);
+    if (!of_close(&of)) {
+        draw_ext_info(NDI_UNIQUE | NDI_ALL_DMS, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOADSAVE, "File write error on server!");
+        return;
+    }
     /** @todo rename/backup, stuff like that */
-    unlink(final);
-    rename(write, final);
 }
 
 /**
@@ -664,7 +672,7 @@ static quest_player *get_or_create_quest(player *pl) {
 }
 
 /* quest_set_state can call itself through the function update_quests, so it needs to be declared here */
-static void quest_set_state(player *pl, sstring quest_code, int state, int started);
+static void quest_set_state(player* dm, player *pl, sstring quest_code, int state, int started);
 
 /**
  * Checks whether the conditions for a given step are met.
@@ -721,7 +729,7 @@ static void update_quests(player *pl) {
         if (new_step > 0) {
             current_step = quest_get_player_state(pl, quest->quest_code);
             if (new_step > current_step) {
-                quest_set_state(pl, quest->quest_code, new_step, 0);
+                quest_set_state(NULL, pl, quest->quest_code, new_step, 0);
             }
         }
         quest = quest->next;
@@ -730,42 +738,55 @@ static void update_quests(player *pl) {
 
 /**
  * Set the state of a quest for a player.
+ * @param dm if NULL then the player is actually playing, else a DM is changing the quest's state manually.
  * @param pl player to set the state for.
  * @param quest_code quest internal code.
  * @param state new state for the quest, must be greater than 0 else forced to 100 and a warning is emitted.
  * @param started if 1, quest must have been started first or a warning is emitted, else it doesn't matter.
  */
-static void quest_set_state(player *pl, sstring quest_code, int state, int started) {
+static void quest_set_state(player* dm, player *pl, sstring quest_code, int state, int started) {
     quest_player *pq = get_or_create_quest(pl);
     quest_state *qs = get_or_create_state(pq, quest_code);
     quest_definition *quest = quest_get(quest_code);
     quest_step_definition *step;
 
     if (!quest) {
-        LOG(llevError, "quest: asking for set_state of unknown quest %s!\n", quest_code);
+        if (dm) {
+            draw_ext_info_format(NDI_UNIQUE, 0, dm->ob, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_FAILURE, "Unknown quest %s!", quest_code);
+        } else {
+            LOG(llevError, "quest: asking for set_state of unknown quest %s!\n", quest_code);
+        }
         return;
     }
 
     if (state <= 0) {
-        LOG(llevDebug, "quest_set_player_state: warning: called with invalid state %d for quest %s, player %s", state, pl->ob->name, quest_code);
+        if (!dm) {
+            LOG(llevDebug, "quest_set_player_state: warning: called with invalid state %d for quest %s, player %s\n", state, pl->ob->name, quest_code);
+        }
         state = 100;
     }
 
     step = quest_get_step(quest, state);
     if (!step) {
-        LOG(llevError, "quest_set_player_state: couldn't find state definition %d for quest %s, player %s", state, quest_code, pl->ob->name);
+        if (dm) {
+            draw_ext_info_format(NDI_UNIQUE, 0, dm->ob, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_FAILURE, "Couldn't find state definition %d for quest %s", state, quest_code);
+        } else {
+            LOG(llevError, "quest_set_player_state: couldn't find state definition %d for quest %s, player %s\n", state, quest_code, pl->ob->name);
+        }
         return;
     }
 
     if (started && qs->state == 0) {
-        LOG(llevDebug, "quest_set_player_state: warning: called for player %s not having started quest %s\n", pl->ob->name, quest_code);
+        if (!dm) {
+            LOG(llevDebug, "quest_set_player_state: warning: called for player %s not having started quest %s\n", pl->ob->name, quest_code);
+        }
     }
 
     qs->state = state;
     if (step->is_completion_step) {
         /* don't send an update note if the quest was already completed, this is just to show the outcome afterwards. */
         if (!qs->is_complete)
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest %s completed.", quest->quest_title);
+            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "Quest %s completed.", quest->quest_title);
         qs->was_completed = 1;
         if (quest->quest_restart)
             qs->state = QC_CAN_RESTART;
@@ -773,8 +794,8 @@ static void quest_set_state(player *pl, sstring quest_code, int state, int start
             qs->is_complete =1;
 
     } else {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "New objective for the quest '%s':", quest->quest_title);
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, step->step_description);
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "New objective for the quest '%s':", quest->quest_title);
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, step->step_description);
     }
 
     if (pl->socket.notifications > 0) {
@@ -790,18 +811,17 @@ static void quest_set_state(player *pl, sstring quest_code, int state, int start
         SockList_AddInt(&sl, quest->client_code);
         if (qs->sent_to_client == 0) {
             SockList_AddLen16Data(&sl, quest->quest_title, strlen(quest->quest_title));
-            if (quest->face && !(pl->socket.faces_sent[quest->face]&NS_FACESENT_FACE))
+            if (quest->face && !(pl->socket.faces_sent[quest->face->number]&NS_FACESENT_FACE))
                 esrv_send_face(&pl->socket, quest->face, 0);
-            SockList_AddInt(&sl, quest->face);
+            SockList_AddInt(&sl, quest->face ? quest->face->number : 0);
             SockList_AddChar(&sl, quest->quest_restart ? 1 : 0);
             SockList_AddInt(&sl, quest->parent ? quest->parent->client_code : 0);
         }
 
         SockList_AddChar(&sl, (step == NULL || step->is_completion_step) ? 1 : 0);
-        if (step != NULL)
-            SockList_AddLen16Data(&sl, step->step_description, strlen(step->step_description));
-        else
-            SockList_AddShort(&sl, 0);
+        assert(step != NULL);
+        SockList_AddLen16Data(&sl, step->step_description,
+                strlen(step->step_description));
 
         Send_With_Handling(&pl->socket, &sl);
         SockList_Term(&sl);
@@ -821,8 +841,9 @@ static void quest_set_state(player *pl, sstring quest_code, int state, int start
  * @param pl player to display list of quests.
  * @param pq quests to display.
  * @param showall if 0, only shows quests in progress and a summary of completed quests, else shows all quests.
+ * @param name either 'You' or the player's name, if pl is a DM asking about another player.
  */
-static void quest_display(player *pl, quest_player *pq, int showall) {
+static void quest_display(player *pl, quest_player *pq, int showall, const char* name) {
     quest_state *state;
     quest_definition *quest;
     const char *restart;
@@ -846,15 +867,15 @@ static void quest_display(player *pl, quest_player *pq, int showall) {
     if (completed_count > 0) {
         if (!showall) {
             if (restart_count > 0)
-                draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                        "You have completed %d quests, of which %d may be restarted", completed_count, restart_count);
+                draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS,
+                        "%s completed %d out of %d quests, of which %d may be restarted", name, completed_count, quests_loaded, restart_count);
             else
-                draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                        "You have completed %d quests", completed_count);
+                draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS,
+                        "%s completed %d quests", name, completed_count);
             current_count = completed_count;
         } else {
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                    "You have completed the following quests:");
+            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS,
+                    "%s completed the following quests:", name);
             state = pq->quests;
             while (state) {
                 quest = quest_get(state->code);
@@ -862,7 +883,7 @@ static void quest_display(player *pl, quest_player *pq, int showall) {
                     if (state->state == QC_CAN_RESTART || state->is_complete) {
 
                         restart = state->state == QC_CAN_RESTART?" (can be replayed)":"";
-                        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+                        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS,
                             "(%3d) %s%s", ++current_count, quest->quest_title, restart);
                     }
                 }
@@ -871,15 +892,15 @@ static void quest_display(player *pl, quest_player *pq, int showall) {
         }
     }
     if (total_count > completed_count) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-                "You have started the following quests:");
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS,
+                "%s started the following quests:", name);
         state = pq->quests;
         while (state) {
             quest = quest_get(state->code);
             if (quest->parent == NULL) {
                 if (state->state != QC_CAN_RESTART && state->is_complete==0) {
                     quest = quest_get(state->code);
-                    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
+                    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS,
                         "(%3d) %s", ++current_count, quest->quest_title);
                 }
             }
@@ -890,20 +911,22 @@ static void quest_display(player *pl, quest_player *pq, int showall) {
 
 /**
  * Display current and completed player quests.
- * @param pl player to display for.
+ * @param pl player to display to.
+ * @param who player to display information for.
  * @param showall - whether to show all of the quests in full, just summary information for the completed ones
+ * @param name either 'You' or the player's name, if pl is a DM asking about another player.
  */
-static void quest_list(player *pl, int showall) {
+static void quest_list(player *pl, player *who, int showall, const char* name) {
     quest_player *pq;
 
     /* ensure we load data if not loaded yet */
-    pq = get_or_create_quest(pl);
+    pq = get_or_create_quest(who);
     if (!pq->quests) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "You didn't start any quest.");
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "%s didn't start any quest.", name);
         return;
     }
 
-    quest_display(pl, pq, showall);
+    quest_display(pl, pq, showall, name);
 }
 
 /**
@@ -911,9 +934,17 @@ static void quest_list(player *pl, int showall) {
  * @param pl player to display help for.
  */
 static void quest_help(player *pl) {
-    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest commands:");
-    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " - list: displays quests you are currently attempting add 'all' to show completed quests also");
-    draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " - info: displays information about the specified (by number) quest");
+    if (QUERY_FLAG(pl->ob, FLAG_WIZ)) {
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "DM Quest commands:");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " - (player name) list: displays quests the player is currently attempting, add 'all' to show completed quests also");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " - (player name) info: displays information about the specified (by number) quest");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " - (player name) set (quest code) (state): set the state of the specified quest");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "Note: (player name) may be a partial name as long as it isn't ambiguous. The player must be online.");
+    } else {
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "Quest commands:");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " - list: displays quests you are currently attempting add 'all' to show completed quests also");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " - info: displays information about the specified (by number) quest");
+    }
 }
 
 /**
@@ -954,43 +985,50 @@ static quest_state *get_quest_by_number(player *pl, int number) {
 /**
  * Give details about a quest.
  * @param pl player to give quest details to.
+ * @param who player to give quest details of.
  * @param qs quest_state to give details about
  * @param level The level of recursion for the quest info that's being provided
  */
-static void quest_info(player *pl, quest_state *qs, int level) {
+static void quest_info(player *pl, player* who, quest_state *qs, int level) {
     quest_definition *quest, *child;
     quest_state *state;
-    quest_player *pq = get_or_create_quest(pl);
+    quest_player *pq = get_or_create_quest(who);
     quest_step_definition *step;
     const char *prefix;
 
     if (!qs) {
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Invalid quest number");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "Invalid quest number");
         return;
     }
     quest = quest_get(qs->code);
     if (!quest) {
         /* already warned by quest_get */
-        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest: (internal error)");
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "Quest: (internal error)");
         return;
     }
 
-    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Quest: %s", quest->quest_title);
-    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "Description: %s", quest->quest_description);
+    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, TAG_START "Quest:" TAG_END " %s", quest->quest_title);
+    if (QUERY_FLAG(pl->ob, FLAG_WIZ)) {
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, TAG_START "Code:" TAG_END " %s", quest->quest_code);
+        for (step = quest->steps; step != NULL; step = step->next) {
+            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " " TAG_START "Step:" TAG_END " %d (%s)", step->step, step->step_description);
+        }
+    }
+    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, TAG_START "Description:" TAG_END " %s", quest->quest_description);
 
     step = quest_get_step(quest, qs->state);
     if (qs->state == QC_CAN_RESTART || qs->is_complete) {
         const char *restart = "";
         if (quest->quest_restart)
             restart = " (can be replayed)";
-        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "This quest has been completed%s.", restart);
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, TAG_START "This quest has been completed%s.[/color]", restart);
     }
     prefix = "";
     if (qs->state != QC_CAN_RESTART) {
         /* ie, if we are in progress or completed for a non-restartable quest */
         if (!step) {
             /* already warned by quest_get_step */
-            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " \nOutcome: (invalid quest)");
+            draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " \nOutcome: (invalid quest)");
             return;
         }
         if (level > 0) {
@@ -999,7 +1037,7 @@ static void quest_info(player *pl, quest_state *qs, int level) {
             prefix = "Outcome";
         else
             prefix = "Current Status";
-        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, " \n%s: %s", prefix, step->step_description);
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, " \n" TAG_START "%s:" TAG_END " %s", prefix, step->step_description);
     }
 
     /* ok, now check all of the player's other quests for any children, and print those in order */
@@ -1007,7 +1045,7 @@ static void quest_info(player *pl, quest_state *qs, int level) {
     while (state) {
         child = quest_get(state->code);
         if (child->parent == quest)
-            quest_info(pl, state, level+1);
+            quest_info(pl, who, state, level+1);
         state = state->next;
     }
     return;
@@ -1081,9 +1119,9 @@ void quest_start(player *pl, sstring quest_code, int state) {
         LOG(llevDebug, "quest_start: warning: player %s has already started quest %s\n", pl->ob->name, quest_code);
     }
 
-    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO, "New quest started: %s", quest->quest_title);
+    draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_QUESTS, "New quest started: %s", quest->quest_title);
 
-    quest_set_state(pl, quest_code, state, 0);
+    quest_set_state(NULL, pl, quest_code, state, 0);
 
     /* saving state will be done in quest_set_state(). */
 }
@@ -1095,7 +1133,7 @@ void quest_start(player *pl, sstring quest_code, int state) {
  * @param state new state for the quest, must be greater than 0 else forced to 100 and a warning is emitted.
  */
 void quest_set_player_state(player *pl, sstring quest_code, int state) {
-    quest_set_state(pl, quest_code, state, 1);
+    quest_set_state(NULL, pl, quest_code, state, 1);
 }
 
 /**
@@ -1117,6 +1155,10 @@ int quest_was_completed(player *pl, sstring quest_code) {
  * @param params extra parameters for command.
  */
 void command_quest(object *op, const char *params) {
+    /* who to display information about, used when called in DM mode */
+    object *who;
+    const char *name;
+
     if (!op->contr) {
         LOG(llevError, "command_quest called for a non player!\n");
         return;
@@ -1126,25 +1168,106 @@ void command_quest(object *op, const char *params) {
         quest_help(op->contr);
         return;
     }
+
+    if (QUERY_FLAG(op, FLAG_WIZ)) {
+        char* dup = strdup(params);
+        char* space = strchr(dup, ' ');
+        player* other;
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_QUESTS, "Command 'quest' called in DM mode.");
+        if (space == NULL) {
+            free(dup);
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_FAILURE, "Please specify a player name.");
+            return;
+        }
+        params = params + (space - dup) + 1;
+        *space = '\0';
+        other = find_player_partial_name(dup);
+        if (other == NULL) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_FAILURE, "%s is not online, or ambiguous name.", dup);
+            free(dup);
+            return;
+        }
+        free(dup);
+        who = other->ob;
+        name = who->name;
+    } else {
+        who = op;
+        name = "You";
+    }
+
     if (strcmp(params, "list all") == 0) {
-        quest_list(op->contr, 1);
+        quest_list(op->contr, who->contr, 1, name);
         return;
     }
 
     if (strcmp(params, "list") == 0) {
-        quest_list(op->contr, 0);
+        quest_list(op->contr, who->contr, 0, name);
         return;
     }
 
     if (strncmp(params, "info ", 5) == 0) {
         int number = atoi(params+5);
-        quest_info(op->contr, get_quest_by_number(op->contr, number), 0);
+        quest_info(op->contr, who->contr, get_quest_by_number(who->contr, number), 0);
+        return;
+    }
+
+    /* 
+     * Quest display for clients using the quest system, similar to 'info' above
+     * but using the (shared) quest's client_code instead of the (player unique) index.
+     */
+    if (strncmp(params, "info_c ", 7) == 0) {
+        int number = atoi(params+7);
+        quest_player *qp = get_quest(who->contr);
+        quest_state *qs = qp ? qp->quests : NULL;
+        while (qs) {
+            quest_definition *q = quest_get_by_code(qs->code);
+            if (q && q->client_code == number) {
+                break;
+            }
+            qs = qs->next;
+        }
+        if (qs) {
+            quest_info(op->contr, who->contr, qs, 0);
+            return;
+        }
+        draw_ext_info(NDI_UNIQUE, 0, who, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE, "Invalid quest number");
+        return;
+    }
+
+    if (QUERY_FLAG(op, FLAG_WIZ) && strncmp(params, "set ", 4) == 0) {
+        char *dup = strdup(params + 4);
+        char *space = strrchr(dup, ' ');
+        int state, number;
+        quest_state* q;
+        if (space == NULL) {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_FAILURE, "Syntax is: quest (player name) (quest number) (state).");
+            free(dup);
+            return;
+        }
+        *space = '\0';
+        number = atoi(dup);
+        q = get_quest_by_number(who->contr, number);
+        if (q == NULL) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_FAILURE, "Invalid quest number %d.", number);
+            free(dup);
+            return;
+        }
+        state = atoi(space + 1);
+        quest_set_state(op->contr, who->contr, q->code, state, 0);
+        free(dup);
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN_DM, MSG_TYPE_COMMAND_SUCCESS, "Set changed.");
         return;
     }
 
     quest_help(op->contr);
 }
 
+/**
+ * Dump all defined quests on the logfile. Will call itself recursively.
+ * @param parent only quests with a parent of this value will be displayed.
+ * Use NULL to display top-level quests.
+ * @param level number of '-' to display before the quest's name.
+ */
 static void output_quests(quest_definition *parent, int level) {
     quest_definition *quest;
     quest_step_definition *step;
@@ -1282,9 +1405,9 @@ void quest_send_initial_states(player *pl) {
 
         SockList_AddInt(&sl, quest->client_code);
         SockList_AddLen16Data(&sl, quest->quest_title, strlen(quest->quest_title));
-        if (quest->face && !(pl->socket.faces_sent[quest->face]&NS_FACESENT_FACE))
+        if (quest->face && !(pl->socket.faces_sent[quest->face->number]&NS_FACESENT_FACE))
             esrv_send_face(&pl->socket, quest->face, 0);
-        SockList_AddInt(&sl, quest->face);
+        SockList_AddInt(&sl, quest->face ? quest->face->number : 0);
         SockList_AddChar(&sl, quest->quest_restart ? 1 : 0);
         SockList_AddInt(&sl, quest->parent ? quest->parent->client_code : 0);
         SockList_AddChar(&sl, (step == NULL || step->is_completion_step) ? 1 : 0);
