@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_resurrection_c =
- *   "$Id: resurrection.c 13308 2010-05-28 06:12:12Z kbulgrien $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Resurrection / raise dead related code.
@@ -36,12 +21,15 @@
 /*  the contents of this file were create solely by peterm@soda.berkeley.edu
     all of the above disclaimers apply.  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
+#include "global.h"
+
 #include <errno.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "output_file.h"
+#include "sproto.h"
+#include "spells.h"
 
 static int resurrection_fails(int levelcaster, int leveldead);
 
@@ -54,21 +42,23 @@ static int resurrection_fails(int levelcaster, int leveldead);
  * the name of the player to resurrect.
  * @param spell
  * spell that was used to resurrect.
+ * @param accountname
+ * the account the resurrected player belongs to
  * @retval 0
  * resurrection failed.
  * @retval 1
  * playername is living again.
  */
-static int resurrect_player(object *op, char *playername, object *spell) {
+static int resurrect_player(object *op, char *playername, object *spell, char *accountname) {
     FILE *deadplayer, *liveplayer;
-
+    OutputFile of;
     char oldname[MAX_BUF];
     char newname[MAX_BUF];
     char path[MAX_BUF];
     char buf[MAX_BUF];
     char buf2[MAX_BUF];
     const char *race = NULL;
-    sint64 exp;
+    int64_t exp;
     int Con;
 
     /* reincarnation, which changes the race */
@@ -118,17 +108,17 @@ static int resurrect_player(object *op, char *playername, object *spell) {
         return 0;
     }
 
-    if (!(liveplayer = fopen(newname, "w"))) {
+    liveplayer = of_open(&of, newname);
+    if (liveplayer == NULL) {
         draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_FAILURE,
                              "The soul of %s cannot be re-embodied at the moment.",
                              playername);
-        LOG(llevError, "Cannot write player in resurrect_player!\n");
         fclose(deadplayer);
         return 0;
     }
 
     while (!feof(deadplayer)) {
-        fgets(buf, 255, deadplayer);
+        if ( fgets(buf, 255, deadplayer) == NULL ) break; /* should never fail */
         sscanf(buf, "%s", buf2);
         if (!(strcmp(buf2, "exp"))) {
             sscanf(buf, "%s %"FMT64, buf2, &exp);
@@ -149,13 +139,22 @@ static int resurrect_player(object *op, char *playername, object *spell) {
         }
         fputs(buf, liveplayer);
     }
-    fclose(liveplayer);
+    if (!of_close(&of)) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_FAILURE,
+                             "The soul of %s cannot be re-embodied at the moment.",
+                             playername);
+        fclose(deadplayer);
+        return 0;
+    }
     fclose(deadplayer);
     unlink(oldname);
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_SUCCESS,
                          "%s lives again!",
                          playername);
-
+    /* Needs to go into the account file and change the player back to isDead = 0 */
+    /* I'm fairly certain the old login did not have the segfault, so just account for the new account system */
+    if (accountname)
+        unmake_perma_dead(accountname, playername);
     return 1;
 }
 
@@ -182,9 +181,11 @@ static int resurrect_player(object *op, char *playername, object *spell) {
 int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, const char *arg) {
     object *temp, *newob;
     char name_to_resurrect[MAX_BUF];
+    char *corpse_account = NULL;
     int leveldead = 25, mflags, clevel;
-    sint16 sx, sy;
+    int16_t sx, sy;
     mapstruct *m;
+    int spell_success = 0;
 
     clevel = caster_level(caster, spell);
 
@@ -195,7 +196,7 @@ int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, co
                                  spell->name);
             return 0;
         }
-        strcpy(name_to_resurrect, arg);
+        safe_strncpy(name_to_resurrect, arg, sizeof(name_to_resurrect));
         temp = NULL;
     } else {
         sx = op->x+freearr_x[dir];
@@ -222,15 +223,23 @@ int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, co
                           "You need a body for this spell.");
             return 0;
         }
-        strcpy(name_to_resurrect, temp->name);
+        safe_strncpy(name_to_resurrect, temp->name, sizeof(name_to_resurrect));
+        /* Only try to copy if dead body is as recent as the patch this is from. */
+        if (temp->slaying){
+            corpse_account = (char *)CALLOC(sizeof(char), MAX_BUF);
+            /* Make sure that the allocation worked. */
+            if (corpse_account) strcpy(corpse_account, temp->slaying);
+        }
     }
-
     /* no matter what, we fry the corpse.  */
     if (temp && temp->map) {
         /* replace corpse object with a burning object */
-        newob = arch_to_object(find_archetype("burnout"));
-        if (newob != NULL)
-            object_insert_in_map_at(newob, temp->map, op, 0, temp->x, temp->y);
+        archetype *arch = find_archetype("burnout");
+        if (arch != NULL) {
+            newob = arch_to_object(arch);
+            if (newob != NULL)
+                object_insert_in_map_at(newob, temp->map, op, 0, temp->x, temp->y);
+        }
         leveldead = temp->level;
         object_remove(temp);
         object_free_drop_inventory(temp);
@@ -244,11 +253,13 @@ int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, co
                 summon_hostile_monsters(op, t->nrof, t->item->name);
             }
         }
-        return 1;
+        spell_success = 1;
     } else {
-        return resurrect_player(op, name_to_resurrect, spell);
+        spell_success = resurrect_player(op, name_to_resurrect, spell, corpse_account);
     }
-    /* Unreachable */
+    /* Reorganized so corpse_account could be deallocated if needed */
+    free(corpse_account);
+    return spell_success;
 }
 
 /**
@@ -297,6 +308,11 @@ void dead_player(object *op) {
     strcat(newname, ".dead");
 
     if (rename(filename, newname) != 0) {
-        LOG(llevError, "Cannot rename dead player's file %s into %s: %s\n", filename, newname, strerror_local(errno, path, sizeof(path)));
+        LOG(llevError, "Cannot rename dead player's file %s into %s: %s\n", filename, newname, strerror(errno));
+    }
+    /* Go into the account file and change isDead for this character to 1. */
+    if (make_perma_dead(op)){
+        /* Make an error message saying that the character could not be changed in the proper account */
+        LOG(llevError, "Could not edit the account to indicate permanent death for %s!\n", op->name);
     }
 }
