@@ -1,49 +1,33 @@
 /*
- * static char *rcsid_rune_c =
- *   "$Id: rune.c 15527 2011-11-06 17:43:01Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2003,2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  *
  * All rune-related functions.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
+#include "global.h"
 
+#include <string.h>
+
+#include "sproto.h"
+#include "spells.h"
 
 #ifndef sqr
 #define sqr(x) ((x)*(x))
 #endif
 
-
 /**
  * Player is attempting to write a magical rune.
  * This function does all checks for paths, sp/gr, ...
@@ -67,7 +51,7 @@ int write_rune(object *op, object *caster, object *spell, int dir, const char *r
     object *rune_spell, *rune;
     char buf[MAX_BUF];
     mapstruct *m;
-    sint16 nx, ny, gr;
+    int16_t nx, ny, gr;
 
     if (!dir) {
         dir = 1;
@@ -315,7 +299,7 @@ void spring_trap(object *trap, object *victim) {
 int dispel_rune(object *op, object *caster, object *spell, object *skill, int dir) {
     object *rune;
     int mflags;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *m;
 
     x = op->x+freearr_x[dir];
@@ -426,7 +410,12 @@ int trap_show(object *trap, object *where) {
     if (where == NULL)
         return 0;
     tmp2 = create_archetype("runedet");
-    tmp2->face = GET_ANIMATION(trap, 0);
+    // Custom-made traps can have no animation.
+    // The GET_ANIMATION macro does not handle this scenario
+    if (trap->animation || trap->temp_animation)
+        tmp2->face = GET_ANIMATION(trap, 0);
+    else
+        tmp2->face = trap->face;
     object_insert_in_map_at(tmp2, where->map, NULL, 0, where->x, where->y);
     return 1;
 }
