@@ -1,108 +1,56 @@
 /*
- * static char *rcsid_server_c =
- *    "$Id: server.c 17458 2012-03-11 17:08:11Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Main server functions.
  */
 
-#include <global.h>
-#include <object.h>
-#include <tod.h>
-#include <version.h>
+#include "global.h"
+
 #include <assert.h>
-
-#ifdef HAVE_DES_H
-#include <des.h>
-#else
-#  ifdef HAVE_CRYPT_H
-#  include <crypt.h>
-#  endif
-#endif
-
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-
-#ifdef HAVE_TIME_H
+#include <ctype.h>
+#include <fcntl.h>
+#include <math.h>
+#include <stdlib.h>
+#include <string.h>
 #include <time.h>
-#endif
 
 #ifndef WIN32
-#  include <unistd.h>
-#  include <sys/types.h>
+#include <unistd.h>
 #endif
 
-#include <../random_maps/random_map.h>
-#include <../random_maps/rproto.h>
+/* FIXME: This is required on certain systems to get crypt(3) to work. */
+#ifdef HAVE_CRYPT_H
+#include <crypt.h>
+#endif
+
+#include "object.h"
 #include "path.h"
+#include "random_maps/random_map.h"
+#include "random_maps/rproto.h"
+#include "sproto.h"
+#include "tod.h"
+#include "version.h"
+#include "server.h"
+
+/** How many minutes before a shutdown to warn players, in reverse order. */
+static const int shutdown_warn_times[] = {120, 90, 60, 45, 30, 15, 10, 5, 4, 3, 2, 1};
 
 /** Ingame days. */
-static const char days[7][4] = {
-    "Sun",
-    "Mon",
-    "Tue",
-    "Wed",
-    "Thu",
-    "Fri",
-    "Sat"
-};
+static const char *days[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
 
-/**
- * Displays basic game version information.
- *
- * @param op
- * who is requesting the verison information.
- */
-void version(object *op) {
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_VERSION,
-                         "This is Crossfire v%s",
-                         FULL_VERSION);
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_VERSION,
-"The authors can be reached at crossfire@metalforge.org");
-}
-
-/**
- * Gives basic start information to player.
- *
- * @param op
- * player.
- */
-void start_info(object *op) {
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_LOGIN,
-                         "Welcome to Crossfire, v%s!\nPress `?' for help\n",
-                         VERSION);
-
-    draw_ext_info_format(NDI_UNIQUE|NDI_ALL|NDI_DK_ORANGE, 5, op,
-                         MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_PLAYER,
-                         "%s entered the game.",
-                         op->name);
-}
+volatile sig_atomic_t shutdown_flag;
 
 /**
  * Encrypt a string. Used for password storage on disk.
@@ -119,40 +67,58 @@ void start_info(object *op) {
  * crypted str.
  * @todo make thread-safe?
  */
-char *crypt_string(const char *str, const char *salt) {
-#if defined(WIN32) || (defined(__FreeBSD__) && !defined(HAVE_LIBDES))
-    return(str);
-#else
-    static const char *const c = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
-    char s[2];
+static char const* crypt_string(char const str[static 1], char const* salt) {
+#if defined(WIN32) || (defined(__FreeBSD__))
+    // Legacy configuration: use crypt everywhere but on Windows and FreeBSD
+    if (settings.crypt_mode == 0) {
+        return str;
+    }
+#endif
+    char s[3];
 
-    if (salt == NULL)
+    if (salt == NULL) {
+	/* Generate a two-character salt for the DES cipher.
+	 * We want the salt to be in this character set.
+	 */
+	static const char *const c =
+	    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
         s[0] = c[RANDOM()%(int)strlen(c)],
         s[1] = c[RANDOM()%(int)strlen(c)];
-    else
+    } else {
         s[0] = salt[0],
         s[1] = salt[1];
+    }
+    s[2] = '\0';
 
-#  ifdef HAVE_LIBDES
-    return (char *)des_crypt(str, s);
-#  endif
-    /* Default case - just use crypt */
-    return (char *)crypt(str, s);
-#endif
+    return crypt(str, s);
+}
+
+char const* newhash(char const password[static 1]) {
+    return crypt_string(password, NULL);
 }
 
 /**
- * Check if 2 passwords match.
- *
+ * Hash a password and compare it to the stored version.
  * @param typed
- * entered password. Not crypted.
+ * Password entered by the user (not hashed).
  * @param crypted
- * password to check against. Must be crypted.
+ * Stored password to check against (hashed).
  * @return
- * 1 if the passwords match, 0 else.
+ * True if the passwords match, false otherwise.
  */
-int check_password(const char *typed, const char *crypted) {
-    return !strcmp(crypt_string(typed, crypted), crypted);
+bool check_password(const char *typed, const char *crypted) {
+    // An empty hashed password only matches an empty input password.
+    if (strlen(crypted) == 0) {
+        return strlen(typed) == 0 ? true : false;
+    }
+
+    const char *typed_hashed = crypt_string(typed, crypted);
+    if (typed_hashed != NULL) {
+        return strcmp(typed_hashed, crypted) == 0;
+    } else {
+        LOG(llevError, "Could not check password with stored hash %s\n", crypted);
+        return false;
+    }
 }
 
 /**
@@ -180,8 +146,9 @@ void enter_player_savebed(object *op) {
      * while we're at it.
      */
     if (oldmap == op->map && strcmp(op->contr->savebed_map, oldmap->path)) {
-        LOG(llevDebug, "Player %s savebed location %s is invalid - going to emergency location (%s)\n", settings.emergency_mapname, op->name, op->contr->savebed_map);
-        strcpy(op->contr->savebed_map, settings.emergency_mapname);
+        LOG(llevDebug, "Player %s savebed location %s is invalid - going to emergency location (%s)\n",
+                settings.emergency_mapname, op->name, op->contr->savebed_map);
+        safe_strncpy(op->contr->savebed_map, settings.emergency_mapname, MAX_BUF);
         op->contr->bed_x = settings.emergency_x;
         op->contr->bed_y = settings.emergency_y;
         free_string(op->contr->savebed_map);
@@ -190,7 +157,7 @@ void enter_player_savebed(object *op) {
         EXIT_Y(tmp) = op->contr->bed_y;
         enter_exit(op, tmp);
     }
-    object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+    object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
 }
 
 /**
@@ -252,25 +219,14 @@ static void enter_map(object *op, mapstruct *newmap, int x, int y) {
      */
     if (!QUERY_FLAG(op, FLAG_REMOVED))
         object_remove(op);
-    if (op->map != NULL) {
-        /* Lauwenmark : Here we handle the MAPLEAVE global event */
-        execute_global_event(EVENT_MAPLEAVE, op, op->map);
-    }
     /* object_remove clears these so they must be reset after the object_remove() call */
     object_insert_in_map_at(op, newmap, NULL, INS_NO_WALK_ON, x, y);
 
-    /* Lauwenmark : Here we handle the MAPENTER global event */
-    execute_global_event(EVENT_MAPENTER, op, op->map);
-
-    if (op->contr) {
-        send_background_music(op->contr, newmap->background_music);
-    }
-
-    newmap->timeout = 0;
     object_set_enemy(op, NULL);
 
     if (op->contr) {
-        strcpy(op->contr->maplevel, newmap->path);
+        safe_strncpy(op->contr->maplevel, newmap->path,
+                sizeof(op->contr->maplevel));
         op->contr->count = 0;
     }
 
@@ -300,15 +256,28 @@ static void enter_map(object *op, mapstruct *newmap, int x, int y) {
      * old map does not work.
      */
     if (oldmap != newmap) {
-        if (oldmap) { /* adjust old map */
-            if (oldmap->players <= 0) /* can be less than zero due to errors in tracking this */
-                set_map_timeout(oldmap);
+        player_map_change_common(op, oldmap, newmap);
+    }
+
+    map_newmap_cmd(&op->contr->socket);
+}
+
+void player_map_change_common(object* op, mapstruct* const oldmap,
+                              mapstruct* const newmap) {
+    if (oldmap != NULL) {
+        /* Lauwenmark : Here we handle the MAPLEAVE global event */
+        execute_global_event(EVENT_MAPLEAVE, op, oldmap);
+
+        /* can be less than zero due to errors in tracking this */
+        if (oldmap->players <= 0) {
+            set_map_timeout(oldmap);
         }
     }
-    swap_below_max(newmap->path);
 
-    if (op->type == PLAYER)
-        map_newmap_cmd(&op->contr->socket);
+    /* Lauwenmark : Here we handle the MAPENTER global event */
+    execute_global_event(EVENT_MAPENTER, op, newmap);
+    newmap->timeout = 0;
+    player_update_bg_music(op);
 }
 
 /**
@@ -353,7 +322,7 @@ void set_map_timeout(mapstruct *oldmap) {
 static char *clean_path(const char *file, char *newpath, int size) {
     char *cp;
 
-    snprintf(newpath, size, "%s", file);
+    strlcpy(newpath, file, size);
     for (cp = newpath; *cp != '\0'; cp++) {
         if (*cp == '/')
             *cp = '_';
@@ -383,9 +352,9 @@ static char *unclean_path(const char *src, char *newpath, int size) {
 
     cp = strrchr(src, '/');
     if (cp)
-        snprintf(newpath, size, "%s", cp+1);
+        strlcpy(newpath, cp+1, size);
     else
-        snprintf(newpath, size, "%s", src);
+        strlcpy(newpath, src, size);
 
     for (cp = newpath; *cp != '\0'; cp++) {
         if (*cp == '_')
@@ -406,7 +375,7 @@ static char *unclean_path(const char *src, char *newpath, int size) {
  */
 static void enter_random_map(object *pl, object *exit_ob) {
     mapstruct *new_map;
-    char newmap_name[HUGE_BUF], *cp;
+    char newmap_name[HUGE_BUF], buf[HUGE_BUF], *cp;
     static int reference_number = 0;
     RMParms rp;
 
@@ -418,7 +387,7 @@ static void enter_random_map(object *pl, object *exit_ob) {
         set_random_map_variable(&rp, exit_ob->msg);
     rp.origin_x = exit_ob->x;
     rp.origin_y = exit_ob->y;
-    strcpy(rp.origin_map, pl->map->path);
+    safe_strncpy(rp.origin_map, pl->map->path, sizeof(rp.origin_map));
 
     /* If we have a final_map, use it as a base name to give some clue
      * as where the player is.  Otherwise, use the origin map.
@@ -430,13 +399,11 @@ static void enter_random_map(object *pl, object *exit_ob) {
         if (!cp)
             cp = rp.final_map;
     } else {
-        char buf[HUGE_BUF];
-
         cp = strrchr(rp.origin_map, '/');
         if (!cp)
             cp = rp.origin_map;
         /* Need to strip of any trailing digits, if it has them */
-        snprintf(buf, sizeof(buf), "%s", cp);
+        strlcpy(buf, cp, sizeof(buf));
         while (isdigit(buf[strlen(buf)-1]))
             buf[strlen(buf)-1] = 0;
         cp = buf;
@@ -459,7 +426,7 @@ static void enter_random_map(object *pl, object *exit_ob) {
         x = EXIT_X(exit_ob) = MAP_ENTER_X(new_map);
         y = EXIT_Y(exit_ob) = MAP_ENTER_Y(new_map);
         EXIT_PATH(exit_ob) = add_string(newmap_name);
-        snprintf(new_map->path, sizeof(new_map->path), "%s", newmap_name);
+        strlcpy(new_map->path, newmap_name, sizeof(new_map->path));
         enter_map(pl, new_map, x, y);
     }
 }
@@ -476,21 +443,20 @@ static void enter_random_map(object *pl, object *exit_ob) {
 static void enter_fixed_template_map(object *pl, object *exit_ob) {
     mapstruct *new_map;
     char tmpnum[32], exitpath[HUGE_BUF], resultname[HUGE_BUF], tmpstring[HUGE_BUF], *sourcemap;
+    char sourcemap_buf[HUGE_BUF];
     char new_map_name[MAX_BUF];
 
     /* Split the exit path string into two parts, one
      * for where to store the map, and one for were
      * to generate the map from.
      */
-    snprintf(exitpath, sizeof(exitpath), "%s", EXIT_PATH(exit_ob)+2);
+    strlcpy(exitpath, EXIT_PATH(exit_ob)+2, sizeof(exitpath));
     sourcemap = strchr(exitpath, '!');
     if (!sourcemap) {
         draw_ext_info_format(NDI_UNIQUE, 0, pl,
                              MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
                              "The %s is closed.",
                              exit_ob->name);
-        /* Should only occur when no source map is set.
-         */
         LOG(llevError, "enter_fixed_template_map: Exit %s (%d,%d) on map %s has no source template.\n", exit_ob->name, exit_ob->x, exit_ob->y, exit_ob->map->path);
         return;
     }
@@ -501,9 +467,8 @@ static void enter_fixed_template_map(object *pl, object *exit_ob) {
      */
     if (!exit_ob->map->is_template) {
         /* We can't use exitpath directly, as sourcemap points there. */
-        path_combine_and_normalize(exit_ob->map->path, sourcemap, tmpstring, sizeof(tmpstring));
-        snprintf(exitpath, sizeof(exitpath), "%s", tmpstring);
-        sourcemap = exitpath;
+        path_combine_and_normalize(exit_ob->map->path, sourcemap, sourcemap_buf, sizeof(sourcemap_buf));
+        sourcemap = sourcemap_buf;
     }
 
     /* Do replacement of %x, %y, and %n to the x coord of the exit, the y coord
@@ -513,10 +478,10 @@ static void enter_fixed_template_map(object *pl, object *exit_ob) {
     replace(exitpath, "%x", tmpnum, resultname,  sizeof(resultname));
 
     snprintf(tmpnum, sizeof(tmpnum), "%d", exit_ob->y);
-    snprintf(tmpstring, sizeof(tmpstring), "%s", resultname);
+    strlcpy(tmpstring, resultname, sizeof(tmpstring));
     replace(tmpstring, "%y", tmpnum, resultname,  sizeof(resultname));
 
-    snprintf(tmpstring, sizeof(tmpstring), "%s", resultname);
+    strlcpy(tmpstring, resultname, sizeof(tmpstring));
     replace(tmpstring, "%n", exit_ob->map->name, resultname,  sizeof(resultname));
 
     /* If we are coming from another template map, use reletive paths unless
@@ -534,27 +499,27 @@ static void enter_fixed_template_map(object *pl, object *exit_ob) {
     new_map = ready_map_name(new_map_name, MAP_PLAYER_UNIQUE);
     if (!new_map) {
         char path[MAX_BUF];
-
         create_pathname(sourcemap, path, MAX_BUF);
-        new_map = load_original_map(path, MAP_PLAYER_UNIQUE);
+        new_map = mapfile_load(path, MAP_PLAYER_UNIQUE);
+        if (!new_map) {
+            draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND,
+                                 MSG_TYPE_COMMAND_FAILURE, "The %s is closed.",
+                                 exit_ob->name);
+            LOG(llevError,
+                "Template exit in '%s' (%d, %d) does not reference a valid "
+                "template. Make sure a template exists at '%s'.\n",
+                exit_ob->map->path, exit_ob->x, exit_ob->y, path);
+            return;
+        }
     }
 
-    if (new_map) {
-        /* set the path of the map to where it should be
-         * so we don't just save over the source map.
-         */
-        snprintf(new_map->path, sizeof(new_map->path), "%s", new_map_name);
-        new_map->is_template = 1;
-        enter_map(pl, new_map, EXIT_X(exit_ob), EXIT_Y(exit_ob));
-    } else {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl,
-                             MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
-                             "The %s is closed.",
-                             exit_ob->name);
-        /* Should only occur when an invalid source map is set.
-         */
-        LOG(llevDebug, "enter_fixed_template_map: Exit %s (%d,%d) on map %s leads no where.\n", exit_ob->name, exit_ob->x, exit_ob->y, exit_ob->map->path);
-    }
+    assert(new_map);
+    /* set the path of the map to where it should be
+     * so we don't just save over the source map.
+     */
+    strlcpy(new_map->path, new_map_name, sizeof(new_map->path));
+    new_map->is_template = 1;
+    enter_map(pl, new_map, EXIT_X(exit_ob), EXIT_Y(exit_ob));
 }
 
 /**
@@ -579,10 +544,10 @@ static void enter_random_template_map(object *pl, object *exit_ob) {
     replace(EXIT_PATH(exit_ob)+3, "%x", tmpnum, resultname,  sizeof(resultname));
 
     snprintf(tmpnum, sizeof(tmpnum), "%d", exit_ob->y);
-    snprintf(tmpstring, sizeof(tmpstring), "%s", resultname);
+    strlcpy(tmpstring, resultname, sizeof(tmpstring));
     replace(tmpstring, "%y", tmpnum, resultname,  sizeof(resultname));
 
-    snprintf(tmpstring, sizeof(tmpstring), "%s", resultname);
+    strlcpy(tmpstring, resultname, sizeof(tmpstring));
     replace(tmpstring, "%n", exit_ob->map->name, resultname,  sizeof(resultname));
 
     /* If we are coming from another template map, use reletive paths unless
@@ -604,7 +569,7 @@ static void enter_random_template_map(object *pl, object *exit_ob) {
             set_random_map_variable(&rp, exit_ob->msg);
         rp.origin_x = exit_ob->x;
         rp.origin_y = exit_ob->y;
-        strcpy(rp.origin_map, pl->map->path);
+        safe_strncpy(rp.origin_map, pl->map->path, sizeof(rp.origin_map));
 
         /* now to generate the actual map. */
         new_map = generate_random_map(new_map_name, &rp, NULL);
@@ -639,11 +604,10 @@ static void enter_unique_map(object *op, object *exit_ob) {
     mapstruct        *newmap;
 
     if (EXIT_PATH(exit_ob)[0] == '/') {
-        snprintf(apartment, sizeof(apartment), "%s/%s/%s/%s", settings.localdir, settings.playerdir, op->name, clean_path(EXIT_PATH(exit_ob), path, sizeof(path)));
+        snprintf(apartment, sizeof(apartment), "~%s/%s", op->name, clean_path(EXIT_PATH(exit_ob), path, sizeof(path)));
         newmap = ready_map_name(apartment, MAP_PLAYER_UNIQUE);
         if (!newmap) {
-            create_pathname(EXIT_PATH(exit_ob), path, sizeof(path));
-            newmap = load_original_map(path, MAP_PLAYER_UNIQUE);
+            newmap = mapfile_load(EXIT_PATH(exit_ob), 0);
         }
     } else { /* relative directory */
         char reldir[HUGE_BUF], tmpc[HUGE_BUF], *cp;
@@ -657,19 +621,18 @@ static void enter_unique_map(object *op, object *exit_ob) {
             if ((cp = strrchr(tmpc, '_')) != NULL)
                 *cp = 0;
 
-            snprintf(apartment, sizeof(apartment), "%s/%s/%s/%s_%s", settings.localdir, settings.playerdir, op->name, tmpc, clean_path(EXIT_PATH(exit_ob), path, sizeof(path)));
+            snprintf(apartment, sizeof(apartment), "~%s/%s_%s", op->name, tmpc, clean_path(EXIT_PATH(exit_ob), path, sizeof(path)));
 
             newmap = ready_map_name(apartment, MAP_PLAYER_UNIQUE);
             if (!newmap) {
-                create_pathname(path_combine_and_normalize(reldir, EXIT_PATH(exit_ob), tmpc, sizeof(tmpc)), path, sizeof(path));
-                newmap = load_original_map(path, MAP_PLAYER_UNIQUE);
+                newmap = mapfile_load(path_combine_and_normalize(reldir, EXIT_PATH(exit_ob), tmpc, sizeof(tmpc)), 0);
             }
         } else {
             /* The exit is unique, but the map we are coming from is not unique.  So
              * use the basic logic - don't need to demangle the path name
              */
             path_combine_and_normalize(exit_ob->map->path, EXIT_PATH(exit_ob), reldir, sizeof(reldir));
-            snprintf(apartment, sizeof(apartment), "%s/%s/%s/%s", settings.localdir, settings.playerdir, op->name, clean_path(reldir, path, sizeof(path)));
+            snprintf(apartment, sizeof(apartment), "~%s/%s", op->name, clean_path(reldir, path, sizeof(path)));
             newmap = ready_map_name(apartment, MAP_PLAYER_UNIQUE);
             if (!newmap) {
                 path_combine_and_normalize(exit_ob->map->path, EXIT_PATH(exit_ob), reldir, sizeof(reldir));
@@ -681,7 +644,7 @@ static void enter_unique_map(object *op, object *exit_ob) {
     }
 
     if (newmap) {
-        snprintf(newmap->path, sizeof(newmap->path), "%s", apartment);
+        strlcpy(newmap->path, apartment, sizeof(newmap->path));
         newmap->unique = 1;
         enter_map(op, newmap, EXIT_X(exit_ob), EXIT_Y(exit_ob));
     } else {
@@ -699,51 +662,41 @@ static void enter_unique_map(object *op, object *exit_ob) {
     }
 }
 
-/**
- * Try to move op to the exit op->maplevel points to.
- * Will abort() if maplevel is invalid and the emergency map is invalid too.
- * @param op what to move, must not be NULL and must be a player.
- */
-static void enter_maplevel_exit(object *op) {
-    int flags = 0;
+void enter_player_maplevel(object *op) {
+    int flags = 0, x = op->x, y = op->y;
     mapstruct *newmap;
 
     assert(op != NULL);
     assert(op->type == PLAYER);
 
-    /* Hypothetically, I guess its possible that a standard map matches
-     * the localdir, but that seems pretty unlikely - unlikely enough that
-     * I'm not going to attempt to try to deal with that possibility.
-     * We use the fact that when a player saves on a unique map, it prepends
-     * the localdir to that name.  So its an easy way to see of the map is
-     * unique or not.
-     */
-    if (!strncmp(op->contr->maplevel, settings.localdir, strlen(settings.localdir)))
-        flags = MAP_PLAYER_UNIQUE;
-
     /* newmap returns the map (if already loaded), or loads it for us. */
     newmap = ready_map_name(op->contr->maplevel, flags);
     if (!newmap) {
-        LOG(llevError, "enter_exit: Pathname to map does not exist! (%s)\n", op->contr->maplevel);
         newmap = ready_map_name(settings.emergency_mapname, 0);
-        op->x = settings.emergency_x;
-        op->y = settings.emergency_y;
-        /* If we can't load the emergency map, something is probably really
-         * screwed up, so bail out now.
-         */
+        x = settings.emergency_x;
+        y = settings.emergency_y;
         if (!newmap) {
-            LOG(llevError, "enter_exit: could not load emergency map? Fatal error\n");
+            LOG(llevError, "Fatal: Could not load emergency map!\n");
             abort();
         }
+
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_MISC, 0,
+                "You find yourself somewhere unexpected...");
     }
-    enter_map(op, newmap, op->x, op->y);
+
+    /* as a special case, if coordinates are (-1, -1), then the item should
+     * be put at the default location. Used for loginmethod 0 (old clients). */
+    if (x == -1 && y == -1) {
+        x = MAP_ENTER_X(newmap);
+        y = MAP_ENTER_Y(newmap);
+    }
+
+    enter_map(op, newmap, x, y);
 }
 
 /**
  * Tries to move 'op' to exit_ob.
  *
- * This is used when loading the player.
- *
  * Largely redone by MSW 2001-01-21 - this function was overly complex
  * and had some obscure bugs.
  *
@@ -752,8 +705,7 @@ static void enter_maplevel_exit(object *op) {
  * @param op
  * character or monster that is using the exit.
  * @param exit_ob
- * exit object (boat, door, teleporter, etc.). if null, then op->contr->maplevel contains that map to
- * move the object to.
+ * exit object (boat, door, teleporter, etc.)
  */
 void enter_exit(object *op, object *exit_ob) {
 #define PORTAL_DESTINATION_NAME "Town portal destination" /* this one should really be in a header file */
@@ -768,12 +720,16 @@ void enter_exit(object *op, object *exit_ob) {
     if (op->contr->transport)
         ob_apply(op->contr->transport, op, AP_UNAPPLY);
 
-    if (!exit_ob) {
-        enter_maplevel_exit(op);
-        return;
-    }
+    assert(exit_ob != NULL);
+    assert(EXIT_PATH(exit_ob) != NULL);
 
-    /* check to see if we make a template map */
+    /* check to see if we make a template map
+     * Template maps have an EXIT_PATH in the form:
+     * /@[!]<MAP_NAME>!<TEMPLATE_PATH>
+     *
+     * <MAP_NAME> is the name of the destination map, which is saved in LOCALDIR/template-maps/.
+     * <TEMPLATE_PATH> is the path to a template map, used when <MAP_NAME> does not exist.
+     */
     if (EXIT_PATH(exit_ob) && EXIT_PATH(exit_ob)[1] == '@') {
         if (EXIT_PATH(exit_ob)[2] == '!') {
             /* generate a template map randomly */
@@ -859,19 +815,19 @@ void enter_exit(object *op, object *exit_ob) {
             tmp = object_find_by_type_and_slaying(op, FORCE, PORTAL_DESTINATION_NAME);
             if (tmp) {
                 object_remove(tmp);
-                object_free2(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
+                object_free(tmp, FREE_OBJ_NO_DESTROY_CALLBACK);
             }
 
             path_combine_and_normalize(exit_ob->map->path, EXIT_PATH(exit_ob), op->contr->savebed_map, sizeof(op->contr->savebed_map));
             op->contr->bed_x = EXIT_X(exit_ob), op->contr->bed_y = EXIT_Y(exit_ob);
             save_player(op, 1);
-            /* LOG(llevDebug, "enter_exit: Taking damned exit %s to (%d,%d) on map %s\n",
-             *     exit_ob->name ? exit_ob->name : "(none)", exit_ob->x, exit_ob->y,
-             *     path_combine_and_normalize(exit_ob->map->path, EXIT_PATH(exit_ob))); */
         }
 
         enter_map(op, newmap, x, y);
     }
+
+    LOG(llevDebug, "%s enters %s\n", op->name, EXIT_PATH(exit_ob));
+
     /* For exits that cause damages (like pits) */
     if (exit_ob->stats.dam && op->type == PLAYER)
         hit_player(op, exit_ob->stats.dam, exit_ob, exit_ob->attacktype, 1);
@@ -915,6 +871,7 @@ static void process_players1(void) {
                         object_remove(pl->ob);
                         object_insert_in_map_at(pl->ob, followed->ob->map, NULL, 0, followed->ob->x+freearr_x[space], followed->ob->y+freearr_y[space]);
                         map_newmap_cmd(&pl->socket);
+                        player_update_bg_music(pl->ob);
                     }
                 } else {
                     draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, "Player %s left or ambiguous name.", pl->followed_player);
@@ -1014,6 +971,10 @@ static void process_players2(void) {
 
 #define SPEED_DEBUG
 
+static bool object_in_icecube(object *op) {
+    return op->env != NULL && strcmp(op->env->arch->name, "icecube") == 0;
+}
+
 /**
  * Process all active objects.
  */
@@ -1065,7 +1026,7 @@ void process_events(void) {
          * The real bug is to try to find out the cause of this - someone
          * is probably calling object_remove() without either an insert_ob or
          * object_free_drop_inventory() afterwards, leaving an object dangling.
-	 * But I'd rather log this and continue on instead of crashing.
+         * But I'd rather log this and continue on instead of crashing.
          * Don't remove players - when a player quits, the object is in
          * sort of a limbo, of removed, but something we want to keep
          * around.
@@ -1083,7 +1044,7 @@ void process_events(void) {
             diff = stringbuffer_finish(sb);
             LOG(llevError, "%s\n", diff);
             free(diff);
-            object_free2(op, FREE_OBJ_NO_DESTROY_CALLBACK);
+            object_free(op, FREE_OBJ_NO_DESTROY_CALLBACK);
             continue;
         }
 
@@ -1118,7 +1079,7 @@ void process_events(void) {
          * is based on ticks, and not the creatures speed?
          */
         if ((op->anim_speed && op->last_anim >= op->anim_speed)
-        || (op->temp_animation_id && op->last_anim >= op->temp_anim_speed)) {
+        || (op->temp_animation && op->last_anim >= op->temp_anim_speed)) {
             op->state++;
             if ((op->type == PLAYER) || (op->type == MONSTER))
                 animate_object(op, op->facing);
@@ -1130,15 +1091,26 @@ void process_events(void) {
         }
 
         if (op->speed_left > 0) {
-            --op->speed_left;
+            // Players are special because their speed_left has already been
+            // reduced in do_server(). Players effectively process every tick
+            // so long they have non-zero speed left.
+            if (op->type != PLAYER) {
+                // Objects in icecubes decay at a slower rate
+                if (object_in_icecube(op)) {
+                    op->speed_left -= 10;
+                } else {
+                    op->speed_left -= 1;
+                }
+            }
             process_object(op);
             if (object_was_destroyed(op, tag))
                 continue;
+        } else {
+            // Custom-made creatures for random maps can still have negative speeds, so catch that with FABS().
+            op->speed_left += FABS(op->speed);
         }
         if (settings.casting_time == TRUE && op->casting_time > 0)
             op->casting_time--;
-        if (op->speed_left <= 0)
-            op->speed_left += FABS(op->speed);
     }
 
     /* Remove marker object from active list */
@@ -1151,36 +1123,29 @@ void process_events(void) {
 }
 
 /**
- * Remove temporary map files.
- *
- * @todo check logic, why is file only removed if map is in memory?
+ * Save unique maps and clean up temporary map files unless recycling
+ * temporary maps. The function name is somewhat misleading.
  */
 void clean_tmp_files(void) {
     mapstruct *m, *next;
-
-    LOG(llevInfo, "Cleaning up...\n");
-
-    /* We save the maps - it may not be intuitive why, but if there are unique
-     * items, we need to save the map so they get saved off.  Perhaps we should
-     * just make a special function that only saves the unique items.
-     */
     for (m = first_map; m != NULL; m = next) {
         next = m->next;
         if (m->in_memory == MAP_IN_MEMORY) {
-            /* If we want to reuse the temp maps, swap it out (note that will also
-             * update the log file.  Otherwise, save the map (mostly for unique item
-             * stuff).  Note that the clean_tmp_map is called after the end of
-             * the for loop but is in the #else bracket.  IF we are recycling the maps,
-             * we certainly don't want the temp maps removed.
-             */
-
-            /* XXX The above comment is dead wrong */
-            if (settings.recycle_tmp_maps == TRUE)
+            // Save all maps currently in memory, because they might contain
+            // unique tiles that have not been written to disk.
+            if (settings.recycle_tmp_maps) {
+                // swap_map() also updates the write log.
                 swap_map(m);
-            else {
-                save_map(m, SAVE_MODE_NORMAL); /* note we save here into a overlay map */
+            } else {
+                save_map(m, SAVE_MODE_NORMAL);
+                // FIXME: Unfortunately, save_map() also unnecessarily saves
+                // non-unique tiles to a new temporary file, so we have to
+                // get rid of it here.
                 clean_tmp_map(m);
             }
+        } else {
+            // Remove the swap file.
+            clean_tmp_map(m);
         }
     }
     write_todclock(); /* lets just write the clock here */
@@ -1188,7 +1153,7 @@ void clean_tmp_files(void) {
 
 /** Clean up everything and exit. */
 void cleanup(void) {
-    LOG(llevDebug, "Cleanup called.  freeing data.\n");
+    LOG(llevInfo, "Cleaning up...\n");
     clean_tmp_files();
     write_book_archive();
     accounts_save();
@@ -1237,7 +1202,7 @@ void leave(player *pl, int draw_exit) {
     }
 
     pl->socket.status = Ns_Dead;
-    LOG(llevInfo, "LOGOUT: Player named %s from ip %s\n", pl->ob->name, pl->socket.host);
+    LOG(llevInfo, "logout: %s from %s\n", pl->ob->name, pl->socket.host);
 
     strcpy(pl->ob->contr->killer, "left");
     hiscore_check(pl->ob, 1);
@@ -1336,6 +1301,65 @@ int forbid_play(void) {
 #endif
 }
 
+static void do_shutdown(void) {
+    draw_ext_info(NDI_UNIQUE | NDI_ALL, 5, NULL, MSG_TYPE_ADMIN,
+            MSG_TYPE_ADMIN_DM, "The server has shut down.");
+
+    /* TODO: Kick everyone out and save player files? */
+    cleanup();
+}
+
+/**
+ * Check if we're ready to shut the server down.
+ */
+static bool check_shutdown(void) {
+    if (shutdown_flag == 1) {
+        LOG(llevInfo, "Received SIGINT; shutting down...\n");
+        return true;
+    }
+
+    /* Zero means that no timed shutdown is pending. */
+    if (cmd_shutdown_time == 0) {
+        return false;
+    }
+
+    /* If a timed shutdown is coming, remind players periodically. */
+    static int next_warn = 0; // FIXME: next_warn not reset if shutdown cancelled
+    time_t time_left = cmd_shutdown_time - time(NULL);
+
+    for (unsigned int i = next_warn; i < sizeof(shutdown_warn_times) / sizeof(int); i++) {
+        if (shutdown_warn_times[i] == (int)ceil(time_left / 60.0)) {
+            draw_ext_info_format(NDI_UNIQUE | NDI_ALL, 0, NULL, MSG_TYPE_ADMIN,
+                    MSG_TYPE_ADMIN_DM,
+                    "Server shutting down in %d minutes.", shutdown_warn_times[i]);
+            next_warn = i + 1;
+            return false;
+        }
+    }
+
+    if (time_left <= 0) {
+        return true;
+    }
+    return false;
+}
+
+/**
+ * Warn op if a server shutdown is scheduled. Used to inform players about
+ * scheduled shutdowns when they log in.
+ */
+void login_check_shutdown(object* const op) {
+    if (cmd_shutdown_time == 0) {
+        return;
+    }
+
+    time_t time_left = cmd_shutdown_time - time(NULL);
+    if (time_left <= 60*shutdown_warn_times[0]) {
+        draw_ext_info_format(
+            NDI_UNIQUE | NDI_ALL, 0, NULL, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM,
+            i18n(op, "Server shutting down in %d minutes."), time_left / 60);
+    }
+}
+
 extern unsigned long todtick;
 
 /**
@@ -1353,6 +1377,14 @@ extern unsigned long todtick;
  * doing the various things.
  */
 static void do_specials(void) {
+    if (check_shutdown()) {
+        do_shutdown();
+    }
+
+#ifdef CS_LOGSTATS
+    if ((time(NULL)-cst_lst.time_start) >= CS_LOGTIME)
+        write_cs_stats();
+#endif
 
     if (!(pticks%10))
         knowledge_process_incremental();
@@ -1397,10 +1429,8 @@ static void do_specials(void) {
  * length of argv.
  * @param argv
  * command-line options.
- * @return
- * 0.
  */
-int server_main(int argc, char **argv) {
+void server_main(int argc, char *argv[]) {
 #ifdef WIN32 /* ---win32 this sets the win32 from 0d0a to 0a handling */
     _fmode = _O_BINARY;
     bRunning = 1;
@@ -1409,10 +1439,10 @@ int server_main(int argc, char **argv) {
 #ifndef WIN32
     /* Here we check that we aren't root or suid */
     if (getuid() == 0 || geteuid() == 0) {
-        fputs("Don't run crossfire as root, it is unsupported.\n", stderr);
-        fputs("Instead run it as a normal unprivileged user.\n", stderr);
-        fputs("Aborting...\n", stderr);
-        return 1;
+        fprintf(stderr,
+                "Running crossfire-server as root is a bad idea; aborting!\n"
+                "Please run it again as a normal, unprivileged user.\n");
+        exit(EXIT_FAILURE);
     }
 #endif
 
@@ -1420,17 +1450,18 @@ int server_main(int argc, char **argv) {
     malloc_debug(DEBUG_MALLOC_LEVEL);
 #endif
 
-    settings.argc = argc;
-    settings.argv = argv;
     init(argc, argv);
     initPlugins();        /* GROS - Init the Plugins */
+    // Give feedback that loading is complete. This prevents confusion on when it is done loading.
+    LOG(llevInfo, "Initialization complete. Waiting for connections.\n");
 #ifdef WIN32
     while (bRunning) {
 #else
-    for (;;) {
+    while (TRUE) {
 #endif
         nroferrors = 0;
 
+        tick_game_time();
         do_server();
         process_events();         /* "do" something with objects with speed */
         cftimer_process_timers(); /* Process the crossfire Timers */
@@ -1438,12 +1469,9 @@ int server_main(int argc, char **argv) {
         execute_global_event(EVENT_CLOCK);
         check_active_maps(); /* Removes unused maps after a certain timeout */
         do_specials();       /* Routines called from time to time. */
-
-        sleep_delta();       /* Sleep proper amount of time before next tick */
+        update_players();
     }
-#if 0 /* unreachable */
-    emergency_save(0);
-    cleanup();
-    return 0;
-#endif
+
+    /* This is unreachable. */
+    abort();
 }
