@@ -1,29 +1,15 @@
 /*
- * static char *rcsid_skill_util_c =
- *   "$Id: skill_util.c 16506 2012-01-07 18:22:35Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
 
 /**
  * @file
@@ -48,14 +34,20 @@
 
 #define WANT_UNARMED_SKILLS
 
-#include <global.h>
-#include <object.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <living.h>     /* for defs of STR,CON,DEX,etc. -b.t.*/
-#include <spells.h>
-#include <skills.h>
+#include "global.h"
+
+#include <assert.h>
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
+#include <math.h>
+
+#include "living.h"
+#include "object.h"
+#include "shop.h"
+#include "skills.h"
+#include "spells.h"
+#include "sproto.h"
 
 static void attack_hth(object *pl, int dir, const char *string, object *skill);
 static void attack_melee_weapon(object *op, int dir, const char *string, object *skill);
@@ -63,76 +55,54 @@ static void attack_melee_weapon(object *op, int dir, const char *string, object
 /**
  * Will contain a number-name mapping for skills, initialized by init_skills().
  */
-const char *skill_names[NUM_SKILLS];
+const char *skill_names[MAX_SKILLS];
 /**
  * Will contain the face numbers for the skills, initialized by init_skill().
  */
-int skill_faces[NUM_SKILLS];
+const Face *skill_faces[MAX_SKILLS];
 
 /**
- * This just sets up the ::skill_names table above. The index into the array is set up by the
- * subtypes.
+ * This just sets up the ::skill_names table above. The index into the array
+ * is simply the order the skill is found.
  */
 void init_skills(void) {
     int i;
     archetype *at;
 
-    for (i = 0; i < NUM_SKILLS; i++) {
+    for (i = 0; i < MAX_SKILLS; i++) {
         skill_names[i] = NULL;
-        skill_faces[i] = -1;
+        skill_faces[i] = NULL;
     }
+    i = 0;
 
     for (at = first_archetype; at != NULL; at = at->next) {
         if (at->clone.type == SKILL) {
-            if (at->clone.subtype >= sizeof(skill_names)/sizeof(*skill_names)) {
-                LOG(llevError, "init_skills: invalid skill subtype %d for skill %s\n", at->clone.subtype, at->clone.skill);
-            } else if (skill_names[at->clone.subtype] != NULL) {
-                LOG(llevError, "init_skills: multiple skill using same subtype %d, %s, %s\n",
-                    at->clone.subtype, skill_names[at->clone.subtype], at->clone.skill);
-            } else {
-                skill_names[at->clone.subtype] = add_refcount(at->clone.skill);
-                if (at->clone.face != NULL)
-                    skill_faces[at->clone.subtype] = at->clone.face->number;
+            if (i == MAX_SKILLS) {
+                LOG(llevError, "init_skills: too many skills, increase MAX_SKILLS and rebuild server!");
+                fatal(SEE_LAST_ERROR);
             }
-        }
-    }
-
-    /* This isn't really an error if there is no skill subtype set, but
-     * checking for this may catch some user errors.
-     * On the other hand, it'll crash later on, which is not nice. Thus giving a dummy name.
-     */
-    for (i = 1; i < NUM_SKILLS; i++) {
-        if (!skill_names[i]) {
-            LOG(llevError, "init_skills: skill subtype %d doesn't have a name?\n", i);
-            skill_names[i] = add_string("dummy skill");
+            skill_names[i] = add_refcount(at->clone.skill);
+            if (at->clone.face != NULL)
+                skill_faces[i] = at->clone.face;
+            i++;
         }
     }
 }
 
 /**
- * This function goes through the player inventory and sets
- * up the last_skills[] array in the player object.
- * The last_skills[] is used to more quickly lookup skills -
- * mostly used for sending exp.
- *
- * @param op
- * player to link skills for. Must be a player.
+ * Return the code of the skill for a client, the index in the skill_names array.
+ * @param skill_name skill name.
+ * @return code, 0 if invalid and asserts.
  */
-void link_player_skills(object *op) {
-    FOR_INV_PREPARE(op, tmp) {
-        if (tmp->type == SKILL) {
-            /* This is really a warning, hence no else below */
-            if (op->contr->last_skill_ob[tmp->subtype] && op->contr->last_skill_ob[tmp->subtype] != tmp) {
-                LOG(llevError, "Multiple skills with the same subtype? %s, %s\n", op->contr->last_skill_ob[tmp->subtype]->skill, tmp->skill);
-            }
-            if (tmp->subtype >= NUM_SKILLS) {
-                LOG(llevError, "Invalid subtype number %d (range 0-%d)\n", tmp->subtype, NUM_SKILLS);
-            } else {
-                op->contr->last_skill_ob[tmp->subtype] = tmp;
-                op->contr->last_skill_exp[tmp->subtype] = -1;
-            }
-        }
-    } FOR_INV_FINISH();
+int get_skill_client_code(const char *skill_name)
+{
+    int index;
+    for (index = 0; index < MAX_SKILLS && skill_names[index] != NULL; index++)
+        if (strcmp(skill_names[index], skill_name) == 0)
+            return index;
+
+    assert("invalid skill!");
+    return 0;
 }
 
 /**
@@ -220,8 +190,8 @@ static object *adjust_skill_tool(object *who, object *skill, object *skill_tool)
  *
  */
 object *find_skill_by_name(object *who, const char *name) {
-    object *skill = NULL, *skills[NUM_SKILLS], *skill_tools[NUM_SKILLS];
-    const char *skill_names[NUM_SKILLS];
+    object *skill = NULL, *skills[MAX_SKILLS], *skill_tools[MAX_SKILLS];
+    const char *skill_names[MAX_SKILLS];
     char *ourname=NULL;
     int num_names, highest_level_skill=0, i;
 
@@ -242,6 +212,7 @@ object *find_skill_by_name(object *who, const char *name) {
         if ((skill_names[0] = strtok_r(ourname, ",", &lasts)) == NULL) {
             /* This should really never happen */
             LOG(llevError, "find_skill_by_name: strtok_r returned null, but strchr did not?\n");
+            free(ourname);
             return NULL;
         } else {
             skill_tools[0] = NULL;
@@ -269,7 +240,7 @@ object *find_skill_by_name(object *who, const char *name) {
         * in length than the passed string. Eg, if we have a skill called
         * 'hi', we don't want to match if the user passed 'high'
         */
-        if (tmp->type == SKILL || tmp->type == SKILL_TOOL) {
+        if (tmp->type == SKILL || (tmp->type == SKILL_TOOL && !QUERY_FLAG(tmp, FLAG_UNPAID))) {
             for (i = 0; i<num_names; i++) {
                 if (!strncasecmp(skill_names[i], tmp->skill, strlen(skill_names[i])) &&
                     strlen(tmp->skill) >= strlen(skill_names[i])) {
@@ -300,7 +271,7 @@ object *find_skill_by_name(object *who, const char *name) {
 }
 
 /**
- * This returns the skill pointer of the given name (the
+ * This returns the first skill pointer of the given subtype (the
  * one that accumulates exp, has the level, etc).
  *
  * It is presumed that the player will be needing to actually
@@ -308,7 +279,11 @@ object *find_skill_by_name(object *who, const char *name) {
  * to benefit from its bonuses.
  *
  * This code is basically the same as find_skill_by_name() above,
- * but instead of a skill name, we search by matching number.
+ * but instead of a skill name, we search by matching subtype.
+ *
+ * Warning: skill subtypes are not unique to skills, various skills (eg
+ * harvesting-like) will share the same subtype, so this function should
+ * only be used if the skill's subtype is known to be used only by one skill.
  *
  * @param who
  * player applying a skill.
@@ -320,8 +295,9 @@ object *find_skill_by_name(object *who, const char *name) {
 object *find_skill_by_number(object *who, int skillno) {
     object *skill = NULL, *skill_tool = NULL;
 
-    if (skillno < 1 || skillno >= NUM_SKILLS)
+    if (skillno <= 0 || skillno > MAX_SKILLS) {
         return NULL;
+    }
 
     FOR_INV_PREPARE(who, tmp) {
         if (tmp->type == SKILL && tmp->subtype == skillno)
@@ -377,9 +353,8 @@ int change_skill(object *who, object *new_skill, int flag) {
         return 1;
     }
 
-    if (!new_skill || who->chosen_skill)
-        if (who->chosen_skill)
-            apply_special(who, who->chosen_skill, AP_UNAPPLY|(flag&AP_NOPRINT));
+    if (who->chosen_skill)
+        apply_special(who, who->chosen_skill, AP_UNAPPLY|(flag&AP_NOPRINT));
 
     /* Only goal in this case was to unapply a skill */
     if (!new_skill)
@@ -449,8 +424,13 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
         object *tmp;
 
         tmp = object_find_by_type_and_skill(op, SKILL, skill->skill);
-        if (!tmp)
+        if (!tmp) {
             tmp = give_skill_by_name(op, skill->skill);
+            if (!tmp) {
+                LOG(llevError, "do_skill: asked for skill %s but couldn't find matching SKILL archetype.\n", skill->skill);
+                return 0;
+            }
+        }
         skill = tmp;
     }
 
@@ -580,7 +560,7 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
         break;
 
     case SK_BARGAINING:
-        success = describe_shop(op);
+        success = shop_describe(op);
         break;
 
     case SK_SORCERY:
@@ -663,8 +643,8 @@ int do_skill(object *op, object *part, object *skill, int dir, const char *strin
  * @return
  * experience for the skill use.
  */
-sint64 calc_skill_exp(const object *who, const object *op, const object *skill) {
-    sint64 op_exp;
+int64_t calc_skill_exp(const object *who, const object *op, const object *skill) {
+    int64_t op_exp;
     int op_lvl;
     float base, value, lvl_mult;
 
@@ -700,7 +680,7 @@ sint64 calc_skill_exp(const object *who, const object *op, const object *skill)
         op_lvl = 1;
 
     if (who->type != PLAYER) {            /* for monsters only */
-        return ((sint64)(op_exp*0.1)+1);  /* we add one to insure positive value is returned */
+        return ((int64_t)(op_exp*0.1)+1);  /* we add one to insure positive value is returned */
     }
 
     /* for players */
@@ -735,7 +715,7 @@ sint64 calc_skill_exp(const object *who, const object *op, const object *skill)
 #ifdef SKILL_UTIL_DEBUG
     LOG(llevDebug, "calc_skill_exp(): who: %s(lvl:%d)  op:%s(lvl:%d)\n", who->name, skill->level, op->name, op_lvl);
 #endif
-    return ((sint64)value);
+    return ((int64_t)value);
 }
 
 /**
@@ -810,7 +790,7 @@ int learn_skill(object *pl, object *scroll) {
  * value between 0 and 100.
  * @todo Probably belongs in some global utils-type file?
  */
-static int clipped_percent(sint64 a, sint64 b) {
+static int clipped_percent(int64_t a, int64_t b) {
     int rv;
 
     if (b <= 0)
@@ -826,6 +806,20 @@ static int clipped_percent(sint64 a, sint64 b) {
     return rv;
 }
 
+
+/**
+ * Gives the number of digits that %ld will print for a long int
+ *
+ * @param num
+ * any integer (negative values not supported)
+ * @return
+ * number of characters to expect
+ */
+static int digits_in_long(int64_t num)
+{
+    return num == 0 ? 1 : floor( log10( labs(num) ) ) + 1;
+}
+
 /**
  * Displays a player's skill list, and some other non skill related info (god,
  * max weapon improvements, item power).
@@ -839,35 +833,50 @@ static int clipped_percent(sint64 a, sint64 b) {
  *
  * @param op
  * player wanting to examine skills.
- * @param search
- * optional string to restrict skills to show.
+ * @param parms
+ * optional parameters, usually a string to restrict skills to show.
  */
-void show_skills(object *op, const char *search) {
+void show_skills(object *op, const char *parms) {
     const char *cp;
     int i, num_skills_found = 0;
-    static const char *const periods = "........................................";
     /* Need to have a pointer and use strdup for qsort to work properly */
-    char skills[NUM_SKILLS][MAX_BUF];
+    char skills[MAX_SKILLS][MAX_BUF];
+    const char *search = parms;
+    bool long_format = false;
 
+    if ( parms && strncmp(parms,"-l",2) == 0 ) {
+       long_format = true;
+       search += 2;
+       while ( *search && *search != ' ' ) ++search; /* move past other parameters */
+       while ( *search == ' ' ) ++search;
+    }
     FOR_INV_PREPARE(op, tmp) {
         if (tmp->type == SKILL) {
-            char buf[MAX_BUF];
-
             if (search && strstr(tmp->name, search) == NULL)
                 continue;
-            /* Basically want to fill this out to 40 spaces with periods */
-            snprintf(buf, sizeof(buf), "%s%s", tmp->name, periods);
-            buf[40] = 0;
 
             if (settings.permanent_exp_ratio) {
-                snprintf(skills[num_skills_found++], MAX_BUF, "%slvl:%3d (xp:%"FMT64"/%"FMT64"/%d%%)",
-                         buf, tmp->level,
-                         tmp->stats.exp,
-                         level_exp(tmp->level+1, op->expmul),
-                         clipped_percent(tmp->perm_exp, tmp->stats.exp));
+                if ( long_format ) {
+                    int perm_level = exp_level(tmp->total_exp * settings.permanent_exp_ratio / 100);
+                    snprintf(skills[num_skills_found++], MAX_BUF, "[fixed]%-20slvl:%3d (xp:%"FMT64"/%"FMT64")%-*sperm lvl:%3d (xp:%"FMT64"/%"FMT64") ",
+                             tmp->name, tmp->level,
+                             tmp->stats.exp,
+                             level_exp(tmp->level+1, op->expmul),
+                             1+2*digits_in_long(op->stats.exp)-digits_in_long(tmp->stats.exp)-digits_in_long(level_exp(tmp->level+1, op->expmul)),
+                             "",
+                             perm_level,
+                             tmp->total_exp * settings.permanent_exp_ratio / 100,
+                             level_exp(perm_level+1, op->expmul));
+                } else {
+                    snprintf(skills[num_skills_found++], MAX_BUF, "[fixed]%-20slvl:%3d (xp:%"FMT64"/%"FMT64"/%d%%)",
+                             tmp->name, tmp->level,
+                             tmp->stats.exp,
+                             level_exp(tmp->level+1, op->expmul),
+                             clipped_percent(PERM_EXP(tmp->total_exp), tmp->stats.exp));
+                }
             } else {
-                snprintf(skills[num_skills_found++], MAX_BUF, "%slvl:%3d (xp:%"FMT64"/%"FMT64")",
-                         buf, tmp->level,
+                snprintf(skills[num_skills_found++], MAX_BUF, "[fixed]%-20slvl:%3d (xp:%"FMT64"/%"FMT64")",
+                         tmp->name, tmp->level,
                          tmp->stats.exp,
                          level_exp(tmp->level+1, op->expmul));
             }
@@ -877,17 +886,22 @@ void show_skills(object *op, const char *search) {
              * it so it doesn't crash the server - otherwise, one character may
              * crash the server numerous times.
              */
-            if (num_skills_found >= NUM_SKILLS) {
+            if (num_skills_found >= MAX_SKILLS) {
                 draw_ext_info(NDI_RED, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                              "Your character has too many skills. "
+                              "Your character has too many skills.\n"
                               "Something isn't right - contact the server admin");
                 break;
             }
         }
     } FOR_INV_FINISH();
 
-    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                  "Player skills:");
+    if ( search && *search ) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
+                             "Player skills%s: (matching '%s')", long_format ? " (long format)":"",search);
+    } else {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
+                             "Player skills%s:", long_format ? " (long format)":"");
+    }
 
     if (num_skills_found > 1)
         qsort(skills, num_skills_found, MAX_BUF, (int (*)(const void *, const void *))strcmp);
@@ -897,19 +911,16 @@ void show_skills(object *op, const char *search) {
                       skills[i]);
     }
 
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                         "You can handle %d weapon improvements.",
-                         op->level/5+5);
-
     cp = determine_god(op);
     if (strcmp(cp, "none") == 0)
         cp = NULL;
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
-                         "You worship %s.",
-                         cp ? cp : "no god at current time");
 
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SKILL, MSG_TYPE_SKILL_LIST,
+                         "You can handle %d weapon improvements.\n"
+                         "You worship %s.\n"
                          "Your equipped item power is %d out of %d\n",
+                         op->level/5+5,
+                         cp ? cp : "no god at current time",
                          op->contr->item_power, op->level);
 }
 
@@ -944,6 +955,7 @@ int use_skill(object *op, const char *string) {
             skop = tmp;
             break;
         } else if (tmp->type == SKILL_TOOL
+        && !QUERY_FLAG(tmp, FLAG_UNPAID) /* Holy symbols could be used unpaid w/o this */
         && !strncasecmp(string, tmp->skill, MIN(strlen(string), strlen(tmp->skill)))) {
             skop = tmp;
             break;
@@ -1167,7 +1179,9 @@ static void do_skill_attack(object *tmp, object *op, const char *string, object
 
             /* Has ready weapon - make sure chosen_skill is set up properly */
             if (!op->chosen_skill || op->current_weapon->skill != op->chosen_skill->skill) {
-                change_skill(op, find_skill_by_name(op, op->current_weapon->skill), 1);
+                object *found_skill = find_skill_by_name(op, op->current_weapon->skill);
+                assert(found_skill != NULL);
+                change_skill(op, found_skill, 1);
             }
         }
     }
@@ -1183,32 +1197,40 @@ static void do_skill_attack(object *tmp, object *op, const char *string, object
 
     success = attack_ob(tmp, op);
 
-    /* print appropriate  messages to the player */
+    /*
+     * print appropriate  messages to the player
+     *
+     * If no string, then we aren't dealing with the
+     * skill attack directly, so the message is printed elsewhere.
+     *
+     * Only print hit/miss if we've got a Fire+Attack.
+     * Otherwise, don't print here at all.
+     */
 
-    if (success && string != NULL && tmp && !QUERY_FLAG(tmp, FLAG_FREED)) {
+    if (string != NULL && tmp && !QUERY_FLAG(tmp, FLAG_FREED)) {
         char op_name[MAX_BUF];
-
-        if (op->type == PLAYER) {
+        if (success){
+            if (op->type == PLAYER) {
+                query_name(tmp, op_name, MAX_BUF);
+                draw_ext_info_format(NDI_UNIQUE, 0, op,
+                                     MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
+                                     "You %s %s!",
+                                     string, op_name);
+            } else if (tmp->type == PLAYER) {
+                query_name(op, op_name, MAX_BUF);
+                draw_ext_info_format(NDI_UNIQUE, 0, tmp,
+                                     MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
+                                     "%s %s you!",
+                                     op_name, string);
+            }
+        }
+        else{
             query_name(tmp, op_name, MAX_BUF);
             draw_ext_info_format(NDI_UNIQUE, 0, op,
-                                 MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_DID_HIT,
-                                 "You %s %s!",
-                                 string, op_name);
-        } else if (tmp->type == PLAYER) {
-            query_name(op, op_name, MAX_BUF);
-            draw_ext_info_format(NDI_UNIQUE, 0, tmp,
-                                 MSG_TYPE_VICTIM, MSG_TYPE_VICTIM_WAS_HIT,
-                                 "%s %s you!",
-                                 op_name, string);
+                                 MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_MISS,
+                                 "You miss %s!",
+                                 op_name);
         }
-    } else if (tmp && !QUERY_FLAG(tmp, FLAG_FREED)) {
-        char op_name[MAX_BUF];
-
-        query_name(tmp, op_name, MAX_BUF);
-        draw_ext_info_format(NDI_UNIQUE, 0, op,
-                             MSG_TYPE_ATTACK, MSG_TYPE_ATTACK_MISS,
-                             "You miss %s!",
-                             op_name);
     }
 }
 
@@ -1236,7 +1258,7 @@ static void do_skill_attack(object *tmp, object *op, const char *string, object
  * attack skill.
  */
 void skill_attack(object *tmp, object *pl, int dir, const char *string, object *skill) {
-    sint16 tx, ty;
+    int16_t tx, ty;
     mapstruct *m;
     int mflags;
 
