@@ -1,44 +1,30 @@
 /*
- * static char *rcsid_skills_c =
- *   "$Id: skills.c 16383 2011-12-30 20:00:44Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2013 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, see the
+ * 'LICENSE' and 'COPYING' files.
+ *
+ * The authors can be reached via e-mail to crossfire-devel@real-time.com
+ *
+ * skills.c -- core skill handling
  */
-/*
-    CrossFire, A Multiplayer game for X-windows
+#define _GNU_SOURCE // strcasestr() is a GNU extension in string.h
 
-    Copyright (C) 2003-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
+#include "global.h"
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+#include <stdlib.h>
+#include <string.h>
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
-/**
- * @file
- * This file contains core skill handling.
- */
-
-#include <global.h>
-#include <object.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <living.h>
-#include <skills.h>
-#include <spells.h>
-#include <book.h>
+#include "book.h"
+#include "living.h"
+#include "object.h"
+#include "skills.h"
+#include "spells.h"
+#include "sproto.h"
 
 /**
  * Computes stealing chance.
@@ -290,7 +276,7 @@ static int attempt_steal(object *op, object *who, object *skill) {
  */
 int steal(object *op, int dir, object *skill) {
     object *tmp;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *m;
     int mflags;
 
@@ -373,11 +359,13 @@ static int attempt_pick_lock(object *door, object *pl, object *skill) {
      * the map level difficulty.
      */
     number = (die_roll(2, 40, pl, PREFER_LOW)-2)/2;
+    // Spring traps even if the lock is picked.
+    if (door->inv && (door->inv->type == RUNE || door->inv->type == TRAP)) {  /* set off any traps? */
+        spring_trap(door->inv, pl);
+    }
     if (number < pl->stats.Dex + skill->level*2 - difficulty ) {
         remove_door(door);
         success = difficulty;
-    } else if (door->inv && (door->inv->type == RUNE || door->inv->type == TRAP)) {  /* set off any traps? */
-        spring_trap(door->inv, pl);
     }
     return success;
 }
@@ -408,7 +396,7 @@ int pick_lock(object *pl, int dir, object *skill) {
         dir = pl->facing;
 
     /* For all the stacked objects at this point find a door*/
-    if (out_of_map(pl->map, x, y)) {
+    if (OUT_OF_REAL_MAP(pl->map, x, y)) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
                       "There is no lock there.");
         return 0;
@@ -423,6 +411,10 @@ int pick_lock(object *pl, int dir, object *skill) {
                       "There is no lock there.");
         return 0;
     }
+
+    if (execute_event(tmp, EVENT_TRIGGER, pl, skill, NULL, SCRIPT_FIX_ALL) != 0)
+        return 0;
+
     if (tmp->type == LOCKED_DOOR) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
                       "You can't pick that lock!");
@@ -542,6 +534,9 @@ int hide(object *op, object *skill) {
 static void stop_jump(object *pl) {
     fix_object(pl);
     object_insert_in_map_at(pl, pl->map, pl, 0, pl->x, pl->y);
+    // Some monsters can also jump. They do not check for pickup with check_pick().
+    if (pl->contr)
+        check_pick(pl);
 }
 
 /**
@@ -558,8 +553,21 @@ static void stop_jump(object *pl) {
  * 1 if jump was successful, 0 else.
  */
 static int attempt_jump(object *pl, int dir, int spaces, object *skill) {
+    /**
+     * Make sure we aren't in a transport.
+     * This should be set to null if we aren't.
+     */
+    if (pl->contr && pl->contr->transport){
+        char trans_name[MAX_BUF];
+        query_name(pl->contr->transport, trans_name, MAX_BUF);
+        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+            "Your bounce off the walls of %s.", trans_name);
+        // We failed to jump. Return as a failure.
+        return 0;
+    }
+
     int i, dx = freearr_x[dir], dy = freearr_y[dir], mflags;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *m;
 
     /* Jump loop. Go through spaces opject wants to jump. Halt the
@@ -793,12 +801,9 @@ static int do_skill_detect_magic(object *pl, object *skill) {
  */
 int identify_object_with_skill(object *tmp, object *pl, object *skill, int print_on_success) {
     int success = 0, chance, ip;
-    int skill_value = skill->level*pl->stats.Int ? pl->stats.Int : 10;
+    int skill_value = (skill->level && pl->stats.Int) ? pl->stats.Int : 10;
 
-    if (!QUERY_FLAG(tmp, FLAG_IDENTIFIED)
-    && !QUERY_FLAG(tmp, FLAG_NO_SKILL_IDENT)
-    && need_identify(tmp)
-    && !tmp->invisible) {
+    if (!tmp->invisible && !QUERY_FLAG(tmp, FLAG_NO_SKILL_IDENT) && !is_identified(tmp)) {
         ip = tmp->magic;
         if (tmp->item_power > ip)
             ip = tmp->item_power;
@@ -811,7 +816,7 @@ int identify_object_with_skill(object *tmp, object *pl, object *skill, int print
 
                 draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
                                      "You identify %s.",
-                                     ob_describe(tmp, pl, desc, sizeof(desc)));
+                                     ob_describe(tmp, pl, 1, desc, sizeof(desc)));
                 if (tmp->msg) {
                     draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_ITEM, MSG_TYPE_ITEM_INFO,
                                          "The item has a story:\n%s",
@@ -862,8 +867,8 @@ static int do_skill_ident(object *pl, int obj_class, object *skill) {
     }
 
     for (i = 0; i < area; i++) {
-        sint16 x = pl->x+freearr_x[i];
-        sint16 y = pl->y+freearr_y[i];
+        int16_t x = pl->x+freearr_x[i];
+        int16_t y = pl->y+freearr_y[i];
         mapstruct *m = pl->map;
         int mflags;
 
@@ -907,7 +912,7 @@ int skill_ident(object *pl, object *skill) {
         if (success)
             draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
                           "...and discover cursed items!");
-            break;
+        break;
 
     case SK_DET_MAGIC:
         success = do_skill_detect_magic(pl, skill);
@@ -961,7 +966,7 @@ int skill_ident(object *pl, object *skill) {
  * check if can't be simplified, code looks duplicated.
  */
 int use_oratory(object *pl, int dir, object *skill) {
-    sint16 x = pl->x+freearr_x[dir], y = pl->y+freearr_y[dir];
+    int16_t x = pl->x+freearr_x[dir], y = pl->y+freearr_y[dir];
     int mflags, chance;
     object *tmp;
     mapstruct *m;
@@ -1057,7 +1062,7 @@ int use_oratory(object *pl, int dir, object *skill) {
 
     /* Ok, got a 'sucker' lets try to make them a follower */
     if (chance > 0 && tmp->level < random_roll(0, chance-1, pl, PREFER_HIGH)-1) {
-        sint64 exp;
+        int64_t exp;
         query_name(tmp, name, MAX_BUF);
         draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
                              "You convince the %s to become your follower.",
@@ -1118,7 +1123,7 @@ int singing(object *pl, int dir, object *skill) {
     int i, exp = 0, chance, mflags;
     object *tmp;
     mapstruct *m;
-    sint16  x, y;
+    int16_t  x, y;
     char name[MAX_BUF];
     const char *value;
 
@@ -1209,7 +1214,7 @@ int singing(object *pl, int dir, object *skill) {
  */
 int find_traps(object *pl, object *skill) {
     int i, expsum = 0, mflags;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *m;
 
     /* First we search all around us for runes and traps, which are
@@ -1281,7 +1286,7 @@ int find_traps(object *pl, object *skill) {
 int remove_trap(object *op, object *skill) {
     int i, success = 0, mflags;
     mapstruct *m;
-    sint16 x, y;
+    int16_t x, y;
 
     for (i = 0; i < 9; i++) {
         x = op->x+freearr_x[i];
@@ -1444,38 +1449,42 @@ void meditate(object *pl, object *skill) {
 }
 
 /**
- * This routine allows players to inscribe messages in
- * ordinary 'books' (anything that is type BOOK). b.t.
+ * Called by write_on_item() to inscribe a message in an ordinary book.
  *
  * @param pl
- * player writing.
+ * Player attempting to write
  * @param item
- * book to write into.
+ * Book to write the message in
  * @param msg
- * message to write.
- * @param skill
- * writing skill object.
+ * Message to write
  * @return
- * experience gained for writing.
+ * Experience gained from using the skill
  * @todo assert() instead of simple check.
  */
-static int write_note(object *pl, object *item, const char *msg, object *skill) {
+static int write_note(object *pl, object *item, const char *msg) {
     char buf[BOOK_BUF];
     object *newBook = NULL;
 
-    /* a pair of sanity checks */
-    if (!item || item->type != BOOK)
-        return 0;
-
-    if (!msg) {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                             "No message to write!\nUsage: use_skill %s <message>",
-                             skill->skill);
+    // The item should exist and be a book, but check it just in case.
+    if (!item || item->type != BOOK) {
+        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
+                "That was interesting...");
+        // TODO: Print a scary log message.
         return 0;
     }
+
+    // The message should never be NULL, but check it just in case.
+    if (!msg) {
+        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
+                "Hmm... what was I going to write?");
+        // TODO: Print a scary log message.
+        return 0;
+    }
+
+    // Don't let the player write a reserved keyword.
     if (strcasestr_local(msg, "endmsg")) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "Trying to cheat now are we?");
+                "Trying to cheat now are we?");
         return 0;
     }
 
@@ -1484,12 +1493,19 @@ static int write_note(object *pl, object *item, const char *msg, object *skill)
         return strlen(msg);
 
     buf[0] = 0;
-    if (!book_overflow(item->msg, msg, BOOK_BUF)) { /* add msg string to book */
-        if (item->msg)
-            snprintf(buf, sizeof(buf), "%s%s\n", item->msg, msg);
-        else
-            snprintf(buf, sizeof(buf), "%s\n", msg);
 
+    // Write the message in the book if it doesn't overflow the buffer.
+    if (!book_overflow(item->msg, msg, BOOK_BUF)) {
+        // TODO: Garble some characters depending on intelligence/skill.
+
+        // If there was already text, append the new message on the end.
+        if (item->msg) {
+            snprintf(buf, sizeof(buf), "%s%s\n", item->msg, msg);
+        } else {
+            snprintf(buf, sizeof(buf), "%s\n", msg);
+        }
+
+        // If there were multiple items in a stack, unstack one and write.
         if (item->nrof > 1) {
             newBook = object_new();
             object_copy(item, newBook);
@@ -1499,30 +1515,32 @@ static int write_note(object *pl, object *item, const char *msg, object *skill)
             newBook = object_insert_in_ob(newBook, pl);
         } else {
             object_set_msg(item, buf);
-            /* This shouldn't be necessary - the object hasn't changed in any
-             * visible way
-             */
-            /*     esrv_send_item(pl, item);*/
-        }
-        query_short_name(item, buf, BOOK_BUF);
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
-                             "You write in the %s.",
-                             buf);
-        return strlen(msg);
-    }
 
-    query_short_name(item, buf, BOOK_BUF);
-    draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                         "Your message won't fit in the %s!",
-                         buf);
-    return 0;
+            // This shouldn't be necessary; the object hasn't changed visibly.
+            //esrv_send_item(pl, item);
+        }
+
+        // Tell the player that he/she wrote in the object.
+        query_short_name(item, buf, BOOK_BUF);
+        draw_ext_info_format(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
+                "You write in the %s.", buf);
+
+        // Give the player experience for writing.
+        return strlen(msg);
+    } else {
+        query_short_name(item, buf, BOOK_BUF);
+        draw_ext_info_format(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "Your message won't fit in the %s!", buf);
+        return 0;
+    }
 }
 
 /**
- * This routine allows players to inscribe spell scrolls
- * of spells which they know. Backfire effects are possible with the
- * severity of the backlash correlated with the difficulty of the scroll
- * that is attempted. -b.t. thomas@astro.psu.edu
+ * Called by write_on_item() to inscribe spell scrolls that the player can
+ * cast. Backfire effects are possible, varying in severity depending on the
+ * difficulty of the spell being attempted.
  *
  * @param pl
  * player writing a scroll.
@@ -1537,71 +1555,80 @@ static int write_scroll(object *pl, object *scroll, object *skill) {
     int success = 0, confused = 0, grace_cost = 0;
     object *newscroll, *chosen_spell, *tmp;
 
-    /* this is a sanity check */
-    if (scroll->type != SCROLL) {
+    // The item should exist and be a scroll, but check it just in case.
+    if (!scroll || scroll->type != SCROLL) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "A spell can only be inscribed into a scroll!");
+                "A spell must be written on a magic scroll!");
+        // TODO: Print a scary log message.
         return 0;
     }
 
-    /* Check if we are ready to attempt inscription */
+    // The player must have a spell readied to inscribe.
     chosen_spell = pl->contr->ranges[range_magic];
     if (!chosen_spell) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "You need a spell readied in order to inscribe!");
+                "You should ready the spell you wish to inscribe.");
         return 0;
     }
-    /* grace can become negative, we don't want a sp spell to block writing. */
+
+    // Make sure the player has enough SP or grace to write the spell.
     grace_cost = SP_level_spellpoint_cost(pl, chosen_spell, SPELL_GRACE);
     if (grace_cost > 0 && grace_cost > pl->stats.grace) {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "You don't have enough grace to write a scroll of %s.",
-                             chosen_spell->name);
+        draw_ext_info_format(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "You don't have enough grace to write a scroll of %s.",
+                chosen_spell->name);
         return 0;
     }
+
     if (SP_level_spellpoint_cost(pl, chosen_spell, SPELL_MANA) > pl->stats.sp) {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "You don't have enough mana to write a scroll of %s.",
-                             chosen_spell->name);
+        draw_ext_info_format(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "You don't have enough mana to write a scroll of %s.",
+                chosen_spell->name);
         return 0;
     }
-    /* Prevent an abuse: write a spell you're denied with, then cast it from the
-     * written scroll - gros, 28th July 2006
-     */
-    if (chosen_spell->path_attuned&pl->path_denied && settings.allow_denied_spells_writing == 0) {
+
+    // Prevent players from writing spells that they are denied.
+    if (chosen_spell->path_attuned & pl->path_denied
+            && settings.allow_denied_spells_writing == 0) {
         char name[MAX_BUF];
 
         query_name(chosen_spell, name, MAX_BUF);
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                             "The simple idea of writing a scroll of %s makes you sick !",
-                             name);
+        draw_ext_info_format(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "Just the idea of writing a scroll of %s makes you sick!",
+                name);
         return 0;
     }
 
-    /* if there is a spell already on the scroll then player could easily
-     * accidently read it while trying to write the new one.  give player
-     * a 50% chance to overwrite spell at their own level
-     */
-    if ((scroll->stats.sp || scroll->inv)
-    && random_roll(0, scroll->level*2, pl, PREFER_LOW) > skill->level) {
+    // If the scroll already had a spell written on it, the player could
+    // accidentally read it while trying to write a new one. Give the player
+    // a 50% chance to overwrite a spell at their own level.
+    if ((scroll->stats.sp || scroll->inv) &&
+            random_roll(0, scroll->level*2, pl, PREFER_LOW) > skill->level) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                      "Oops! You accidently read it while trying to write on it.");
+                "Oops! You accidently read it while trying to write on it.");
         apply_manual(pl, scroll, 0);
         return 0;
     }
 
-    if (execute_event(scroll, EVENT_TRIGGER, pl, chosen_spell, NULL, 0) != 0)
+    if (execute_event(scroll, EVENT_TRIGGER, pl, chosen_spell, NULL, 0) != 0) {
         return 0;
+    }
 
-    /* ok, we are ready to try inscription */
-    if (QUERY_FLAG(pl, FLAG_CONFUSED))
+    // Find out if the player is confused or not.
+    if (QUERY_FLAG(pl, FLAG_CONFUSED)) {
         confused = 1;
+    }
 
-    /* Lost mana/grace no matter what */
+    // Mana or grace is lost no matter if the inscription is successful.
     pl->stats.grace -= SP_level_spellpoint_cost(pl, chosen_spell, SPELL_GRACE);
     pl->stats.sp -= SP_level_spellpoint_cost(pl, chosen_spell, SPELL_MANA);
 
-    if (random_roll(0, chosen_spell->level*4-1, pl, PREFER_LOW) < skill->level) {
+    if (random_roll(0, chosen_spell->level * 4 - 1, pl, PREFER_LOW) <
+            skill->level) {
+        // If there were multiple items in a stack, unstack one.
         if (scroll->nrof > 1) {
             newscroll = object_new();
             object_copy(scroll, newscroll);
@@ -1611,86 +1638,102 @@ static int write_scroll(object *pl, object *scroll, object *skill) {
             newscroll = scroll;
         }
 
+        // Write spell if not confused; otherwise write random spell.
+        newscroll->level = MAX(skill->level, chosen_spell->level);
+
         if (!confused) {
-            newscroll->level = MAX(skill->level, chosen_spell->level);
-            draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
-                          "You succeed in writing a new scroll.");
+            draw_ext_info(
+                    NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_SUCCESS,
+                    "You succeed in writing a new scroll.");
         } else {
             chosen_spell = find_random_spell_in_ob(pl, NULL);
-            if (!chosen_spell)
+            if (!chosen_spell) {
                 return 0;
+            }
 
-            newscroll->level = MAX(skill->level, chosen_spell->level);
-            draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                          "In your confused state, you write down some odd spell.");
+            draw_ext_info(
+                    NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                    "In your confused state, you write down some odd spell.");
         }
 
         if (newscroll->inv) {
             object *ninv;
-
             ninv = newscroll->inv;
             object_remove(ninv);
-            object_free2(ninv, FREE_OBJ_NO_DESTROY_CALLBACK);
+            object_free(ninv, FREE_OBJ_NO_DESTROY_CALLBACK);
         }
+
         tmp = object_new();
         object_copy(chosen_spell, tmp);
         object_insert_in_ob(tmp, newscroll);
-        /* This is needed so casting from the scroll correctly works with moving_ball types, which
-           check attunements. */
+
+        // This is needed so casting from the scroll works correctly with
+        // moving_ball types, which checks attunements.
         newscroll->path_attuned = tmp->path_repelled;
 
-        /* Same code as from treasure.c - so they can better merge.
-         * if players want to sell them, so be it.
-         */
-        newscroll->value =  newscroll->arch->clone.value*newscroll->inv->value*(newscroll->level+50)/(newscroll->inv->level+50);
-        newscroll->stats.exp = newscroll->value/5;
+        // Same code as from treasure.c - so they can better merge.
+        // If players want to sell them, so be it.
+        newscroll->value = newscroll->arch->clone.value *
+            newscroll->inv->value * (newscroll->level + 50 ) /
+            (newscroll->inv->level + 50);
+        newscroll->stats.exp = newscroll->value / 5;
 
-        /* wait until finished manipulating the scroll before inserting it */
+        // Finish manipulating the scroll before inserting it.
         if (newscroll == scroll) {
-            /* Remove to correctly merge with other items which may exist in inventory */
+            // Remove object to stack correctly with other items.
             object_remove(newscroll);
         }
+
         newscroll = object_insert_in_ob(newscroll, pl);
         success = calc_skill_exp(pl, newscroll, skill);
-        if (!confused)
+
+        if (!confused) {
             success *= 2;
-        success = success*skill->level;
+        }
+
+        success = success * skill->level;
         return success;
     }
 
-    /* Inscription has failed */
-    if (chosen_spell->level > skill->level || confused) { /*backfire!*/
-        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                      "Ouch! Your attempt to write a new scroll strains your mind!");
-        if (random_roll(0, 1, pl, PREFER_LOW) == 1)
+    // Inscription wasn't successful; do something bad to the player.
+    if (chosen_spell->level > skill->level || confused) {
+        draw_ext_info(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "Ouch! Your attempt to write a new scroll strains your mind!");
+
+        // Either drain a stat or subtract experience.
+        if (random_roll(0, 1, pl, PREFER_LOW) == 1) {
             drain_specific_stat(pl, 4);
-        else {
+        } else {
             confuse_living(pl, pl, 99);
-            return (-30*chosen_spell->level);
+            return -30 * chosen_spell->level;
         }
     } else if (random_roll(0, pl->stats.Int-1, pl, PREFER_HIGH) < 15) {
-        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                      "Your attempt to write a new scroll rattles your mind!");
+        draw_ext_info(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "Your attempt to write a new scroll rattles your mind!");
         confuse_living(pl, pl, 99);
-    } else
-        draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
-                      "You fail to write a new scroll.");
+    } else {
+        draw_ext_info(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE,
+                "You fail to write a new scroll.");
+    }
+
     return 0;
 }
 
 /**
- * Writing skill handling.
- *
- * Wrapper for write_note() and write_scroll().
+ * Implement the 'inscription' skill, which checks for the required skills and
+ * marked items before running either write_note() or write_scroll().
  *
  * @param pl
- * player writing.
+ * Player attempting to write
  * @param params
- * message to inscribe.
+ * Message to write, blank to write a spell
  * @param skill
- * writing skill.
+ * Writing skill
  * @return
- * experience gained for using the skill.
+ * Experience gained from using the skill
  */
 int write_on_item(object *pl, const char *params, object *skill) {
     object *item;
@@ -1698,66 +1741,75 @@ int write_on_item(object *pl, const char *params, object *skill) {
     int msgtype;
     archetype *skat;
 
-    if (pl->type != PLAYER)
+    // Only players can use the inscription skill.
+    if (pl->type != PLAYER) {
         return 0;
+    }
 
+    // No message was given, so both strings are set to empty.
     if (!params) {
         params = "";
         string = params;
     }
+
+    // You must be able to read before you can write.
     skat = get_archetype_by_type_subtype(SKILL, SK_LITERACY);
 
-    /* Need to be able to read before we can write! */
     if (!find_skill_by_name(pl, skat->clone.skill)) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_MISSING,
-                      "You must learn to read before you can write!");
+                "You must learn to read before you can write!");
         return 0;
     }
 
+    // You must not be blind to write, unless you're a DM.
     if (QUERY_FLAG(pl, FLAG_BLIND) && !QUERY_FLAG(pl, FLAG_WIZ)) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-            "You are unable to write while blind.");
+                "You are unable to write while blind.");
         return 0;
     }
 
+    // If a message was given, write a book. Otherwise, write a scroll.
+    if (string[0] != '\0') {
+        msgtype = BOOK;
+    } else {
+        msgtype = SCROLL;
+    }
 
-    /* if there is a message then it goes in a book and no message means
-     * write active spell into the scroll
-     */
-    msgtype = (string[0] != '\0') ? BOOK : SCROLL;
-
-    /* find an item of correct type to write on */
+    // Find and attempt to write on the player's marked item.
     item = find_marked_object(pl);
     if (item == NULL) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "You don't have any marked item to write on.");
+                "You haven't marked any items to write on yet.");
         return 0;
     }
 
+    // Don't let the player write on an unpaid item.
     if (QUERY_FLAG(item, FLAG_UNPAID)) {
         draw_ext_info(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                      "You had better pay for that before you write on it.");
+                "You had better pay for that before you write on it!");
         return 0;
     }
+
+    // Check if the marked item is the type of item we're writing.
     if (msgtype != item->type) {
-        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
-                             "You have no %s to write %s",
-                             msgtype == BOOK ? "book" : "scroll",
-                             msgtype == BOOK ? "on" : "your spell down");
+        draw_ext_info_format(
+                NDI_UNIQUE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_ERROR,
+                "You must mark a %s to write %s.",
+                msgtype == BOOK ? "book" : "magic scroll",
+                msgtype == BOOK ? "your message on" : "your spell down");
         return 0;
     }
 
-    if (msgtype == SCROLL) {
-        return write_scroll(pl, item, skill);
-    }
     if (msgtype == BOOK) {
-        return write_note(pl, item, string, skill);
+        return write_note(pl, item, string);
+    } else if (msgtype == SCROLL) {
+        return write_scroll(pl, item, skill);
+    } else {
+        // This case should never be reached.
+        abort();
     }
-    return 0;
 }
 
-
-
 /**
  * Find an object to throw.
  * If there is a suitable marked item, use it.
@@ -1914,7 +1966,7 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
     int pause_f, weight_f = 0, mflags;
     float str_factor = 1.0, load_factor = 1.0, item_factor = 1.0;
     mapstruct *m;
-    sint16  sx, sy;
+    int16_t  sx, sy;
     tag_t tag;
     char name[MAX_BUF];
 
@@ -1952,12 +2004,26 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
     }
 
     /* the more we carry, the less we can throw. Limit only on players */
-    /* This logic is basically grabbed right out of fix_object() */
     if (op->type == PLAYER
     && op->carrying > (get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT)
     && (FREE_PLAYER_LOAD_PERCENT < 1.0)) {
-        int extra_weight = op->carrying-get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT;
-        load_factor = (float)extra_weight/(float)(get_weight_limit(op->stats.Str)*(1.0-FREE_PLAYER_LOAD_PERCENT));
+        /**
+         * Fixed logic here
+         *
+         * The old code made you have a better factor the more encumbered past the LOAD_PCT
+         * you were. This new way essentially flips the variable to be higher at lower encumbrance.
+         *
+         * It also only encumbers throwing once carryng more than FREE_PLAYER_LOAD_PERCENT of our capacity,
+         * since we have a MIN on load_factor when we use it, so before the limit it is clipped to 1.0.
+         *
+         * Also ,use 2.0f since op->carrying/(get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT) = 1 if
+         * they are equal. As op->carrying increases, this slides higher, making the factor lower.
+         *
+         * SilverNexus 2019-01-03
+         */
+        load_factor = 2.0f - (float)(op->carrying) / (float)(get_weight_limit(op->stats.Str)*FREE_PLAYER_LOAD_PERCENT);
+        // Only clip to 1.0 if we get in here, since it is 1.0 if we do not get in here.
+        load_factor = MIN(load_factor, 1.0f);
     }
 
     /* lighter items are thrown harder, farther, faster */
@@ -1971,7 +2037,7 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
         return 0;
     }
 
-    eff_str = str*MIN(load_factor, 1.0);
+    eff_str = str*load_factor;
     eff_str = (float)eff_str*item_factor*str_factor;
 
     /* alas, arrays limit us to a value of settings.max_stat (30). Use str_factor to
@@ -2143,7 +2209,7 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
      * In short summary, a throw can take anywhere between speed 5 and
      * speed 0.5
      */
-    op->speed_left -= 50/pause_f;
+    op->speed_left -= 50.0/pause_f;
 
     object_update_speed(throw_ob);
     throw_ob->speed_left = 0;
