@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_spell_attack_c =
- *   "$Id: spell_attack.c 17054 2012-02-11 16:43:03Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002-2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * This file contains all the spell attack code.  Grouping this code
@@ -36,19 +21,20 @@
  * put parameters in the same order, use same name.
  */
 
-#include <global.h>
-#include <object.h>
-#include <living.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <spells.h>
-#include <sounds.h>
+#include "global.h"
+
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "object.h"
+#include "living.h"
+#include "sproto.h"
+#include "spells.h"
+#include "sounds.h"
 
 /***************************************************************************
- *
  * BOLT CODE
- *
  ***************************************************************************/
 
 /**
@@ -203,6 +189,8 @@ void explode_bullet(object *op) {
      * the count of the parent should work fine.
      */
     tmp->stats.maxhp = op->count;
+    if (tmp->stats.maxhp == 0)
+        tmp->stats.maxhp = 1;
 
     /* Set direction of cone explosion */
     if (tmp->type == SPELL_EFFECT && tmp->subtype == SP_CONE)
@@ -230,7 +218,7 @@ void check_bullet(object *op) {
     tag_t op_tag = op->count, tmp_tag;
     int dam, mflags;
     mapstruct *m;
-    sint16 sx, sy;
+    int16_t sx, sy;
 
     mflags = get_map_flags(op->map, &m, op->x, op->y, &sx, &sy);
 
@@ -310,7 +298,7 @@ int cast_cone(object *op, object *caster, int dir, object *spell) {
     object *tmp;
     int i, success = 0, range_min = -1, range_max = 1;
     mapstruct *m;
-    sint16 sx, sy;
+    int16_t sx, sy;
     MoveType movetype;
 
     if (!spell->other_arch)
@@ -333,7 +321,7 @@ int cast_cone(object *op, object *caster, int dir, object *spell) {
     movetype = spell->other_arch->clone.move_type;
 
     for (i = range_min; i <= range_max; i++) {
-        sint16 x, y, d;
+        int16_t x, y, d;
 
         /* We can't use absdir here, because it never returns
          * 0.  If this is a rune, we want to hit the person on top
@@ -418,12 +406,15 @@ int cast_cone(object *op, object *caster, int dir, object *spell) {
         if (!tmp->move_on && tmp->stats.dam) {
             LOG(llevDebug, "cast_cone(): arch %s doesn't have move_on set\n", spell->other_arch->name);
         }
-        object_insert_in_map_at(tmp, m, op, 0, sx, sy);
 
         /* This is used for tracking spells so that one effect doesn't hit
          * a single space too many times.
          */
         tmp->stats.maxhp = tmp->count;
+        if (tmp->stats.maxhp == 0)
+            tmp->stats.maxhp = 1;
+
+        object_insert_in_map_at(tmp, m, op, 0, sx, sy);
 
         if (tmp->other_arch)
             cone_drop(tmp);
@@ -456,7 +447,7 @@ int cast_cone(object *op, object *caster, int dir, object *spell) {
 int create_bomb(object *op, object *caster, int dir, object *spell) {
     object *tmp;
     int mflags;
-    sint16 dx = op->x+freearr_x[dir], dy = op->y+freearr_y[dir];
+    int16_t dx = op->x+freearr_x[dir], dy = op->y+freearr_y[dir];
     mapstruct *m;
 
     mflags = get_map_flags(op->map, &m, dx, dy, &dx, &dy);
@@ -504,7 +495,7 @@ int create_bomb(object *op, object *caster, int dir, object *spell) {
  */
 static object *get_pointed_target(object *op, int dir, int range, int type) {
     object *target;
-    sint16 x, y;
+    int16_t x, y;
     int dist, mflags;
     mapstruct *mp;
 
@@ -605,7 +596,7 @@ int cast_smite_spell(object *op, object *caster, int dir, object *spell) {
         effect->level = spell->stats.dam+SP_level_dam_adjust(caster, spell);
 
         /* casting death spells at undead isn't a good thing */
-        if QUERY_FLAG(target, FLAG_UNDEAD) {
+        if (QUERY_FLAG(target, FLAG_UNDEAD)) {
             if (random_roll(0, 2, op, PREFER_LOW)) {
                 draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_FAILURE, "Idiot! Your spell boomerangs!");
                 effect->x = op->x;
@@ -636,6 +627,8 @@ int cast_smite_spell(object *op, object *caster, int dir, object *spell) {
          * spells.
          */
         effect->stats.maxhp = spell->count;
+        if (effect->stats.maxhp == 0)
+            effect->stats.maxhp = 1;
     }
 
     object_set_owner(effect, op);
@@ -713,9 +706,9 @@ static int make_object_glow(object *op, int radius, int time) {
  */
 int cast_destruction(object *op, object *caster, object *spell_ob) {
     int i, j, range, mflags, friendly = 0, dam, dur;
-    sint16 sx, sy;
+    int16_t sx, sy;
     mapstruct *m;
-    object *tmp;
+    object *tmp, *found_skill;
     const char *skill;
 
     range = spell_ob->range+SP_level_range_adjust(caster, spell_ob);
@@ -738,7 +731,9 @@ int cast_destruction(object *op, object *caster, object *spell_ob) {
     else
         op->skill = NULL;
 
-    change_skill(op, find_skill_by_name(op, op->skill), 1);
+    /* The skill may be NULL (if op is not a player but eg a bulletwall), but that's ok because change_skill does what is right. */
+    found_skill = find_skill_by_name(op, op->skill);
+    change_skill(op, found_skill, 1);
 
     for (i = -range; i < range; i++) {
         for (j = -range; j < range; j++) {
@@ -879,6 +874,11 @@ int cast_curse(object *op, object *caster, object *spell_ob, int dir) {
     change_abil(tmp, force);     /* Mostly to display any messages */
     object_insert_in_ob(force, tmp);
     fix_object(tmp);
+
+    if (spell_ob->other_arch != NULL && tmp->map != NULL) {
+        object_insert_in_map_at(arch_to_object(spell_ob->other_arch), tmp->map, NULL, INS_ON_TOP, tmp->x, tmp->y);
+    }
+
     return 1;
 }
 
@@ -905,7 +905,7 @@ int mood_change(object *op, object *caster, object *spell) {
     object *tmp, *head;
     const object *god;
     int done_one, range, mflags, level, at, best_at, immunity_chance = 50;
-    sint16 x, y, nx, ny;
+    int16_t x, y, nx, ny;
     mapstruct *m;
     const char *race;
 
@@ -1018,10 +1018,17 @@ int mood_change(object *op, object *caster, object *spell) {
             /* aggravation */
             if (QUERY_FLAG(spell, FLAG_MONSTER)) {
                 CLEAR_FLAG(head, FLAG_SLEEP);
+                // Make sure unaggressive monsters also go to attack
+                CLEAR_FLAG(head, FLAG_UNAGGRESSIVE);
                 if (QUERY_FLAG(head, FLAG_FRIENDLY))
                     remove_friendly_object(head);
 
                 done_one = 1;
+                // Prevent three-player abuse w/ two casting aggravation at a distance and the last punishing the
+                // monsters who can't decide who to attack.
+                // Also prevents aggravation -> singing -> aggravation loops
+                object_set_value(head, "no_mood_change", "1", 1);
+                // They big mad
                 object_set_enemy(head, op);
             }
 
@@ -1131,7 +1138,7 @@ int fire_swarm(object *op, object *caster, object *spell, int dir) {
  */
 int cast_light(object *op, object *caster, object *spell, int dir) {
     object *target = NULL, *tmp = NULL;
-    sint16 x, y;
+    int16_t x, y;
     int dam, mflags;
     mapstruct *m;
 
@@ -1201,7 +1208,7 @@ int cast_light(object *op, object *caster, object *spell, int dir) {
  * at least one living was affected.
  */
 int cast_cause_disease(object *op, object *caster, object *spell, int dir) {
-    sint16 x, y;
+    int16_t x, y;
     int i, mflags, range, dam_mod, dur_mod;
     object *target_head;
     mapstruct *m;
