@@ -1,48 +1,34 @@
 /*
- * static char *rcsid_spell_util_c =
- *   "$Id: spell_util.c 16793 2012-01-22 15:44:23Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Spell-related helper functions.
  */
 
-#include <global.h>
-#include <spells.h>
-#include <object.h>
-#include <errno.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <sounds.h>
+#include "global.h"
+
 #include <assert.h>
+#include <ctype.h>
+#include <errno.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "living.h"
+#include "object.h"
+#include "sounds.h"
+#include "spells.h"
+#include "sproto.h"
 
 extern const char *const spell_mapping[];
 
@@ -130,10 +116,10 @@ void check_spells(void) {
     for (at = first_archetype; at; at = at->next) {
         if (at->clone.type == SPELL) {
             if (at->clone.skill) {
-                for (i = 1; i < NUM_SKILLS; i++)
+                for (i = 0; i < MAX_SKILLS && skill_names[i]; i++)
                     if (!strcmp(skill_names[i], at->clone.skill))
                         break;
-                if (i == NUM_SKILLS) {
+                if (i == MAX_SKILLS || !skill_names[i]) {
                     LOG(llevError, "Spell %s has improper associated skill %s\n", at->name, at->clone.skill);
                 }
             }
@@ -249,14 +235,12 @@ int caster_level(const object *caster, const object *spell) {
 
     /* If this is a player, try to find the matching skill */
     if (caster->type == PLAYER && spell->skill) {
-        int i;
-
-        for (i = 0; i < NUM_SKILLS; i++)
-            if (caster->contr->last_skill_ob[i]
-            && caster->contr->last_skill_ob[i]->skill == spell->skill) {
-                level = caster->contr->last_skill_ob[i]->level;
-                break;
-            }
+        object* skill = find_applied_skill_by_name(caster, spell->skill);
+        if (skill != NULL) {
+            level = skill->level;
+        } else {
+            level = 0;
+        }
     }
     /* Got valid caster level.  Now adjust for attunement */
     level += ((caster->path_repelled&spell->path_attuned) ? -2 : 0)
@@ -288,7 +272,7 @@ int caster_level(const object *caster, const object *spell) {
  * @return
  * sp/mana points cost.
  */
-sint16 SP_level_spellpoint_cost(object *caster, object *spell, int flags) {
+int16_t SP_level_spellpoint_cost(object *caster, object *spell, int flags) {
     int sp, grace, level = caster_level(caster, spell);
 
     if (settings.spellpoint_level_depend == TRUE) {
@@ -535,7 +519,6 @@ int reflwall(mapstruct *m, int x, int y, object *sp_op) {
 
 /**
  * Creates object new_op in direction dir or if that is blocked, beneath the player (op).
- * we pass 'caster', but don't use it for anything.
  * This is really just a simple wrapper function .
  *
  * @param op
@@ -543,13 +526,13 @@ int reflwall(mapstruct *m, int x, int y, object *sp_op) {
  * @param new_op
  * object to insert.
  * @param dir
- * direction to insert into. Can be 0.
+ * direction to insert into. Can be 0 to insert below.
  * @return
  * direction that the object was actually placed in.
  */
 int cast_create_obj(object *op, object *new_op, int dir) {
     mapstruct *m;
-    sint16  sx, sy;
+    int16_t  sx, sy;
 
     if (dir
     && ((get_map_flags(op->map, &m, op->x+freearr_x[dir], op->y+freearr_y[dir], &sx, &sy)&P_OUT_OF_MAP)
@@ -583,7 +566,7 @@ int cast_create_obj(object *op, object *new_op, int dir) {
  * @return
  * 1 if we can add op, 0 else.
  */
-int ok_to_put_more(mapstruct *m, sint16 x, sint16 y, object *op, uint32 immune_stop) {
+int ok_to_put_more(mapstruct *m, int16_t x, int16_t y, object *op, uint32_t immune_stop) {
     int mflags;
     mapstruct *mp;
 
@@ -645,9 +628,11 @@ int ok_to_put_more(mapstruct *m, sint16 x, sint16 y, object *op, uint32 immune_s
                 int i;
 
                 for (i = 0; i < SPELL_TAG_SIZE; i++) {
-                    if (op->spell_tags[i] && op->spell_tags[i] == tmp->spell_tags[i])
+                    if (op->spell_tags[i] &&
+                            op->spell_tags[i] == tmp->spell_tags[i]) {
                         statistics.spell_suppressions++;
-                    return 0;
+                        return 0;
+                    }
                 }
             }
         }
@@ -679,10 +664,8 @@ int ok_to_put_more(mapstruct *m, sint16 x, sint16 y, object *op, uint32 immune_s
  * @return
  * 0 on failure, 1 on success.
  * @todo check the note?
- * @note
- * fire_bullet() has been merged into fire_arch_from_position()
  */
-int fire_arch_from_position(object *op, object *caster, sint16 x, sint16 y, int dir, object *spell) {
+int fire_arch_from_position(object *op, object *caster, int16_t x, int16_t y, int dir, object *spell) {
     object *tmp;
     int mflags;
     mapstruct *m;
@@ -736,11 +719,14 @@ int fire_arch_from_position(object *op, object *caster, sint16 x, sint16 y, int
         if (spell->subtype != SP_MAGIC_MISSILE && spell->subtype != SP_MOVING_BALL)
             LOG(llevError, "Unexpected object subtype %d in fire_arch_from_position for %s\n", spell->subtype, spell->name);
 
-        if (OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(tmp->map, tmp->x, tmp->y))) {
-            if (caster->type == PLAYER)
-                /* If caster is not player, it's for instance a swarm, so don't say there's an issue. */
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_ERROR,
-                              "You can't cast the spell on top of a wall!");
+        /*
+         * Things like firewalls and such can fire even if blocked, since the
+         * origin is themselves which block things...
+         * This fixes bug #3536508.
+         */
+        if (caster->type == PLAYER && OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(tmp->map, tmp->x, tmp->y))) {
+            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_ERROR,
+                "You can't cast the spell on top of a wall!");
             object_free_drop_inventory(tmp);
             return 0;
         }
@@ -866,7 +852,7 @@ void drain_wand_charge(object *wand) {
 object *find_target_for_friendly_spell(object *op, int dir) {
     object *tmp;
     mapstruct *m;
-    sint16 x, y;
+    int16_t x, y;
     int mflags;
 
     /* I don't really get this block - if op isn't a player or rune,
@@ -938,7 +924,7 @@ object *find_target_for_friendly_spell(object *op, int dir) {
  */
 int spell_find_dir(mapstruct *m, int x, int y, object *exclude) {
     int i;
-    sint16 nx, ny;
+    int16_t nx, ny;
     int owner_type = 0, mflags;
     object *tmp;
     mapstruct *mp;
@@ -1203,6 +1189,8 @@ void spell_failure(object *op, int failure, int power, object *skill) {
                 tmp->stats.dam = power; /* nasty recoils! */
 
             tmp->stats.maxhp = tmp->count;
+            if (tmp->stats.maxhp == 0)
+                tmp->stats.maxhp = 1;
             object_insert_in_map_at(tmp, op->map, NULL, 0, op->x, op->y);
         }
     }
@@ -1280,7 +1268,7 @@ static void transmute_item_to_flower(object *op) {
     if (QUERY_FLAG(item, FLAG_APPLIED))
         apply_manual(op, item, AP_NOPRINT|AP_IGNORE_CURSE|AP_UNAPPLY);
     object_remove(item);
-    flower->weight = item->nrof ? ((sint32)item->nrof)*item->weight : item->weight;
+    flower->weight = item->nrof ? ((int32_t)item->nrof)*item->weight : item->weight;
     item->weight = 0;
     esrv_del_item(op->contr, item);
     object_insert_in_ob(item, force);
@@ -1366,7 +1354,7 @@ static int spell_consume_items(object *op, const object *spell_ob) {
     char *ingredients[10];
     object *found[10];
     int count, i;
-    uint32 nrof[10];
+    uint32_t nrof[10];
     char name_ob[MAX_BUF];
     const char *name2;
 
@@ -1482,6 +1470,7 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
     rangetype old_shoottype;
     object *skill = NULL;
     int confusion_effect = 0;
+    float cost_multiplier = 1.0f;
 
     old_shoottype = op->contr ? op->contr->shoottype : range_none;
 
@@ -1522,6 +1511,51 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
         return 0;
     }
 
+
+    mflags = get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL);
+
+    /* See if we can cast a spell here.  If the caster and op are
+     * not alive, then this would mean that the mapmaker put the
+     * objects on the space - presume that they know what they are
+     * doing.
+     */
+    if (spell_ob->type == SPELL
+    && caster->type != POTION
+    && !QUERY_FLAG(op, FLAG_WIZCAST)
+    && (QUERY_FLAG(caster, FLAG_ALIVE) || QUERY_FLAG(op, FLAG_ALIVE))
+    && !QUERY_FLAG(op, FLAG_MONSTER)
+    && (((mflags&P_NO_MAGIC) && spell_ob->stats.sp) || ((mflags&P_NO_CLERIC) && spell_ob->stats.grace))) {
+        if (op->type != PLAYER)
+            return 0;
+
+        cast_create_obj(op, create_archetype(ARCH_SPELL_BLOCKED), 0);
+
+        if ((mflags&P_NO_CLERIC) && spell_ob->stats.grace)
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_ERROR,
+                                 "This ground is unholy!  %s ignores you.",
+                                 godname);
+        else
+            switch (op->contr->shoottype) {
+            case range_magic:
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_ERROR,
+                              "Something blocks your spellcasting.");
+                break;
+
+            case range_misc:
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
+                              "Something blocks the magic of your item.");
+                break;
+            case range_golem:
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
+                              "Something blocks the magic of your scroll.");
+                break;
+
+            default:
+                break;
+            }
+        return 0;
+    }
+
     /* if it is a player casting the spell, and they are really casting it
      * (vs it coming from a wand, scroll, or whatever else), do some
      * checks.  We let monsters do special things - eg, they
@@ -1585,6 +1619,12 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
                     op->casting_time = -1;
                 }
                 op->stats.grace -= random_roll(1, SP_level_spellpoint_cost(caster, spell_ob, SPELL_GRACE), op, PREFER_LOW);
+                object *bungle = create_archetype(ARCH_SPELL_BUNGLE);
+                if (bungle->arch != NULL) {
+                    cast_create_obj(op, bungle, 0);
+                } else {
+                    LOG(llevError, "cast_spell: Could not create spell_bungle arch\n");
+                }
                 return 0;
             } else if (spell_ob->stats.sp) {
                 int failure = random_roll(0, 199, op, PREFER_HIGH)-op->contr->encumbrance+op->level-spell_ob->level+35;
@@ -1597,6 +1637,12 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
                         spell_failure(op, failure, SP_level_spellpoint_cost(caster, spell_ob, SPELL_MANA), skill);
                     op->contr->shoottype = old_shoottype;
                     op->stats.sp -= random_roll(0, SP_level_spellpoint_cost(caster, spell_ob, SPELL_MANA), op, PREFER_LOW);
+                    object *bungle = create_archetype(ARCH_SPELL_BUNGLE);
+                    if (bungle->arch != NULL) {
+                        cast_create_obj(op, bungle, 0);
+                    } else {
+                        LOG(llevError, "cast_spell: Could not create spell_bungle arch\n");
+                    }
                     return 0;
                 }
             }
@@ -1608,48 +1654,6 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
         }
     }
 
-    mflags = get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL);
-
-    /* See if we can cast a spell here.  If the caster and op are
-     * not alive, then this would mean that the mapmaker put the
-     * objects on the space - presume that they know what they are
-     * doing.
-     */
-    if (spell_ob->type == SPELL
-    && caster->type != POTION
-    && !QUERY_FLAG(op, FLAG_WIZCAST)
-    && (QUERY_FLAG(caster, FLAG_ALIVE) || QUERY_FLAG(op, FLAG_ALIVE))
-    && !QUERY_FLAG(op, FLAG_MONSTER)
-    && (((mflags&P_NO_MAGIC) && spell_ob->stats.sp) || ((mflags&P_NO_CLERIC) && spell_ob->stats.grace))) {
-        if (op->type != PLAYER)
-            return 0;
-
-        if ((mflags&P_NO_CLERIC) && spell_ob->stats.grace)
-            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_ERROR,
-                                 "This ground is unholy!  %s ignores you.",
-                                 godname);
-        else
-            switch (op->contr->shoottype) {
-            case range_magic:
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_ERROR,
-                              "Something blocks your spellcasting.");
-                break;
-
-            case range_misc:
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                              "Something blocks the magic of your item.");
-                break;
-            case range_golem:
-                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_APPLY, MSG_TYPE_APPLY_ERROR,
-                              "Something blocks the magic of your scroll.");
-                break;
-
-            default:
-                break;
-            }
-        return 0;
-    }
-
     if (caster == op && caster->type ==PLAYER && settings.casting_time == TRUE && spell_ob->type == SPELL) {
         if (op->casting_time == -1) { /* begin the casting */
             op->casting_time = spell_ob->casting_time*PATH_TIME_MULT(op, spell_ob);
@@ -1700,11 +1704,6 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
         }
     }
 
-    if (op->type == PLAYER && op == caster && !QUERY_FLAG(caster, FLAG_WIZ)) {
-        op->stats.grace -= SP_level_spellpoint_cost(caster, spell_ob, SPELL_GRACE);
-        op->stats.sp -= SP_level_spellpoint_cost(caster, spell_ob, SPELL_MANA);
-    }
-
     /* We want to try to find the skill to properly credit exp.
      * for spell casting objects, the exp goes to the skill the casting
      * object requires.
@@ -1741,6 +1740,13 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
                              MSG_TYPE_SPELL, MSG_TYPE_SPELL_FAILURE,
                              "In your confused state, you can't control the magic!");
         handle_spell_confusion(op);
+
+        /* Still subtract full grace and sp. */
+        if (op->type == PLAYER && op == caster && !QUERY_FLAG(caster, FLAG_WIZ)) {
+            op->stats.grace -= SP_level_spellpoint_cost(caster, spell_ob, SPELL_GRACE);
+            op->stats.sp -= SP_level_spellpoint_cost(caster, spell_ob, SPELL_MANA);
+        }
+
         return 0;
     }
 
@@ -1872,6 +1878,9 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
 
     case SP_SUMMON_MONSTER:
         success = pets_summon_object(op, caster, spell_ob, dir, stringarg);
+        if (!success) {
+            cost_multiplier = 0.0f;
+        }
         break;
 
     case SP_CHARGING:
@@ -1985,6 +1994,12 @@ int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stri
         LOG(llevError, "cast_spell: Unhandled spell subtype %d\n", spell_ob->subtype);
     }
 
+    /* Subtract grace and sp. */
+    if (op->type == PLAYER && op == caster && !QUERY_FLAG(caster, FLAG_WIZ)) {
+        op->stats.grace -= (int)(0.5 + cost_multiplier * SP_level_spellpoint_cost(caster, spell_ob, SPELL_GRACE));
+        op->stats.sp -= (int)(0.5 + cost_multiplier * SP_level_spellpoint_cost(caster, spell_ob, SPELL_MANA));
+    }
+
     /* FIXME - we need some better sound suppport */
     /* free the spell arg */
     if (settings.casting_time == TRUE) {
@@ -2035,7 +2050,7 @@ void store_spell_expiry(object *spell) {
 void check_spell_expiry(object *spell) {
     const char *key;
 
-    if (!spell->env || !spell->env->type == PLAYER)
+    if (!spell->env || !IS_PLAYER(spell->env))
         return;
 
     key = object_get_value(spell, "spell_expiry_warn_1");
