@@ -1,41 +1,28 @@
 /*
- * static char *rcsid_swap_c =
- *    "$Id: swap.c 13308 2010-05-28 06:12:12Z kbulgrien $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * @file
- * All those functions deal with swapping maps to disk to claim memory space.
+ * Functions that deal with swapping maps to disk to reclaim memory.
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
-#include <object.h>
+#include "global.h"
+
+#include <string.h>
+
+#include "object.h"
+#include "output_file.h"
+#include "sproto.h"
 
 /**
  * Writes out information on all the temporary maps.  It is called by
@@ -45,15 +32,15 @@
  */
 static void write_map_log(void) {
     FILE *fp;
+    OutputFile of;
     mapstruct *map;
     char buf[MAX_BUF];
     long current_time = time(NULL);
 
     snprintf(buf, sizeof(buf), "%s/temp.maps", settings.localdir);
-    if (!(fp = fopen(buf, "w"))) {
-        LOG(llevError, "Could not open %s for writing\n", buf);
+    fp = of_open(&of, buf);
+    if (fp == NULL)
         return;
-    }
     for (map = first_map; map != NULL; map = map->next) {
         /* If tmpname is null, it is probably a unique player map,
          * so don't save information on it.
@@ -67,12 +54,12 @@ static void write_map_log(void) {
              * to work.
              */
             fprintf(fp, "%s:%s:%ld:0:0:%d:0:%d\n", map->path, map->tmpname,
-                    (map->reset_time == -1 ? -1 : map->reset_time-current_time),
+                    (map->reset_time == (uint32_t)-1 ? -1 : map->reset_time-current_time),
                     map->difficulty,
                     map->darkness);
         }
     }
-    fclose(fp);
+    of_close(&of);
 }
 
 /**
@@ -105,7 +92,7 @@ void read_map_log(void) {
             continue;
         }
 
-        strcpy(map->path, tmp[0]);
+        safe_strncpy(map->path, tmp[0], sizeof(map->path));
         map->tmpname = strdup_local(tmp[1]);
 
         /* Lock is left over from the lock items - we just toss it now.
@@ -159,6 +146,7 @@ int swap_map(mapstruct *map) {
 
     if (pl != NULL) {
         LOG(llevDebug, "Wanted to swap out map with player.\n");
+        map->timeout = 0;
         return SAVE_ERROR_PLAYER;
     }
     pets_remove_all(); /* Give them a chance to follow */
@@ -191,6 +179,7 @@ int swap_map(mapstruct *map) {
         return res;
     }
 
+    map->timeout = 0;
     free_map(map);
 
     if (settings.recycle_tmp_maps == TRUE)
@@ -200,76 +189,25 @@ int swap_map(mapstruct *map) {
 }
 
 /**
- * Finds maps in memory to reset.
- *
- * @todo
- * The check for MAX_OBJECTS_LWM is wrongly placed, and should be moved elsewhere.
+ * Finds maps in memory to swap.
  */
 void check_active_maps(void) {
-    mapstruct *map, *next;
+    /* Swapping can take many tens of milliseconds. Swapping too many maps in
+     * one tick can cause enough latency for the server to skip time. */
+    int num_to_swap = 1;
 
+    mapstruct *map, *next;
     for (map = first_map; map != NULL; map = next) {
         next = map->next;
-        if (map->in_memory != MAP_IN_MEMORY)
-            continue;
-        if (!map->timeout)
-            continue;
-        if (--(map->timeout) > 0)
-            continue;
-        /* If LWM is set, we only swap maps out when we run out of objects */
-#ifndef MAX_OBJECTS_LWM
-        swap_map(map);
-#endif
-    }
-}
-
-/**
- * Returns the map with the lowest timeout variable (not 0).
- *
- * @param except_level
- * path of map to ignore for reset. Musn't be NULL.
- * @return
- * map, or NULL if no map is ready for reset.
- */
-static mapstruct *map_least_timeout(const char *except_level) {
-    mapstruct *map, *chosen = NULL;
-    int timeout = MAP_MAXTIMEOUT+1;
-
-    for (map = first_map; map != NULL; map = map->next)
-        if (map->in_memory == MAP_IN_MEMORY
-        && strcmp(map->path, except_level)
-        && map->timeout
-        && map->timeout < timeout)
-            chosen = map, timeout = map->timeout;
-    return chosen;
-}
-
-/**
- * Tries to swap out maps which are still in memory, because
- * of MAP_TIMEOUT until used objects is below MAX_OBJECTS or there are
- * no more maps to swap.
- *
- * @param except_level
- * path of map to ignore for reset. Musn't be NULL.
- */
-void swap_below_max(const char *except_level) {
-    mapstruct *map;
-
-    if (nrofallocobjects-nroffreeobjects < MAX_OBJECTS)
-        return;
-    for (;;) {
-#ifdef MAX_OBJECTS_LWM
-        if (nrofallocobjects-nroffreeobjects < MAX_OBJECTS_LWM)
-            return;
-#else
-        if (nrofallocobjects-nroffreeobjects < MAX_OBJECTS)
-            return;
-#endif
-        if ((map = map_least_timeout(except_level)) == NULL)
-            return;
-        LOG(llevDebug, "Trying to swap out %s before its time.\n", map->path);
-        map->timeout = 0;
-        swap_map(map);
+        if (map->in_memory == MAP_IN_MEMORY) {
+            // map->timeout == 0 means to never swap
+            if (map->timeout > 1) {
+                map->timeout -= 1;
+            } else if (num_to_swap > 0 && map->timeout == 1) {
+                swap_map(map);
+                num_to_swap -= 1;
+            }
+        }
     }
 }
 
