@@ -1,42 +1,28 @@
 /*
- * static char *rcsid_time_c =
- *    "$Id: time.c 16785 2012-01-22 10:18:46Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * @file
  * Routines that is executed from objects based on their speed have been
  * collected in this file.
  */
 
-#include <global.h>
-#include <spells.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <stdlib.h>
+
+#include "spells.h"
+#include "sproto.h"
 
 /**
  * Remove non locked doors. The functions check to see if similar
@@ -201,16 +187,15 @@ static int generate_monster_arch(object *gen) {
         op->x = nx+at->clone.x;
         op->y = ny+at->clone.y;
 
-        if (head != NULL)
-            op->head = head,
+        if (head != NULL) {
+            op->head = head;
             prev->more = op;
+        }
 
         if (rndm(0, 9))
             generate_artifact(op, gen->map->difficulty);
 
         code = object_get_value(gen, "generator_code");
-        if (code)
-            object_set_value(head, "generator_code", code, 1);
 
         object_insert_in_map(op, gen->map, gen, 0);
         /* Did generate a monster, just didn't live very long */
@@ -222,8 +207,12 @@ static int generate_monster_arch(object *gen) {
                 monster_check_apply_all(op);
             }
         }
-        if (head == NULL)
+        if (head == NULL) {
             head = op;
+            if (code) {
+                object_set_value(head, "generator_code", code, 1);
+            }
+        }
         prev = op;
         at = at->more;
     }
@@ -237,8 +226,8 @@ static int generate_monster_arch(object *gen) {
  * generator.
  */
 static void generate_monster(object *gen) {
-    sint8 children, max_children;
-    sint8 x, y;
+    int8_t children, max_children;
+    int8_t x, y;
     const char *code, *value;
     int did_gen = 0;
 
@@ -247,7 +236,7 @@ static void generate_monster(object *gen) {
 
     value = object_get_value(gen, "generator_max_map");
     if (value) {
-        max_children = (sint8)strtol(value, NULL, 10);
+        max_children = (int8_t)strtol(value, NULL, 10);
         if (max_children < 1)
             return;
         code = object_get_value(gen, "generator_code");
@@ -345,7 +334,7 @@ static void remove_force(object *op) {
             object *pl = object_get_player_container(op);
 
             object_remove(inv);
-            inv->weight = (inv->nrof ? (sint32)(op->env->weight/inv->nrof) : op->env->weight);
+            inv->weight = (inv->nrof ? (int32_t)(op->env->weight/inv->nrof) : op->env->weight);
             if (op->env->env) {
                 object_insert_in_ob(inv, op->env->env);
                 if (pl) {
@@ -577,7 +566,7 @@ int free_no_drop(object *op) {
         object_remove(op);
     }
 
-    object_free2(op, FREE_OBJ_FREE_INVENTORY);
+    object_free(op, FREE_OBJ_FREE_INVENTORY);
     return 1;
 }
 
@@ -590,8 +579,12 @@ int free_no_drop(object *op) {
  *
  * @param op
  * object to change. Will be removed and replaced.
+ *
+ * @note Now handles multipart objects!
+ * If it fails to insert a multipart object, it re-inserts the original object.
+ * So, FLAG_FREED and FLAG_REMOVED will not be set in op if it is put back.
  */
-void change_object(object *op) { /* Doesn`t handle linked objs yet */
+void change_object(object *op) {
     object *env;
     int i;
     int friendly;
@@ -616,7 +609,7 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
     friendly = QUERY_FLAG(op, FLAG_FRIENDLY);
     unaggressive = QUERY_FLAG(op, FLAG_UNAGGRESSIVE);
     owner = object_get_owner(op);
-    for (i = 0; i < op->stats.food; i++) { /* This doesn't handle op->more yet */
+    for (i = 0; i < op->stats.food; i++) {
         object *tmp;
 
         tmp = arch_to_object(op->other_arch);
@@ -636,8 +629,31 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
             tmp->x = env->x,
             tmp->y = env->y;
             tmp = object_insert_in_ob(tmp, env);
-        } else
+        }
+        // If there is more to the object, put in the map,
+        // then initiate the process of setting up multipartdom.
+        else if (tmp->arch->more)
+        {
+            tmp->map = op->map;
+            // Get the best free spot for the object
+            // Using the clone of the arch because I do not think that the actual object is ready yet.
+            if (object_find_multi_free_spot_around(&tmp->arch->clone, op, &tmp->x, &tmp->y) != 0)
+            {
+                LOG(llevInfo, "change_object: Failed to find a spot to put changing multipart object\n");
+                // Put the orignal object back.
+                object_insert_in_map(op, op->map, NULL, INS_NO_MERGE|INS_ABOVE_FLOOR_ONLY|INS_NO_WALK_ON);
+                // Free the failed object
+                object_free_drop_inventory(tmp);
+                // Bail out so we don't break things.
+                return;
+            }
+            object_insert_in_map(tmp, op->map, op, INS_NO_MERGE|INS_ABOVE_FLOOR_ONLY|INS_NO_WALK_ON);
+            object_fix_multipart(tmp);
+        }
+        else{
+            // Single-size objects work here.
             object_insert_to_free_spot_or_free(tmp, op->map, op->x, op->y, 1, SIZEOFFREE1+1, op);
+        }
     }
     if (friendly)
         remove_friendly_object(op);
@@ -648,7 +664,7 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
  * Move for ::FIREWALL.
  *
  * firewalls fire other spells.
- * The direction of the wall is stored in op->stats.sp.
+ * The direction of the wall is stored in op->direction.
  * walls can have hp, so they can be torn down.
  *
  * @param op
@@ -662,11 +678,11 @@ void move_firewall(object *op) {
 
     spell = op->inv;
     if (!spell) {
-        LOG(llevError, "move_firewall: no spell specified (%s, %s, %d, %d)\n", op->name, op->map->name, op->x, op->y);
+        LOG(llevError, "firewall '%s' in (%s, %d, %d) has no spell\n", op->name, op->map->path, op->x, op->y);
         return;
     }
 
-    cast_spell(op, op, op->stats.sp ? op->stats.sp : get_random_dir(), spell, NULL);
+    cast_spell(op, op, op->direction ? op->direction : get_random_dir(), spell, NULL);
 }
 
 
@@ -685,7 +701,7 @@ void move_firewall(object *op) {
  */
 void move_player_mover(object *op) {
     int dir = op->stats.sp;
-    sint16 nx, ny;
+    int16_t nx, ny;
     mapstruct *m;
 
     if (!op->map) {
@@ -788,7 +804,7 @@ int process_object(object *op) {
             return 1;
 
     if ((QUERY_FLAG(op, FLAG_ANIMATE) && op->anim_speed == 0)
-    || (op->temp_animation_id && op->temp_anim_speed == 0)) {
+    || (op->temp_animation && op->temp_anim_speed == 0)) {
         op->state++;
         if (op->type == PLAYER)
             animate_object(op, op->facing);
@@ -805,14 +821,30 @@ int process_object(object *op) {
     if (QUERY_FLAG(op, FLAG_GENERATOR) && !QUERY_FLAG(op, FLAG_FRIENDLY))
         generate_monster(op);
 
-    if (QUERY_FLAG(op, FLAG_IS_USED_UP) && --op->stats.food <= 0) {
-        if (QUERY_FLAG(op, FLAG_APPLIED))
+    /* If object can be used up, decrement 'food' and eventually remove it. */
+    if (QUERY_FLAG(op, FLAG_IS_USED_UP) && op->stats.food-- <= 0) {
+        if (QUERY_FLAG(op, FLAG_APPLIED)) {
             remove_force(op);
-        else {
+        } else {
+            if (op->env != NULL && op->env->type == PLAYER) {
+                sstring key;
+                key_value *used_up_message;
+
+                key = add_string("used_up_message");
+                used_up_message = object_get_key_value(op, key);
+                free_string(key);
+
+                if (used_up_message != NULL) {
+                    draw_ext_info_format(
+                        NDI_BLACK, 0, op->env, MSG_TYPE_ITEM, MSG_TYPE_ITEM_REMOVE,
+                        "The %s %s.", op->name, used_up_message->value);
+                }
+            }
+
             object_remove(op);
             if (QUERY_FLAG(op, FLAG_SEE_ANYWHERE))
                 make_sure_not_seen(op);
-            object_free2(op, FREE_OBJ_DROP_ABOVE_FLOOR);
+            object_free(op, FREE_OBJ_DROP_ABOVE_FLOOR);
         }
         return 1;
     }
