@@ -1,24 +1,15 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2001-2007 Yann Chachkoff & Crossfire Development Team
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
 /**
  * @file
@@ -37,10 +28,10 @@
  * Note that timers are one shot only, they reset after they activate.
  */
 
-#include <timers.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include <string.h>
+
+#include "timers.h"
+#include "sproto.h"
 
 /* Extern in header. */
 cftimer timers_table[MAX_TIMERS];
@@ -147,7 +138,7 @@ int cftimer_destroy(int id) {
  * Finds a free ID for a new timer.
  * @retval ::TIMER_ERR_ID
  * no free ID available.
- * @retval >0
+ * @retval >=0
  * an available ID.
  */
 int cftimer_find_free_id(void) {
