@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_sock_info_c =
- *   "$Id: info.c 14057 2010-11-06 11:22:31Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2006,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * \file
  * Basic client output functions.
@@ -36,25 +21,32 @@
  * client.  Basically, things like sending text strings along
  */
 
-#include <global.h>
-#include <sproto.h>
-#include <stdarg.h>
-#include <spells.h>
-#include <skills.h>
+#include "global.h"
 
+#include <stdarg.h>
+#include <string.h>
+
+#include "skills.h"
+#include "spells.h"
+#include "sproto.h"
 
 /**
- * Draws   an extended message on the client.
- * ns      the socket to send message to
- * color   color informations (used mainly if client does not support message type)
- * type,
- * subtype type and subtype of text message
- * intro   Intro message to send with main message if client does not support the message type
- * message The main message
- * make this non static, so other files can use this to send messages to client before
+ * Draws an extended message on the client.
+ *
+ * @param ns The socket to send message to
+ *
+ * @param color Color informations (used mainly if client does not support message type)
+ *
+ * @param type The type of text message
+ *
+ * @param subtype The subtype of text message
+ *
+ * @param message The main message
+ *
+ * make this non-static, so other files can use this to send messages to client before
  * player has been added.
  */
-void print_ext_msg(socket_struct *ns, int color, uint8 type, uint8 subtype, const char *message) {
+void print_ext_msg(socket_struct *ns, int color, uint8_t type, uint8_t subtype, const char *message) {
     SockList sl;
 
     SockList_Init(&sl);
@@ -92,8 +84,8 @@ void print_ext_msg(socket_struct *ns, int color, uint8 type, uint8 subtype, cons
  * @param message The message to send.
  */
 void draw_ext_info(
-    int flags, int pri, const object *pl, uint8 type,
-    uint8 subtype, const char *message) {
+    int flags, int pri, const object *pl, uint8_t type,
+    uint8_t subtype, const char *message) {
 
     if ((flags&NDI_ALL) || (flags&NDI_ALL_DMS)) {
         player *tmppl;
@@ -104,6 +96,7 @@ void draw_ext_info(
             draw_ext_info((flags&~NDI_ALL&~NDI_ALL_DMS), pri, tmppl->ob, type, subtype, message);
         }
 
+        LOG(llevInfo, "-- %s\n", message);
         return;
     }
 
@@ -137,11 +130,11 @@ void draw_ext_info(
  *
  * @param type The type MSG_TYPE for the type of message.
  *
- * @param subtype The type MSG_TYPE for the type of message.
+ * @param subtype The type MSG_TYPE for the subtype of message.
  *
  * @param format The message to send, with optional format specifiers.
  */
-void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *format, ...) {
+void draw_ext_info_format(int flags, int pri, const object *pl, uint8_t type, uint8_t subtype, const char *format, ...) {
     char buf[HUGE_BUF];
     va_list ap;
 
@@ -154,8 +147,18 @@ void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint
 
 /**
  * Writes to everyone on the specified map
+ *
+ * @param color The color of the message
+ *
+ * @param map The map where the players receiving the message are located
+ *
+ * @param type The message type
+ *
+ * @param subtype The message subtype
+ *
+ * @param str1 The message to be sent
  */
-void ext_info_map(int color, const mapstruct *map, uint8 type, uint8 subtype, const char *str1) {
+void ext_info_map(int color, const mapstruct *map, uint8_t type, uint8_t subtype, const char *str1) {
     player *pl;
 
     for (pl = first_player; pl != NULL; pl = pl->next)
@@ -165,9 +168,21 @@ void ext_info_map(int color, const mapstruct *map, uint8 type, uint8 subtype, co
 }
 
 /**
- * Writes to everyone on the map *except *op.  This is useful for emotions.
+ * Writes to everyone on the map except *op.  This is useful for emotions.
+ *
+ * @param color The color of the message
+ *
+ * @param map The map where the players receiving the message are located
+ *
+ * @param op The player object to skip sending the message to on the map
+ *
+ * @param type The message type
+ *
+ * @param subtype The message subtype
+ *
+ * @param str1 The message to be sent
  */
-void ext_info_map_except(int color, const mapstruct *map, const object *op, uint8 type, uint8 subtype, const char *str1) {
+void ext_info_map_except(int color, const mapstruct *map, const object *op, uint8_t type, uint8_t subtype, const char *str1) {
     player *pl;
 
     for (pl = first_player; pl != NULL; pl = pl->next)
@@ -178,6 +193,20 @@ void ext_info_map_except(int color, const mapstruct *map, const object *op, uint
 
 /**
  * Writes to everyone on the map except op1 and op2
+ *
+ * @param color The message color
+ *
+ * @param map The map where the players receiving the message are located
+ *
+ * @param op1 The first player object to skip sending the message to
+ *
+ * @param op2 The second player object to skip sending the message to
+ *
+ * @param type The message type
+ *
+ * @param subtype The message subtype
+ *
+ * @param str1 The message to be sent
  */
 void ext_info_map_except2(int color, const mapstruct *map, const object *op1, const object *op2, int type, int subtype, const char *str1) {
     player *pl;
@@ -191,6 +220,12 @@ void ext_info_map_except2(int color, const mapstruct *map, const object *op1, co
 
 /**
  * Get player's current range attack in obuf.
+ *
+ * @param pl The player who is being evaluated
+ *
+ * @param obuf The buffer for the range attack string
+ *
+ * @param len The length of the buffer
  */
 void rangetostring(const object *pl, char *obuf, size_t len) {
     char name[MAX_BUF];
@@ -215,15 +250,13 @@ void rangetostring(const object *pl, char *obuf, size_t len) {
         break;
 
     case range_magic:
-        if (settings.casting_time == TRUE) {
-            if (pl->casting_time > -1) {
-                if (pl->casting_time == 0)
-                    snprintf(obuf, len, "Range: Holding spell (%s)", pl->spell->name);
-                else
-                    snprintf(obuf, len, "Range: Casting spell (%s)", pl->spell->name);
-            } else
-                snprintf(obuf, len, "Range: spell (%s)", pl->contr->ranges[range_magic]->name);
-        } else
+        if (settings.casting_time == TRUE && pl->casting_time > -1) {
+            if (pl->casting_time == 0)
+                snprintf(obuf, len, "Range: Holding spell (%s)", pl->spell->name);
+            else
+                snprintf(obuf, len, "Range: Casting spell (%s)", pl->spell->name);
+        }
+        else
             snprintf(obuf, len, "Range: spell (%s)", pl->contr->ranges[range_magic]->name);
         break;
 
@@ -276,8 +309,14 @@ void set_title(const object *pl, char *buf, size_t len) {
  * Helper for magic map creation.
  *
  * Takes a player, the map_mark array and an x and y starting position.
- * pl is the player.
- * px, py are offsets from the player.
+ *
+ * @param pl The player creating the magic map.
+ *
+ * @param map_mark Holds flags for each tile analyzed
+ *
+ * @param px The x offset from the player to start calculations
+ *
+ * @param py The y offset from the player to start calculations
  *
  * This function examines all the adjacant spaces next to px, py.
  * It updates the map_mark arrow with the color and high bits set
@@ -285,9 +324,9 @@ void set_title(const object *pl, char *buf, size_t len) {
  */
 static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int py) {
     int x, y, dx, dy, mflags, l;
-    sint16 nx, ny;
+    int16_t nx, ny;
     mapstruct *mp;
-    const New_Face *f;
+    const Face *f;
     object *ob;
 
     for (dx = -1; dx <= 1; dx++) {
@@ -334,6 +373,14 @@ static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int
 /**
  * Creates magic map for player.
  *
+ * @param pl The player making the magic map
+ *
+ * @param map_mark Holds flags for each tile analyzed. Should have been
+ * initialized to zero before this function is called.
+ *
+ * @param strength An initial strength*2 rectangular area that we
+ * automatically see in/penetrate through.
+ *
  * Note:  For improved magic mapping display, the space that blocks
  * the view is now marked with value 2.  Any dependencies of map_mark
  * being nonzero have been changed to check for 1.  Also, since
@@ -341,16 +388,13 @@ static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int
  *
  * This function examines the map the player is on, and determines what
  * is visible.  2 is set for walls or objects that blocks view.  1
- * is for open spaces.  map_mark should already have been initialized
- * to zero before this is called.
- * strength is an initial strength*2 rectangular area that we automatically
- * see in/penetrate through.
+ * is for open spaces.
  */
 void magic_mapping_mark(object *pl, char *map_mark, int strength) {
     int x, y, mflags, l;
-    sint16 nx, ny;
+    int16_t nx, ny;
     mapstruct *mp;
-    const New_Face *f;
+    const Face *f;
     object *ob;
 
     for (x = -strength; x < strength; x++) {
@@ -386,6 +430,8 @@ void magic_mapping_mark(object *pl, char *map_mark, int strength) {
 /**
  * Creates and sends magic map to player.
  *
+ * @ param pl The player to send the magic map to
+ *
  * The following function is a lot messier than it really should be,
  * but there is no real easy solution.
  *
