@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_init_c =
- *    "$Id: init.c 17000 2012-02-07 18:56:31Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006,2010 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * \file
  * Socket general functions
@@ -37,10 +22,16 @@
  * The reading of data is handled in ericserver.c
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#endif
+#include "global.h"
+
+#include <arpa/inet.h>
+#include <assert.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+
 #ifndef WIN32 /* ---win32 exclude include files */
 #include <sys/types.h>
 #include <sys/time.h>
@@ -48,15 +39,9 @@
 #include <netdb.h>
 #endif /* win32 */
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-#include <newserver.h>
-#include <loader.h>
+#include "image.h"
+#include "newserver.h"
+#include "sproto.h"
 
 /** Socket information. */
 Socket_Info socket_info;
@@ -70,28 +55,9 @@ Socket_Info socket_info;
  */
 socket_struct *init_sockets;
 
-/**
- * Initializes a connection. Really, it just sets up the data structure,
- * socket setup is handled elsewhere.  We do send a version to the
- * client.
- */
-void init_connection(socket_struct *ns, const char *from_ip) {
-    SockList sl;
-    int bufsize = 65535; /*Supposed absolute upper limit */
+static void set_output_sock_buf(socket_struct *ns, int bufsize) {
     int oldbufsize;
-    socklen_t buflen = sizeof(int);
-
-#ifdef WIN32 /* ***WIN32 SOCKET: init win32 non blocking socket */
-    int temp = 1;
-
-    if (ioctlsocket(ns->fd, FIONBIO , &temp) == -1)
-        LOG(llevError, "init_connection:  Error on ioctlsocket.\n");
-#else
-    if (fcntl(ns->fd, F_SETFL, O_NONBLOCK) == -1) {
-        LOG(llevError, "init_connection:  Error on fcntl.\n");
-    }
-#endif /* end win32 */
-
+    socklen_t buflen = sizeof(oldbufsize);
     if (getsockopt(ns->fd, SOL_SOCKET, SO_SNDBUF, (char *)&oldbufsize, &buflen) == -1)
         oldbufsize = 0;
     if (oldbufsize < bufsize) {
@@ -107,6 +73,28 @@ void init_connection(socket_struct *ns, const char *from_ip) {
 #ifdef ESRV_DEBUG
     LOG(llevDebug, "Socket buffer size now %d bytes\n", oldbufsize);
 #endif
+}
+
+/**
+ * Initializes a connection. Really, it just sets up the data structure,
+ * socket setup is handled elsewhere.  We do send a version to the
+ * client.
+ */
+void init_connection(socket_struct *ns, const char *from_ip) {
+    SockList sl;
+
+#ifdef WIN32 /* ***WIN32 SOCKET: init win32 non blocking socket */
+    int temp = 1;
+
+    if (ioctlsocket(ns->fd, FIONBIO , &temp) == -1)
+        LOG(llevError, "init_connection:  Error on ioctlsocket.\n");
+#else
+    if (fcntl(ns->fd, F_SETFL, O_NONBLOCK) == -1) {
+        LOG(llevError, "init_connection:  Error on fcntl.\n");
+    }
+#endif /* end win32 */
+
+    set_output_sock_buf(ns, SOCKETBUFSIZE);
 
     ns->faceset = 0;
     ns->facecache = 0;
@@ -130,6 +118,7 @@ void init_connection(socket_struct *ns, const char *from_ip) {
     ns->account_chars = NULL;
     ns->login_method = 0;
     ns->notifications = 0;
+    ns->heartbeat = 0;
 
     /* we should really do some checking here - if total clients overflows
      * we need to do something more intelligent, because client id's will start
@@ -145,8 +134,8 @@ void init_connection(socket_struct *ns, const char *from_ip) {
     memset(ns->inbuf.buf, 0, sizeof(ns->inbuf.buf));
     memset(&ns->lastmap, 0, sizeof(struct Map));
     if (!ns->faces_sent)
-        ns->faces_sent = calloc(sizeof(*ns->faces_sent), nrofpixmaps);
-    ns->faces_sent_len = nrofpixmaps;
+        ns->faces_sent = calloc(sizeof(*ns->faces_sent), get_faces_count());
+    ns->faces_sent_len = get_faces_count();
 
     memset(&ns->anims_sent, 0, sizeof(ns->anims_sent));
     memset(&ns->stats, 0, sizeof(struct statsinfo));
@@ -158,9 +147,6 @@ void init_connection(socket_struct *ns, const char *from_ip) {
      */
     ns->faces_sent[0] = NS_FACESENT_FACE;
 
-    ns->outputbuffer.start = 0;
-    ns->outputbuffer.len = 0;
-    ns->can_write = 1;
     ns->password_fails = 0;
 
     ns->host = strdup_local(from_ip);
@@ -185,7 +171,6 @@ void init_connection(socket_struct *ns, const char *from_ip) {
 void init_listening_socket(socket_struct *ns) {
     struct linger linger_opt;
     char buf1[MAX_BUF], buf2[MAX_BUF];
-    char err[MAX_BUF];
 
     if (ns == NULL || ns->listen == NULL) {
         LOG(llevError, "init_listening_socket: missing listen info in socket_struct?!\n");
@@ -196,14 +181,14 @@ void init_listening_socket(socket_struct *ns) {
 
     ns->fd = socket(ns->listen->family, ns->listen->socktype, ns->listen->protocol);
     if (ns->fd == -1) {
-        LOG(llevError, "Cannot create socket: %s\n", strerror_local(errno, err, sizeof(err)));
+        LOG(llevError, "Cannot create socket: %s\n", strerror(errno));
         return;
     }
 
     linger_opt.l_onoff = 0;
     linger_opt.l_linger = 0;
     if (setsockopt(ns->fd, SOL_SOCKET, SO_LINGER, (char *)&linger_opt, sizeof(struct linger))) {
-        LOG(llevError, "Cannot setsockopt(SO_LINGER): %s\n", strerror_local(errno, err, sizeof(err)));
+        LOG(llevError, "Cannot setsockopt(SO_LINGER): %s\n", strerror(errno));
     }
 /* Would be nice to have an autoconf check for this.  It appears that
  * these functions are both using the same calling syntax, just one
@@ -222,17 +207,17 @@ void init_listening_socket(socket_struct *ns) {
 #endif
 
         if (setsockopt(ns->fd, SOL_SOCKET, SO_REUSEADDR, &tmp, sizeof(tmp))) {
-            LOG(llevError, "Cannot setsockopt(SO_REUSEADDR): %s\n", strerror_local(errno, err, sizeof(err)));
+            LOG(llevError, "Cannot setsockopt(SO_REUSEADDR): %s\n", strerror(errno));
         }
 #ifdef HAVE_GETADDRINFO
         if ((ns->listen->family == AF_INET6) && setsockopt(ns->fd, IPPROTO_IPV6, IPV6_V6ONLY, &tmp, sizeof(tmp))) {
-            LOG(llevError, "Cannot setsockopt(IPV6_V6ONLY): %s\n", strerror_local(errno, err, sizeof(err)));
+            LOG(llevError, "Cannot setsockopt(IPV6_V6ONLY): %s\n", strerror(errno));
         }
 #endif
     }
 #else
     if (setsockopt(ns->fd, SOL_SOCKET, SO_REUSEADDR, (char *)NULL, 0)) {
-        LOG(llevError, "Cannot setsockopt(SO_REUSEADDR): %s\n", strerror_local(errno, err, sizeof(err)));
+        LOG(llevError, "Cannot setsockopt(SO_REUSEADDR): %s\n", strerror(errno));
     }
 #endif
 
@@ -248,7 +233,7 @@ void init_listening_socket(socket_struct *ns) {
         snprintf(buf1, sizeof(buf1), "%ld.%ld.%ld.%ld", (ip>>24)&255, (ip>>16)&255, (ip>>8)&255, ip&255);
         snprintf(buf2, sizeof(buf2), "%d", port&65535);
 #endif
-        LOG(llevError, "Cannot bind socket to [%s]:%s: %s\n", buf1, buf2, strerror_local(errno, err, sizeof(err)));
+        LOG(llevError, "Cannot bind socket to [%s]:%s: %s\n", buf1, buf2, strerror(errno));
 #ifdef WIN32 /* ***win32: close() -> closesocket() */
         shutdown(ns->fd, SD_BOTH);
         closesocket(ns->fd);
@@ -259,7 +244,7 @@ void init_listening_socket(socket_struct *ns) {
         return;
     }
     if (listen(ns->fd, 5) == (-1))  {
-        LOG(llevError, "Cannot listen on socket: %s\n", strerror_local(errno, err, sizeof(err)));
+        LOG(llevError, "Cannot listen on socket: %s\n", strerror(errno));
 #ifdef WIN32 /* ***win32: close() -> closesocket() */
         shutdown(ns->fd, SD_BOTH);
         closesocket(ns->fd);
@@ -344,6 +329,8 @@ void init_server(void) {
     for (ai_p = ai; ai_p != NULL; ai_p = ai_p->ai_next) {
         listen_socket_count++;
     }
+
+    assert(listen_socket_count > 0);
 #else
     listen_socket_count = 1;
 #endif
