@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_item_c =
- *    "$Id: item.c 14838 2011-07-10 11:00:56Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-  CrossFire, A Multiplayer game for X-windows
-
-  Copyright (C) 2002,2006 Mark Wedel & Crossfire Development Team
-  Copyright (C) 1992 Frank Tore Johansen
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-  The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * \file
  * Client/server logic.
@@ -37,11 +22,15 @@
  * the logic for what items should be sent.
  */
 
-#include <global.h>
-#include <object.h> /* LOOK_OBJ */
-#include <shared/newclient.h>
-#include <newserver.h>
-#include <sproto.h>
+#include "global.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "newserver.h"
+#include "object.h"
+#include "shared/newclient.h"
+#include "sproto.h"
 
 /** This is the maximum number of bytes we expect any one item to take up */
 #define MAXITEMLEN  300
@@ -99,7 +88,7 @@ static unsigned int query_flags(const object *op) {
     && ((op->env && op->env->container == op) || (!op->env && QUERY_FLAG(op, FLAG_APPLIED))))
         flags |= F_OPEN;
 
-    if (!QUERY_FLAG(op, FLAG_IDENTIFIED) && need_identify(op))
+    if (!is_identified(op))
         flags |= F_UNIDENTIFIED;
 
     if (QUERY_FLAG(op, FLAG_KNOWN_CURSED)) {
@@ -134,14 +123,14 @@ static void add_object_to_socklist(socket_struct *ns, SockList *sl, object *head
         flags |= F_NOPICK;
 
     if (!(ns->faces_sent[head->face->number]&NS_FACESENT_FACE))
-        esrv_send_face(ns, head->face->number, 0);
+        esrv_send_face(ns, head->face, 0);
 
     if (QUERY_FLAG(head, FLAG_ANIMATE)) {
-        if (head->animation_id == 0) {
+        if (head->animation == NULL) {
             LOG(llevError, "Item %s in %s (%d,%d) has FLAG_ANIMATE but animation_id 0\n", head->name, (head->env ? head->env->name : (head->map ? head->map->path : "???")), head->x, head->y);
             CLEAR_FLAG(head, FLAG_ANIMATE);
-        } else if (!ns->anims_sent[head->animation_id])
-            esrv_send_animation(ns, head->animation_id);
+        } else if (!ns->anims_sent[head->animation->num])
+            esrv_send_animation(ns, head->animation);
     }
 
     SockList_AddInt(sl, head->count);
@@ -155,10 +144,10 @@ static void add_object_to_socklist(socket_struct *ns, SockList *sl, object *head
         len = strlen(item_n);
         query_base_name(head, 1, item_p, MAX_BUF);
     } else {
-        strncpy(item_n, head->custom_name, 127);
+        safe_strncpy(item_n, head->custom_name, 127);
         item_n[127] = 0;
         len = strlen(item_n);
-        strncpy(item_p, head->custom_name, MAX_BUF);
+        safe_strncpy(item_p, head->custom_name, sizeof(item_p));
     }
     strncpy(item_n+len+1, item_p, 127);
     /* This is needed because strncpy may not add a ending \0 if the string is long enough. */
@@ -166,7 +155,7 @@ static void add_object_to_socklist(socket_struct *ns, SockList *sl, object *head
     len += strlen(item_n+1+len)+1;
     SockList_AddLen8Data(sl, item_n, len);
 
-    SockList_AddShort(sl, head->animation_id);
+    SockList_AddShort(sl, (QUERY_FLAG(head, FLAG_ANIMATE) && head->animation) ? head->animation->num : 0);
     anim_speed = 0;
     if (QUERY_FLAG(head, FLAG_ANIMATE)) {
         if (head->anim_speed)
@@ -230,7 +219,7 @@ void esrv_draw_look(object *pl) {
     SockList_AddInt(&sl, 0);
 
     if (!(pl->contr->socket.faces_sent[empty_face->number]&NS_FACESENT_FACE))
-        esrv_send_face(&pl->contr->socket, empty_face->number, 0);
+        esrv_send_face(&pl->contr->socket, empty_face, 0);
 
     if (pl->contr->socket.look_position) {
         int overhead = 1+(pl->contr->transport != NULL);
@@ -429,11 +418,13 @@ void esrv_update_item(int flags, object *pl, object *op) {
          */
     }
     if (!QUERY_FLAG(op, FLAG_CLIENT_SENT)) {
-        /* FLAG_CLIENT_SENT is debug only.  We are using it to see where
-         * this is happening - we can set a breakpoint here in the debugger
-         * and track back the call.
-         */
-        LOG(llevDebug, "We have not sent item %s (%d)\n", op->name, op->count);
+        // Sometimes, we try to update an item that we haven't sent to the
+        // client. Don't! This can happen, for example, when a button under the
+        // floor gets toggled, but objects under floor tiles are generally not
+        // sent. There are some other places where this happens that we haven't
+        // tracked down, but in general, just don't.
+        //LOG(llevDebug, "We have not sent item %s (%d)\n", op->name, op->count);
+        return;
     }
 
     SockList_Init(&sl);
@@ -450,7 +441,7 @@ void esrv_update_item(int flags, object *pl, object *op) {
         SockList_AddInt(&sl, query_flags(op));
 
     if (flags&UPD_WEIGHT) {
-        sint32 weight = WEIGHT(op);
+        int32_t weight = WEIGHT(op);
 
         /* TRANSPORTS are odd - they sort of look like containers,
          * yet can't be picked up.  So we don't to send the weight,
@@ -465,7 +456,7 @@ void esrv_update_item(int flags, object *pl, object *op) {
 
     if (flags&UPD_FACE) {
         if (!(pl->contr->socket.faces_sent[op->face->number]&NS_FACESENT_FACE))
-            esrv_send_face(&pl->contr->socket, op->face->number, 0);
+            esrv_send_face(&pl->contr->socket, op->face, 0);
         SockList_AddInt(&sl, op->face->number);
     }
     if (flags&UPD_NAME) {
@@ -474,24 +465,20 @@ void esrv_update_item(int flags, object *pl, object *op) {
         char item_n[MAX_BUF];
 
         if (!op->custom_name) {
-            query_base_name(op, 0, item_n, MAX_BUF);
-            len = strlen(item_n);
-            query_base_name(op, 1, item_p, MAX_BUF);
+            query_base_name(op, 0, item_n, sizeof(item_n)-1);
+            query_base_name(op, 1, item_p, sizeof(item_p));
         } else {
-            strncpy(item_n, op->custom_name, MAX_BUF-1);
-            item_n[MAX_BUF-1] = 0;
-            len = strlen(item_n);
-            strncpy(item_p, op->custom_name, MAX_BUF-1);
-            item_p[MAX_BUF-1] = 0;
+            strlcpy(item_n, op->custom_name, sizeof(item_n)-1);
+            strlcpy(item_p, op->custom_name, sizeof(item_p));
         }
 
-        strncpy(item_n+len+1, item_p, 127);
-        item_n[254] = 0;
-        len += strlen(item_n+1+len)+1;
+        len = strlen(item_n)+1;
+        snprintf(item_n+len, sizeof(item_n)-len, "%s", item_p);
+        len += strlen(item_n+len);
         SockList_AddLen8Data(&sl, item_n, len);
     }
     if (flags&UPD_ANIM)
-        SockList_AddShort(&sl, op->animation_id);
+        SockList_AddShort(&sl, op->animation ? op->animation->num : 0);
 
     if (flags&UPD_ANIMSPEED) {
         int anim_speed = 0;
@@ -644,7 +631,7 @@ void examine_cmd(char *buf, int len, player *pl) {
 
 /** Client wants to apply some object.  Lets do so. */
 void apply_cmd(char *buf, int len, player *pl) {
-    uint32 tag;
+    uint32_t tag;
     object *op;
 
     if (!buf || len <= 0) {
@@ -683,10 +670,11 @@ void apply_cmd(char *buf, int len, player *pl) {
 }
 
 /** Client wants to apply some object.  Lets do so. */
-void lock_item_cmd(uint8 *data, int len, player *pl) {
+void lock_item_cmd(uint8_t *data, int len, player *pl) {
     int flag, tag;
     object *op;
     object *tmp;
+    char name[HUGE_BUF];
 
     if (len != 5) {
         LOG(llevDebug, "Player '%s' sent bogus lock_item_cmd information\n", pl->ob->name);
@@ -713,10 +701,16 @@ void lock_item_cmd(uint8 *data, int len, player *pl) {
         return;
     }
 
-    if (!flag)
+    query_short_name(op, name, HUGE_BUF);
+    if (!flag) {
         CLEAR_FLAG(op, FLAG_INV_LOCKED);
-    else
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
+                             "Unlocked %s.", name);
+    } else {
         SET_FLAG(op, FLAG_INV_LOCKED);
+        draw_ext_info_format(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_FAILURE,
+                             "Locked %s.", name);
+    }
 
     tmp = object_merge(op, NULL);
     if (tmp == NULL) {
@@ -735,7 +729,7 @@ void lock_item_cmd(uint8 *data, int len, player *pl) {
  * @param pl
  * player.
  */
-void mark_item_cmd(uint8 *data, int len, player *pl) {
+void mark_item_cmd(uint8_t *data, int len, player *pl) {
     int tag;
     object *op;
     char name[MAX_BUF];
@@ -769,7 +763,7 @@ void mark_item_cmd(uint8 *data, int len, player *pl) {
 void look_at(object *op, int dx, int dy) {
     object *tmp;
     int flag = 0;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *m;
     char name[MAX_BUF];
 
@@ -835,6 +829,11 @@ void look_at_cmd(char *buf, int len, player *pl) {
     int dx, dy;
     char *cp;
 
+    if (len <= 0 || !buf) {
+        LOG(llevDebug, "Player '%s' sent bogus look_at_cmd information\n", pl->ob->name);
+        return;
+    }
+
     dx = atoi(buf);
     if (!(cp = strchr(buf, ' '))) {
         return;
@@ -938,8 +937,8 @@ void inscribe_scroll_cmd(char *buf, int len, player *pl) {
             LOG(llevDebug, "Player %s sent an invalid inscribe command.\n", pl->ob->name);
             return;
         }
-        tscroll = GetInt_String((uint8 *)buf+1);
-        tspell = GetInt_String((uint8 *)buf+5);
+        tscroll = GetInt_String((uint8_t *)buf+1);
+        tspell = GetInt_String((uint8_t *)buf+5);
 
         scroll = esrv_get_ob_from_count(pl->ob, tscroll);
         if (!scroll) {
