@@ -1,32 +1,16 @@
-
 /*
- * static char *rcsid_loop_c =
- *    "$Id: loop.c 16608 2012-01-14 18:09:49Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2006,2010 Mark Wedel & The Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * \file
  * Main client/server loops.
@@ -38,11 +22,15 @@
  * The reading of data is handled in lowlevel.c
  */
 
-#include <global.h>
-#ifndef __CEXTRACT__
-#include <sproto.h>
-#include <sockproto.h>
-#endif
+#include "global.h"
+
+#include <arpa/inet.h>
+#include <assert.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #ifndef WIN32 /* ---win32 exclude unix headers */
 #include <sys/types.h>
@@ -51,16 +39,10 @@
 #include <netdb.h>
 #endif /* end win32 */
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
-
-#include <loader.h>
-#include <newserver.h>
+#include "image.h"
+#include "newserver.h"
+#include "sockproto.h"
+#include "sproto.h"
 
 /*****************************************************************************
  * Start of command dispatch area.
@@ -90,7 +72,7 @@ typedef void (*func_uint8_int_pl)(char *, int, player *);
 struct player_cmd_mapping {
     const char *cmdname;             /**< Command name. */
     const func_uint8_int_pl cmdproc; /**< Function to call. */
-    const uint8 flag;                /**< If set, the player must be in the ST_PLAYING state for this command to be available. */
+    const uint8_t flag;                /**< If set, the player must be in the ST_PLAYING state for this command to be available. */
 };
 
 /**
@@ -117,7 +99,6 @@ static const struct player_cmd_mapping player_commands[] = {
     { "lookat",     look_at_cmd,                       1 },
     { "lock",       (func_uint8_int_pl)lock_item_cmd,  1 },
     { "mark",       (func_uint8_int_pl)mark_item_cmd,  1 },
-    { "mapredraw",  map_redraw_cmd,                    0 },  /* Added: phil */
     { "inscribe",   inscribe_scroll_cmd,               0 },
     { NULL,         NULL,                              0 }   /* terminator */
 };
@@ -126,6 +107,7 @@ static const struct player_cmd_mapping player_commands[] = {
 static const struct client_cmd_mapping client_commands[] = {
     { "addme",               add_me_cmd },
     { "askface",             send_face_cmd },             /* Added: phil */
+    { "beat",                NULL },
     { "requestinfo",         request_info_cmd },
     { "setup",               set_up_cmd },
     { "version",             version_cmd },
@@ -143,12 +125,20 @@ static const struct client_cmd_mapping client_commands[] = {
  * request_info_cmd is sort of a meta command. There is some specific
  * request of information, but we call other functions to provide
  * that information.
+ * @param buf buffer containing the information requested.
+ * @param len length of buf, ignored.
+ * @param ns socket to write data to.
  */
 void request_info_cmd(char *buf, int len, socket_struct *ns) {
     char *params = NULL, *cp;
     /* No match */
     SockList sl;
 
+    if (len <= 0 || !buf) {
+        LOG(llevDebug, "IP '%s' sent bogus request_info_cmd information\n", ns->host);
+        return;
+    }
+
     /* Set up replyinfo before we modify any of the buffers - this is used
      * if we don't find a match.
      */
@@ -166,7 +156,7 @@ void request_info_cmd(char *buf, int len, socket_struct *ns) {
             break;
         }
     if (!strcmp(buf, "image_info"))
-        send_image_info(ns, params);
+        send_image_info(ns);
     else if (!strcmp(buf, "image_sums"))
         send_image_sums(ns, params);
     else if (!strcmp(buf, "skill_info"))
@@ -200,87 +190,115 @@ void request_info_cmd(char *buf, int len, socket_struct *ns) {
     SockList_Term(&sl);
 }
 
-/**
- * Handle client commands.
- *
- * We only get here once there is input, and only do basic connection checking.
- *
- * @param ns
- * socket sending the command. Will be set to Ns_Dead if read error.
- * @param pl
- * player associated to the socket. If NULL, only commands in client_cmd_mapping will be checked.
- */
-void handle_client(socket_struct *ns, player *pl) {
-    int len, i;
-    unsigned char *data;
-
-    /* Loop through this - maybe we have several complete packets here. */
-    while (1) {
-        /* If it is a player, and they don't have any speed left, we
-         * return, and will read in the data when they do have time.
-         */
-        if (pl && pl->state == ST_PLAYING && pl->ob != NULL && pl->ob->speed_left < 0) {
-            return;
-        }
-
-        i = SockList_ReadPacket(ns->fd, &ns->inbuf, sizeof(ns->inbuf.buf)-1);
-        if (i < 0) {
-#ifdef ESRV_DEBUG
-            LOG(llevDebug, "handle_client: Read error on connection player %s\n", (pl ? pl->ob->name : "None"));
-#endif
-            /* Caller will take care of cleaning this up */
-            ns->status = Ns_Dead;
-            return;
-        }
-        /* Still dont have a full packet */
-        if (i == 0)
-            return;
-
-        SockList_NullTerminate(&ns->inbuf); /* Terminate buffer - useful for string data */
-
-        /* First, break out beginning word.  There are at least
-         * a few commands that do not have any paremeters.  If
-         * we get such a command, don't worry about trying
-         * to break it up.
-         */
-        data = (unsigned char *)strchr((char *)ns->inbuf.buf+2, ' ');
-        if (data) {
-            *data = '\0';
-            data++;
-            len = ns->inbuf.len-(data-ns->inbuf.buf);
-        } else
-            len = 0;
-
-        for (i = 0; client_commands[i].cmdname != NULL; i++) {
-            if (strcmp((char *)ns->inbuf.buf+2, client_commands[i].cmdname) == 0) {
-                client_commands[i].cmdproc((char *)data, len, ns);
-                SockList_ResetRead(&ns->inbuf);
-                return;
-            }
-        }
-        /* Player must be in the playing state or the flag on the
-         * the command must be zero for the user to use the command -
-         * otherwise, a player cam save, be in the play_again state, and
-         * the map they were on getsswapped out, yet things that try to look
-         * at the map causes a crash.  If the command is valid, but
-         * one they can't use, we still swallow it up.
-         */
-        if (pl)
-            for (i = 0; player_commands[i].cmdname != NULL; i++) {
-                if (strcmp((char *)ns->inbuf.buf+2, player_commands[i].cmdname) == 0) {
-                    if (pl->state == ST_PLAYING || player_commands[i].flag == 0)
-                        player_commands[i].cmdproc((char *)data, len, pl);
-                    SockList_ResetRead(&ns->inbuf);
-                    return;
-                }
-            }
-        /* If we get here, we didn't find a valid command.  Logging
-         * this might be questionable, because a broken client/malicious
-         * user could certainly send a whole bunch of invalid commands.
-         */
-        LOG(llevDebug, "Bad command from client (%s)\n", ns->inbuf.buf+2);
-        SockList_ResetRead(&ns->inbuf);
+static int
+handle_cmd(socket_struct *ns, player *pl, char *cmd, char *data, int len) {
+    /* Fuzz testing indicated a way to get a null command here
+     * --> make an empty command, but have a length.
+     * So, if we get here with a null command, log it and exit the function.
+     * Daniel Hawkins 2020-01-16
+     */
+    if (cmd == NULL) {
+        LOG(llevDebug, "%s: missing command. Sending garbage?\n", ns->host);
+        return 0;
     }
+    for (int i = 0; client_commands[i].cmdname != NULL; i++) {
+        if (strcmp(cmd, client_commands[i].cmdname) == 0) {
+            if (client_commands[i].cmdproc != NULL) {
+                client_commands[i].cmdproc(data, len, ns);
+            }
+            return 0;
+        }
+    }
+    /* Player must be in the playing state or the flag on the
+     * the command must be zero for the user to use the command -
+     * otherwise, a player cam save, be in the play_again state, and
+     * the map they were on getsswapped out, yet things that try to look
+     * at the map causes a crash.  If the command is valid, but
+     * one they can't use, we still swallow it up.
+     */
+    if (pl) {
+        for (int i = 0; player_commands[i].cmdname != NULL; i++) {
+            if (strcmp(cmd, player_commands[i].cmdname) == 0) {
+                if (pl->state == ST_PLAYING || player_commands[i].flag == 0) {
+                    player_commands[i].cmdproc(data, len, pl);
+                }
+                return 1;
+            }
+        }
+    }
+    LOG(llevDebug, "%s: invalid command '%s'\n", ns->host, cmd);
+    return 0;
+}
+
+/**
+ * Handle commands from a client. This function should only be called when
+ * the socket is readable. If an error occurs, the socket status will be set
+ * to Ns_Dead; it is up to the caller to clean it up.
+ *
+ * @param ns Socket sending the command
+ * @param pl Player associated with this socket, or NULL
+ * @return If the main loop should continue processing this client.
+ */
+bool handle_client(socket_struct *ns, player *pl) {
+    /* Loop through this - maybe we have several complete packets here. */
+    /* Command_count is used to limit the number of requests from
+     * clients that have not logged in - we do not want an unauthenticated
+     * connection to spew us with hundreds of requests.  As such,
+     * this counter is only increased in the case of socket level commands.
+     * Note that this also has the effect of throttling down face and other
+     * socket commands from the client.  As such, if we have a player attached,
+     * we will process more of these, as getting a fair number when entering
+     * a map may not be uncommon.
+     */
+    int command_count = 0;
+    while (command_count < 5 || (pl && command_count < 25)) {
+        if (pl && pl->state == ST_PLAYING && pl->ob != NULL && pl->ob->speed_left < 0) {
+            // Skip processing players with no turns left.
+            return false;
+        }
+
+        int status = SockList_ReadPacket(ns->fd, &ns->inbuf, sizeof(ns->inbuf.buf)-1);
+        if (status != 1) {
+            if (status < 0) {
+                ns->status = Ns_Dead;
+            }
+            return false;
+        }
+
+        /* Since we have a full packet, reset last tick time. */
+        ns->last_tick = 0;
+
+        SockList_NullTerminate(&ns->inbuf);
+        assert(ns->inbuf.len >= 2);
+        char *data;
+        char *cmd = strtok_r((char *)ns->inbuf.buf + 2, " ", &data);
+
+        int got_player_cmd;
+        if (data != NULL) {
+            int rem = ns->inbuf.len - ((unsigned char *)data - ns->inbuf.buf);
+            got_player_cmd = handle_cmd(ns, pl, cmd, data, rem);
+        } else {
+            got_player_cmd = handle_cmd(ns, pl, cmd, NULL, 0);
+        }
+
+        SockList_ResetRead(&ns->inbuf);
+        if (got_player_cmd) {
+            return true;
+        }
+
+        command_count += 1;
+        /* Evil case, and not a nice workaround, but well...
+         * If we receive eg an accountplay command, the socket is copied
+         * to the player structure, and its faces_sent is set to NULL.
+         * This leads to issues when processing the next commands in the queue,
+         * especially if related to faces...
+         * So get out of here in this case, which we detect because faces_sent is NULL.
+         */
+        if (ns->faces_sent == NULL) {
+            command_count = 6;
+        }
+    }
+    return false;
 }
 
 /*****************************************************************************
@@ -318,7 +336,9 @@ void watchdog(void) {
 
 extern unsigned long todtick;
 
-/** Waits for new connection */
+/**
+ * Waits for new connection when there is no one connected.
+ */
 static void block_until_new_connection(void) {
     struct timeval Timeout;
     fd_set readfs;
@@ -339,7 +359,7 @@ static void block_until_new_connection(void) {
         FD_ZERO(&readfs);
         for (i = 0; i < socket_info.allocated_sockets && init_sockets[i].listen; i++)
             if (init_sockets[i].status == Ns_Add)
-                FD_SET((uint32)init_sockets[i].fd, &readfs);
+                FD_SET((uint32_t)init_sockets[i].fd, &readfs);
 
         /* If fastclock is set, we need to seriously slow down the updates
          * to the metaserver as well as watchdog.  Do same for flush_old_maps() -
@@ -389,6 +409,10 @@ static int is_fd_valid(int fd) {
 #endif
 }
 
+/**
+ * Handle a new connection from a client.
+ * @param listen_fd file descriptor the request came from.
+ */
 static void new_connection(int listen_fd) {
     int newsocknum = -1, j;
 #ifdef HAVE_GETNAMEINFO
@@ -397,7 +421,6 @@ static void new_connection(int listen_fd) {
     struct sockaddr_in addr;
 #endif
     socklen_t addrlen = sizeof(addr);
-    char err[MAX_BUF];
 
 #ifdef ESRV_DEBUG
     LOG(llevDebug, "do_server: New Connection\n");
@@ -417,8 +440,8 @@ static void new_connection(int listen_fd) {
         newsocknum = socket_info.allocated_sockets;
         socket_info.allocated_sockets++;
         init_sockets[newsocknum].listen = NULL;
-        init_sockets[newsocknum].faces_sent_len = nrofpixmaps;
-        init_sockets[newsocknum].faces_sent = calloc(1, nrofpixmaps*sizeof(*init_sockets[newsocknum].faces_sent));
+        init_sockets[newsocknum].faces_sent_len = get_faces_count();
+        init_sockets[newsocknum].faces_sent = calloc(get_faces_count(), sizeof(*init_sockets[newsocknum].faces_sent));
         if (!init_sockets[newsocknum].faces_sent)
             fatal(OUT_OF_MEMORY);
         init_sockets[newsocknum].status = Ns_Avail;
@@ -431,7 +454,7 @@ static void new_connection(int listen_fd) {
 
     init_sockets[newsocknum].fd = accept(listen_fd, (struct sockaddr *)&addr, &addrlen);
     if (init_sockets[newsocknum].fd == -1) {
-        LOG(llevError, "accept failed: %s\n", strerror_local(errno, err, sizeof(err)));
+        LOG(llevError, "accept failed: %s\n", strerror(errno));
     } else {
         char buf[MAX_BUF];
 #ifndef HAVE_GETNAMEINFO
@@ -458,6 +481,51 @@ static void new_connection(int listen_fd) {
     }
 }
 
+/**
+ * Check whether the given socket's connection is alive or not.
+ *
+ * @return True if connection is active, false if not.
+ */
+bool connection_alive(socket_struct socket) {
+    // If the client doesn't send heartbeats, assume it's connected.
+    if (!socket.heartbeat) {
+        return true;
+    }
+
+    // If a client message was received recently, it's connected.
+    if (socket.last_tick < tick_length(BEAT_INTERVAL + 1)) {
+        return true;
+    }
+
+    return false;
+}
+
+/**
+ * Send updated stats, map, look, and inventory to the player.
+ */
+static void send_updates(player *pl) {
+    /* Update the players stats once per tick.  More efficient than
+     * sending them whenever they change, and probably just as useful
+     */
+    esrv_update_stats(pl);
+    if (pl->last_weight != -1 && pl->last_weight != WEIGHT(pl->ob)) {
+        esrv_update_item(UPD_WEIGHT, pl->ob, pl->ob);
+        if (pl->last_weight != WEIGHT(pl->ob))
+            LOG(llevError, "esrv_update_item(UPD_WEIGHT) did not set player weight: is %lu, should be %lu\n", (unsigned long)pl->last_weight, (unsigned long)WEIGHT(pl->ob));
+    }
+    /* draw_client_map does sanity checking that map is
+     * valid, so don't do it here.
+     */
+    draw_client_map(pl->ob);
+    if (pl->socket.update_look)
+        esrv_draw_look(pl->ob);
+    if (pl->socket.update_inventory) {
+        if (pl->ob->container != NULL)
+            esrv_send_inventory(pl->ob, pl->ob->container);
+        pl->socket.update_inventory = 0;
+    }
+}
+
 /**
  * This checks the sockets for input and exceptions, does the right thing.
  *
@@ -466,26 +534,18 @@ static void new_connection(int listen_fd) {
  *
  */
 void do_server(void) {
-    int i, pollret, active = 0;
-    fd_set tmp_read, tmp_exceptions, tmp_write;
-    player *pl, *next;
-    char err[MAX_BUF];
-
-#ifdef CS_LOGSTATS
-    if ((time(NULL)-cst_lst.time_start) >= CS_LOGTIME)
-        write_cs_stats();
-#endif
-
+    fd_set tmp_read, tmp_exceptions;
+    int active = 0;
     FD_ZERO(&tmp_read);
-    FD_ZERO(&tmp_write);
     FD_ZERO(&tmp_exceptions);
 
-    for (i = 0; i < socket_info.allocated_sockets; i++) {
+    for (int i = 0; i < socket_info.allocated_sockets; i++) {
         if (init_sockets[i].status == Ns_Add && !is_fd_valid(init_sockets[i].fd)) {
             LOG(llevError, "do_server: invalid waiting fd %d\n", i);
             init_sockets[i].status = Ns_Dead;
         }
         if (init_sockets[i].status == Ns_Dead) {
+            LOG(llevInfo, "Disconnected from %s\n", init_sockets[i].host);
             if (init_sockets[i].listen) {
                 /* try to reopen the listening socket */
                 init_listening_socket(&init_sockets[i]);
@@ -494,9 +554,8 @@ void do_server(void) {
                 init_sockets[i].status = Ns_Avail;
             }
         } else if (init_sockets[i].status != Ns_Avail) {
-            FD_SET((uint32)init_sockets[i].fd, &tmp_read);
-            FD_SET((uint32)init_sockets[i].fd, &tmp_write);
-            FD_SET((uint32)init_sockets[i].fd, &tmp_exceptions);
+            FD_SET((uint32_t)init_sockets[i].fd, &tmp_read);
+            FD_SET((uint32_t)init_sockets[i].fd, &tmp_exceptions);
             active++;
         }
     }
@@ -504,6 +563,7 @@ void do_server(void) {
     /* Go through the players.  Let the loop set the next pl value,
      * since we may remove some
      */
+    player *pl, *next;
     for (pl = first_player; pl != NULL; ) {
         if (pl->socket.status != Ns_Dead && !is_fd_valid(pl->socket.fd)) {
             LOG(llevError, "do_server: invalid file descriptor for player %s [%s]: %d\n", (pl->ob && pl->ob->name) ? pl->ob->name : "(unnamed player?)", (pl->socket.host) ? pl->socket.host : "(unknown ip?)", pl->socket.fd);
@@ -518,9 +578,8 @@ void do_server(void) {
             final_free_player(pl);
             pl = npl;
         } else {
-            FD_SET((uint32)pl->socket.fd, &tmp_read);
-            FD_SET((uint32)pl->socket.fd, &tmp_write);
-            FD_SET((uint32)pl->socket.fd, &tmp_exceptions);
+            FD_SET((uint32_t)pl->socket.fd, &tmp_read);
+            FD_SET((uint32_t)pl->socket.fd, &tmp_exceptions);
             pl = pl->next;
         }
     }
@@ -528,28 +587,32 @@ void do_server(void) {
     if (active == 1 && first_player == NULL)
         block_until_new_connection();
 
-    /* Reset timeout each time, since some OS's will change the values on
-     * the return from select.
-     */
-    socket_info.timeout.tv_sec = 0;
-    socket_info.timeout.tv_usec = 0;
-
-    pollret = select(socket_info.max_filedescriptor, &tmp_read, &tmp_write, &tmp_exceptions, &socket_info.timeout);
-
-    if (pollret == -1) {
-        LOG(llevError, "select failed: %s\n", strerror_local(errno, err, sizeof(err)));
-        return;
+    long sleep_time = get_sleep_remaining();
+    if (sleep_time < 0) {
+        LOG(llevInfo, "skipping time (over by %ld ms)\n", -sleep_time/1000);
+        jump_time();
     }
 
-    /* We need to do some of the processing below regardless */
-    /*    if (!pollret) return;*/
+    while (sleep_time > 0) {
+        socket_info.timeout.tv_sec = 0;
+        socket_info.timeout.tv_usec = sleep_time;
+        int pollret = select(socket_info.max_filedescriptor, &tmp_read, NULL,
+                             &tmp_exceptions, &socket_info.timeout);
+        if (pollret == -1) {
+            if (errno != EINTR) {
+                LOG(llevError, "select failed: %s\n", strerror(errno));
+            }
+            return;
+        } else if (!pollret) {
+            return;
+        }
 
-    /* Check for any exceptions/input on the sockets */
-    if (pollret)
-        for (i = 0; i < socket_info.allocated_sockets; i++) {
+        /* Check for any exceptions/input on the sockets */
+        for (int i = 0; i < socket_info.allocated_sockets; i++) {
             /* listen sockets can stay in status Ns_Dead */
-            if (init_sockets[i].status != Ns_Add)
+            if (init_sockets[i].status != Ns_Add) {
                 continue;
+            }
             if (FD_ISSET(init_sockets[i].fd, &tmp_exceptions)) {
                 free_newsocket(&init_sockets[i]);
                 init_sockets[i].status = Ns_Avail;
@@ -561,81 +624,72 @@ void do_server(void) {
                 else
                     handle_client(&init_sockets[i], NULL);
             }
-            if (FD_ISSET(init_sockets[i].fd, &tmp_write)) {
-                init_sockets[i].can_write = 1;
-            }
         }
 
-    /* This does roughly the same thing, but for the players now */
-    for (pl = first_player; pl != NULL; pl = next) {
-        next = pl->next;
-        if (pl->socket.status == Ns_Dead)
-            continue;
-
-        if (FD_ISSET(pl->socket.fd, &tmp_write)) {
-            if (!pl->socket.can_write)  {
-                pl->socket.can_write = 1;
-                write_socket_buffer(&pl->socket);
-            }
-            /* if we get an error on the write_socket buffer, no reason to
-             * continue on this socket.
-             */
+        /* This does roughly the same thing, but for the players now */
+        for (pl = first_player; pl != NULL; pl = next) {
+            next = pl->next;
             if (pl->socket.status == Ns_Dead)
                 continue;
-        } else
-            pl->socket.can_write = 0;
 
-        if (FD_ISSET(pl->socket.fd, &tmp_exceptions)) {
-            save_player(pl->ob, 0);
-            leave(pl, 1);
-            final_free_player(pl);
-        } else {
-            handle_client(&pl->socket, pl);
-
-            /* There seems to be rare cases where next points to a removed/freed player.
-             * My belief is that this player does something (shout, move, whatever)
-             * that causes data to be sent to the next player on the list, but
-             * that player is defunct, so the socket codes removes that player.
-             * End result is that next now points at the removed player, and
-             * that has garbage data so we crash.  So update the next pointer
-             * while pl is still valid.  MSW 2007-04-21
-             */
-            next = pl->next;
-
-
-            /* If the player has left the game, then the socket status
-             * will be set to this be the leave function.  We don't
-             * need to call leave again, as it has already been called
-             * once.
-             */
-            if (pl->socket.status == Ns_Dead) {
+            if (FD_ISSET(pl->socket.fd, &tmp_exceptions)) {
                 save_player(pl->ob, 0);
                 leave(pl, 1);
                 final_free_player(pl);
             } else {
-                /* Update the players stats once per tick.  More efficient than
-                 * sending them whenever they change, and probably just as useful
-                 */
-                esrv_update_stats(pl);
-                if (pl->last_weight != -1 && pl->last_weight != WEIGHT(pl->ob)) {
-                    esrv_update_item(UPD_WEIGHT, pl->ob, pl->ob);
-                    if (pl->last_weight != WEIGHT(pl->ob))
-                        LOG(llevError, "esrv_update_item(UPD_WEIGHT) did not set player weight: is %lu, should be %lu\n", (unsigned long)pl->last_weight, (unsigned long)WEIGHT(pl->ob));
+                bool keep_processing = handle_client(&pl->socket, pl);
+                if (!keep_processing) {
+                    FD_CLR(pl->socket.fd, &tmp_read);
                 }
-                /* draw_client_map does sanity checking that map is
-                 * valid, so don't do it here.
+
+                /* There seems to be rare cases where next points to a removed/freed player.
+                 * My belief is that this player does something (shout, move, whatever)
+                 * that causes data to be sent to the next player on the list, but
+                 * that player is defunct, so the socket codes removes that player.
+                 * End result is that next now points at the removed player, and
+                 * that has garbage data so we crash.  So update the next pointer
+                 * while pl is still valid.  MSW 2007-04-21
                  */
-                draw_client_map(pl->ob);
-                if (pl->socket.update_look)
-                    esrv_draw_look(pl->ob);
-                if (pl->socket.update_inventory) {
-                    if (pl->ob->container != NULL)
-                        esrv_send_inventory(pl->ob, pl->ob->container);
-                    pl->socket.update_inventory = 0;
+                next = pl->next;
+
+
+                /* If the player has left the game, then the socket status
+                 * will be set to this be the leave function.  We don't
+                 * need to call leave again, as it has already been called
+                 * once.
+                 */
+                if (pl->socket.status == Ns_Dead) {
+                    save_player(pl->ob, 0);
+                    leave(pl, 1);
+                    final_free_player(pl);
+                } else {
+                    send_updates(pl);
                 }
-                if (pl->socket.tick)
-                    send_tick(pl);
             }
         }
+        sleep_time = get_sleep_remaining();
+    }
+}
+
+/**
+ * Send updates to players. Called once per tick.
+ */
+void update_players() {
+    player *pl, *next;
+    for (pl = first_player; pl != NULL; pl = next) {
+        next = pl->next;
+        send_updates(pl);
+        /* Increment time since last contact only if logged in. */
+        if (pl->state == ST_PLAYING) {
+            pl->socket.last_tick++;
+
+            if (!connection_alive(pl->socket)) {
+                // TODO: Handle a lost client connection.
+                LOG(llevDebug, "Lost client connection!\n");
+            }
+
+            if (pl->socket.tick)
+                send_tick(pl);
+        }
     }
 }
