@@ -1,30 +1,16 @@
 /*
- * static char *rcsid_sockets_c =
- *    "$Id: lowlevel.c 14914 2011-07-24 10:02:41Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The author can be reached via e-mail to mark@pyramid.com
-*/
-
 /**
  * \file
  * Low-level socket-related functions.
@@ -37,12 +23,16 @@
  * CFCLIENT defined as part of its compile flags.
  */
 
+#include "global.h"
+
 #include <assert.h>
-#include <stdarg.h>
-#include <global.h>
-#include <shared/newclient.h>
-#include <sproto.h>
 #include <errno.h>
+#include <stdarg.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "shared/newclient.h"
+#include "sproto.h"
 
 /***********************************************************************
  *
@@ -105,7 +95,7 @@ static void SockList_Ensure(const SockList *sl, size_t size) {
  * @param sl the SockList instance to add to
  * @param data the value to add
  */
-void SockList_AddChar(SockList *sl, char data) {
+void SockList_AddChar(SockList *sl, unsigned char data) {
     SockList_Ensure(sl, 1);
     sl->buf[sl->len++] = data;
 }
@@ -115,7 +105,7 @@ void SockList_AddChar(SockList *sl, char data) {
  * @param sl the SockList instance to add to
  * @param data the value to add
  */
-void SockList_AddShort(SockList *sl, uint16 data) {
+void SockList_AddShort(SockList *sl, uint16_t data) {
     SockList_Ensure(sl, 2);
     sl->buf[sl->len++] = (data>>8)&0xff;
     sl->buf[sl->len++] = data&0xff;
@@ -126,7 +116,7 @@ void SockList_AddShort(SockList *sl, uint16 data) {
  * @param sl the SockList instance to add to
  * @param data the value to add
  */
-void SockList_AddInt(SockList *sl, uint32 data) {
+void SockList_AddInt(SockList *sl, uint32_t data) {
     SockList_Ensure(sl, 4);
     sl->buf[sl->len++] = (data>>24)&0xff;
     sl->buf[sl->len++] = (data>>16)&0xff;
@@ -139,7 +129,7 @@ void SockList_AddInt(SockList *sl, uint32 data) {
  * @param sl the SockList instance to add to
  * @param data the value to add
  */
-void SockList_AddInt64(SockList *sl, uint64 data) {
+void SockList_AddInt64(SockList *sl, uint64_t data) {
     SockList_Ensure(sl, 8);
     sl->buf[sl->len++] = (char)((data>>56)&0xff);
     sl->buf[sl->len++] = (char)((data>>48)&0xff);
@@ -276,7 +266,6 @@ short GetShort_String(const unsigned char *data) {
  */
 int SockList_ReadPacket(int fd, SockList *sl, int len) {
     int stat, toread;
-    char err[MAX_BUF];
 
     /* We already have a partial packet */
     if (sl->len < 2) {
@@ -304,11 +293,11 @@ int SockList_ReadPacket(int fd, SockList *sl, int len) {
             }
 #else
             if (errno == ECONNRESET) {
-                LOG(llevDebug, "ReadPacket got error %s, returning -1\n", strerror_local(errno, err, sizeof(err)));
+                LOG(llevDebug, "ReadPacket got error %s, returning -1\n", strerror(errno));
                 return -1;
             }
             if (errno != EAGAIN && errno != EWOULDBLOCK) {
-                LOG(llevDebug, "ReadPacket got error %s, returning 0\n", strerror_local(errno, err, sizeof(err)));
+                LOG(llevDebug, "ReadPacket got error %s, returning 0\n", strerror(errno));
             }
 #endif
             return 0; /*Error */
@@ -334,10 +323,11 @@ int SockList_ReadPacket(int fd, SockList *sl, int len) {
          * shouldn't hurt.
          */
 #ifdef WIN32 /* ***win32 SockList_ReadPacket: change read() to recv() */
-        recv(fd, sl->buf+2, 100, 0);
+        stat = recv(fd, sl->buf+2, 100, 0);
 #else
-        read(fd, sl->buf+2, 100);
+        stat = read(fd, sl->buf+2, 100);
 #endif /* end win32 */
+        (void) stat; // Don't care how much we read; avoid complier warnings
 
         /* return error so the socket is closed */
         return -1;
@@ -362,7 +352,7 @@ int SockList_ReadPacket(int fd, SockList *sl, int len) {
             }
 #else
             if (errno != EAGAIN && errno != EWOULDBLOCK) {
-                LOG(llevDebug, "ReadPacket got error %s, returning 0\n", strerror_local(errno, err, sizeof(err)));
+                LOG(llevDebug, "ReadPacket got error %s, returning 0\n", strerror(errno));
             }
 #endif
             return 0; /*Error */
@@ -391,98 +381,6 @@ int SockList_ReadPacket(int fd, SockList *sl, int len) {
  *
  ******************************************************************************/
 
-/**
- * Adds data to a socket buffer for whatever reason.
- *
- * ns is the socket we are adding the data to, buf is the start of the
- * data, and len is the number of bytes to add.
- */
-static void add_to_buffer(socket_struct *ns, const unsigned char *buf, int len) {
-    int avail, end;
-
-    if ((len+ns->outputbuffer.len) > SOCKETBUFSIZE) {
-        LOG(llevDebug, "Socket on fd %d has overrun internal buffer - marking as dead\n", ns->fd);
-        ns->status = Ns_Dead;
-        return;
-    }
-
-    /* data + end is where we start putting the new data.  The last byte
-     * currently in use is actually data + end -1
-     */
-
-    end = ns->outputbuffer.start+ns->outputbuffer.len;
-    /* The buffer is already in a wrapped state, so adjust end */
-    if (end >= SOCKETBUFSIZE)
-        end -= SOCKETBUFSIZE;
-    avail = SOCKETBUFSIZE-end;
-
-    /* We can all fit it behind the current data without wrapping */
-    if (avail >= len) {
-        memcpy(ns->outputbuffer.data+end, buf, len);
-    } else {
-        memcpy(ns->outputbuffer.data+end, buf, avail);
-        memcpy(ns->outputbuffer.data, buf+avail, len-avail);
-    }
-    ns->outputbuffer.len += len;
-}
-
-/**
- * Writes data to socket.
- *
- * When the socket is clear to write, and we have backlogged data, this
- * is called to write it out.
- */
-void write_socket_buffer(socket_struct *ns) {
-    int amt, max;
-
-    if (ns->outputbuffer.len == 0) {
-        LOG(llevDebug, "write_socket_buffer called when there is no data, fd=%d\n", ns->fd);
-        return;
-    }
-
-    do {
-        max = SOCKETBUFSIZE-ns->outputbuffer.start;
-        if (ns->outputbuffer.len < max)
-            max = ns->outputbuffer.len;
-
-#ifdef WIN32 /* ***win32 write_socket_buffer: change write() to send() */
-        amt = send(ns->fd, ns->outputbuffer.data+ns->outputbuffer.start, max, 0);
-#else
-        do {
-            amt = write(ns->fd, ns->outputbuffer.data+ns->outputbuffer.start, max);
-        } while ((amt < 0) && (errno == EINTR));
-#endif
-
-        if (amt < 0) { /* We got an error */
-#ifdef WIN32 /* ***win32 write_socket_buffer: change error handling */
-            if (amt == -1 && WSAGetLastError() != WSAEWOULDBLOCK) {
-                LOG(llevError, "New socket write failed (wsb) (%d).\n", WSAGetLastError());
-#else
-            if (errno != EWOULDBLOCK) {
-                char err[MAX_BUF];
-
-                LOG(llevError, "New socket write failed (wsb) (%d: %s).\n", errno, strerror_local(errno, err, sizeof(err)));
-#endif
-                ns->status = Ns_Dead;
-                return;
-            } else { /* EWOULDBLOCK */
-                /* can't write it, so store it away. */
-                ns->can_write = 0;
-                return;
-            }
-        }
-        ns->outputbuffer.start += amt;
-        /* wrap back to start of buffer */
-        if (ns->outputbuffer.start == SOCKETBUFSIZE)
-            ns->outputbuffer.start = 0;
-        ns->outputbuffer.len -= amt;
-#ifdef CS_LOGSTATS
-        cst_tot.obytes += amt;
-        cst_lst.obytes += amt;
-#endif
-    } while (ns->outputbuffer.len > 0);
-}
-
 /**
  * This writes data to the socket. - It is very low level -
  * all we try and do is write out the data to the socket
@@ -490,61 +388,42 @@ void write_socket_buffer(socket_struct *ns) {
  * of bytes to write.  IT doesn't return anything - rather, it
  * updates the ns structure if we get an  error.
  */
-static void Write_To_Socket(socket_struct *ns, const unsigned char *buf, int len) {
-    int amt = 0;
-    const unsigned char *pos = buf;
-    char err[MAX_BUF];
-
+static void Write_To_Socket(socket_struct* ns, const unsigned char* buf, const int len) {
     if (ns->status == Ns_Dead || !buf) {
         LOG(llevDebug, "Write_To_Socket called with dead socket\n");
         return;
     }
 
-#ifndef __GNU__ /* This caused problems on Hurd */
-    if (!ns->can_write) {
-        add_to_buffer(ns, buf, len);
+    const int amt = send(ns->fd, buf, len, 0);
+    if (amt < 0) { /* We got an error */
+#ifdef WIN32       /* ***win32 Write_To_Socket: change error handling */
+        if (amt == -1 && WSAGetLastError() != WSAEWOULDBLOCK) {
+            LOG(llevInfo, "New socket write failed WTS (%d).\n",
+                WSAGetLastError());
+#else
+        if (errno != EWOULDBLOCK) {
+            LOG(llevInfo, "New socket write failed WTS: %s\n",
+                strerror(errno));
+#endif
+            ns->status = Ns_Dead;
+            return;
+        } else { /* EWOULDBLOCK */
+            LOG(llevError,
+                "Write_To_Socket: write would block; disconnecting. Try "
+                "increasing SOCKETBUFSIZE.\n");
+            ns->status = Ns_Dead;
+            return;
+        }
+    } else if (amt != len) {
+        LOG(llevError, "Write_To_Socket: write wrote less than requested; "
+                       "disconnecting. Try increasing SOCKETBUFSIZE.\n");
+        ns->status = Ns_Dead;
         return;
     }
-#endif
-    /* If we manage to write more than we wanted, take it as a bonus */
-    while (len > 0) {
-#ifdef WIN32 /* ***win32 Write_To_Socket: change write() to send() */
-        amt = send(ns->fd, pos, len, 0);
-#else
-        do {
-            amt = write(ns->fd, pos, len);
-        } while ((amt < 0) && (errno == EINTR));
-#endif
-
-        if (amt < 0) { /* We got an error */
-#ifdef WIN32 /* ***win32 Write_To_Socket: change error handling */
-            if (amt == -1 && WSAGetLastError() != WSAEWOULDBLOCK) {
-                LOG(llevError, "New socket write failed WTS (%d).\n", WSAGetLastError());
-#else
-            if (errno != EWOULDBLOCK) {
-                LOG(llevError, "New socket write failed WTS (%d: %s).\n", errno, strerror_local(errno, err, sizeof(err)));
-#endif
-                ns->status = Ns_Dead;
-                return;
-            } else { /* EWOULDBLOCK */
-                /* can't write it, so store it away. */
-                add_to_buffer(ns, pos, len);
-                ns->can_write = 0;
-                return;
-            }
-        /* amt gets set to 0 above in blocking code, so we do this as
-         * an else if to make sure we don't reprocess it.
-         */
-        } else if (amt == 0) {
-            LOG(llevError, "Write_To_Socket: No data written out.\n");
-        }
-        len -= amt;
-        pos += amt;
 #ifdef CS_LOGSTATS
-        cst_tot.obytes += amt;
-        cst_lst.obytes += amt;
+    cst_tot.obytes += amt;
+    cst_lst.obytes += amt;
 #endif
-    }
 }
 
 /**
