@@ -1,38 +1,28 @@
 /*
- * static char *rcsid_metaserver_c =
- *    "$Id: metaserver.c 15225 2011-10-21 17:41:22Z ryo_saeba $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-    CrossFire, A Multiplayer game for X-windows
-
-    Copyright (C) 2002,2007 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
 /**
  * \file
- * \date 2003-12-02
  * Meta-server related functions.
+ * \date 2003-12-02
  */
 
-#include <global.h>
+#include "global.h"
+
+#include <ctype.h>
+#include <pthread.h>
+#include <stdlib.h>
+#include <string.h>
 
 #ifndef WIN32 /* ---win32 exclude unix header files */
 #include <sys/types.h>
@@ -42,11 +32,10 @@
 
 #endif /* end win32 */
 
-#include <pthread.h>
-#include <metaserver2.h>
-#include <version.h>
+#include "metaserver2.h"
+#include "version.h"
 
-#ifdef HAVE_CURL_CURL_H
+#ifdef HAVE_LIBCURL
 #include <curl/curl.h>
 #include <curl/easy.h>
 #endif
@@ -61,6 +50,7 @@ void metaserver_update(void) {
     char num_players = 0;
     player *pl;
 
+#ifdef HAVE_LIBCURL
     /* We could use socket_info.nconns, but that is not quite as accurate,
      * as connections in the progress of being established, are listening
      * but don't have a player, etc.  The checks below are basically the
@@ -92,9 +82,10 @@ void metaserver_update(void) {
     metaserver2_updateinfo.out_bytes = cst_tot.obytes;
     metaserver2_updateinfo.uptime  = (long)time(NULL)-cst_tot.time_start;
     pthread_mutex_unlock(&ms2_info_mutex);
+#endif
 }
 
-/**
+/*
  * Start of metaserver2 logic
  * Note: All static structures in this file should be treated as strictly
  * private.  The metaserver2 update logic runs in its own thread,
@@ -106,12 +97,12 @@ void metaserver_update(void) {
  * This is a linked list of all the metaservers -
  * never really know how many we have.
  */
-
 typedef struct _MetaServer2 {
-    char *hostname;
-    struct _MetaServer2 *next;
+    char *hostname;             /**< Hostname to contact. */
+    struct _MetaServer2 *next;  /**< Next element in the list. */
 } MetaServer2;
 
+/** Metaservers to send information to. */
 static MetaServer2 *metaserver2;
 
 /**
@@ -123,22 +114,24 @@ static MetaServer2 *metaserver2;
  * or something)
  */
 typedef struct _LocalMeta2Info {
-    int     notification;  /* if true, do updates to metaservers */
-    char    *hostname;     /* Hostname of this server */
-    int     portnumber;    /* Portnumber of this server */
-    char    *html_comment; /* html comment to send to metaservers */
-    char    *text_comment; /* text comment to send to metaservers */
-    char    *archbase;     /* Different sources for arches, maps */
-    char    *mapbase;      /* and server */
+    int     notification;  /**< If true, do updates to metaservers. */
+    char    *hostname;     /**< Hostname of this server. */
+    int     portnumber;    /**< Portnumber of this server. */
+    char    *html_comment; /**< html comment to send to metaservers. */
+    char    *text_comment; /**< text comment to send to metaservers. */
+    char    *archbase;     /**< Different sources for arches, maps */
+    char    *mapbase;      /**< and server. */
     char    *codebase;
-    char    *flags;        /* Short flags to send to metaserver */
+    char    *flags;        /**< Short flags to send to metaserver. */
 } LocalMeta2Info;
 
+/** Non volatile information on the server. */
 static LocalMeta2Info local_info;
 
-/* These two are globals, but we declare them here. */
+/** Mutex to protect access to ::metaserver2_updateinfo. */
 pthread_mutex_t ms2_info_mutex;
 
+/** Statistics on players and such sent to the metaserver2. */
 MetaServer2_UpdateInfo metaserver2_updateinfo;
 
 /**
@@ -146,6 +139,7 @@ MetaServer2_UpdateInfo metaserver2_updateinfo;
  * including the pointer itself.  Caller is responsible for updating
  * pointers (ms->next) - really only used when wanting to free
  * all data.
+ * @param ms data to free, pointer becomes invalid.
  */
 static void free_metaserver2(MetaServer2 *ms) {
     free(ms->hostname);
@@ -175,7 +169,7 @@ int metaserver2_init(void) {
 
     dummy[0] = '\0';
 
-#ifdef HAVE_CURL_CURL_H
+#ifdef HAVE_LIBCURL
     if (!has_init) {
         memset(&local_info, 0, sizeof(LocalMeta2Info));
         memset(&metaserver2_updateinfo, 0, sizeof(MetaServer2_UpdateInfo));
@@ -293,7 +287,7 @@ int metaserver2_init(void) {
     if (!local_info.hostname)
         local_info.notification = 0;
 
-#ifndef HAVE_CURL_CURL_H
+#ifndef HAVE_LIBCURL
     if (local_info.notification) {
         LOG(llevError, "metaserver2 file is set to do notification, but libcurl is not found.\n");
         LOG(llevError, "Either fix your compilation, or turn off metaserver2 notification in \n");
@@ -340,26 +334,20 @@ int metaserver2_init(void) {
  * We treat the data as a string.  We should really pay attention to the
  * header data, and do something clever if we get 404 codes
  * or the like.
+ * @param ptr actual data.
+ * @param size size of the data.
+ * @param nmemb number of elements.
+ * @param data user-provided data, unused.
+ * @return number of bytes processed, always the full size.
  */
 static size_t metaserver2_writer(void *ptr, size_t size, size_t nmemb, void *data) {
     size_t realsize = size*nmemb;
-
-    LOG(llevDebug, "metaserver2_writer- Start of text:\n%s\n", (const char*)ptr);
-    LOG(llevDebug, "metaserver2_writer- End of text:\n");
-
+    LOG(llevError, "Message from metaserver:\n%s\n", (const char*)ptr);
     return realsize;
 }
 
-
-/**
- * This sends an update to the various metaservers.
- * It generates the form, and then sends it to the
- * server
- */
-static void metaserver2_updates(void) {
-#ifdef HAVE_CURL_CURL_H
-    MetaServer2 *ms2;
-    struct curl_httppost *formpost = NULL;
+#ifdef HAVE_LIBCURL
+static void metaserver2_build_form(struct curl_httppost **formpost) {
     struct curl_httppost *lastptr = NULL;
     char buf[MAX_BUF];
 
@@ -369,43 +357,43 @@ static void metaserver2_updates(void) {
      * The string after CURLFORM_COPYNAME is the name of the POST variable
      * as the
      */
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "hostname",
                  CURLFORM_COPYCONTENTS, local_info.hostname,
                  CURLFORM_END);
 
     snprintf(buf, sizeof(buf), "%d", local_info.portnumber);
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "port",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
 
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "html_comment",
                  CURLFORM_COPYCONTENTS, local_info.html_comment,
                  CURLFORM_END);
 
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "text_comment",
                  CURLFORM_COPYCONTENTS, local_info.text_comment,
                  CURLFORM_END);
 
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "archbase",
                  CURLFORM_COPYCONTENTS, local_info.archbase,
                  CURLFORM_END);
 
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "mapbase",
                  CURLFORM_COPYCONTENTS, local_info.mapbase,
                  CURLFORM_END);
 
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "codebase",
                  CURLFORM_COPYCONTENTS, local_info.codebase,
                  CURLFORM_END);
 
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "flags",
                  CURLFORM_COPYCONTENTS, local_info.flags,
                  CURLFORM_END);
@@ -413,25 +401,25 @@ static void metaserver2_updates(void) {
     pthread_mutex_lock(&ms2_info_mutex);
 
     snprintf(buf, sizeof(buf), "%d", metaserver2_updateinfo.num_players);
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "num_players",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
 
     snprintf(buf, sizeof(buf), "%d", metaserver2_updateinfo.in_bytes);
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "in_bytes",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
 
     snprintf(buf, sizeof(buf), "%d", metaserver2_updateinfo.out_bytes);
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "out_bytes",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
 
-    snprintf(buf, sizeof(buf), "%ld", metaserver2_updateinfo.uptime);
-    curl_formadd(&formpost, &lastptr,
+    snprintf(buf, sizeof(buf), "%ld", (long)metaserver2_updateinfo.uptime);
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "uptime",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
@@ -441,28 +429,37 @@ static void metaserver2_updates(void) {
     /* Following few fields are global variables,
      * but are really defines, so won't ever change.
      */
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "version",
                  CURLFORM_COPYCONTENTS, FULL_VERSION,
                  CURLFORM_END);
 
     snprintf(buf, sizeof(buf), "%d", VERSION_SC);
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "sc_version",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
 
     snprintf(buf, sizeof(buf), "%d", VERSION_CS);
-    curl_formadd(&formpost, &lastptr,
+    curl_formadd(formpost, &lastptr,
                  CURLFORM_COPYNAME, "cs_version",
                  CURLFORM_COPYCONTENTS, buf,
                  CURLFORM_END);
+}
+#endif
 
-    for (ms2 = metaserver2; ms2; ms2 = ms2->next) {
-        CURL *curl;
-        CURLcode res;
+/**
+ * This sends an update to the various metaservers.
+ * It generates the form, and then sends it to the
+ * server
+ */
+static void metaserver2_updates(void) {
+#ifdef HAVE_LIBCURL
+    struct curl_httppost *formpost = NULL;
+    metaserver2_build_form(&formpost);
 
-        curl = curl_easy_init();
+    for (MetaServer2 *ms2 = metaserver2; ms2; ms2 = ms2->next) {
+        CURL *curl = curl_easy_init();
         if (curl) {
             /* what URL that receives this POST */
             curl_easy_setopt(curl, CURLOPT_URL, ms2->hostname);
@@ -473,13 +470,19 @@ static void metaserver2_updates(void) {
              * we want to take care of it ourselves.
              */
             curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, metaserver2_writer);
-            res = curl_easy_perform(curl);
 
-            if (res)
-                fprintf(stderr, "easy_perform got error %d\n", res);
+            char errbuf[CURL_ERROR_SIZE];
+            curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errbuf);
+
+            CURLcode res = curl_easy_perform(curl);
+            if (res) {
+                LOG(llevError, "metaserver update failed: %s\n", errbuf);
+            }
 
             /* always cleanup */
             curl_easy_cleanup(curl);
+        } else {
+            LOG(llevError, "metaserver: could not initialize curl\n");
         }
     }
     /* then cleanup the formpost chain */
@@ -496,6 +499,7 @@ static void metaserver2_updates(void) {
  * do a time() call and see how long the update takes,
  * and sleep according to that.
  *
+ * @param junk unused.
  * @return
  * This function should never return/exit.
  */
