@@ -1,31 +1,16 @@
 /*
- * static char *rcsid_requestinfo_c =
- *    "$Id: request.c 13308 2010-05-28 06:12:12Z kbulgrien $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/*
-  CrossFire, A Multiplayer game for X-windows
-
-  Copyright (C) 2001-2006,2010 Mark Wedel
-  Copyright (C) 1992 Frank Tore Johansen
-
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-  The author can be reached via e-mail to crossfire-devel@real-time.com
-*/
-
 /**
  * \file
  * requestinfo protocol handling.
@@ -38,14 +23,14 @@
  *
  */
 
-#include <assert.h>
-#include <global.h>
-#include <sproto.h>
+#include "global.h"
 
-#include <shared/newclient.h>
-#include <newserver.h>
-#include <living.h>
-#include <commands.h>
+#include <assert.h>
+#include <fcntl.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/time.h>
+#include <unistd.h>
 
 /* This block is basically taken from socket.c - I assume if it works there,
  * it should work here.
@@ -58,15 +43,12 @@
 #include <netdb.h>
 #endif /* win32 */
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-
+#include "commands.h"
+#include "living.h"
+#include "newserver.h"
+#include "shared/newclient.h"
 #include "sounds.h"
+#include "sproto.h"
 
 /* Note that following protocol commands (was corresponding function)
  * are in image.c and not this file, even though they are requestinfo
@@ -88,7 +70,7 @@ void send_skill_info(socket_struct *ns, char *params) {
 
     SockList_Init(&sl);
     SockList_AddString(&sl, "replyinfo skill_info\n");
-    for (i = 1; i < NUM_SKILLS; i++) {
+    for (i = 0; i < MAX_SKILLS && skill_names[i]; i++) {
         size_t len;
 
         len = 16+strlen(skill_names[i]); /* upper bound for length */
@@ -98,9 +80,9 @@ void send_skill_info(socket_struct *ns, char *params) {
         }
 
         if (params != NULL && *params == '1') {
-            if ((skill_faces[i] != -1) && !(ns->faces_sent[skill_faces[i]]&NS_FACESENT_FACE))
+            if ((skill_faces[i] != NULL) && !(ns->faces_sent[skill_faces[i]->number]&NS_FACESENT_FACE))
                 esrv_send_face(ns, skill_faces[i], 0);
-            SockList_AddPrintf(&sl, "%d:%s:%d\n", i+CS_STAT_SKILLINFO, skill_names[i], skill_faces[i]);
+            SockList_AddPrintf(&sl, "%d:%s:%d\n", i+CS_STAT_SKILLINFO, skill_names[i], skill_faces[i] ? skill_faces[i]->number : 0);
         } else
             SockList_AddPrintf(&sl, "%d:%s\n", i+CS_STAT_SKILLINFO, skill_names[i]);
     }
@@ -139,7 +121,7 @@ void send_spell_paths(socket_struct *ns, char *params) {
 void send_exp_table(socket_struct *ns, char *params) {
     SockList sl;
     int i;
-    extern sint64 *levels;
+    extern int64_t *levels;
 
     SockList_Init(&sl);
     SockList_AddString(&sl, "replyinfo exp_table\n");
@@ -177,8 +159,8 @@ void send_exp_table(socket_struct *ns, char *params) {
  */
 static void send_arch_info(SockList *sl, const object *op)
 {
-    SockList_AddString(sl, "name ");
     if (op->name) {
+        SockList_AddString(sl, "name ");
         SockList_AddLen8Data(sl, op->name, strlen(op->name));
     }
 
@@ -518,11 +500,11 @@ void send_new_char_info(socket_struct *ns) {
              settings.starting_stat_min, settings.starting_stat_max);
     SockList_AddLen8Data(&sl, buf, strlen(buf) + 1);
 
-    snprintf(buf, MAX_BUF, "V statname ");
+    snprintf(buf, MAX_BUF, "V statname");
     len = strlen(buf);
     for (i=0; i<NUM_STATS; i++) {
-        safe_strcat(buf, short_stat_name[i], &len, MAX_BUF);
         safe_strcat(buf, " ", &len, MAX_BUF);
+        safe_strcat(buf, short_stat_name[i], &len, MAX_BUF);
     }
 
     SockList_AddLen8Data(&sl, buf, strlen(buf) + 1);
