@@ -1,10 +1,16 @@
 /*
- * static char *rcsid_sound_c =
- *   "$Id: sounds.c 11218 2009-01-22 20:03:52Z akirschbaum $";
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
  */
 
-/* Send bug reports to Raphael Quinet (quinet@montefiore.ulg.ac.be) */
-
 /**
  * \file
  * Sound-related functions.
@@ -12,9 +18,14 @@
  * \date 2003-12-02
  */
 
-#include <global.h>
-#include <sproto.h>
-#include <sounds.h>
+#include "global.h"
+
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "sounds.h"
+#include "sproto.h"
 
 /**
  * Maximum distance a player may hear a sound from.
@@ -37,7 +48,7 @@
  * @param action
  * sound name to play.
  */
-void play_sound_player_only(player *pl, sint8 sound_type, object *emitter, int dir, const char *action) {
+void play_sound_player_only(player *pl, int8_t sound_type, object *emitter, int dir, const char *action) {
     SockList sl;
     int volume = 50;
     sstring name;
@@ -52,8 +63,14 @@ void play_sound_player_only(player *pl, sint8 sound_type, object *emitter, int d
 
     source = emitter->map ? emitter : emitter->env;
 
-    if ((RANDOM()%100) >= emitter->sound_chance)
-        return;
+    // Approximate the distance to the emitter from the source
+    int dx = FABS(source->x-pl->ob->x),
+        dy = FABS(source->y-pl->ob->y);
+    int distance = (MIN(dx, dy) * 3 + FABS(dx-dy) * 2) / 2;
+    // Make the sound dissipation more gradual.
+    distance >>= 1;
+    // Downscale the volume by distance
+    volume = distance ? volume / distance : volume;
 
     pl->socket.sounds_this_tick = 0;
 
@@ -63,8 +80,8 @@ void play_sound_player_only(player *pl, sint8 sound_type, object *emitter, int d
 
     SockList_Init(&sl);
     SockList_AddString(&sl, "sound2 ");
-    SockList_AddChar(&sl, (sint8)(source->x-pl->ob->x));
-    SockList_AddChar(&sl, (sint8)(source->y-pl->ob->y));
+    SockList_AddChar(&sl, (int8_t)(source->x-pl->ob->x));
+    SockList_AddChar(&sl, (int8_t)(source->y-pl->ob->y));
     SockList_AddChar(&sl, dir);
     SockList_AddChar(&sl, volume);
     SockList_AddChar(&sl, sound_type);
@@ -87,13 +104,10 @@ void play_sound_player_only(player *pl, sint8 sound_type, object *emitter, int d
  * @param action
  * sound name to play.
  */
-void play_sound_map(sint8 sound_type, object *emitter, int dir, const char *action) {
+void play_sound_map(int8_t sound_type, object *emitter, int dir, const char *action) {
     player *pl;
     object *source;
 
-    if ((RANDOM()%100) >= emitter->sound_chance)
-        return;
-
     if (!emitter->map && !(emitter->env && emitter->env->map))
         return;
 
@@ -130,3 +144,17 @@ void send_background_music(player *pl, const char *music) {
     Send_With_Handling(&pl->socket, &sl);
     SockList_Term(&sl);
 }
+
+static char const* pick_bg_music(mapstruct map[static 1]) {
+    if (map->background_music != NULL) {
+        return map->background_music;
+    }
+    return get_name_of_region_for_map(map);
+}
+
+void player_update_bg_music(object player[static 1]) {
+    assert(player->contr);
+    assert(player->type == PLAYER);
+    send_background_music(player->contr, pick_bg_music(player->map));
+}
+
