@@ -36,11 +36,11 @@
 #include <global.h>
 #include <sproto.h>
 
-void setup(void) {
+static void setup(void) {
     /* put any initialisation steps here, they will be run before each testcase */
 }
 
-void teardown(void) {
+static void teardown(void) {
     /* put any cleanup steps here, they will be run after each testcase */
 }
 
@@ -212,10 +212,7 @@ static void local_check_loaded_object(object *op) {
 
     /* objects now have a materialname.  try to patch it in */
     if (!(IS_WEAPON(op) && op->level > 0)) {
-        if (op->map != NULL)
-            set_materialname(op, op->map->difficulty, NULL);
-        else
-            set_materialname(op, 5, NULL);
+        set_materialname(op);
     }
     /* only do these when program is first run - a bit
      * excessive to do this at every run - most of this is
@@ -421,7 +418,7 @@ START_TEST(test_randommaps) {
 }
 END_TEST
 
-Suite *bug_suite(void) {
+static Suite *bug_suite(void) {
     Suite *s = suite_create("bug");
     TCase *tc_core = tcase_create("Core");
 
