@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_check_item_c =
- *   "$Id: check_item.c 15379 2011-11-01 19:40:45Z ryo_saeba $";
+ *   "$Id$";
  */
 
 /*
@@ -92,39 +92,39 @@ START_TEST(test_describe_item) {
         NULL };
     static const char *arch_results[] = {
         "",
-        "(food+200)",
+        "(food+600)",
         "(glowing)",
         "",
-        "",
-        "",
+        "of adornment",
+        "of adornment",
         "(slow movement)(wield weapon)(archer)(wear armour)(wear ring)",
         "",
         "(dam+6)(weapon speed 10)(Attacks: physical)",
         "(wc+1)(dam+15)(Attacks: physical)",
-        "(Int+2)(Pow+2)(ac+2)(item_power +5)(magic+1)(armour +5)",
-        "(Str+1)(Wis+2)(Cha+2)(dam+15)(item_power +25)(weapon speed 6)(Attacks: weaponmagic, blinding)(resist magic +30)(resist drain +100)",
+        "(Int+2)(Pow+2)(ac+2)(item_power +5)(magic+1)[color=#FF15CD](armour +5)[/color]",
+        "(Str+1)(Wis+2)(Cha+2)(dam+15)(item_power +25)(weapon speed 6)(Attacks: weaponmagic, blinding)[color=#930C76](resist magic +30)[/color][ul](resist drain +100)[/ul]",
         "(dam+8)(weapon speed 8)(Attacks: physical)",
-        "(ac+1)(armour +5)",
+        "(ac+1)[color=#FF15CD](armour +5)[/color]",
         "(food+600)",
-        "(food+100)(slay vial_poison:poison)",
+        "(food+65)(slay vial_poison:poison)",
         "",
         "(extremely fast movement)(undead)(spellcaster)(Spell abilities:)(paralyze)(fear ability)(cause black death)(cause red death)(face of death)(meteor swarm)(hellfire)(burning hands)(large fireball)(mass confusion)(negative energy bolt)(negative energy ball)(slow ability)(Attacks: physical, cold)(armour +75)(resist magic +100)(resist fire +100)(resist electricity +90)(resist cold +100)(resist confusion +100)(resist acid +90)(resist drain +100)(resist weaponmagic +80)(resist ghosthit +100)(resist poison +100)(resist slow +100)(resist paralyzation +100)(resist fear +100)(resist cancellation +65)(resist depletion +100)(resist death +100)(resist chaos +100)(resist counterspell +65)(resist god power +80)(resist blindness +100)",
         "(slow movement)(undead)(Attacks: physical)(resist cold +50)(resist fear +100)",
         "",
         "",
-        "(normal movement)(see invisible)(spellcaster)(Spell abilities:)(burning hands ability)(medium fireball ability)(fear ability)(Attacks: physical)(resist fire +100)(resist cold -100)(resist confusion -100)(resist fear +100)(resist blindness +50)",
+        "(normal movement)(see invisible)(spellcaster)(Spell abilities:)(breath flame)(medium fireball ability)(fear ability)(Attacks: physical)(resist fire +100)(resist cold -100)(resist confusion -100)(resist fear +100)(resist blindness +50)",
         "(fast movement)(see invisible)(wield weapon)(archer)(wear armour)(wear ring)(read scroll)(fires wand/rod/horn)(spellcaster)(Spell abilities:)(paralyze)(fear ability)(small lightning)(large lightning)(slow ability)(resist magic +50)(resist electricity +100)(resist fear +100)",
-        "(speed +6)(item_power +6)(armour +3)",
+        "(speed +6)(item_power +6)[color=#FF15CD](armour +3)[/color]",
         "(dam+3)(weapon speed 2)(Attacks: physical)",
         "(Con+2)(item_power +1)",
-        "(Str+2)(Dex-1)(dam+3)(item_power +2)(armour +3)",
-        "(Str+2)(dam+3)(item_power +1)(armour +3)",
-        "(Cha+1)(ac+1)(Spell regen penalty 4)(armour +4)(resist blindness +1)",
-        "(ac+5)(Max speed 0.70)(Spell regen penalty 30)(armour +40)",
+        "(Str+2)(Dex-1)(dam+3)(item_power +2)[color=#FF15CD](armour +3)[/color]",
+        "(Str+2)(dam+3)(item_power +1)[color=#FF15CD](armour +3)[/color]",
+        "(Cha+1)(ac+1)(Spell regen penalty 4)[color=#FF15CD](armour +4)[/color](resist blindness +1)",
+        "(ac+5)(Max speed 0.70)(Spell regen penalty 30)[color=#FF15CD](armour +40)[/color]",
         "(ac+1)(Max speed 1.20)",
-        "(ac+3)(Max speed 0.90)(Spell regen penalty 10)(armour +20)",
-        "(Cha-5)(ac+7)(item_power +10)(reflect spells)(reflect missiles)(armour +15)(resist fire +30)(resist drain +100)(resist ghosthit +80)",
-        "(ac+4)(item_power +6)(armour +10)(resist drain +100)(resist ghosthit +50)",
+        "(ac+3)(Max speed 0.90)(Spell regen penalty 10)[color=#FF15CD](armour +20)[/color]",
+        "(Cha-5)(ac+7)(item_power +10)(reflect spells)(reflect missiles)[color=#FF15CD](armour +15)[/color][color=red](resist fire +30)[/color][ul](resist drain +100)[/ul](resist ghosthit +80)",
+        "(ac+4)(item_power +6)[color=#FF15CD](armour +10)[/color][ul](resist drain +100)[/ul](resist ghosthit +50)",
         NULL };
 
     /* if you change the order, the result will quite certainly change, as the generation depends on the value returned
@@ -150,13 +150,13 @@ START_TEST(test_describe_item) {
         "(wc+1)(dam+2)(Attacks: physical)",
         "",
         "(dam+6)(weapon speed 9)(Attacks: physical)",
-        "(food+200)",
-        "(Con+2)(Cha-1)(dam+10)(item_power +15)(weapon speed 5)(regeneration+1)(Attacks: weaponmagic)(resist drain +100)(resist poison +30)",
+        "(food+70)",
+        "(Con+2)(Cha-1)(dam+10)(item_power +15)(weapon speed 5)(regeneration+1)(Attacks: weaponmagic)[ul](resist drain +100)[/ul][color=green](resist poison +30)[/color]",
         "",
-        "",
-        "(Str+1)(dam+9)(item_power +20)(weapon speed 6)(regeneration+1)(Attacks: electricity, drain)(resist magic +30)(resist electricity +30)(resist drain +100)",
-        "",
-        "(Str+1)(armour +3)",
+        "of adornment",
+        "(Str+1)(dam+9)(item_power +20)(weapon speed 6)(regeneration+1)(Attacks: electricity, drain)[color=#930C76](resist magic +30)[/color][color=blue](resist electricity +30)[/color][ul](resist drain +100)[/ul]",
+        "of adornment",
+        "(Str+1)[color=#FF15CD](armour +3)[/color]",
         "",
         NULL
     };
@@ -165,7 +165,7 @@ START_TEST(test_describe_item) {
         test = cctk_create_game_object(archs[check]);
         fail_unless(test != NULL, "couldn't create arch %s", archs[check]);
         SET_FLAG(test, FLAG_IDENTIFIED);
-        buf = stringbuffer_finish(describe_item(test, NULL, NULL));
+        buf = stringbuffer_finish(describe_item(test, NULL, 0, NULL));
 
         /* if you're adding items, uncomment that so make finding the good value easier. */
         /*
@@ -187,7 +187,7 @@ START_TEST(test_describe_item) {
         test = generate_treasure(list, 50);
         fail_if(test == NULL, "couldn't create item from treasure list %s", treasures[check]);
         SET_FLAG(test, FLAG_IDENTIFIED);
-        buf = stringbuffer_finish(describe_item(test, NULL, NULL));
+        buf = stringbuffer_finish(describe_item(test, NULL, 0, NULL));
 
         /* if you're adding lists, uncomment that so make finding the good value easier. */
         /*
@@ -385,15 +385,15 @@ START_TEST(test_describe_monster_rewrite) {
         ob->contr = &pl;
 
         old_describe_monster(ob, buf, sizeof(buf));
-        compat = stringbuffer_finish(describe_item(ob, NULL, NULL));
+        compat = stringbuffer_finish(describe_item(ob, NULL, 0, NULL));
         fail_unless(strcmp(buf, compat) == 0, "(compat) description change:\n%s\n  === vs ===\n%s", buf, compat);
         free(compat);
 
-        final = stringbuffer_finish(describe_monster(ob, NULL));
+        final = stringbuffer_finish(describe_monster(ob, 0, NULL));
 
         fail_unless(strcmp(buf, final) == 0, "description change: \"%s\" vs \"%s\"", buf, final);
         free(final);
-        object_free2(ob, FREE_OBJ_NO_DESTROY_CALLBACK | FREE_OBJ_FREE_INVENTORY);
+        object_free(ob, FREE_OBJ_NO_DESTROY_CALLBACK | FREE_OBJ_FREE_INVENTORY);
     }
 
 } END_TEST
