@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_check_object_c =
- *   "$Id: check_object.c 16239 2011-12-20 21:39:15Z ryo_saeba $";
+ *   "$Id$";
  */
 
 /*
@@ -442,8 +442,8 @@ END_TEST
  */
 START_TEST(test_object_update_turn_face) {
     object *ob1;
-    const New_Face *face1;
-    const New_Face *face2;
+    const Face *face1;
+    const Face *face2;
 
     ob1 = cctk_create_game_object("arrow");
     ob1->direction = 1;
@@ -857,7 +857,7 @@ START_TEST(test_object_replace_insert_in_map) {
 END_TEST
 
 /** This is the test to check the behaviour of the method
- *  object *object_split(object *orig_ob, uint32 nr);
+ *  object *object_split(object *orig_ob, uint32_t nr);
  */
 START_TEST(test_object_split) {
     object *first = NULL;
@@ -884,7 +884,7 @@ START_TEST(test_object_split) {
 END_TEST
 
 /** This is the test to check the behaviour of the method
- *  object *object_decrease_nrof(object *op, uint32 i);
+ *  object *object_decrease_nrof(object *op, uint32_t i);
  */
 START_TEST(test_object_decrease_nrof) {
     object *first = NULL;
