@@ -47,7 +47,7 @@ void stop_projectile(object *op) {
     if (object_was_destroyed(event, tag)) {
         if (event != op) {
             object_remove(op);
-            object_free2(op, FREE_OBJ_FREE_INVENTORY);
+            object_free(op, FREE_OBJ_FREE_INVENTORY);
         }
         return;
     }
@@ -77,8 +77,8 @@ void stop_projectile(object *op) {
 
 method_ret common_process_projectile(ob_methods *context, object *op) {
     object *tmp;
-    sint16 new_x, new_y;
-    int was_reflected, mflags;
+    int16_t new_x, new_y;
+    int mflags;
     mapstruct *m;
 
     if (op->map == NULL) {
@@ -91,7 +91,6 @@ method_ret common_process_projectile(ob_methods *context, object *op) {
     /* Calculate target map square */
     new_x = op->x+DIRX(op);
     new_y = op->y+DIRY(op);
-    was_reflected = 0;
 
     m = op->map;
     mflags = get_map_flags(m, &m, new_x, new_y, &new_x, &new_y);
@@ -120,9 +119,8 @@ method_ret common_process_projectile(ob_methods *context, object *op) {
                 op->direction = absdir(op->direction+4);
                 op->state = 0;
                 if (GET_ANIM_ID(op)) {
-                    SET_ANIMATION(op, op->direction);
+                    object_update_turn_face(op);
                 }
-                was_reflected = 1;   /* skip normal movement calculations */
             } else {
                 /* Attack the object. */
                 op = hit_with_arrow(op, tmp);
@@ -157,7 +155,7 @@ method_ret common_process_projectile(ob_methods *context, object *op) {
             while (retry < 2) {
                 int left, right, mflags;
                 mapstruct *m1;
-                sint16  x1, y1;
+                int16_t  x1, y1;
 
                 retry++;
 
@@ -198,7 +196,7 @@ method_ret common_process_projectile(ob_methods *context, object *op) {
             /* update object image for new facing */
             /* many thrown objects *don't *have more than one face */
             if (GET_ANIM_ID(op))
-                SET_ANIMATION(op, op->direction);
+                object_update_turn_face(op);
         } /* object is reflected */
     } /* object ran into a wall */
 
