@@ -1,35 +1,29 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
-    Copyright (C) 2007 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
-/** @file gate/gate.c
+/**
+ * @file
  * The implementation of the Gate and Timed Gate classes of objects.
  * @todo merge GATE and TIMED_GATE object types.
  */
-#include <global.h>
-#include <ob_methods.h>
-#include <ob_types.h>
-#include <sounds.h>
-#include <sproto.h>
+#include "global.h"
+
+#include <stdlib.h>
+
+#include "ob_methods.h"
+#include "ob_types.h"
+#include "sounds.h"
+#include "sproto.h"
 
 static method_ret gate_type_process(ob_methods *context, object *op);
 static method_ret timed_gate_type_process(ob_methods *context, object *op);
@@ -49,13 +43,13 @@ void init_type_gate(void) {
  * @return METHOD_OK
  */
 static method_ret gate_type_process(ob_methods *context, object *op) {
-    object *tmp;
+    object *tmp, *part;
 
     if (op->stats.wc < 0 || (int)op->stats.wc >= NUM_ANIMATIONS(op)) {
         StringBuffer *sb;
         char *diff;
 
-        LOG(llevError, "Gate error: animation was %d, max=%d\n", op->stats.wc, NUM_ANIMATIONS(op));
+        LOG(llevError, "Gate error: animation was %d, max=%d, on %s (%d, %d)\n", op->stats.wc, NUM_ANIMATIONS(op), map_get_path(op), op->x, op->y);
         sb = stringbuffer_new();
         object_dump(op, sb);
         diff = stringbuffer_finish(sb);
@@ -75,13 +69,17 @@ static method_ret gate_type_process(ob_methods *context, object *op) {
                 object_update_speed(op);
             }
         }
-        if ((int)op->stats.wc < (NUM_ANIMATIONS(op)/2+1)) {
-            op->move_block = 0;
-            CLEAR_FLAG(op, FLAG_BLOCKSVIEW);
-            update_all_los(op->map, op->x, op->y);
+
+        for (part = op; part != NULL; part = part->more) {
+            if ((int)op->stats.wc < (NUM_ANIMATIONS(op)/2+1)) {
+                part->move_block = 0;
+                CLEAR_FLAG(part, FLAG_BLOCKSVIEW);
+                update_all_los(part->map, part->x, part->y);
+            }
+            SET_ANIMATION(part, op->stats.wc);
         }
-        SET_ANIMATION(op, op->stats.wc);
         object_update(op, UP_OBJ_CHANGE);
+        play_sound_map(SOUND_TYPE_GROUND, op, 0, "gate moving");
         return METHOD_OK;
     }
 
@@ -168,15 +166,23 @@ static method_ret gate_type_process(ob_methods *context, object *op) {
             if (tmp) {
                 op->stats.food = 1;
             } else {
-                op->move_block = MOVE_ALL;
-                if (!op->arch->clone.stats.ac)
-                    SET_FLAG(op, FLAG_BLOCKSVIEW);
-                update_all_los(op->map, op->x, op->y);
+                object* part;
+                for (part = op; part != NULL; part = part->more) {
+                    if (!part->stats.luck)
+                        part->move_block = MOVE_ALL;
+                    if (!part->stats.ac) {
+                        SET_FLAG(part, FLAG_BLOCKSVIEW);
+                        update_all_los(part->map, part->x, part->y);
+                    }
+                }
             }
         } /* gate is halfway up */
 
-        SET_ANIMATION(op, op->stats.wc);
+        for (part = op; part != NULL; part = part->more) {
+            SET_ANIMATION(part, op->stats.wc);
+        }
         object_update(op, UP_OBJ_CHANGE);
+        play_sound_map(SOUND_TYPE_GROUND, op, 0, "gate moving");
     } /* gate is going up */
 
     return METHOD_OK;
