@@ -1,33 +1,29 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
-    Copyright (C) 2007 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-/** @file potion.c
+/**
+ * @file
  * The implementation of the Potion class of objects.
  */
-#include <global.h>
-#include <ob_methods.h>
-#include <ob_types.h>
-#include <sounds.h>
-#include <sproto.h>
+
+#include "global.h"
+
+#include <string.h>
+
+#include "ob_methods.h"
+#include "ob_types.h"
+#include "sounds.h"
+#include "sproto.h"
 
 static method_ret potion_type_apply(ob_methods *context, object *potion,
     object *applier, int aflags);
@@ -40,7 +36,7 @@ void init_type_potion(void) {
 }
 
 /**
- * Handles applying a potion.
+ * Handles applying a potion, dust, balm, or figurine.
  * @param context The method context
  * @param potion The potion to apply
  * @param applier The object attempting to apply the potion
@@ -57,6 +53,7 @@ static method_ret potion_type_apply(ob_methods *context, object *potion,
             potion = identify(potion);
     }
 
+    play_sound_map(SOUND_TYPE_ITEM, applier, 0, "drink");
     apply_handle_yield(potion);
 
     /* Potion of restoration - only for players */
@@ -114,7 +111,7 @@ static method_ret potion_type_apply(ob_methods *context, object *potion,
             if (got_one) {
                 fix_object(applier);
                 draw_ext_info(NDI_UNIQUE, 0, applier, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
-                              "The Gods smile upon you and remake you a little more in their image."
+                              "The Gods smile upon you and remake you a little more in their image. "
                               "You feel a little more perfect.");
             } else
                 draw_ext_info(NDI_UNIQUE, 0, applier, MSG_TYPE_APPLY, MSG_TYPE_APPLY_FAILURE,
@@ -167,8 +164,8 @@ static method_ret potion_type_apply(ob_methods *context, object *potion,
                 force = create_archetype(FORCE_NAME);
             memcpy(force->resist, potion->resist, sizeof(potion->resist));
             force->type = POTION_RESIST_EFFECT;
-            force->speed = 0.2;
-            force->duration = 100;
+            force->speed = MOVE_PER_SECOND;
+            force->duration = 60;
             break;  /* Only need to find one protection since we cappliery entire batch */
         }
     }
@@ -196,6 +193,11 @@ static method_ret potion_type_apply(ob_methods *context, object *potion,
         SET_FLAG(force, FLAG_APPLIED);
         change_abil(applier, force);
         object_decrease_nrof_by_one(potion);
+
+        if (potion->other_arch != NULL && applier->map != NULL) {
+            object_insert_in_map_at(arch_to_object(potion->other_arch), applier->map, NULL, INS_ON_TOP, applier->x, applier->y);
+        }
+
         return METHOD_OK;
     }
 
