@@ -24,11 +24,12 @@
 /** @file shop_mat.c
  * The implementation of the Shop Mat class of objects.
  */
-#include <global.h>
-#include <ob_methods.h>
-#include <ob_types.h>
-#include <sounds.h>
-#include <sproto.h>
+#include "global.h"
+#include "ob_methods.h"
+#include "ob_types.h"
+#include "shop.h"
+#include "sounds.h"
+#include "sproto.h"
 
 static method_ret shop_mat_type_move_on(ob_methods *context, object *trap, object *victim, object *originator);
 
@@ -89,7 +90,7 @@ static method_ret shop_mat_type_move_on(ob_methods *context, object *trap, objec
         rv = teleport(trap, SHOP_MAT, victim);
     /* immediate block below is only used for players */
     } else if (can_pay(victim)) {
-        get_payment(victim, victim->inv);
+        shop_pay_unpaid(victim, victim->inv);
         rv = teleport(trap, SHOP_MAT, victim);
         if (trap->msg) {
             draw_ext_info(NDI_UNIQUE, 0, victim, MSG_TYPE_APPLY, MSG_TYPE_APPLY_SUCCESS,
@@ -99,8 +100,8 @@ static method_ret shop_mat_type_move_on(ob_methods *context, object *trap, objec
          * but there is never a guarantee that the bottom space on the map is
          * actually the shop floor.
          */
-        else if (!rv && !is_in_shop(victim)) {
-            opinion = shopkeeper_approval(victim->map, victim);
+        else if (!rv && !shop_contains(victim)) {
+            opinion = shop_approval(victim->map, victim);
             if (opinion > 0.9)
                 draw_ext_info(NDI_UNIQUE, 0, victim, MSG_TYPE_SHOP, MSG_TYPE_SHOP_MISC,
                     "The shopkeeper gives you a friendly wave.");
