@@ -158,7 +158,7 @@ static method_ret spell_effect_type_process(ob_methods *context, object *op) {
 static void move_bolt(object *op) {
     object *tmp;
     int mflags;
-    sint16 x, y;
+    int16_t x, y;
     mapstruct *m;
 
     if (--(op->duration) < 0) {
@@ -167,9 +167,8 @@ static void move_bolt(object *op) {
         return;
     }
     hit_map(op, 0, op->attacktype, 1);
-    
-    if (op->weight)
-        check_spell_knockback(op);
+
+    check_spell_knockback(op);
 
     if (!op->direction)
         return;
@@ -260,7 +259,7 @@ static void move_bolt(object *op) {
  * @param op The bullet being moved.
  */
 static void move_bullet(object *op) {
-    sint16 new_x, new_y;
+    int16_t new_x, new_y;
     int mflags;
     mapstruct *m;
 
@@ -324,13 +323,12 @@ static void explosion(object *op) {
         return;
     }
     hit_map(op, 0, op->attacktype, 0);
-//     if (op->weight)
     check_spell_knockback(op);
 
     if (op->range > 0) {
         for (i = 1; i < 9; i++) {
-            sint16 dx, dy;
-	    sint16 Dx, Dy;
+            int16_t dx, dy;
+            int16_t Dx, Dy;
             dx = op->x+freearr_x[i];
             dy = op->y+freearr_y[i];
             /* ok_to_put_more already does things like checks for walls,
@@ -343,35 +341,35 @@ static void explosion(object *op) {
                 tmp->speed_left = -0.21;
                 tmp->range--;
                 tmp->value = 0;
-		Dx=dx-op->x;
-		Dy=dy-op->y;
-		if (Dx==-1 && Dy==-1){
-		  tmp->direction=8;
-		}
-		if (Dx==0 && Dy==-1){
-		  tmp->direction=1;
-		}
-		if (Dx==1 && Dy==-1){
-		  tmp->direction=2;
-		}
-		if (Dx==1 && Dy==0){
-		  tmp->direction=3;
-		}
-		if (Dx==1 && Dy==1){
-		  tmp->direction=4;
-		}
-		if (Dx==0 && Dy==1){
-		  tmp->direction=5;
-		}
-		if (Dx==-1 && Dy==-1){
-		  tmp->direction=6;
-		}
-		if (Dx==-1 && Dy==0){
-		  tmp->direction=7;
-		}
-		
+                Dx=dx-op->x;
+                Dy=dy-op->y;
+                if (Dx==-1 && Dy==-1){
+                    tmp->direction=8;
+                }
+                if (Dx==0 && Dy==-1){
+                    tmp->direction=1;
+                }
+                if (Dx==1 && Dy==-1){
+                    tmp->direction=2;
+                }
+                if (Dx==1 && Dy==0){
+                    tmp->direction=3;
+                }
+                if (Dx==1 && Dy==1){
+                    tmp->direction=4;
+                }
+                if (Dx==0 && Dy==1){
+                    tmp->direction=5;
+                }
+                if (Dx==-1 && Dy==-1){
+                    tmp->direction=6;
+                }
+                if (Dx==-1 && Dy==0){
+                    tmp->direction=7;
+                }
+
                 object_insert_in_map_at(tmp, m, op, 0, dx, dy);
-		
+
             }
         }
         /* Reset range so we don't try to propogate anymore.
@@ -390,7 +388,7 @@ static void explosion(object *op) {
 static void move_cone(object *op) {
     int i;
     tag_t tag;
-    
+
     /* if no map then hit_map will crash so just ignore object */
     if (!op->map) {
         LOG(llevError, "Tried to move_cone object %s without a map.\n", op->name ? op->name : "unknown");
@@ -412,9 +410,7 @@ static void move_cone(object *op) {
      * Spell objects with weight push whatever they encounter to some
      * degree.
      */
-    if (op->weight)
-	
-        check_spell_knockback(op);
+    check_spell_knockback(op);
 
     if (object_was_destroyed(op, tag))
         return;
@@ -424,6 +420,7 @@ static void move_cone(object *op) {
         object_free_drop_inventory(op);
         return;
     }
+
     /* Object has hit maximum range, so don't have it move
      * any further.  When the duration above expires,
      * then the object will get removed.
@@ -434,8 +431,8 @@ static void move_cone(object *op) {
     }
 
     for (i = -1; i < 2; i++) {
-        sint16 x = op->x+freearr_x[absdir(op->stats.sp+i)];
-        sint16 y = op->y+freearr_y[absdir(op->stats.sp+i)];
+        int16_t x = op->x+freearr_x[absdir(op->stats.sp+i)];
+        int16_t y = op->y+freearr_y[absdir(op->stats.sp+i)];
 
         if (ok_to_put_more(op->map, x, y, op, op->attacktype)) {
             object *tmp = object_new();
@@ -511,7 +508,7 @@ static void animate_bomb(object *op) {
  */
 static void move_missile(object *op) {
     int i, mflags;
-    sint16 new_x, new_y;
+    int16_t new_x, new_y;
     mapstruct *m;
 
     if (op->range-- <= 0) {
@@ -566,12 +563,12 @@ static void execute_word_of_recall(object *op) {
     while (op != NULL && op->type != PLAYER)
         op = op->env;
 
-    if (op != NULL && op->map) {
-        if ((get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL)&P_NO_CLERIC) && (!QUERY_FLAG(op, FLAG_WIZCAST)))
-            draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_SPELL, MSG_TYPE_SPELL_FAILURE,
-                "You feel something fizzle inside you.");
-        else
-            enter_exit(op, wor);
+    if (op != NULL) {
+        // Drop any unpaid items that would be carried.
+        // This prevents abuse of balms of return home in shops.
+        remove_unpaid_objects(op->inv, op, 0);
+
+        enter_exit(op, wor);
     }
     object_remove(wor);
     object_free_drop_inventory(wor);
@@ -584,7 +581,7 @@ static void execute_word_of_recall(object *op) {
  */
 static void move_ball_spell(object *op) {
     int i, j, dam_save, dir, mflags;
-    sint16 nx, ny, hx, hy;
+    int16_t nx, ny, hx, hy;
     object *owner;
     mapstruct *m;
 
@@ -684,7 +681,7 @@ static void move_ball_spell(object *op) {
 static void move_swarm_spell(object *op) {
     static int cardinal_adjust[9] = { -3, -2, -1, 0, 0, 0, 1, 2, 3 };
     static int diagonal_adjust[10] = { -3, -2, -2, -1, 0, 0, 1, 2, 2, 3 };
-    sint16 target_x, target_y, origin_x, origin_y;
+    int16_t target_x, target_y, origin_x, origin_y;
     int basedir, adjustdir;
     mapstruct *m;
     object *owner;
@@ -782,7 +779,7 @@ static void move_aura(object *aura) {
     object_insert_in_map_at(aura, env->map, aura, 0, env->x, env->y);
 
     for (i = 1; i < 9; i++) {
-        sint16 nx, ny;
+        int16_t nx, ny;
 
         nx = aura->x+freearr_x[i];
         ny = aura->y+freearr_y[i];
@@ -819,7 +816,7 @@ static void forklightning(object *op, object *tmp) {
     int new_dir = 1;  /* direction or -1 for left, +1 for right 0 if no new bolt */
     int t_dir; /* stores temporary dir calculation */
     mapstruct *m;
-    sint16 sx, sy;
+    int16_t sx, sy;
     object *new_bolt;
 
     /* pick a fork direction.  tmp->stats.Con is the left bias
@@ -871,14 +868,14 @@ static void check_spell_knockback(object *op) {
     int weight_move;
     int frictionmod = 2; /*poor man's physics - multipy targets weight by this amount */
 
-    if (!op->weight) { /*shouldn't happen but if cone object has no weight drop out*/
-        LOG(llevDebug, "DEBUG: arch weighs nothing in check_spell_knockback\n");
+    /* if cone object has no weight drop out */
+    if (!op->weight) {
         return;
-    } else {
-        weight_move = op->weight+(op->weight*op->level)/3;
-        /*LOG(llevDebug, "DEBUG: arch weighs %d and masses %d (%s,level %d)\n", op->weight, weight_move, op->name, op->level);*/
     }
 
+    weight_move = op->weight+(op->weight*op->level)/3;
+    /*LOG(llevDebug, "DEBUG: arch weighs %d and masses %d (%s,level %d)\n", op->weight, weight_move, op->name, op->level);*/
+
     for (tmp = GET_MAP_OB(op->map, op->x, op->y); tmp != NULL; tmp = tmp->above) {
         int num_sections = 1;
 
@@ -915,18 +912,18 @@ static void check_spell_knockback(object *op) {
              * I don't see us doing anything useful with that information
              * right now.
              */
-// 	    LOG(llevDebug, "trying move\n");
-	    if (op->direction){
-	      move_object(tmp,absdir(op->direction));
-	    }
-	    
+//             LOG(llevDebug, "trying move\n");
+            if (op->direction){
+                move_object(tmp,absdir(op->direction));
+            }
+
             else {
-	      (move_object(tmp, absdir(op->stats.sp)));
-	      
-	    }
+                (move_object(tmp, absdir(op->stats.sp)));
+
+            }
         }
         else{
-// 	  LOG(llevDebug, "did not try move, don't know why\n");
-	}
+//           LOG(llevDebug, "did not try move, don't know why\n");
+        }
     }
 }
