@@ -1,38 +1,32 @@
 /*
-    CrossFire, A Multiplayer game for X-windows
+ * Crossfire -- cooperative multi-player graphical RPG and adventure game
+ *
+ * Copyright (c) 1999-2014 Mark Wedel and the Crossfire Development Team
+ * Copyright (c) 1992 Frank Tore Johansen
+ *
+ * Crossfire is free software and comes with ABSOLUTELY NO WARRANTY. You are
+ * welcome to redistribute it under certain conditions. For details, please
+ * see COPYING and LICENSE.
+ *
+ * The authors can be reached via e-mail at <crossfire@metalforge.org>.
+ */
 
-    Copyright (C) 2007 Mark Wedel & Crossfire Development Team
-    Copyright (C) 1992 Frank Tore Johansen
-
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-    The authors can be reached via e-mail at crossfire-devel@real-time.com
-*/
-
-/** @file weapon_improver.c
+/**
+ * @file
  * The implementation of the Weapon Improver class of objects.
  */
-#include <global.h>
-#include <ob_methods.h>
-#include <ob_types.h>
-#include <sounds.h>
-#include <sproto.h>
+#include "global.h"
+
+#include <string.h>
+
+#include "ob_methods.h"
+#include "ob_types.h"
+#include "sounds.h"
+#include "sproto.h"
 
 static method_ret weapon_improver_type_apply(ob_methods *context, object *op, object *applier, int aflags);
 static int check_item(object *op, const char *item);
-static void eat_item(object *op, const char *item, uint32 nrof);
+static void eat_item(object *op, const char *item, uint32_t nrof);
 static int check_sacrifice(object *op, const object *improver);
 static int improve_weapon_stat(object *op, object *improver, object *weapon, signed char *stat, int sacrifice_count, const char *statname);
 static int prepare_weapon(object *op, object *improver, object *weapon);
@@ -133,7 +127,7 @@ static int check_item(object *op, const char *item) {
  * couldn't item be a shared string, and use == instead of strcmp?
  * also, the remove logic is wrong - op->nrof will be 0 after decreat_ob_nr in the 2nd case.
  */
-static void eat_item(object *op, const char *item, uint32 nrof) {
+static void eat_item(object *op, const char *item, uint32_t nrof) {
     object *prev;
 
     prev = op;
@@ -391,6 +385,7 @@ static int improve_weapon(object *op, object *improver, object *weapon) {
 
         weapon->item_power++;
         object_decrease_nrof_by_one(improver);
+        esrv_update_item(UPD_WEIGHT, op, weapon);
         return 1;
     }
 
@@ -405,6 +400,7 @@ static int improve_weapon(object *op, object *improver, object *weapon) {
         weapon->last_eat++;
         weapon->item_power++;
         object_decrease_nrof_by_one(improver);
+        esrv_update_item(UPD_WEIGHT, op, weapon);
         return 1;
     }
 
